/*												-*- c++ -*-
 * File:    wx_win.h
 * Purpose: 
 * Author:  Julian Smart
 * Created: 1993
 * Updated: 
 * Copyright:   (c) 1993, AIAI, University of Edinburgh
 */

/* sccsid[] = "%W% %G%" */

/*
 * Purpose:  wxWindow class declaration. Base class for all windows and
 *           panel items.
 */


#ifndef wx_winh
#define wx_winh

#include "wb_win.h"

#include <os2.h>

/*
 * Base class for frame, panel, canvas, panel items, dialog box.
 *
 */

#ifdef IN_CPROTO
typedef       void    *wxWindow ;
#else

class wxWindow: public wxbWindow
{
    DECLARE_ABSTRACT_CLASS(wxWindow)
protected:
    Bool inOnSize; // Protection against OnSize reentry

public:
    int windows_id;
    Bool mouseInWindow;
    Bool winEnabled;
    int wxx, wxy, wxh, wxw;	// wxWindows's opinion of the position and size of this window

    // Caret data
    int caretWidth;
    int caretHeight;
    Bool caretEnabled;
    Bool caretShown;

    virtual BOOL OS2Control(WORD param, WORD id);
    virtual wxWindow *FindItem(int id);
    virtual wxWindow *FindItemByHWND(HWND hWnd);
    virtual wxWindow *FindItemByStatic(HWND hWnd);
    virtual HWND GetHWND(void);

    // Constructors/Destructors
    wxWindow(void);
    virtual ~wxWindow(void);

    virtual Bool Show(Bool show);
    virtual wxCursor *SetCursor(wxCursor *cursor);
    virtual void SetColourMap(wxColourMap *cmap);

    virtual float GetCharHeight(void);
    virtual float GetCharWidth(void);
    virtual void GetTextExtent(const char *string, float *x, float *y,
			       float *descent = NULL, float *externalLeading = NULL,
			       wxFont *theFont = NULL, Bool use16bit = FALSE);

    void GetSize(int *width, int *height);
    void GetPosition(int *x, int *y);
    void GetClientSize(int *width, int *height); // Size client can use
    void SetSize(int x, int y, int width, int height, int sizeFlags = wxSIZE_AUTO);
    void SetSize(int width, int height) { wxbWindow::SetSize(width, height); }
    void SetClientSize(int width, int size);
    void ClientToScreen(int *x, int *y);
    void ScreenToClient(int *x, int *y);
    void SetFocus(void);
    void CaptureMouse(void);
    void ReleaseMouse(void);
    void Enable(Bool enable);
    void DragAcceptFiles(Bool accept);
    inline void SetTitle(const char *WXUNUSED(title)) {};
    inline void Fit(void) {};
    inline void Centre(int WXUNUSED(direction)) {};
    Bool PopupMenu(wxMenu *menu, float x, float y);

    void Refresh(void);

    void OnScroll(wxCommandEvent& event);
    void SetScrollPos(int orient, int pos);
    void SetScrollRange(int orient, int range);
    void SetScrollPage(int orient, int page);
    int GetScrollPos(int orient);
    int GetScrollRange(int orient);
    int GetScrollPage(int orient);

    // Caret manipulation
    void CreateCaret(int w, int h);
    void CreateCaret(wxBitmap *bitmap);
    void DestroyCaret(void);
    void ShowCaret(Bool show);
    void SetCaretPos(int x, int y);
    void GetCaretPos(int *x, int *y);

    // The default implementation sets scroll ranges, if any
    void OnSize(int w, int h);

    // Internal function to update scrollbars
    void DoScroll(wxCommandEvent& event);

    // Calculate scroll increment
    int CalcScrollInc(wxCommandEvent& event);

    void MoveWindow(int x, int y, int h, int w, Bool show = FALSE);

    // Presentation Parameters
    virtual void SetBackgroundColour(const wxColour& col);
    virtual void OnBackgroundColourChange(const wxColour& col);
    virtual void OnFontChange(const wxFont& fnt);
};

// Window specific (so far)
wxWindow *wxGetActiveWindow(void);

// Allows iteration through damaged rectangles in OnPaint
class wxUpdateIterator
{
  int rects;				// How many rects in Update region
  int current;				// Current rectangle index
  RECTL rectlUpdate;			// current rectangle

 public:
  wxUpdateIterator(wxWindow* wnd);
  ~wxUpdateIterator(void);

  operator int (void);
  wxUpdateIterator* operator ++(int);
  void GetRect(wxRectangle *rect);
  int GetX();
  int GetY();
  int GetW();
  int GetH();
};

int wxCharCodeOS2ToWX(int keySym);
int wxCharCodeWXToOS2(int id, Bool *IsVirtual);

void wxGetCharSize(HWND wnd, int *x, int *y);
LONG wxGetCharHeight(HWND);
LONG wxGetCharWidth(HWND);
void wxGetTextExtents(HWND wnd, const char *str, int *cx, int *cy);
int wxGetTextWidth(HWND hwnd, const char *str = NULL);

// Allocates control ids
int NewControlId(void);

#endif // IN_CPROTO
#endif
