/* -*-c++-*-
 * File:    wx_dc.h
 * Purpose:    wxDC device context declaration
 * Author:    Julian Smart
 * Created:    1993
 * Updated:    
 * Copyright:    (c) 1993, AIAI, University of Edinburgh
 */

/* sccsid[] = "%W% %G%" */


#ifndef wx_dch
#define wx_dch

#include "wx_gdi.h"
#include "wb_dc.h"

#ifdef IN_CPROTO
typedef       void    *wxDC ;
typedef       void    *wxPrinterDC;
#else

// Since Windows handles DCs quite uniformly, we can have
// a general wxWindowsDC, and derive canvas and printer DCs from this
// with minimum extra functionality.

/*
 * The wxDC class holds information for one PM presentation
 * space. a wxDC may or may not be connected with a wxWnd.
 * Screen wxDC's always are connected with a wxWnd, but
 * memory and printer wxDC's are not.
 */
class wxDC: public wxbDC
{
    DECLARE_ABSTRACT_CLASS(wxDC)

public:
    int clip_x1;
    int clip_y1;
    int clip_x2;
    int clip_y2;
    Bool dont_delete;
    float system_scale_x;
    float system_scale_y;

    wxCanvas *canvas;
    wxBitmap *selected_bitmap;
    char *filename;

    HPS hpsDC;				// Presentation space associated with the DC

#if 0
    HDC      cur_dc ;
    COLORREF cur_bk ;
#else
    UINT     cur_cpen ;
    UINT     cur_cbrush ;
    HPS      cur_ps;			/* PS cur_ is valid for */
    LONG     cur_bk;			/* Current Backgroup Color */
#endif

    wxDC(void);
    ~wxDC(void);

    virtual void BeginDrawing(void) ;
    virtual void EndDrawing(void) ;

    void FloodFill(float x1, float y1, wxColour *col, int style=wxFLOOD_SURFACE) ;
    Bool GetPixel(float x1, float y1, wxColour *col) ;

    void DrawLine(float x1, float y1, float x2, float y2);
    void IntDrawLine(int x1, int y1, int x2, int y2);
    void CrossHair(int x, int y) ;
    void DrawArc(float x1,float y1,float x2,float y2,float xc,float yc);
    void DrawPoint(float x, float y);

    void DrawLines(int n, wxPoint points[], float xoffset = 0, float yoffset = 0);
    void DrawLines(int n, wxIntPoint points[], int xoffset = 0, int yoffset = 0);
    // MS C7 complains if this overloaded function isn't explicitly mentioned
    inline void DrawLines(wxList *list, float xoffset = 0, float yoffset = 0)
    { wxbDC::DrawLines(list, xoffset, yoffset); }

    void DrawPolygon(int n, wxPoint points[], float xoffset = 0, float yoffset = 0, int fillStyle=wxODDEVEN_RULE);
    // See MS C7 comment above
    inline void DrawPolygon(wxList *list, float xoffset = 0, float yoffset = 0, int fillStyle=wxODDEVEN_RULE)
    { wxbDC::DrawPolygon(list, xoffset, yoffset,fillStyle); }

    void DrawRectangle(float x, float y, float width, float height);
    void DrawRoundedRectangle(float x, float y, float width, float height, float radius = 20.0);
    void DrawEllipse(float x, float y, float width, float height);

    void DrawIcon(wxIcon *icon, float x, float y);

    void Clear(void);
    void SetFont(wxFont *font);
    void SetPen(wxPen *pen);
    void SetBrush(wxBrush *brush);
    void SetLogicalFunction(int function);
    void SetBackground(wxBrush *brush);
    void SetBackgroundMode(int mode);
    void SetClippingRegion(float x, float y, float width, float height);
    void SetColourMap(wxColourMap *cmap);
    void DestroyClippingRegion(void);
    void DrawText(const char *text, float x, float y, Bool use16bit = FALSE);

    float GetCharHeight(void);
    float GetCharWidth(void);
    void GetTextExtent(const char *string, float *x, float *y,
		       float *descent = NULL, float *externalLeading = NULL,
		       wxFont *theFont = NULL, Bool use16bit = FALSE);
    void GetSize(float *width, float *height);
    void GetSizeMM(float *width, float *height);
    Bool StartDoc(char *message);
    void EndDoc(void);
    void StartPage(void);
    void EndPage(void);
    void SetMapMode(int mode);
    void SetUserScale(float x, float y);
    void SetSystemScale(float x, float y);
    void SetLogicalOrigin(float x, float y);
    void SetDeviceOrigin(float x, float y);

    /*
     * Coordinate system transformations
     */
    float DeviceToLogicalX(int x);
    float DeviceToLogicalY(int y);
    float DeviceToLogicalXRel(int x);
    float DeviceToLogicalYRel(int y);
    int LogicalToDeviceX(float x);
    int LogicalToDeviceY(float y);
    int LogicalToDeviceXRel(float x);
    int LogicalToDeviceYRel(float y);

    /*
     * Transformation between WX and PM coordinate systems
     *
     * the X versions are just here for symmetri
     */
    inline int WxToPmY(int y);//		{ (wxWnd *)canvas->handle->WxToPmY(y); }
    inline int WxToPmX(int x);//     	{ return x; }
    inline int PmToWxY(int y);//		{ (wxWnd *)canvas->handle->PmToWxY(y); }
    inline int PmToWxX(int x);//     	{ return x; }

    Bool Blit(float xdest, float ydest, float width, float height,
	      wxCanvasDC *source, float xsrc, float ysrc, int rop = wxCOPY);
            
    Bool CanDrawBitmap(void);
    Bool CanGetTextExtent(void);

    void SetRop(HPS cps);
    void DoClipping(HPS cps);
    virtual HPS GetHPS(void);
    virtual void FreeHPS(void);
    void SetupPS(HPS hps);
};

// This class specific to Windows 3.1
class wxPrinterDC: public wxDC
{
 public:
  DECLARE_CLASS(wxPrinterDC)

  // Create a printer DC
  wxPrinterDC(char *driver, char *device, char *output, Bool interactive = TRUE);
  wxPrinterDC(HDC theDC);

  ~wxPrinterDC(void);
};

// Gets an HDC for the default printer configuration
HDC wxGetPrinterDC(void);

// Logical to device
// Absolute
#define XLOG2DEV(x) (int)(floor(x))

#define YLOG2DEV(y) (int)(floor(y))

// Relative
#define XLOG2DEVREL(x) (int)(floor(x))
#define YLOG2DEVREL(y) (int)(floor(y))

// Device to logical
// Absolute
#define XDEV2LOG(x) (x)

#define YDEV2LOG(y) (y)

// Relative
#define XDEV2LOGREL(x) (x)
#define YDEV2LOGREL(y) (y)

#endif // IN_CPROTO
#endif // wx_dc.h

