/* -*- c++ -*-
 * File:    wx_item.h
 * Purpose:    Declares base panel item class
 * Author:    Julian Smart
 * Created:    1993
 * Updated:    
 * Copyright:    (c) 1993, AIAI, University of Edinburgh
 */

/* sccsid[] = "@(#)wx_item.h    1.2 5/9/94" */

#ifndef wx_itemh
#define wx_itemh

#include "wb_item.h"

#ifdef IN_CPROTO
typedef       void    *wxItem ;
#else

// General item class
class wxBrush;
class wxFont;
class wxColour;
class wxItem: public wxbItem
{
    DECLARE_ABSTRACT_CLASS(wxItem)

protected:

    HWND hwndLabel;			// Label handle, or NULLHANDLE if no label
    int	label_x;
    int	label_y;
    int	label_width;			// Width of label
    int	label_height;			// Height of label
    int fixed_length_label_size;	// Number of chars in fixed length label, or 0

    void RefreshLabelSize(const char *text = NULL);

public:
    HWND hwndItem;			// Item window handle

    wxList subControls;			// For controls like radiobuttons which are really composite
    PFNWP oldWndProc;			// For subclassed controls
    wxBrush *backBrush;
    int os2LastXPos, os2LastYPos;
    int os2LastEvent;
    ULONG buttonstate;			// State of the buttons (used in subclass)

    int currentX, currentY, currentW, currentH;	// "Current" size/position for the whole item set

//   HWND labelhWnd;
//   HBRUSH backgroundBrush;

    wxItem(void);
    ~wxItem(void);

    void GetSize(int *width, int *height);
    void GetPosition(int *x, int *y);
    void SetSize(int x, int y, int width, int height, int sizeFlags = wxSIZE_AUTO);
    virtual void GetAutoSize(int *cx, int *cy);
    virtual void PositionItem(int x, int y, int width, int height);
    void SetClientSize(int width, int height);
    void SetFocus(void);
    void SetLabel(char *label);
    char *GetLabel(void);
    void AdjustLabelPos(int *x, int *y, int *width, int *height);
    void CreateLabel(HWND hwndParent, const char *text, const Bool isFixedLength = FALSE);

    Bool Show(Bool show = TRUE);
    float GetCharHeight(void);
    float GetCharWidth(void);
    void GetTextExtent(char *string, float *x, float *y,
		       float *descent = NULL, float *externalLeading = NULL,
		       wxFont *the_font=NULL);

    virtual void SetBackgroundColour(wxColour& col);
    virtual void SetLabelColour(wxColour& col);
    virtual void SetButtonColour(wxColour& col);

    virtual void SetLabelFont(wxFont *font);
    virtual void SetButtonFont(wxFont *font);

    virtual void Enable(Bool enable);

    inline virtual HWND GetHWND(void) { return hwndItem; }
    inline virtual HWND GetLabelHWND(void) { return hwndLabel; };

    // Windows subclassing
    void SubclassControl(HWND hWnd);
#if 0
    // For ownerdraw items
    virtual Bool MSWOnDraw(DRAWITEMSTRUCT *item) { return FALSE; };
    virtual Bool MSWOnMeasure(MEASUREITEMSTRUCT *item) { return FALSE; };
#endif
   
//   virtual HBRUSH GetBackgroundHBRUSH(void) { return backgroundBrush; }
   
    inline Bool HaveLabel(void) { return hwndLabel != NULLHANDLE; }
};

#endif // IN_CPROTO
#endif // wx_itemh
