/* -*- c++ -*-
 * File:        wx_privt.h
 * Purpose:     Private class declarations.
 * Author:      Julian Smart
 * Created:     1993
 * Updated:    
 * Copyright:   (c) 1993, AIAI, University of Edinburgh
 */

/* sccsid[] = "@(#)wx_privt.h    1.2 5/9/94" */

#ifndef wx_privth
#define wx_privth

#include "common.h"
#include "wx_obj.h"

/*
 * These classes are exactly like the os/2 classes
 * RECTL and POINTL, except that they use
 * the wxWindows coordinate system.
 */
struct wxRECTL {
	LONG	xLeft;
	LONG	yBottom;
	LONG	xRight;
	LONG	yTop;
};

struct wxPOINTL {
	LONG	x;
	LONG	y;
};

#define wxMAX_STATUS   5

void wx_WindowRect(HWND, RECTL *, HWND = NULLHANDLE);
void wx_WindowRect(HWND, wxRECTL *, HWND = NULLHANDLE);
void wx_ClientRect(HWND, RECTL *);
void wx_ClientRect(HWND, wxRECTL *);
void wx_MoveWindow(HWND, int, int, int, int, Bool show = FALSE);
void wx_GetWindowSize(HWND, int *, int *);
void wx_SetWindowSize(HWND, int width, int height);

inline int rcHeight(RECTL rc) { return rc.yTop - rc.yBottom; };
inline int rcHeight(PRECTL rc) { return rc->yTop - rc->yBottom; };
inline int rcWidth(RECTL rc)  { return rc.xRight - rc.xLeft; }
inline int rcWidth(PRECTL rc) { return rc->xRight - rc->xLeft; }

inline int rcHeight(wxRECTL rc) { return rc.yBottom - rc.yTop; };
inline int rcHeight(wxRECTL *rc) { return rc->yBottom - rc->yTop; };
inline int rcWidth(wxRECTL rc)  { return rc.xRight - rc.xLeft; }
inline int rcWidth(wxRECTL *rc) { return rc->xRight - rc->xLeft; }

void wx_ScreenToClient(HWND hwnd, wxPOINTL *ppoint);
void wx_ClientToScreen(HWND hwnd, wxPOINTL *ppoint);

#ifdef IN_CPROTO
typedef       void    *wxWnd;
typedef       void    *wxSubWnd;
typedef       void    *wxCanvasWnd;
typedef	      void    *wxPanelWnd;
typedef       void    *wxFrameWnd;
typedef       void    *wxMDIFrameWnd;
typedef       void    *wxMDIChildWnd;
#else

class wxWindow;
class wxBrush;
class wxColour;

/*
 * The wxWnd class is holding the information for 
 * one os/2 window.
 */
class wxWnd : public wxObject
{
public:
    UINT last_msg;			// Parameters from the last window message
    MPARAM last_mp1;
    MPARAM last_mp2;

    Bool x_scrolling_enabled;
    Bool y_scrolling_enabled;
    Bool calcScrolledOffset;		// If TRUE, wxCanvasDC uses scrolled offsets

    int xscroll_pixels_per_line;
    int yscroll_pixels_per_line;
    int xscroll_lines;
    int yscroll_lines;
    int xscroll_lines_per_page;
    int yscroll_lines_per_page;
    int xscroll_position;
    int yscroll_position;
    float last_x_pos;
    float last_y_pos;
    int last_event;

    HWND hwndWnd;			// OS/2 window handle
    HPOINTER os2_cursor;		// Cursor that should be used for this window

    int win_height;			// Height of window
    int win_width;			// Width of window
    
    ULONG buttonstate;			// Button state of this window
#define	BUTTON1	1
#define	BUTTON2	2
#define	BUTTON3	4

    // GetClient: returns handle for all windows, hwndClient for frame windows
    virtual HWND GetClientHWND() const	{ return hwndWnd; };
    virtual HWND GetFrameHWND() const	{ assert(FALSE); return NULLHANDLE; }

    HACCEL accelerator_table;
    HWND hwndMenu;			// Menu, if any

    Bool userColours;			// Usually FALSE, wxUSER_COLOURS overrides CTL3D etc. settings
//    wxBrush *background_brush;
    COLOR background_colour;		// The current background colour presentation parameter

    Bool background_transparent;
    Bool canDeleteBackgroundBrush;

    wxWindow *wx_window;		// Back pointer to wxWindow class

    // The following two are only valid in a WM_PAINT
    // hpsUpdate is NULL otherwise
    HPS hpsUpdate;			// Update presentation space.
    RECTL rectlUpdate;			// Current update rectangle

    wxWnd(void);
    ~wxWnd(void);

    void Create(wxWnd *parent, char *wclass, wxWindow *wx_win, const char *title,
		int x, int y, int width, int height,
		ULONG style, char *dialog_template = NULL,
		ULONG exendedStyle = 0);

    // Calculates the position of a point on the window
    // taking into account the position of scrollbars.
    // Windows doesn't automatically reflect the position of the
    // scrollbars - (0, 0) is always the top left of the visible window,
    // whereas in XView, (0, 0) moves according to scrollbar positions.
    virtual void CalcScrolledPosition(int x, int y, int *xx, int *yy);

    // Actually defined in wx_canvs.cc since requires wxCanvas declaration
    void DeviceToLogical(float *x, float *y);

    // Calculate logical (scroll-bar/scaling aware) position from
    // device (pixel) position
    virtual void CalcUnscrolledPosition(int x, int y, float *xx, float *yy);

    // Handlers
    virtual void OnCreate(PCREATESTRUCT cs);
    virtual BOOL OnPaint(HWND hwnd);
    virtual void OnSize(int x, int y, UINT flag);
    virtual void OnHScroll(int uscmd, int sslider, int usidentifier);
    virtual void OnVScroll(int uscmd, int sslider, int usidentifier);
    virtual BOOL OS2Command(int cmd, int source, int pointer);
    virtual UINT OnCtlColor(HPS dc, HWND pWnd, UINT nCtlColor,
			    ULONG msg, MPARAM mp1, MPARAM mp2);
    virtual BOOL OnColorChange(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2);
    virtual BOOL OnEraseBkgnd(HPS ps, PRECTL pprcPaint = NULL);
    virtual void OnMenuSelect(ULONG item, ULONG flags, HWND sysmenu);
//    virtual void OnInitMenuPopup(HWND menu, int pos, Bool isSystem);
    virtual BOOL OnClose(void);
    virtual BOOL OnDestroy(void);
    virtual BOOL OnSetFocus(HWND wnd);
    virtual BOOL OnKillFocus(HWND wnd);
    virtual void OnDropFiles(MPARAM mp1);

    // Canvas-type events
    virtual void OnLButtonDown(int x, int y, UINT flags);
    virtual void OnLButtonUp(int x, int y, UINT flags);
    virtual void OnLButtonDClick(int x, int y, UINT flags);

    virtual void OnMButtonDown(int x, int y, UINT flags);
    virtual void OnMButtonUp(int x, int y, UINT flags);
    virtual void OnMButtonDClick(int x, int y, UINT flags);

    virtual void OnRButtonDown(int x, int y, UINT flags);
    virtual void OnRButtonUp(int x, int y, UINT flags);
    virtual void OnRButtonDClick(int x, int y, UINT flags);

    virtual void OnMouseMove(int x, int y, UINT flags);
    virtual void OnMouseEnter(int x, int y, UINT flags);
    virtual void OnMouseLeave(int x, int y, UINT flags);

    virtual void OnChar(USHORT chvk, Bool isASCII = FALSE);

    virtual BOOL OnActivate(BOOL flag, BOOL minimized, HWND activate);
    virtual BOOL OnMDIActivate(BOOL flag, HWND activate, HWND deactivate);

#if 0
    virtual BOOL OnDrawItem(int id, DRAWITEMSTRUCT *item);
    virtual BOOL OnMeasureItem(int id, MEASUREITEMSTRUCT *item);
#endif

    virtual MRESULT DefWindowProc(ULONG nMsg, MPARAM mp1, MPARAM mp2);
    virtual BOOL ProcessMessage(QMSG* pMsg);
    virtual void DestroyWindow(void);

    // Set background brush, deleting old one if necessary and
    // noting whether we can delete the new one in future.
//    void SetBackgroundBrush(wxBrush *br, Bool canDelete = FALSE);
    void SetBackgroundColour(const wxColour& col);
    wxColour GetBackgroundColour(void);

    // Detach "Window" menu from menu bar so it doesn't get deleted
    void DetachWindowMenu(void);

    /*
     * Coordinate transformation routines
     * the X versions are here just for symmetri
     */
    inline int WxToPmY(int y) const	{ return win_height - y; }
    inline int WxToPmX(int x) const	{ return x; }
    inline int PmToWxY(int y) const     { return win_height - y; }
    inline int PmToWxX(int x) const     { return x; }

    void OS2BackgroundColourChanged(void);
    void OS2FontChanged(void);
};


/*
 * This is a Windows 3 subwindow - panel or canvas
 */

class wxSubWnd : public wxWnd
{
public:
    Bool mouse_in_window ;

    wxSubWnd(wxWnd *parent, char *wclass, wxWindow *wx_win,
                int x, int y, int width, int height,
                ULONG style, char *dialog_template = NULL);
    ~wxSubWnd(void);

    // Handlers
    virtual BOOL OnPaint(HWND hwnd);
    void OnSize(int x, int y, UINT flag);
    BOOL OS2Command(int cmd, int source, int pointer);

    // Canvas-type events
    void OnLButtonDown(int x, int y, UINT flags);
    void OnLButtonUp(int x, int y, UINT flags);
    void OnLButtonDClick(int x, int y, UINT flags);

    void OnMButtonDown(int x, int y, UINT flags);
    void OnMButtonUp(int x, int y, UINT flags);
    void OnMButtonDClick(int x, int y, UINT flags);

    void OnRButtonDown(int x, int y, UINT flags);
    void OnRButtonUp(int x, int y, UINT flags);
    void OnRButtonDClick(int x, int y, UINT flags);

    void OnMouseMove(int x, int y, UINT flags);
    void OnMouseEnter(int x, int y, UINT flags);
    void OnMouseLeave(int x, int y, UINT flags);

    void OnChar(USHORT chvk, Bool isASCII = FALSE);

    virtual void OnHScroll(int uscmd, int sslider, int usidentifier);
    virtual void OnVScroll(int uscmd, int sslider, int usidentifier);
};

class wxCanvasWnd : public wxSubWnd
{
public:
    wxCanvasWnd(wxWnd *parent, char *winClass, wxWindow *wx_win,
		int x, int y, int width, int height,
		ULONG style);

    HPS hpsCanvas;			// Canvas Permanent presentation space copy from the DC

    int unit;
    float xmult, ymult, xoff, yoff;
    float fxmult, fymult;		// Font multipliers

    HWND hwndVertScroll;		// Window handle for vertical scroll bar
    HWND hwndHorzScroll;		// Window handle for horizontal scroll bar

    // Handlers
//    BOOL OnEraseBkgnd(HPS ps, PRECTL pprcPaint = NULL);
    virtual BOOL OnPaint(HWND hwnd);
    virtual void OnHScroll(int uscmd, int sslider, int usidentifier);
    virtual void OnVScroll(int uscmd, int sslider, int usidentifier);
};

class wxFrameWnd : public wxWnd
{
public:
    static PFNWP  wxFrameProc;		// Pointer to superclassed windows procedure
    Bool iconized;			// True if frame is iconized
    HPOINTER icon;			// Icon for this frame
    HPOINTER defaultIcon;		// Default icon if icon is not set
    int frame_height, frame_width;	// Frame window height and width

    HWND hwndClient;			// Client window handle
    SWP swpClient;			// Client window space rectangle
    HWND hwndStatusLine;		// Status line window handle
    HWND status_item[wxMAX_STATUS]; 	// Status line field window handles
    int nitems;				// Number of status line items

    wxFrameWnd(void);
    wxFrameWnd(wxWnd *parent, wxWindow *wx_win, const char *title,
	       int x, int y, int width, int height,
	       long style);
    void Create(wxWnd *parent, char *wclass, wxWindow *wx_win, const char *title,
		int x, int y, int width, int height,
		long style);
    ~wxFrameWnd(void);

    virtual HWND GetClientHWND() const	{ return hwndClient; };
    virtual HWND GetFrameHWND() const	{ return hwndWnd; }

    // Handlers
    BOOL OnPaint(HWND hwnd);
    HPOINTER OnQueryDragIcon(void);
    void OnSize(int x, int y, UINT flag);
    BOOL OS2Command(int cmd, int source, int pointer);
    BOOL OnClose(void);
    void OnMenuSelect(ULONG item, ULONG flags, HWND sysmenu);
    BOOL ProcessMessage(QMSG *msg);
    
    MRESULT DefWindowProc(ULONG nMsg, MPARAM mp1, MPARAM mp2);
};

class wxMDIFrameWnd : public wxFrameWnd
{
public:
    wxWnd *current_child;
    HWND window_menu;
    Bool parent_frame_active; // TRUE if MDI Frame is intercepting
                              // commands, not child

    wxMDIFrameWnd(wxWnd *parent, wxWindow *wx_win, const char *title=NULL,
                int x=-1, int y=-1, int width=-1, int height=-1, long style = 0);
    ~wxMDIFrameWnd(void);

    void OnCreate(PCREATESTRUCT cs);
// PGD: Commented out with wx163 distribution
//    BOOL OnPaint(HWND hwnd);
//    BOOL OnClose(void);
    void OnSize(int x, int y, UINT);
    BOOL OS2Command(int cmd, int source, int pointer);
    void OnMenuSelect(ULONG, ULONG, HWND);
//    MRESULT DefWindowProc(ULONG msg, MPARAM mp1, MPARAM mp2);
    BOOL ProcessMessage(QMSG *msg);
//    BOOL OnEraseBkgnd(HPS ps, PRECTL pprcPaint = NULL);
    BOOL OnDestroy(void);
};

class wxMDIChildWnd : public wxFrameWnd
{
public:
    Bool active;

    HPOINTER icon;
    wxMDIChildWnd(wxMDIFrameWnd *parent, wxWindow *wx_win, const char *title=NULL,
                int x=-1, int y=-1, int width=-1, int height=-1, long style = 0);
    ~wxMDIChildWnd(void);

    BOOL OnMDIActivate(BOOL bActivate, HWND, HWND);
//    BOOL OnPaint(HWND hwnd);
    BOOL OnClose(void);
    void OnSize(int x, int y, UINT);
    BOOL OS2Command(int cmd, int source, int pointer);
//    void OnMenuSelect(ULONG, ULONG, HWND);
//    MRESULT DefWindowProc(ULONG msg, MPARAM mp1, MPARAM mp2);
    BOOL ProcessMessage(QMSG *msg);
    void DestroyWindow(void);
};

class wxPanelWnd : public wxSubWnd
{
public:
    wxPanelWnd(wxWnd *parent, char *winClass, wxWindow *wx_win,
	       int x, int y, int width, int height, DWORD flags);

    // Handlers
    MRESULT DefWindowProc(ULONG nMsg, MPARAM mp1, MPARAM mp2);
    Bool ProcessMessage(QMSG* pMsg);
//  Bool OnEraseBkgnd(HPS hps, PRECTL pprcPaint = NULL);
};


class wxDialogWnd : public wxSubWnd
{
public:
    HWND hwndClient;			// Dialog Client window handle

    wxDialogWnd(wxWnd *parent, wxWindow *wx_win,
		int x, int y, int width, int height,
		char *dialog_template);
    wxDialogWnd(void);
    virtual HWND GetClientHWND() const	{ return hwndClient; };
    virtual HWND GetFrameHWND()	const	{ return hwndWnd; }

    // Handlers
    MRESULT DefWindowProc(ULONG nMsg, MPARAM mp1, MPARAM mp2);
    BOOL ProcessMessage(QMSG* pMsg);
//  BOOL OnEraseBkgnd(HPS ps, PRECTL pprcPaint = NULL);
    BOOL OnClose(void);
};

#define VIEWPORT_EXTENT 1000

// Define 'undocumented' apis
#define MM_SETITEMATTRBYPOS     0x1F6
#define MM_SETITEMTEXTBYPOS     0x1F9

// MDI Messages
#define	WM_MDIGETACTIVE		(WM_USER+1)
#define	WM_MDICASCADE		(WM_USER+2)
#define	WM_MDITILE		(WM_USER+3)
#define	WM_MDIICONARRANGE	(WM_USER+4)
#define	WM_MDINEXT		(WM_USER+5)
#define	WM_MDISETMENU		(WM_USER+6)
#define	WM_MDIDESTROY		(WM_USER+7)

/*
 * MDI Command ID's
 */
#define IDM_WINDOWTILE 		4001
#define IDM_WINDOWCASCADE	4002
#define IDM_WINDOWICONS		4003
#define IDM_WINDOWNEXT		4004


class wxFont ;

void wxSliderEvent(int usidentifier, int uscmd, int sslider);
wxWnd *wxFindWinFromHandle(HWND hWnd);
void wxScrollBarEvent(HWND hbar, int mp1, int pos);
void wxSetupPS(HPS hps);
HWND wxMakeWindowMenu(HWND);

extern HPOINTER wxSTD_FRAME_ICON;
extern HPOINTER wxSTD_MDIPARENTFRAME_ICON;
extern HPOINTER wxSTD_MDICHILDFRAME_ICON;
extern HPOINTER wxDEFAULT_FRAME_ICON;
extern HPOINTER wxDEFAULT_MDIPARENTFRAME_ICON;
extern HPOINTER wxDEFAULT_MDICHILDFRAME_ICON;
extern FATTRS wxSTATUS_LINE_FONT;

#endif // IN_CPROTO
#endif // wx_privth

