/*
 * File:	wx_cmdlg.cc
 * Purpose:	Common dialogs: MS Windows implementation
 * Author:	Julian Smart
 * Created:	1995
 * Updated:	
 * Copyright:	(c) 1995, Julian Smart
 */

static const char sccsid[] = "%W% %G%";

#ifdef __GNUG__
#pragma implementation "wx_cmdlg.h"
#endif

#if !defined(_MSC_VER) && !defined(wx_wxh)
#define wx_wxh
#endif

#include "common.h"
#include "wx_gdi.h"
#include "wx_utils.h"
#include "wx_types.h"
#include "wx_frame.h"
#include "wx_dialg.h"

#include "wx.h"

#include "wx_privt.h"
#include "wx_cmdlg.h"

#define wxDIALOG_DEFAULT_X 300
#define wxDIALOG_DEFAULT_Y 300

/*
 * wxColourDialog
 */

#if XXX
IMPLEMENT_DYNAMIC_CLASS(wxColourDialog, wxDialogBox)

wxColourDialog::wxColourDialog(void)
{
  dialogParent = NULL;
}

wxColourDialog::wxColourDialog(wxWindow *parent, wxColourData *data)
{
  Create(parent, data);
}

Bool wxColourDialog::Create(wxWindow *parent, wxColourData *data)
{
  dialogParent = parent;
  
  if (data)
    colourData = *data;
  return TRUE;
}

Bool wxColourDialog::Show(Bool flag)
{
  if (flag)
  {
    CHOOSECOLOR chooseColorStruct;
    COLORREF custColours[16];
    memset(&chooseColorStruct, 0, sizeof(CHOOSECOLOR));

    for (int i = 0; i < 16; i++)
      custColours[i] = RGB(colourData.custColours[i].Red(), colourData.custColours[i].Green(), colourData.custColours[i].Blue());

    chooseColorStruct.lStructSize = sizeof(CHOOSECOLOR);
    chooseColorStruct.hwndOwner = (dialogParent ? dialogParent->GetHWND() : NULL);
    chooseColorStruct.rgbResult = RGB(colourData.dataColour.Red(), colourData.dataColour.Green(), colourData.dataColour.Blue());
    chooseColorStruct.lpCustColors = custColours;

    chooseColorStruct.Flags = CC_RGBINIT;

    if (!colourData.GetChooseFull())
      chooseColorStruct.Flags |= CC_PREVENTFULLOPEN;

    // Do the modal dialog
    Bool success = ChooseColor(&(chooseColorStruct));

#if XXX
    // Try to highlight the correct window (the parent)
    HWND hWndParent = 0;
    if (GetParent())
    {
      hWndParent = GetParent()->GetHWND();
      if (hWndParent)
        ::BringWindowToTop(hWndParent);
    }
#endif


    // Restore values
    for (i = 0; i < 16; i++)
    {
      colourData.custColours[i].Set(GetRValue(custColours[i]), GetGValue(custColours[i]),
         GetBValue(custColours[i]));
    }

    colourData.dataColour.Set(GetRValue(chooseColorStruct.rgbResult), GetGValue(chooseColorStruct.rgbResult),
     GetBValue(chooseColorStruct.rgbResult));

    return success;
  }
  return TRUE;
}
#endif

/*
 * wxFontDialog
 */

#if XXX
void wxFillLogFont(LOGFONT *logFont, wxFont *font);
wxFont *wxCreateFontFromLogFont(LOGFONT *logFont);

IMPLEMENT_DYNAMIC_CLASS(wxFontDialog, wxDialogBox)

wxFontDialog::wxFontDialog(void)
{
    dialogParent = NULL;
}

wxFontDialog::wxFontDialog(wxWindow *parent, wxFontData *data)
{
  Create(parent, data);
}

Bool wxFontDialog::Create(wxWindow *parent, wxFontData *data)
{
  dialogParent = parent;
  
  if (data)
    fontData = *data;
  return TRUE;
}

Bool wxFontDialog::Show(Bool flag)
{
    if (flag) {
	CHOOSEFONT chooseFontStruct;
	LOGFONT logFont;

	DWORD flags = CF_TTONLY | CF_SCREENFONTS | CF_NOSIMULATIONS;

	memset(&chooseFontStruct, 0, sizeof(CHOOSEFONT));

	chooseFontStruct.lStructSize = sizeof(CHOOSEFONT);
	chooseFontStruct.hwndOwner = (dialogParent ? dialogParent->GetHWND() : NULL);
	chooseFontStruct.lpLogFont = &logFont;

	if (fontData.initialFont)
	{
	    flags |= CF_INITTOLOGFONTSTRUCT;
	    wxFillLogFont(&logFont, fontData.initialFont);
	}

	chooseFontStruct.iPointSize = 0;
	chooseFontStruct.rgbColors = RGB(fontData.fontColour.Red(), fontData.fontColour.Green(), fontData.fontColour.Blue());

	if (!fontData.GetAllowSymbols())
	    flags |= CF_ANSIONLY;
	if (fontData.GetEnableEffects())
	    flags |= CF_EFFECTS;
	if (fontData.GetShowHelp())
	    flags |= CF_SHOWHELP;
	if (!(fontData.minSize == 0 && fontData.maxSize == 0))
	{
	    chooseFontStruct.nSizeMin = fontData.minSize;
	    chooseFontStruct.nSizeMax = fontData.maxSize;
	    flags |= CF_LIMITSIZE;
	}

	chooseFontStruct.Flags = flags;
	chooseFontStruct.nFontType = SCREEN_FONTTYPE;
	Bool success = ChooseFont(&(chooseFontStruct));

	// Restore values
	if (success)
	{
	    fontData.fontColour.Set(GetRValue(chooseFontStruct.rgbColors), GetGValue(chooseFontStruct.rgbColors),
				    GetBValue(chooseFontStruct.rgbColors));
	    fontData.chosenFont = wxCreateFontFromLogFont(&logFont);
	}

	return success;
    }
    return TRUE;
}

void wxFillLogFont(LOGFONT *logFont, wxFont *font)
{
    TRACE("wxFillLogFont(logFont=%#x, font=%#x)", (int)logFont, (int)font);
    BYTE ff_italic;
    int ff_weight = 0;
    int ff_family = 0;
    char *ff_face = NULL;

    switch (font->GetFamily()) {
    case wxSCRIPT:
	ff_family = FF_SCRIPT ;
	ff_face = "Script" ;
	break ;
    case wxDECORATIVE:
	ff_family = FF_DECORATIVE;
	break;
    case wxROMAN:
	ff_family = FF_ROMAN;
	ff_face = "Times New Roman" ;
	break;
    case wxTELETYPE:
    case wxMODERN:
	ff_family = FF_MODERN;
	ff_face = "Courier New" ;
	break;
    case wxSWISS:
	ff_family = FF_SWISS;
	ff_face = "Helvetica";
	break;
    case wxDEFAULT:
    default:
	ff_family = FF_SWISS;
	ff_face = "MS Sans Serif" ; 
    }

    if (font->GetStyle() == wxITALIC || font->GetStyle() == wxSLANT)
	ff_italic = 1;
    else
	ff_italic = 0;

    if (font->GetWeight() == wxNORMAL)
	ff_weight = FW_NORMAL;
    else if (font->GetWeight() == wxLIGHT)
	ff_weight = FW_LIGHT;
    else if (font->GetWeight() == wxBOLD)
	ff_weight = FW_BOLD;

    // Have to get screen DC Caps, because a metafile will return 0.
    HDC dc2 = ::GetDC(NULL);
#if FONT_SIZE_COMPATIBILITY
    // Incorrect, but compatible with old wxWindows behaviour
    int nHeight = (font->GetPointSize()*GetDeviceCaps(dc2, LOGPIXELSY)/72);
#else
    // Correct for Windows compatibility
    int nHeight = - (font->GetPointSize()*GetDeviceCaps(dc2, LOGPIXELSY)/72);
#endif
    ::ReleaseDC(NULL, dc2);

    Bool ff_underline = font->GetUnderlined();

    
    if (font->GetFaceName())
      ff_face = font->GetFaceName();
  
    logFont->lfHeight = nHeight;
    logFont->lfWidth = 0;
    logFont->lfEscapement = 0;
    logFont->lfOrientation = 0;
    logFont->lfWeight = ff_weight;
    logFont->lfItalic = ff_italic;
    logFont->lfUnderline = (BYTE)ff_underline;
    logFont->lfStrikeOut = 0;
    logFont->lfCharSet = ANSI_CHARSET;
    logFont->lfOutPrecision = OUT_DEFAULT_PRECIS;
    logFont->lfClipPrecision = CLIP_DEFAULT_PRECIS;
    logFont->lfQuality = PROOF_QUALITY;
    logFont->lfPitchAndFamily = DEFAULT_PITCH | ff_family;
    strcpy(logFont->lfFaceName, ff_face);
}

wxFont *wxCreateFontFromLogFont(LOGFONT *logFont)
{
    TRACE("wxCreateFontFromLogFont(logFont=%#x)", (int)logFont);
    int fontFamily = wxSWISS;
    int fontStyle = wxNORMAL;
    int fontWeight = wxNORMAL;
    int fontPoints = 10;
    Bool fontUnderline = FALSE;
    char *fontFace = NULL;

//  int lfFamily = logFont->lfPitchAndFamily & 0xF0;
    int lfFamily = logFont->lfPitchAndFamily;
    if (lfFamily & FIXED_PITCH)
	lfFamily -= FIXED_PITCH;
    if (lfFamily & VARIABLE_PITCH)
	lfFamily -= VARIABLE_PITCH;
  
    switch (lfFamily) {
    case FF_ROMAN:
	fontFamily = wxROMAN;
	break;
    case FF_SWISS:
	fontFamily = wxSWISS;
	break;
    case FF_SCRIPT:
	fontFamily = wxSCRIPT;
	break;
    case FF_MODERN:
	fontFamily = wxMODERN;
	break;
    case FF_DECORATIVE:
	fontFamily = wxDECORATIVE;
	break;
    default:
	fontFamily = wxSWISS;
	break;
    }

    switch (logFont->lfWeight) {
    case FW_LIGHT:
	fontWeight = wxLIGHT;
	break;
    case FW_NORMAL:
	fontWeight = wxNORMAL;
	break;
    case FW_BOLD:
	fontWeight = wxBOLD;
	break;
    default:
	fontWeight = wxNORMAL;
	break;
    }
    if (logFont->lfItalic)
	fontStyle = wxITALIC;
    else
	fontStyle = wxNORMAL;

    if (logFont->lfUnderline)
	fontUnderline = TRUE;
    
  if  (logFont->lfFaceName)
    fontFace = logFont->lfFaceName;
    
  HDC dc2 = ::GetDC(NULL);
  fontPoints = abs(72*logFont->lfHeight/GetDeviceCaps(dc2, LOGPIXELSY));
  ::ReleaseDC(NULL, dc2);

  return new wxFont(fontPoints, fontFamily, fontStyle, fontWeight, fontUnderline, fontFace);
}
#endif

/*
 * The following code was formerly in wx_dialg.cc.
 */


/*
 * Common dialogs
 *
 */
 
// Pop up a message box
int wxMessageBox(const char *message, const char *caption, long type, wxWindow *parent, int x, int y)
{
    TRACE("wxMessageBox(message='%s', caption='%s', type=%d, parent=%#x, x=%d, y=%d)", message, caption, (int)type, x, y);
    if ((type & wxCENTRE) == wxCENTRE)
	return wxbMessageBox(message, caption, type, parent, x, y);
    
    HWND hWnd = 0;
    if (parent) hWnd = parent->GetHWND();
    unsigned int msStyle = MB_OK;
    if (type & wxYES_NO) {
	if (type & wxCANCEL)
	    msStyle = MB_YESNOCANCEL;
	else
	    msStyle = MB_YESNO;
    }
    if (type & wxOK) {
	if (type & wxCANCEL)
	    msStyle = MB_OKCANCEL;
	else
	    msStyle = MB_OK;
    }
    if (type & wxICON_EXCLAMATION)
	msStyle |= MB_ICONEXCLAMATION;
    else if (type & wxICON_HAND)
	msStyle |= MB_ICONHAND;
    else if (type & wxICON_INFORMATION)
	msStyle |= MB_ICONASTERISK;
    else if (type & wxICON_QUESTION)
	msStyle |= MB_ICONQUESTION;

    if (hWnd)
	msStyle |= MB_APPLMODAL;
    else
	msStyle |= MB_SYSTEMMODAL;
    
    int msAns = WinMessageBox(HWND_DESKTOP, hWnd, (PSZ)message, (PSZ)caption, 0, msStyle);
    int ans = wxOK;
    switch (msAns)
    {
    case MBID_CANCEL:
	ans = wxCANCEL;
	break;
    case MBID_OK:
	ans = wxOK;
	break;
    case MBID_YES:
	ans = wxYES;
	break;
    case MBID_NO:
	ans = wxNO;
	break;
    }
    return ans;
}

char *wxFileSelector(Constdata char *message,
                     Const char *default_path, Const char *default_filename, 
                     Const char *default_extension, Const char *wildcard, int flags,
                     wxWindow *parent, int x, int y)
{
TRACE("wxFileSelector(message='%s', default_path='%s', default_filename='%s', default_extension='%s', wildcard='%s', flags=%d, parent=%#x, x=%d, y=%d)", message, default_path, default_filename, default_extension, wildcard, flags, (int)parent, x, y);

    static FILEDLG fild;			// File dialog info structure
    HWND hwndDlg;				// File dialog window

    HWND hwndMain;
    if (parent) {
        wxWnd *wnd = (wxWnd *)parent->handle;
        hwndMain = wnd->hwndWnd;
    } else
	hwndMain = HWND_DESKTOP;
 
    memset(&fild, 0, sizeof(FILEDLG));
    fild.cbSize = sizeof(FILEDLG);		// Size of structure
    if (x > 0 && y > 0) {
	    fild.x = x;
	    fild.y = y;
    } else
	    fild.fl |= FDS_CENTER;
    if (flags & wxSAVE)
	    fild.fl |= FDS_SAVEAS_DIALOG;
    else
	    fild.fl |= FDS_OPEN_DIALOG;
    fild.pszTitle = (PSZ)message;		// Dialog title string
    if (default_path) {
	    strcpy(fild.szFullFile, default_path);
	    strcat(fild.szFullFile, "\\");
    }
    if (wildcard)
	    strcat(fild.szFullFile, wildcard);
    else if (default_extension || default_filename) {
	    if (default_filename)
		    strcat(fild.szFullFile, default_filename);
	    else
		    strcat(fild.szFullFile, "*");
	    strcat(fild.szFullFile, ".");
	    if (default_extension)
		    strcat(fild.szFullFile, default_extension);
	    else
		    strcat(fild.szFullFile, "*");
    } else
	    strcat(fild.szFullFile, "*");

    hwndDlg = WinFileDlg(HWND_DESKTOP, hwndMain, &fild);
    
    if (fild.lReturn == DID_OK)
	    return fild.szFullFile;
    return NULL;
}



