# define INCL_DOS
# define INCL_PM
# define INCL_ERRORS
# define INCL_WINHELP
#include <os2.h>
#include <assert.h>
#include <stdarg.h>

#if defined (__cplusplus)
extern "C" {
#endif
int printf (__const__ char *format, ...);
int sprintf (char *buffer, __const__ char *format, ...);
int vsprintf (char *buffer, __const__ char *format, va_list arg_ptr);
#if defined (__cplusplus)
}
#endif

int traceon = 1;

void
TRACE(const char *fmt ...)
{
    va_list ap;
    static char buffer[2048];
    static unsigned long stack[512];
    static int stkp = -1;
    int i;
    unsigned long sp;
    char *cp;

    if (!traceon)
	return;

    sp = (unsigned long)&ap;
    while (stkp >= 0 && stack[stkp] <= sp)
	stkp--;
    stack[++stkp] = sp;

    cp = buffer;
    for (i = 0; i < stkp; i++)
	*cp++ = ' ';

    va_start(ap, fmt);

    vsprintf(cp, fmt, ap);

    i = printf("%s\n", buffer);
    if (i <= 0)
	traceon = 0;

    va_end(ap);
}



char *getpmerrmsg(void);
extern HAB wxhAb;

// Fatal error (exits)
extern const char *wxFatalErrorStr;
void wxFatalError(const char *msg, const char *title = wxFatalErrorStr);

extern "C" {
void _assert(__const__ char *string, __const__ char *fname, unsigned line)
{
	printf("Assert violation on line %d in file %s:\n%s\n", line, fname, string);
	char buffer[512];
	sprintf(buffer, "Assert violation on line %d in file %s:\n%s\n", line, fname, string);
	wxFatalError(buffer);
	WinTerminate(wxhAb);
	abort();
}

}


void
debugfatal(const char *fcn, const char *file, int line)
{
	printf("%s failed, file=%s, line=%d, %s\n", fcn, file, line, getpmerrmsg());
	WinTerminate(wxhAb);
	abort();
}

HDC DebugDevOpenDC(const char *file, int line, HAB hab, LONG lType, PCSZ pszToken, LONG lCount, PDEVOPENDATA pdopData, HDC hdcComp)
{
	HDC res = DevOpenDC(hab, lType, pszToken, lCount, pdopData, hdcComp);
	if (!res)
		debugfatal("DevOpenDC", file, line);
	return res;
}

BOOL DebugDevQueryCaps(const char *file, int line, HDC hdc, LONG lStart, LONG lCount, PLONG alArray)
{
	BOOL res = DevQueryCaps(hdc, lStart, lCount, alArray);
	if (!res)
		debugfatal("DevQueryCaps", file, line);
	return res;
}

ULONG DebugDosAllocSharedMem(const char *file, int line, PPVOID pBaseAddress, PCSZ pszName, ULONG ulObjectSize, ULONG ulAllocationFlags)
{
	ULONG res = DosAllocSharedMem(pBaseAddress, pszName, ulObjectSize, ulAllocationFlags);
	if (!res)
		debugfatal("DosAllocSharedMem", file, line);
	return res;
}

ULONG DebugDosFreeMem(const char *file, int line, PVOID pBaseAddress)
{
	ULONG res = DosFreeMem(pBaseAddress);
	if (!res)
		debugfatal("DosFreeMem", file, line);
	return res;
}

ULONG DebugDosGetInfoBlocks(const char *file, int line, PTIB *ptib, PPIB *ppib)
{
	ULONG res = DosGetInfoBlocks(ptib, ppib);
	if (!res)
		debugfatal("DosGetInfoBlocks", file, line);
	return res;
}

ULONG DebugDosGiveSharedMem(const char *file, int line, CPVOID pBaseAddress, PID idProcessId, ULONG ulAttributeFlags)
{
	ULONG res = DosGiveSharedMem(pBaseAddress, idProcessId, ulAttributeFlags);
	if (!res)
		debugfatal("DosGiveSharedMem", file, line);
	return res;
}

ULONG DebugDosOpenQueue(const char *file, int line, PPID ppid, PHQUEUE phq, PCSZ pszName)
{
	ULONG res = DosOpenQueue(ppid, phq, pszName);
	if (!res)
		debugfatal("DosOpenQueue", file, line);
	return res;
}

ULONG DebugDosWriteQueue(const char *file, int line, HQUEUE hq, ULONG ulRequest, ULONG ulDataLength, PVOID pData, ULONG ulPriority)
{
	ULONG res = DosWriteQueue(hq, ulRequest, ulDataLength, pData, ulPriority);
	if (!res)
		debugfatal("DosWriteQueue", file, line);
	return res;
}

BOOL DebugGpiBeginPath(const char *file, int line, HPS hps, LONG lPath)
{
	BOOL res = GpiBeginPath(hps, lPath);
	if (!res)
		debugfatal("GpiBeginPath", file, line);
	return res;
}

LONG DebugGpiBitBlt(const char *file, int line, HPS hpsTarget, HPS hpsSource, LONG lCount, __const__ POINTL *aptlPoints, LONG lRop, ULONG flOptions)
{
	LONG res = GpiBitBlt(hpsTarget, hpsSource, lCount, aptlPoints, lRop, flOptions);
	if (!res)
		debugfatal("GpiBitBlt", file, line);
	return res;
}

LONG DebugGpiBox(const char *file, int line, HPS hps, LONG lControl, __const__ POINTL *pptlPoint, LONG lHRound, LONG lVRound)
{
	LONG res = GpiBox(hps, lControl, pptlPoint, lHRound, lVRound);
	if (!res)
		debugfatal("GpiBox", file, line);
	return res;
}

LONG DebugGpiCharStringAt(const char *file, int line, HPS hps, __const__ POINTL *pptlPoint, LONG lCount,  PCCH pchString)
{
	LONG res = GpiCharStringAt(hps, pptlPoint, lCount,  pchString);
	if (!res)
		debugfatal("GpiCharStringAt", file, line);
	return res;
}

HBITMAP DebugGpiCreateBitmap(const char *file, int line, HPS hps, __const__ BITMAPINFOHEADER2 *pbmpNew, ULONG flOptions, __const__ BYTE *pbInitData, __const__ BITMAPINFO2 *pbmiInfoTable)
{
	HBITMAP res = GpiCreateBitmap(hps, pbmpNew, flOptions, pbInitData, pbmiInfoTable);
	if (!res)
		debugfatal("GpiCreateBitmap", file, line);
	return res;
}

BOOL DebugGpiCreateLogColorTable(const char *file, int line, HPS hps, ULONG flOptions, LONG lFormat, LONG lStart, LONG lCount, __const__ LONG *alTable)
{
	BOOL res = GpiCreateLogColorTable(hps, flOptions, lFormat, lStart, lCount, alTable);
	if (!res)
		debugfatal("GpiCreateLogColorTable", file, line);
	return res;
}

HPS DebugGpiCreatePS(const char *file, int line, HAB hab, HDC hdc, PSIZEL psizlSize, ULONG flOptions)
{
	HPS res = GpiCreatePS(hab, hdc, psizlSize, flOptions);
	if (!res)
		debugfatal("GpiCreatePS", file, line);
	return res;
}

BOOL DebugGpiDeleteBitmap(const char *file, int line, HBITMAP hbm)
{
	BOOL res = GpiDeleteBitmap(hbm);
	if (!res)
		debugfatal("GpiDeleteBitmap", file, line);
	return res;
}

BOOL DebugGpiEndPath(const char *file, int line, HPS hps)
{
	BOOL res = GpiEndPath(hps);
	if (!res)
		debugfatal("GpiEndPath", file, line);
	return res;
}

LONG DebugGpiFillPath(const char *file, int line, HPS hps, LONG lPath, LONG lOptions)
{
	LONG res = GpiFillPath(hps, lPath, lOptions);
	if (!res)
		debugfatal("GpiFillPath", file, line);
	return res;
}

LONG DebugGpiFullArc(const char *file, int line, HPS hps, LONG lControl, FIXED fxMultiplier)
{
	LONG res = GpiFullArc(hps, lControl, fxMultiplier);
	if (!res)
		debugfatal("GpiFullArc", file, line);
	return res;
}

LONG DebugGpiLine(const char *file, int line, HPS hps, __const__ POINTL *pptlEndPoint)
{
	LONG res = GpiLine(hps, pptlEndPoint);
	if (!res)
		debugfatal("GpiLine", file, line);
	return res;
}

BOOL DebugGpiMove(const char *file, int line, HPS hps, __const__ POINTL *pptlPoint)
{
	BOOL res = GpiMove(hps, pptlPoint);
	if (!res)
		debugfatal("GpiMove", file, line);
	return res;
}

LONG DebugGpiPartialArc(const char *file, int line, HPS hps, __const__ POINTL *pptlCenter, FIXED fxMultiplier, FIXED fxStartAngle, FIXED fxSweepAngle)
{
	LONG res = GpiPartialArc(hps, pptlCenter, fxMultiplier, fxStartAngle, fxSweepAngle);
	if (!res)
		debugfatal("GpiPartialArc", file, line);
	return res;
}

LONG DebugGpiPolyLine(const char *file, int line, HPS hps, LONG lCount, __const__ POINTL *aptlPoints)
{
	LONG res = GpiPolyLine(hps, lCount, aptlPoints);
	if (!res)
		debugfatal("GpiPolyLine", file, line);
	return res;
}

LONG DebugGpiQueryBackColor(const char *file, int line, HPS hps)
{
	LONG res = GpiQueryBackColor(hps);
	if (res == CLR_ERROR) {
		printf("GpiQueryBackColor failed, file=%s, line=%d, %s\n", file, line, getpmerrmsg());
		WinTerminate(wxhAb);
		abort();
	}
	return res;
}

LONG DebugGpiQueryBackMix(const char *file, int line, HPS hps)
{
	LONG res = GpiQueryBackMix(hps);
	if (res == BM_ERROR) {
		printf("GpiQueryBackMix failed, file=%s, line=%d, %s\n", file, line, getpmerrmsg());
		WinTerminate(wxhAb);
		abort();
	}
	return res;
}

LONG DebugGpiQueryCharSet(const char *file, int line, HPS hps)
{
	LONG res = GpiQueryCharSet(hps);
	if (res == LCID_ERROR) {
		printf("GpiQueryCharSet failed, file=%s, line=%d, %s\n", file, line, getpmerrmsg());
		WinTerminate(wxhAb);
		abort();
	}
	return res;
}

HDC DebugGpiQueryDevice(const char *file, int line, HPS hps)
{
	HDC res = GpiQueryDevice(hps);
	if (res == HDC_ERROR) {
		printf("GpiQueryDevice failed, file=%s, line=%d, %s\n", file, line, getpmerrmsg());
		WinTerminate(wxhAb);
		abort();
	}
	return res;
}

BOOL DebugGpiQueryFontMetrics(const char *file, int line, HPS hps, LONG lMetricsLength, PFONTMETRICS pfmMetrics)
{
	BOOL res = GpiQueryFontMetrics(hps, lMetricsLength, pfmMetrics);
	if (!res)
		debugfatal("GpiQueryFontMetrics", file, line);
	return res;
}

BOOL DebugGpiQueryTextBox(const char *file, int line, HPS hps, LONG lCount1, PCH pchString, LONG lCount2, PPOINTL aptlPoints)
{
	BOOL res = GpiQueryTextBox(hps, lCount1, pchString, lCount2, aptlPoints);
	if (!res)
		debugfatal("GpiQueryTextBox", file, line);
	return res;
}

HPAL DebugGpiSelectPalette(const char *file, int line, HPS hps, HPAL hpal)
{
	HPAL res = GpiSelectPalette(hps, hpal);
	if (!res)
		debugfatal("GpiSelectPalette", file, line);
	return res;
}

BOOL DebugGpiSetArcParams(const char *file, int line, HPS hps, __const__ ARCPARAMS *parcpArcParams)
{
	BOOL res = GpiSetArcParams(hps, parcpArcParams);
	if (!res)
		debugfatal("GpiSetArcParams", file, line);
	return res;
}

BOOL DebugGpiSetAttrs(const char *file, int line, HPS hps, LONG lPrimType, ULONG flAttrMask, ULONG flDefMask, __const__ VOID *ppbunAttrs)
{
	BOOL res = GpiSetAttrs(hps, lPrimType, flAttrMask, flDefMask, ppbunAttrs);
	if (!res)
		debugfatal("GpiSetAttrs", file, line);
	return res;
}

BOOL DebugGpiSetBackColor(const char *file, int line, HPS hps, LONG lColor)
{
	BOOL res = GpiSetBackColor(hps, lColor);
	if (!res)
		debugfatal("GpiSetBackColor", file, line);
	return res;
}

BOOL DebugGpiSetBackMix(const char *file, int line, HPS hps, LONG lMixMode)
{
	BOOL res = GpiSetBackMix(hps, lMixMode);
	if (!res)
		debugfatal("GpiSetBackMix", file, line);
	return res;
}

HBITMAP DebugGpiSetBitmap(const char *file, int line, HPS hps, HBITMAP hbm)
{
	HBITMAP res = GpiSetBitmap(hps, hbm);
	if (res == HBM_ERROR)
		debugfatal("GpiSetBitmap", file, line);
	return res;
}

BOOL DebugGpiSetCharSet(const char *file, int line, HPS hps, LONG llcid)
{
	BOOL res = GpiSetCharSet(hps, llcid);
	if (!res)
		debugfatal("GpiSetCharSet", file, line);
	return res;
}

BOOL DebugGpiSetColor(const char *file, int line, HPS hps, LONG lColor)
{
	BOOL res = GpiSetColor(hps, lColor);
	if (!res)
		debugfatal("GpiSetColor", file, line);
	return res;
}

BOOL DebugGpiSetMix(const char *file, int line, HPS hps, LONG lMixMode)
{
	BOOL res = GpiSetMix(hps, lMixMode);
	if (!res)
		debugfatal("GpiSetMix", file, line);
	return res;
}

LONG DebugGpiSetPel(const char *file, int line, HPS hps, __const__ POINTL *pptlPoint)
{
	LONG res = GpiSetPel(hps, pptlPoint);
	if (!res)
		debugfatal("GpiSetPel", file, line);
	return res;
}

BOOL DebugPrfCloseProfile(const char *file, int line, HINI hini)
{
	BOOL res = PrfCloseProfile(hini);
	if (!res)
		debugfatal("PrfCloseProfile", file, line);
	return res;
}

HINI DebugPrfOpenProfile(const char *file, int line, HAB hab, PCSZ pszFileName)
{
	HINI res = PrfOpenProfile(hab, pszFileName);
	if (!res)
		debugfatal("PrfOpenProfile", file, line);
	return res;
}

BOOL DebugPrfQueryProfileData(const char *file, int line, HINI hini, PCSZ pszApp, PCSZ pszKey, PVOID pBuffer, PULONG pulBufferLength)
{
	BOOL res = PrfQueryProfileData(hini, pszApp, pszKey, pBuffer, pulBufferLength);
	if (!res)
		debugfatal("PrfQueryProfileData", file, line);
	return res;
}

BOOL DebugPrfWriteProfileData(const char *file, int line, HINI hini, PCSZ pszApp, PCSZ pszKey, CPVOID pData, ULONG ulDataLength)
{
	BOOL res = PrfWriteProfileData(hini, pszApp, pszKey, pData, ulDataLength);
	if (!res)
		debugfatal("PrfWriteProfileData", file, line);
	return res;
}

HPS DebugWinBeginPaint(const char *file, int line, HWND hwnd, HPS hps, PRECTL prclPaint)
{
	HPS res = WinBeginPaint(hwnd, hps, prclPaint);
	if (!res)
		debugfatal("WinBeginPaint", file, line);
	return res;
}

HWND DebugWinCreateMenu(const char *file, int line, HWND hwndParent, CPVOID lpmt)
{
	HWND res = WinCreateMenu(hwndParent, lpmt);
	if (!res)
		debugfatal("WinCreateMenu", file, line);
	return res;
}

HMQ DebugWinCreateMsgQueue(const char *file, int line, HAB hab, LONG cmsg)
{
	HMQ res = WinCreateMsgQueue(hab, cmsg);
	if (!res)
		debugfatal("WinCreateMsgQueue", file, line);
	return res;
}

BOOL DebugGpiSetPS(const char *file, int line, HPS hps, __const__ SIZEL *psizlsize, ULONG flOptions)
{
	BOOL res = GpiSetPS(hps,psizlsize, flOptions);
	if (!res)
		debugfatal("GpiSetPS", file, line);
	return res;
}

BOOL DebugGpiQueryPageViewport(const char *file, int line, HPS hps, PRECTL prclViewport)
{
	BOOL res = GpiQueryPageViewport(hps, prclViewport);
	if (!res)
		debugfatal("GpiQueryPageViewport", file, line);
	return res;
}

BOOL DebugGpiSetPageViewport(const char *file, int line, HPS hps, __const__ RECTL *prclViewport)
{
	BOOL res = GpiSetPageViewport(hps, prclViewport);
	if (!res)
		debugfatal("GpiSetPageViewport", file, line);
	return res;
}

HWND DebugWinCreateWindow(const char *file, int line, HWND hwndParent, PCSZ pszClass, PCSZ pszName, ULONG flStyle, LONG x, LONG y, LONG cx, LONG cy, HWND hwndOwner, HWND hwndInsertBehind, ULONG id, PVOID pCtlData, PVOID pPresParams)
{
	HWND res = WinCreateWindow(hwndParent, pszClass, pszName, flStyle, x, y, cx, cy, hwndOwner, hwndInsertBehind, id, pCtlData, pPresParams);
	char *cn;
	switch ((int)pszClass) {
	case WC_FRAME:		cn = "WC_FRAME"; break;
	case WC_COMBOBOX:	cn = "WC_COMBOBOX"; break;
	case WC_BUTTON:		cn = "WC_BUTTON"; break;
	case WC_MENU:		cn = "WC_MENU"; break;
	case WC_STATIC:		cn = "WC_STATIC"; break;
	case WC_ENTRYFIELD:	cn = "WC_ENTRYFIELD"; break;
	case WC_LISTBOX:	cn = "WC_LISTBOX"; break;
	case WC_SCROLLBAR:	cn = "WC_SCROLLBAR"; break;
	case WC_TITLEBAR:	cn = "WC_TITLEBAR"; break;
	case WC_MLE:		cn = "WC_MLE"; break;
	case WC_APPSTAT:	cn = "WC_APPSTAT"; break;
	case WC_KBDSTAT:	cn = "WC_KBDSTAT"; break;
	case WC_PECIC:		cn = "WC_PECIC"; break;
	case WC_DBE_KKPOPUP:	cn = "WC_DBE_KKPOPUP"; break;
	case WC_SPINBUTTON:	cn = "WC_SPINBUTTON"; break;
	case WC_CONTAINER:	cn = "WC_CONTAINER"; break;
	case WC_SLIDER:		cn = "WC_SLIDER"; break;
	case WC_VALUESET:	cn = "WC_VALUESET"; break;
	case WC_NOTEBOOK:	cn = "WC_NOTEBOOK"; break;
	case WC_PENFIRST:	cn = "WC_PENFIRST"; break;
	case WC_PENLAST:	cn = "WC_PENLAST"; break;
	case WC_MMPMFIRST:	cn = "WC_MMPMFIRST"; break;
	case WC_MMPMLAST:	cn = "WC_MMPMLAST"; break;
	default:		cn = (char *)pszClass;
	}
TRACE("# WinCreateWindow(hwndParent=%#x, pszClass=%s, pszName='%s', flStyle=%#x, x=%d, y=%d, cx=%d, cy=%d, hwndOwner=%#x, hwndInsertBehind=%#x, id=%d, ...) hwnd=%#x called from file %s line %d", (int)hwndParent, cn, pszName, (int)flStyle, (int)x, (int)y, (int)cx, (int)cy, (int)hwndOwner, (int)hwndInsertBehind, (int)id, (int)res, file, line);
assert(x >= 0 && y >= 0 && cx >= 0 && cy >= 0);
	if (!res)
		debugfatal("WinCreateWindow", file, line);
	return res;
}

BOOL DebugWinDestroyPointer(const char *file, int line, HPOINTER hptr)
{
	BOOL res = WinDestroyPointer(hptr);
	if (!res)
		debugfatal("WinDestroyPointer", file, line);
	return res;
}

BOOL DebugWinDestroyWindow(const char *file, int line, HWND hwnd)
{
TRACE("# WinDestroyWindow(hwnd=%#x) called from file %s line %d", (int)hwnd, file, line);
	BOOL res = WinDestroyWindow(hwnd);
	if (!res)
		debugfatal("WinDestroyWindow", file, line);
	return res;
}

BOOL DebugWinDrawPointer(const char *file, int line, HPS hps, LONG x, LONG y, HPOINTER hptr, ULONG fs)
{
	BOOL res = WinDrawPointer(hps, x, y, hptr, fs);
	assert(x >= 0 && y >= 0);
	if (!res)
		debugfatal("WinDrawPointer", file, line);
	return res;
}

BOOL DebugWinEnableWindow(const char *file, int line, HWND hwnd, BOOL fEnable)
{
	BOOL res = WinEnableWindow(hwnd, fEnable);
	if (!res)
		debugfatal("WinEnableWindow", file, line);
	return res;
}

BOOL DebugWinEndPaint(const char *file, int line, HPS hps)
{
	BOOL res = WinEndPaint(hps);
	if (!res)
		debugfatal("WinEndPaint", file, line);
	return res;
}

BOOL DebugWinFillRect(const char *file, int line, HPS hps, __const__ RECTL *prcl, LONG lColor)
{
	BOOL res = WinFillRect(hps, prcl, lColor);
	assert(prcl->xLeft <= prcl->xRight && prcl->yTop >= prcl->yBottom);
	if (!res)
		debugfatal("WinFillRect", file, line);
	return res;
}

BOOL DebugWinGetMsg(const char *file, int line, HAB hab, PQMSG pqmsg, HWND hwndFilter, ULONG msgFilterFirst, ULONG msgFilterLast)
{
	BOOL res = WinGetMsg(hab, pqmsg, hwndFilter, msgFilterFirst, msgFilterLast);
	return res;
}

HPS DebugWinGetPS(const char *file, int line, HWND hwnd)
{
	HPS res = WinGetPS(hwnd);
	if (!res)
		debugfatal("WinGetPS", file, line);
	return res;
}

HAB DebugWinInitialize(const char *file, int line, ULONG fsOptions)
{
	HAB res = WinInitialize(fsOptions);
	if (!res)
		debugfatal("WinInitialize", file, line);
	return res;
}

BOOL DebugWinInvalidateRect(const char *file, int line, HWND hwnd, __const__ RECTL *prcl, BOOL fIncludeChildren)
{
	if (prcl)
		assert(prcl->xLeft <= prcl->xRight && prcl->yTop > prcl->yBottom);
	BOOL res = WinInvalidateRect(hwnd, prcl, fIncludeChildren);
	if (!res)
		debugfatal("WinInvalidateRect", file, line);
	return res;
}

BOOL DebugWinInvalidateRegion(const char *file, int line, HWND hwnd, HRGN hrgn, BOOL fIncludeChildren)
{
	BOOL res = WinInvalidateRegion(hwnd, hrgn, fIncludeChildren);
	if (!res)
		debugfatal("WinInvalidateRegion", file, line);
	return res;
}

HPOINTER DebugWinLoadPointer(const char *file, int line, HWND hwndDesktop, HMODULE hmod, ULONG idres)
{
	HPOINTER res = WinLoadPointer(hwndDesktop, hmod, idres);
	if (!res)
		debugfatal("WinLoadPointer", file, line);
	return res;
}

BOOL DebugWinMapWindowPoints(const char *file, int line, HWND hwndFrom, HWND hwndTo, PPOINTL prgptl, LONG cwpt)
{
	BOOL res = WinMapWindowPoints(hwndFrom, hwndTo, prgptl, cwpt);
	if (!res)
		debugfatal("WinMapWindowPoints", file, line);
	return res;
}

ULONG DebugWinMessageBox(const char *file, int line, HWND hwndParent, HWND hwndOwner, PCSZ pszText, PCSZ pszCaption, ULONG idWindow, ULONG flStyle)
{
	ULONG res = WinMessageBox(hwndParent, hwndOwner, pszText, pszCaption, idWindow, flStyle);
	if (!res)
		debugfatal("WinMessageBox", file, line);
	return res;
}

BOOL DebugWinPostMsg(const char *file, int line, HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
	BOOL res = WinPostMsg(hwnd, msg, mp1, mp2);
	if (!res)
		debugfatal("WinPostMsg", file, line);
	return res;
}

BOOL DebugWinQueryClassInfo(const char *file, int line, HAB hab, PCSZ pszClassName, PCLASSINFO pClassInfo)
{
	BOOL res = WinQueryClassInfo(hab, pszClassName, pClassInfo);
	if (!res)
		debugfatal("WinQueryClassInfo", file, line);
	return res;
}

BOOL DebugWinQueryPointerPos(const char *file, int line, HWND hwndDesktop, PPOINTL pptl)
{
	BOOL res = WinQueryPointerPos(hwndDesktop, pptl);
	if (!res)
		debugfatal("WinQueryPointerPos", file, line);
	return res;
}

BOOL DebugWinQueryUpdateRect(const char *file, int line, HWND hwnd, PRECTL prcl)
{
	BOOL res = WinQueryUpdateRect(hwnd, prcl);
	assert(prcl->xLeft <= prcl->xRight && prcl->yTop > prcl->yBottom);
	if (!res)
		debugfatal("WinQueryUpdateRect", file, line);
	return res;
}

HWND DebugWinQueryWindow(const char *file, int line, HWND hwnd, LONG cmd)
{
	HWND res = WinQueryWindow(hwnd, cmd);
	if (!res)
		debugfatal("WinQueryWindow", file, line);
	return res;
}

BOOL DebugWinQueryWindowRect(const char *file, int line, HWND hwnd, PRECTL prclDest)
{
	BOOL res = WinQueryWindowRect(hwnd, prclDest);
	if (!res)
		debugfatal("WinQueryWindowRect", file, line);
	return res;
}

LONG DebugWinRealizePalette(const char *file, int line, HWND hwnd, HPS hps, PULONG pcclr)
{
	LONG res = WinRealizePalette(hwnd, hps, pcclr);
	if (!res)
		debugfatal("WinRealizePalette", file, line);

	return res;
}

BOOL DebugWinRegisterClass(const char *file, int line, HAB hab, PCSZ pszClassName, PFNWP pfnWndProc, ULONG flStyle, ULONG cbWindowData)
{
	BOOL res = WinRegisterClass(hab, pszClassName, pfnWndProc, flStyle, cbWindowData);
	if (!res)
		debugfatal("WinRegisterClass", file, line);
	return res;
}

BOOL DebugWinReleasePS(const char *file, int line, HPS hps)
{
	BOOL res = WinReleasePS(hps);
	if (!res)
		debugfatal("WinReleasePS", file, line);
	return res;
}

LONG DebugWinScrollWindow(const char *file, int line, HWND hwnd, LONG dx, LONG dy, __const__ RECTL *prclScroll, __const__ RECTL *prclClip, HRGN hrgnUpdate, PRECTL prclUpdate, ULONG rgfsw)
{
    LONG res = WinScrollWindow(hwnd, dx, dy, prclScroll, prclClip, hrgnUpdate, prclUpdate, rgfsw);
    if (!res)
	debugfatal("WinScrollWindow", file, line);
    return res;
}
    

BOOL DebugWinSetCapture(const char *file, int line, HWND hwndDesktop, HWND hwnd)
{
	BOOL res = WinSetCapture(hwndDesktop, hwnd);
	if (!res)
		debugfatal("WinSetCapture", file, line);
	return res;
}

BOOL DebugWinSetFocus(const char *file, int line, HWND hwndDesktop, HWND hwndSetFocus)
{
	BOOL res = WinSetFocus(hwndDesktop, hwndSetFocus);
// The WinSetFocus function randomly fails with error 0x1202, so we disable the error checking here
//	if (!res)
//		debugfatal("WinSetFocus", file, line);
	return res;
}

BOOL DebugWinSetPointer(const char *file, int line, HWND hwndDesktop, HPOINTER hptrNew)
{
	BOOL res = WinSetPointer(hwndDesktop, hptrNew);
	if (!res)
		debugfatal("WinSetPointer", file, line);
	return res;
}

BOOL DebugWinSetPointerPos(const char *file, int line, HWND hwndDesktop, LONG x, LONG y)
{
	BOOL res = WinSetPointerPos(hwndDesktop, x, y);
	assert(x >= 0 && y >= 0);
	if (!res)
		debugfatal("WinSetPointerPos", file, line);
	return res;
}

BOOL DebugWinSetPresParam(const char *file, int line, HWND hwnd, ULONG id, ULONG cbParam, PVOID pbParam)
{
	BOOL res = WinSetPresParam(hwnd, id, cbParam, pbParam);
	if (!res)
		debugfatal("WinSetPresParam", file, line);
	return res;
}

BOOL DebugWinSetWindowPos(const char *file, int line, HWND hwnd, HWND hwndInsertBehind, LONG x, LONG y, LONG cx, LONG cy, ULONG fl)
{
TRACE("# WinSetWindowPos(hwnd=%#x, hwndInsertBehind=%#x, x=%d, y=%d, cx=%d, cy=%d, fl=%#x(%s%s%s%s%s%s%s%s%s%s%s%s )", (int)hwnd, (int)hwndInsertBehind, x, y, cx, cy, (int)fl, \
      fl&SWP_SIZE?" SWP_SIZE":"", \
      fl&SWP_MOVE?" SWP_MOVE":"", \
      fl&SWP_ZORDER?" SWP_ZORDER":"", \
      fl&SWP_SHOW?" SWP_SHOW":"", \
      fl&SWP_HIDE?" SWP_HIDE":"", \
      fl&SWP_NOREDRAW?" SWP_NOREDRAW":"", \
      fl&SWP_NOADJUST?" SWP_NOADJUST":"", \
      fl&SWP_ACTIVATE?" SWP_ACTIVATE":"", \
      fl&SWP_DEACTIVATE?" SWP_DEACTIVATE":"", \
      fl&SWP_MINIMIZE?" SWP_MINIMIZE":"", \
      fl&SWP_MAXIMIZE?" SWP_MAXIMIZE":"", \
      fl&SWP_RESTORE?" SWP_RESTORE":"" \
    );
	assert(!(fl & SWP_SIZE) || cx >= 0 && cy >= 0);
	if ((fl & SWP_MOVE) && (x < 0 && y < 0))
	    printf("!!! WARNING WinSetWindowPos arg out of range, hwnd=%#x, x=%d, y=%d, cx=%d, cy=%d\n", (int)hwnd, (int)x, (int)y, (int)cx, (int)cy);
	BOOL res = WinSetWindowPos(hwnd, hwndInsertBehind, x, y, cx, cy, fl);
	if (!res)
		debugfatal("WinSetWindowPos", file, line);
	return res;
}

BOOL DebugWinSetWindowText(const char *file, int line, HWND hwnd, PCSZ pszText)
{
	BOOL res = WinSetWindowText(hwnd, pszText);
//	if (!res)
//		debugfatal("WinSetWindowText", file, line);
	return res;
}

BOOL DebugWinSetWindowULong(const char *file, int line, HWND hwnd, LONG index, ULONG ul)
{
	BOOL res = WinSetWindowULong(hwnd, index, ul);
	if (!res)
		debugfatal("WinSetWindowULong", file, line);
	return res;
}

BOOL DebugWinSetWindowPtr(const char *file, int line, HWND hwnd, LONG index, PVOID p)
{
	BOOL res = WinSetWindowPtr(hwnd, index, p);
	if (!res)
		debugfatal("WinSetWindowPtr", file, line);
	return res;
}

BOOL DebugWinShowWindow(const char *file, int line, HWND hwnd, BOOL fShow)
{
	BOOL res = WinShowWindow(hwnd, fShow);
	if (!res)
		debugfatal("WinShowWindow", file, line);
	return res;
}

ULONG DebugWinStartTimer(const char *file, int line, HAB hab, HWND hwnd, ULONG idTimer, ULONG dtTimeout)
{
	ULONG res = WinStartTimer(hab, hwnd, idTimer, dtTimeout);
	if (!res)
		debugfatal("WinStartTimer", file, line);
	return res;
}

BOOL DebugWinStopTimer(const char *file, int line, HAB hab, HWND hwnd, ULONG idTimer)
{
	BOOL res = WinStopTimer(hab, hwnd, idTimer);
	if (!res)
		debugfatal("WinStopTimer", file, line);
	return res;
}

BOOL DebugWinTerminate(const char *file, int line, HAB hab)
{
	BOOL res = WinTerminate(hab);
	if (!res)
		debugfatal("WinTerminate", file, line);
	return res;
}

BOOL DebugWinUpdateWindow(const char *file, int line, HWND hwnd)
{
	BOOL res = WinUpdateWindow(hwnd);
	if (!res)
		debugfatal("WinUpdateWindow", file, line);
	return res;
}

HWND DebugWinWindowFromID(const char *file, int line, HWND hwndParent, ULONG id)
{
	HWND res = WinWindowFromID(hwndParent, id);
//	if (!res)
//		debugfatal("WinWindowFromID", file, line);
	return res;
}

HPOINTER DebugWinQuerySysPointer(const char *file, int line, HWND hwndDesktop, LONG lId, BOOL fCopy)
{
	HPOINTER res = WinQuerySysPointer(hwndDesktop, lId, fCopy);
	if (!res)
		debugfatal("WinQuerySysPointer", file, line);
	return res;
}

BOOL DebugWinCreateFrameControls(const char *file, int line, HWND hwndFrame, PFRAMECDATA pfcdata, PCSZ pszTitle)
{
	BOOL res = WinCreateFrameControls(hwndFrame, pfcdata, pszTitle);
	if (!res)
		debugfatal("WinCreateFrameControls", file, line);
	return res;
}

BOOL DebugGpiQueryBitmapParameters(const char *file, int line, HBITMAP hbm, PBITMAPINFOHEADER pbmpData)
{
	BOOL res = GpiQueryBitmapParameters(hbm,  pbmpData);
	if (!res)
		debugfatal("GpiQueryBitmapParameterss", file, line);
	return res;
}


HWND DebugWinFileDlg(const char *file, int line, HWND hwndP, HWND hwndO, PFILEDLG pfild)
{
        HWND res = WinFileDlg(hwndP, hwndO, pfild);
	if (!res)
		debugfatal("WinFileDlg", file, line);
	return res;
}

HPOINTER DebugWinQueryPointer(const char *file, int line, HWND hwndDesktop)
{
        HPOINTER res = WinQueryPointer(hwndDesktop);
TRACE("# DebugWinQueryPointer(%#x)... WinQueryPointer returns %#x", (int)hwndDesktop, (int)res);

	if (!res)
		debugfatal("WinQueryPointer", file, line);
	return res;
}

BOOL DebugGpiSetClipPath(const char *file, int line, HPS hps, LONG lPath, LONG lOptions)
{
    LONG res = GpiSetClipPath(hps, lPath, lOptions);
    if (!res)
	debugfatal("GpiSetClipPath", file, line);
    return res;
}

LONG DebugGpiSetClipRegion(const char *file, int line, HPS hps, HRGN hrgn, PHRGN phrgnOld)
{
    BOOL res = GpiSetClipRegion(hps, hrgn, phrgnOld);
    if (!res)
	debugfatal("GpiSetClipRegion", file, line);
    return res;
}


HDC DebugWinOpenWindowDC(const char *file, int line, HWND hwnd)
{
    HDC hdc = WinOpenWindowDC(hwnd);
    if (!hdc)
	debugfatal("WinOpenWindowDC", file, line);
    return hdc;
}

BOOL DebugGpiDestroyPS(const char *file, int line, HPS hps)
{
    BOOL res = GpiDestroyPS(hps);
    if (!res)
	debugfatal("GpiDestroyPS", file, line);
    return res;
}


LONG DebugGpiCreateLogFont(const char *file, int line, HPS hps, const STR8 *pName, LONG lLcid,
			   const FATTRS *pfatAttrs)
{
    LONG res = GpiCreateLogFont(hps, pName, lLcid, pfatAttrs);
    if (!res)
	debugfatal("GpiCreateLogFont", file, line);
    return res;
}
	
BOOL DebugGpiSetCharBox(const char *file, int line, HPS hps, __const__ SIZEF *psizfxBox)
{
    BOOL res = GpiSetCharBox(hps, psizfxBox);
    if (!res)
	debugfatal("GpiSetCharBox", file, line);
    return res;
}

BOOL DebugGpiDeleteSetId(const char *file, int line, HPS hps, LONG lLcid)
{
    BOOL res = GpiDeleteSetId(hps, lLcid);
    if (!res)
	debugfatal("GpiDeleteSetId", file, line);
    return res;
}

LONG DebugGpiDrawBits(const char *file, int line, HPS hps, __const__ VOID *pBits,
		      __const__ BITMAPINFO2 *pbmiInfoTable, LONG lCount,
		      __const__ POINTL *aptlPoints, LONG lRop, ULONG flOptions)
{
    LONG res = GpiDrawBits(hps, pBits, pbmiInfoTable, lCount, aptlPoints, lRop, flOptions);
    if (!res)
	debugfatal("GpiDrawBits", file, line);
    return res;
}

BOOL DebugWinCalcFrameRect(const char *file, int line, HWND hwndFrame, PRECTL prcl, BOOL fClient)
{
    BOOL res = WinCalcFrameRect(hwndFrame, prcl, fClient);
    if (!res)
	debugfatal("WinCalcFrameRect", file, line);
    return res;
}

ULONG DebugWinProcessDlg(const char *file, int line, HWND hwndDlg)
{
    ULONG res = WinProcessDlg(hwndDlg);
    return res;
}

BOOL DebugWinDismissDlg(const char *file, int line, HWND hwndDlg, ULONG usResult)
{
    BOOL res = WinDismissDlg(hwndDlg, usResult);
    return res;
}

LONG DebugGpiQueryRGBColor(const char *file, int line, HPS hps, ULONG flOptions, LONG lColorIndex)
{
    LONG res = GpiQueryRGBColor(hps, flOptions, lColorIndex);
    if (res == GPI_ALTERROR)
	debugfatal("GpiQueryRGBColor", file, line);
    return res;
}

BOOL DebugGpiErase(const char *file, int line, HPS hps)
{
    BOOL res = GpiErase(hps);
    if (!res)
	debugfatal("GpiErase", file, line);
    return res;
}

BOOL DebugGpiQueryBitmapInfoHeader(const char *file, int line, HBITMAP hbm, PBITMAPINFOHEADER2 pbmpData)
{
    BOOL res = GpiQueryBitmapInfoHeader(hbm, pbmpData);
    if (!res)
	debugfatal("GpiQueryBitmapInfoHeader", file, line);
    return res;
}


BOOL DebugWinDestroyAccelTable(const char *file, int line, HACCEL haccel)
{
    BOOL res = WinDestroyAccelTable(haccel);
    if (!res)
	debugfatal("WinDestroyAccelTable", file, line);
    return res;
}

HACCEL DebugWinLoadAccelTable(const char *file, int line, HAB hab, HMODULE hmod, ULONG idAccelTable)
{
    HACCEL res = WinLoadAccelTable(hab, hmod, idAccelTable);
    if (!res)
	debugfatal("WinLoadAccelTable", file, line);
    return res;
}

BOOL DebugWinSetAccelTable(const char *file, int line, HAB hab, HACCEL haccel, HWND hwndFrame)
{
    BOOL res = WinSetAccelTable(hab, haccel, hwndFrame);
    if (!res)
	debugfatal("WinSetAccelTable", file, line);
    return res;
}

HWND DebugWinLoadMenu(const char *file, int line, HWND hwndFrame, HMODULE hmod, ULONG idMenu)
{
    HWND res = WinLoadMenu(hwndFrame, hmod, idMenu);
    if (!res)
	debugfatal("WinLoadMenu", file, line);
    return res;
}


BOOL DebugWinQueryWindowPos(const char *file, int line, HWND hwnd, PSWP pswp)
{
    BOOL res = WinQueryWindowPos(hwnd, pswp);
    if (!res)
	debugfatal("WinQueryWindowPos", file, line);
    return res;
}

BOOL DebugWinPopupMenu(const char *file, int line, HWND hwndParent, HWND hwndOwner, HWND hwndMenu,
			LONG x, LONG y, LONG idItem, ULONG fs)
{
    BOOL res = WinPopupMenu(hwndParent, hwndOwner, hwndMenu, x, y, idItem, fs);
    if (!res)
	debugfatal("WinPopupMenu", file, line);
    return res;
}


struct {
	int	errcode;
	char	*errmsg;
} pmerrtab[] = {
	{0x1001, 	"PMERR_INVALID_HWND"},
	{0x1002, 	"PMERR_INVALID_HMQ"},
	{0x1003, 	"PMERR_PARAMETER_OUT_OF_RANGE"},
	{0x1004, 	"PMERR_WINDOW_LOCK_UNDERFLOW"},
	{0x1005, 	"PMERR_WINDOW_LOCK_OVERFLOW"},
	{0x1006, 	"PMERR_BAD_WINDOW_LOCK_COUNT"},
	{0x1007, 	"PMERR_WINDOW_NOT_LOCKED"},
	{0x1008, 	"PMERR_INVALID_SELECTOR"},
	{0x1009, 	"PMERR_CALL_FROM_WRONG_THREAD"},
	{0x100A, 	"PMERR_RESOURCE_NOT_FOUND"},
	{0x100B, 	"PMERR_INVALID_STRING_PARM"},
	{0x100C, 	"PMERR_INVALID_HHEAP"},
	{0x100D, 	"PMERR_INVALID_HEAP_POINTER"},
	{0x100E, 	"PMERR_INVALID_HEAP_SIZE_PARM"},
	{0x100F, 	"PMERR_INVALID_HEAP_SIZE"},
	{0x1010, 	"PMERR_INVALID_HEAP_SIZE_WORD"},
	{0x1011, 	"PMERR_HEAP_OUT_OF_MEMORY"},
	{0x1012, 	"PMERR_HEAP_MAX_SIZE_REACHED"},
	{0x1013, 	"PMERR_INVALID_HATOMTBL"},
	{0x1014, 	"PMERR_INVALID_ATOM"},
	{0x1015, 	"PMERR_INVALID_ATOM_NAME"},
	{0x1016, 	"PMERR_INVALID_INTEGER_ATOM"},
	{0x1017, 	"PMERR_ATOM_NAME_NOT_FOUND"},
	{0x1018, 	"PMERR_QUEUE_TOO_LARGE"},
	{0x1019, 	"PMERR_INVALID_FLAG"},
	{0x101A, 	"PMERR_INVALID_HACCEL"},
	{0x101B, 	"PMERR_INVALID_HPTR"},
	{0x101C, 	"PMERR_INVALID_HENUM"},
	{0x101D, 	"PMERR_INVALID_SRC_CODEPAGE"},
	{0x101E, 	"PMERR_INVALID_DST_CODEPAGE"},
	{0x101F, 	"PMERR_UNKNOWN_COMPONENT_ID"},
	{0x1020, 	"PMERR_UNKNOWN_ERROR_CODE"},
	{0x1021, 	"PMERR_SEVERITY_LEVELS"},
	{0x1022, 	"WINDBG_HWND_NOT_DESTROYED"},
	{0x1023, 	"WINDBG_HPTR_NOT_DESTROYED"},
	{0x1024, 	"WINDBG_HACCEL_NOT_DESTROYED"},
	{0x1025, 	"WINDBG_HENUM_NOT_DESTROYED"},
	{0x1026, 	"WINDBG_VISRGN_SEM_BUSY"},
	{0x1027, 	"WINDBG_USER_SEM_BUSY"},
	{0x1028, 	"WINDBG_DC_CACHE_BUSY"},
	{0x1029, 	"WINDBG_HOOK_STILL_INSTALLED"},
	{0x102A, 	"WINDBG_WINDOW_STILL_LOCKED"},
	{0x102B, 	"WINDBG_UPDATEPS_ASSERTION_FAIL"},
	{0x102C, 	"WINDBG_SENDMSG_WITHIN_USER_SEM"},
	{0x102D, 	"WINDBG_USER_SEM_NOT_ENTERED"},
	{0x102E, 	"WINDBG_PROC_NOT_EXPORTED"},
	{0x102F, 	"WINDBG_BAD_SENDMSG_HWND"},
	{0x1030, 	"WINDBG_ABNORMAL_EXIT"},
	{0x1031, 	"WINDBG_INTERNAL_REVISION"},
	{0x1032, 	"WINDBG_INITSYSTEM_FAILED"},
	{0x1033, 	"WINDBG_HATOMTBL_NOT_DESTROYED"},
	{0x1034, 	"PMERR_INVALID_RESOURCE_FORMAT"},
	{0x1035, 	"WINDBG_WINDOW_UNLOCK_WAIT"},
	{0x1036, 	"PMERR_NO_MSG_QUEUE"},
	{0x1037, 	"PMERR_CANNOT_SET_FOCUS"},
	{0x1038, 	"PMERR_QUEUE_FULL"},
	{0x1039, 	"PMERR_LIBRARY_LOAD_FAILED"},
	{0x103A, 	"PMERR_PROCEDURE_LOAD_FAILED"},
	{0x103B, 	"PMERR_LIBRARY_DELETE_FAILED"},
	{0x103C, 	"PMERR_PROCEDURE_DELETE_FAILED"},
	{0x103D, 	"PMERR_ARRAY_TOO_LARGE"},
	{0x103E, 	"PMERR_ARRAY_TOO_SMALL"},
	{0x103F, 	"PMERR_DATATYPE_ENTRY_BAD_INDEX"},
	{0x1040, 	"PMERR_DATATYPE_ENTRY_CTL_BAD"},
	{0x1041, 	"PMERR_DATATYPE_ENTRY_CTL_MISS"},
	{0x1042, 	"PMERR_DATATYPE_ENTRY_INVALID"},
	{0x1043, 	"PMERR_DATATYPE_ENTRY_NOT_NUM"},
	{0x1044, 	"PMERR_DATATYPE_ENTRY_NOT_OFF"},
	{0x1045, 	"PMERR_DATATYPE_INVALID"},
	{0x1046, 	"PMERR_DATATYPE_NOT_UNIQUE"},
	{0x1047, 	"PMERR_DATATYPE_TOO_LONG"},
	{0x1048, 	"PMERR_DATATYPE_TOO_SMALL"},
	{0x1049, 	"PMERR_DIRECTION_INVALID"},
	{0x104A, 	"PMERR_INVALID_HAB"},
	{0x104D, 	"PMERR_INVALID_HSTRUCT"},
	{0x104E, 	"PMERR_LENGTH_TOO_SMALL"},
	{0x104F, 	"PMERR_MSGID_TOO_SMALL"},
	{0x1050, 	"PMERR_NO_HANDLE_ALLOC"},
	{0x1051, 	"PMERR_NOT_IN_A_PM_SESSION"},
	{0x1052, 	"PMERR_MSG_QUEUE_ALREADY_EXISTS"},
	{0x1055, 	"PMERR_OLD_RESOURCE"},
	{0x1056, 	"PMERR_WPDSERVER_IS_ACTIVE"},
	{0x1057, 	"PMERR_WPDSERVER_NOT_STARTED"},
	{0x1058, 	"PMERR_SOMDD_IS_ACTIVE"},
	{0x1059, 	"PMERR_SOMDD_NOT_STARTED"},
	{0x10F0, 	"PMERR_BIDI_FIRST"},
	{0x10FF, 	"PMERR_BIDI_LAST"},
	{0x1101, 	"PMERR_INVALID_PIB"},
	{0x1102, 	"PMERR_INSUFF_SPACE_TO_ADD"},
	{0x1103, 	"PMERR_INVALID_GROUP_HANDLE"},
	{0x1104, 	"PMERR_DUPLICATE_TITLE"},
	{0x1105, 	"PMERR_INVALID_TITLE"},
	{0x1106, 	"PMERR_INVALID_TARGET_HANDLE"},
	{0x1107, 	"PMERR_HANDLE_NOT_IN_GROUP"},
	{0x1108, 	"PMERR_INVALID_PATH_STATEMENT"},
	{0x1109, 	"PMERR_NO_PROGRAM_FOUND"},
	{0x110A, 	"PMERR_INVALID_BUFFER_SIZE"},
	{0x110B, 	"PMERR_BUFFER_TOO_SMALL"},
	{0x110C, 	"PMERR_PL_INITIALISATION_FAIL"},
	{0x110D, 	"PMERR_CANT_DESTROY_SYS_GROUP"},
	{0x110E, 	"PMERR_INVALID_TYPE_CHANGE"},
	{0x110F, 	"PMERR_INVALID_PROGRAM_HANDLE"},
	{0x1110, 	"PMERR_NOT_CURRENT_PL_VERSION"},
	{0x1111, 	"PMERR_INVALID_CIRCULAR_REF"},
	{0x1112, 	"PMERR_MEMORY_ALLOCATION_ERR"},
	{0x1113, 	"PMERR_MEMORY_DEALLOCATION_ERR"},
	{0x1114, 	"PMERR_TASK_HEADER_TOO_BIG"},
	{0x1115, 	"PMERR_INVALID_INI_FILE_HANDLE"},
	{0x1116, 	"PMERR_MEMORY_SHARE"},
	{0x1117, 	"PMERR_OPEN_QUEUE"},
	{0x1118, 	"PMERR_CREATE_QUEUE"},
	{0x1119, 	"PMERR_WRITE_QUEUE"},
	{0x111A, 	"PMERR_READ_QUEUE"},
	{0x111B, 	"PMERR_CALL_NOT_EXECUTED"},
	{0x111C, 	"PMERR_UNKNOWN_APIPKT"},
	{0x111D, 	"PMERR_INITHREAD_EXISTS"},
	{0x111E, 	"PMERR_CREATE_THREAD"},
	{0x111F, 	"PMERR_NO_HK_PROFILE_INSTALLED"},
	{0x1120, 	"PMERR_INVALID_DIRECTORY"},
	{0x1121, 	"PMERR_WILDCARD_IN_FILENAME"},
	{0x1122, 	"PMERR_FILENAME_BUFFER_FULL"},
	{0x1123, 	"PMERR_FILENAME_TOO_LONG"},
	{0x1124, 	"PMERR_INI_FILE_IS_SYS_OR_USER"},
	{0x1125, 	"PMERR_BROADCAST_PLMSG"},
	{0x1126, 	"PMERR_190_INIT_DONE"},
	{0x1127, 	"PMERR_HMOD_FOR_PMSHAPI"},
	{0x1128, 	"PMERR_SET_HK_PROFILE"},
	{0x1129, 	"PMERR_API_NOT_ALLOWED"},
	{0x112A, 	"PMERR_INI_STILL_OPEN"},
	{0x112B, 	"PMERR_PROGDETAILS_NOT_IN_INI"},
	{0x112C, 	"PMERR_PIBSTRUCT_NOT_IN_INI"},
	{0x112D, 	"PMERR_INVALID_DISKPROGDETAILS"},
	{0x112E, 	"PMERR_PROGDETAILS_READ_FAILURE"},
	{0x112F, 	"PMERR_PROGDETAILS_WRITE_FAILURE"},
	{0x1130, 	"PMERR_PROGDETAILS_QSIZE_FAILURE"},
	{0x1131, 	"PMERR_INVALID_PROGDETAILS"},
	{0x1132, 	"PMERR_SHEPROFILEHOOK_NOT_FOUND"},
	{0x1133, 	"PMERR_190PLCONVERTED"},
	{0x1134, 	"PMERR_FAILED_TO_CONVERT_INI_PL"},
	{0x1135, 	"PMERR_PMSHAPI_NOT_INITIALISED"},
	{0x1136, 	"PMERR_INVALID_SHELL_API_HOOK_ID"},
	{0x1200, 	"PMERR_DOS_ERROR"},
	{0x1201, 	"PMERR_NO_SPACE"},
	{0x1202, 	"PMERR_INVALID_SWITCH_HANDLE"},
	{0x1203, 	"PMERR_NO_HANDLE"},
	{0x1204, 	"PMERR_INVALID_PROCESS_ID"},
	{0x1205, 	"PMERR_NOT_SHELL"},
	{0x1206, 	"PMERR_INVALID_WINDOW"},
	{0x1207, 	"PMERR_INVALID_POST_MSG"},
	{0x1208, 	"PMERR_INVALID_PARAMETERS"},
	{0x1209, 	"PMERR_INVALID_PROGRAM_TYPE"},
	{0x120A, 	"PMERR_NOT_EXTENDED_FOCUS"},
	{0x120B, 	"PMERR_INVALID_SESSION_ID"},
	{0x120C, 	"PMERR_SMG_INVALID_ICON_FILE"},
	{0x120D, 	"PMERR_SMG_ICON_NOT_CREATED"},
	{0x120E, 	"PMERR_SHL_DEBUG"},
	{0x1301, 	"PMERR_OPENING_INI_FILE"},
	{0x1302, 	"PMERR_INI_FILE_CORRUPT"},
	{0x1303, 	"PMERR_INVALID_PARM"},
	{0x1304, 	"PMERR_NOT_IN_IDX"},
	{0x1305, 	"PMERR_NO_ENTRIES_IN_GROUP"},
	{0x1306, 	"PMERR_INI_WRITE_FAIL"},
	{0x1307, 	"PMERR_IDX_FULL"},
	{0x1308, 	"PMERR_INI_PROTECTED"},
	{0x1309, 	"PMERR_MEMORY_ALLOC"},
	{0x130A, 	"PMERR_INI_INIT_ALREADY_DONE"},
	{0x130B, 	"PMERR_INVALID_INTEGER"},
	{0x130C, 	"PMERR_INVALID_ASCIIZ"},
	{0x130D, 	"PMERR_CAN_NOT_CALL_SPOOLER"},
	{0x1401, 	"PMERR_WARNING_WINDOW_NOT_KILLED"},
	{0x1402, 	"PMERR_ERROR_INVALID_WINDOW"},
	{0x1403, 	"PMERR_ALREADY_INITIALIZED"},
	{0x1405, 	"PMERR_MSG_PROG_NO_MOU"},
	{0x1406, 	"PMERR_MSG_PROG_NON_RECOV"},
	{0x1407, 	"PMERR_WINCONV_INVALID_PATH"},
	{0x1408, 	"PMERR_PI_NOT_INITIALISED"},
	{0x1409, 	"PMERR_PL_NOT_INITIALISED"},
	{0x140A, 	"PMERR_NO_TASK_MANAGER"},
	{0x140B, 	"PMERR_SAVE_NOT_IN_PROGRESS"},
	{0x140C, 	"PMERR_NO_STACK_SPACE"},
	{0x140D, 	"PMERR_INVALID_COLR_FIELD"},
	{0x140E, 	"PMERR_INVALID_COLR_VALUE"},
	{0x140F, 	"PMERR_COLR_WRITE"},
	{0x1501, 	"PMERR_TARGET_FILE_EXISTS"},
	{0x1502, 	"PMERR_SOURCE_SAME_AS_TARGET"},
	{0x1503, 	"PMERR_SOURCE_FILE_NOT_FOUND"},
	{0x1504, 	"PMERR_INVALID_NEW_PATH"},
	{0x1505, 	"PMERR_TARGET_FILE_NOT_FOUND"},
	{0x1506, 	"PMERR_INVALID_DRIVE_NUMBER"},
	{0x1507, 	"PMERR_NAME_TOO_LONG"},
	{0x1508, 	"PMERR_NOT_ENOUGH_ROOM_ON_DISK"},
	{0x1509, 	"PMERR_NOT_ENOUGH_MEM"},
	{0x150B, 	"PMERR_LOG_DRV_DOES_NOT_EXIST"},
	{0x150C, 	"PMERR_INVALID_DRIVE"},
	{0x150D, 	"PMERR_ACCESS_DENIED"},
	{0x150E, 	"PMERR_NO_FIRST_SLASH"},
	{0x150F, 	"PMERR_READ_ONLY_FILE"},
	{0x151F, 	"PMERR_GROUP_PROTECTED"},
	{0x152F, 	"PMERR_INVALID_PROGRAM_CATEGORY"},
	{0x1530, 	"PMERR_INVALID_APPL"},
	{0x1531, 	"PMERR_CANNOT_START"},
	{0x1532, 	"PMERR_STARTED_IN_BACKGROUND"},
	{0x1533, 	"PMERR_INVALID_HAPP"},
	{0x1534, 	"PMERR_CANNOT_STOP"},
	{0x1601, 	"PMERR_INTERNAL_ERROR_1"},
	{0x1602, 	"PMERR_INTERNAL_ERROR_2"},
	{0x1603, 	"PMERR_INTERNAL_ERROR_3"},
	{0x1604, 	"PMERR_INTERNAL_ERROR_4"},
	{0x1605, 	"PMERR_INTERNAL_ERROR_5"},
	{0x1606, 	"PMERR_INTERNAL_ERROR_6"},
	{0x1607, 	"PMERR_INTERNAL_ERROR_7"},
	{0x1608, 	"PMERR_INTERNAL_ERROR_8"},
	{0x1609, 	"PMERR_INTERNAL_ERROR_9"},
	{0x160A, 	"PMERR_INTERNAL_ERROR_10"},
	{0x160B, 	"PMERR_INTERNAL_ERROR_11"},
	{0x160C, 	"PMERR_INTERNAL_ERROR_12"},
	{0x160D, 	"PMERR_INTERNAL_ERROR_13"},
	{0x160E, 	"PMERR_INTERNAL_ERROR_14"},
	{0x160F, 	"PMERR_INTERNAL_ERROR_15"},
	{0x1610, 	"PMERR_INTERNAL_ERROR_16"},
	{0x1611, 	"PMERR_INTERNAL_ERROR_17"},
	{0x1612, 	"PMERR_INTERNAL_ERROR_18"},
	{0x1613, 	"PMERR_INTERNAL_ERROR_19"},
	{0x1614, 	"PMERR_INTERNAL_ERROR_20"},
	{0x1615, 	"PMERR_INTERNAL_ERROR_21"},
	{0x1616, 	"PMERR_INTERNAL_ERROR_22"},
	{0x1617, 	"PMERR_INTERNAL_ERROR_23"},
	{0x1618, 	"PMERR_INTERNAL_ERROR_24"},
	{0x1619, 	"PMERR_INTERNAL_ERROR_25"},
	{0x161A, 	"PMERR_INTERNAL_ERROR_26"},
	{0x161B, 	"PMERR_INTERNAL_ERROR_27"},
	{0x161C, 	"PMERR_INTERNAL_ERROR_28"},
	{0x161D, 	"PMERR_INTERNAL_ERROR_29"},
	{0x1630, 	"PMERR_INVALID_FREE_MESSAGE_ID"},
	{0x1641, 	"PMERR_FUNCTION_NOT_SUPPORTED"},
	{0x1642, 	"PMERR_INVALID_ARRAY_COUNT"},
	{0x1643, 	"PMERR_INVALID_LENGTH"},
	{0x1644, 	"PMERR_INVALID_BUNDLE_TYPE"},
	{0x1645, 	"PMERR_INVALID_PARAMETER"},
	{0x1646, 	"PMERR_INVALID_NUMBER_OF_PARMS"},
	{0x1647, 	"PMERR_GREATER_THAN_64K"},
	{0x1648, 	"PMERR_INVALID_PARAMETER_TYPE"},
	{0x1649, 	"PMERR_NEGATIVE_STRCOND_DIM"},
	{0x164A, 	"PMERR_INVALID_NUMBER_OF_TYPES"},
	{0x164B, 	"PMERR_INCORRECT_HSTRUCT"},
	{0x164C, 	"PMERR_INVALID_ARRAY_SIZE"},
	{0x164D, 	"PMERR_INVALID_CONTROL_DATATYPE"},
	{0x164E, 	"PMERR_INCOMPLETE_CONTROL_SEQU"},
	{0x164F, 	"PMERR_INVALID_DATATYPE"},
	{0x1650, 	"PMERR_INCORRECT_DATATYPE"},
	{0x1651, 	"PMERR_NOT_SELF_DESCRIBING_DTYP"},
	{0x1652, 	"PMERR_INVALID_CTRL_SEQ_INDEX"},
	{0x1653, 	"PMERR_INVALID_TYPE_FOR_LENGTH"},
	{0x1654, 	"PMERR_INVALID_TYPE_FOR_OFFSET"},
	{0x1655, 	"PMERR_INVALID_TYPE_FOR_MPARAM"},
	{0x1656, 	"PMERR_INVALID_MESSAGE_ID"},
	{0x1657, 	"PMERR_C_LENGTH_TOO_SMALL"},
	{0x1658, 	"PMERR_APPL_STRUCTURE_TOO_SMALL"},
	{0x1659, 	"PMERR_INVALID_ERRORINFO_HANDLE"},
	{0x165A, 	"PMERR_INVALID_CHARACTER_INDEX"},
	{0x1F00, 	"PMERR_NOT_DRAGGING"},
	{0x1F01, 	"PMERR_ALREADY_DRAGGING"},
	{0x1F02, 	"PMERR_NOFILTERED_ITEMS"},
	{0x1F03, 	"PMERR_COMPARISON_FAILED"},
	{0x1F04, 	"PMERR_RECORD_CURRENTLY_INSERTED"},
	{0x1F05, 	"PMERR_FI_CURRENTLY_INSERTED"},
	{0x1F06, 	"PMERR_UPDATE_IN_PROGRESS"},
	{0x2001, 	"PMERR_ALREADY_IN_AREA"},
	{0x2002, 	"PMERR_ALREADY_IN_ELEMENT"},
	{0x2003, 	"PMERR_ALREADY_IN_PATH"},
	{0x2004, 	"PMERR_ALREADY_IN_SEG"},
	{0x2005, 	"PMERR_AREA_INCOMPLETE"},
	{0x2006, 	"PMERR_BASE_ERROR"},
	{0x2007, 	"PMERR_BITBLT_LENGTH_EXCEEDED"},
	{0x2008, 	"PMERR_BITMAP_IN_USE"},
	{0x2009, 	"PMERR_BITMAP_IS_SELECTED"},
	{0x200A, 	"PMERR_BITMAP_NOT_FOUND"},
	{0x200B, 	"PMERR_BITMAP_NOT_SELECTED"},
	{0x200C, 	"PMERR_BOUNDS_OVERFLOW"},
	{0x200D, 	"PMERR_CALLED_SEG_IS_CHAINED"},
	{0x200E, 	"PMERR_CALLED_SEG_IS_CURRENT"},
	{0x200F, 	"PMERR_CALLED_SEG_NOT_FOUND"},
	{0x2010, 	"PMERR_CANNOT_DELETE_ALL_DATA"},
	{0x2011, 	"PMERR_CANNOT_REPLACE_ELEMENT_0"},
	{0x2012, 	"PMERR_COL_TABLE_NOT_REALIZABLE"},
	{0x2013, 	"PMERR_COL_TABLE_NOT_REALIZED"},
	{0x2014, 	"PMERR_COORDINATE_OVERFLOW"},
	{0x2015, 	"PMERR_CORR_FORMAT_MISMATCH"},
	{0x2016, 	"PMERR_DATA_TOO_LONG"},
	{0x2017, 	"PMERR_DC_IS_ASSOCIATED"},
	{0x2018, 	"PMERR_DESC_STRING_TRUNCATED"},
	{0x2019, 	"PMERR_DEVICE_DRIVER_ERROR_1"},
	{0x201A, 	"PMERR_DEVICE_DRIVER_ERROR_2"},
	{0x201B, 	"PMERR_DEVICE_DRIVER_ERROR_3"},
	{0x201C, 	"PMERR_DEVICE_DRIVER_ERROR_4"},
	{0x201D, 	"PMERR_DEVICE_DRIVER_ERROR_5"},
	{0x201E, 	"PMERR_DEVICE_DRIVER_ERROR_6"},
	{0x201F, 	"PMERR_DEVICE_DRIVER_ERROR_7"},
	{0x2020, 	"PMERR_DEVICE_DRIVER_ERROR_8"},
	{0x2021, 	"PMERR_DEVICE_DRIVER_ERROR_9"},
	{0x2022, 	"PMERR_DEVICE_DRIVER_ERROR_10"},
	{0x2023, 	"PMERR_DEV_FUNC_NOT_INSTALLED"},
	{0x2024, 	"PMERR_DOSOPEN_FAILURE"},
	{0x2025, 	"PMERR_DOSREAD_FAILURE"},
	{0x2026, 	"PMERR_DRIVER_NOT_FOUND"},
	{0x2027, 	"PMERR_DUP_SEG"},
	{0x2028, 	"PMERR_DYNAMIC_SEG_SEQ_ERROR"},
	{0x2029, 	"PMERR_DYNAMIC_SEG_ZERO_INV"},
	{0x202A, 	"PMERR_ELEMENT_INCOMPLETE"},
	{0x202B, 	"PMERR_ESC_CODE_NOT_SUPPORTED"},
	{0x202C, 	"PMERR_EXCEEDS_MAX_SEG_LENGTH"},
	{0x202D, 	"PMERR_FONT_AND_MODE_MISMATCH"},
	{0x202E, 	"PMERR_FONT_FILE_NOT_LOADED"},
	{0x202F, 	"PMERR_FONT_NOT_LOADED"},
	{0x2030, 	"PMERR_FONT_TOO_BIG"},
	{0x2031, 	"PMERR_HARDWARE_INIT_FAILURE"},
	{0x2032, 	"PMERR_HBITMAP_BUSY"},
	{0x2033, 	"PMERR_HDC_BUSY"},
	{0x2034, 	"PMERR_HRGN_BUSY"},
	{0x2035, 	"PMERR_HUGE_FONTS_NOT_SUPPORTED"},
	{0x2036, 	"PMERR_ID_HAS_NO_BITMAP"},
	{0x2037, 	"PMERR_IMAGE_INCOMPLETE"},
	{0x2038, 	"PMERR_INCOMPAT_COLOR_FORMAT"},
	{0x2039, 	"PMERR_INCOMPAT_COLOR_OPTIONS"},
	{0x203A, 	"PMERR_INCOMPATIBLE_BITMAP"},
	{0x203B, 	"PMERR_INCOMPATIBLE_METAFILE"},
	{0x203C, 	"PMERR_INCORRECT_DC_TYPE"},
	{0x203D, 	"PMERR_INSUFFICIENT_DISK_SPACE"},
	{0x203E, 	"PMERR_INSUFFICIENT_MEMORY"},
	{0x203F, 	"PMERR_INV_ANGLE_PARM"},
	{0x2040, 	"PMERR_INV_ARC_CONTROL"},
	{0x2041, 	"PMERR_INV_AREA_CONTROL"},
	{0x2042, 	"PMERR_INV_ARC_POINTS"},
	{0x2043, 	"PMERR_INV_ATTR_MODE"},
	{0x2044, 	"PMERR_INV_BACKGROUND_COL_ATTR"},
	{0x2045, 	"PMERR_INV_BACKGROUND_MIX_ATTR"},
	{0x2046, 	"PMERR_INV_BITBLT_MIX"},
	{0x2047, 	"PMERR_INV_BITBLT_STYLE"},
	{0x2048, 	"PMERR_INV_BITMAP_DIMENSION"},
	{0x2049, 	"PMERR_INV_BOX_CONTROL"},
	{0x204A, 	"PMERR_INV_BOX_ROUNDING_PARM"},
	{0x204B, 	"PMERR_INV_CHAR_ANGLE_ATTR"},
	{0x204C, 	"PMERR_INV_CHAR_DIRECTION_ATTR"},
	{0x204D, 	"PMERR_INV_CHAR_MODE_ATTR"},
	{0x204E, 	"PMERR_INV_CHAR_POS_OPTIONS"},
	{0x204F, 	"PMERR_INV_CHAR_SET_ATTR"},
	{0x2050, 	"PMERR_INV_CHAR_SHEAR_ATTR"},
	{0x2051, 	"PMERR_INV_CLIP_PATH_OPTIONS"},
	{0x2052, 	"PMERR_INV_CODEPAGE"},
	{0x2053, 	"PMERR_INV_COLOR_ATTR"},
	{0x2054, 	"PMERR_INV_COLOR_DATA"},
	{0x2055, 	"PMERR_INV_COLOR_FORMAT"},
	{0x2056, 	"PMERR_INV_COLOR_INDEX"},
	{0x2057, 	"PMERR_INV_COLOR_OPTIONS"},
	{0x2058, 	"PMERR_INV_COLOR_START_INDEX"},
	{0x2059, 	"PMERR_INV_COORD_OFFSET"},
	{0x205A, 	"PMERR_INV_COORD_SPACE"},
	{0x205B, 	"PMERR_INV_COORDINATE"},
	{0x205C, 	"PMERR_INV_CORRELATE_DEPTH"},
	{0x205D, 	"PMERR_INV_CORRELATE_TYPE"},
	{0x205E, 	"PMERR_INV_CURSOR_BITMAP"},
	{0x205F, 	"PMERR_INV_DC_DATA"},
	{0x2060, 	"PMERR_INV_DC_TYPE"},
	{0x2061, 	"PMERR_INV_DEVICE_NAME"},
	{0x2062, 	"PMERR_INV_DEV_MODES_OPTIONS"},
	{0x2063, 	"PMERR_INV_DRAW_CONTROL"},
	{0x2064, 	"PMERR_INV_DRAW_VALUE"},
	{0x2065, 	"PMERR_INV_DRAWING_MODE"},
	{0x2066, 	"PMERR_INV_DRIVER_DATA"},
	{0x2067, 	"PMERR_INV_DRIVER_NAME"},
	{0x2068, 	"PMERR_INV_DRAW_BORDER_OPTION"},
	{0x2069, 	"PMERR_INV_EDIT_MODE"},
	{0x206A, 	"PMERR_INV_ELEMENT_OFFSET"},
	{0x206B, 	"PMERR_INV_ELEMENT_POINTER"},
	{0x206C, 	"PMERR_INV_END_PATH_OPTIONS"},
	{0x206D, 	"PMERR_INV_ESC_CODE"},
	{0x206E, 	"PMERR_INV_ESCAPE_DATA"},
	{0x206F, 	"PMERR_INV_EXTENDED_LCID"},
	{0x2070, 	"PMERR_INV_FILL_PATH_OPTIONS"},
	{0x2071, 	"PMERR_INV_FIRST_CHAR"},
	{0x2072, 	"PMERR_INV_FONT_ATTRS"},
	{0x2073, 	"PMERR_INV_FONT_FILE_DATA"},
	{0x2074, 	"PMERR_INV_FOR_THIS_DC_TYPE"},
	{0x2075, 	"PMERR_INV_FORMAT_CONTROL"},
	{0x2076, 	"PMERR_INV_FORMS_CODE"},
	{0x2077, 	"PMERR_INV_FONTDEF"},
	{0x2078, 	"PMERR_INV_GEOM_LINE_WIDTH_ATTR"},
	{0x2079, 	"PMERR_INV_GETDATA_CONTROL"},
	{0x207A, 	"PMERR_INV_GRAPHICS_FIELD"},
	{0x207B, 	"PMERR_INV_HBITMAP"},
	{0x207C, 	"PMERR_INV_HDC"},
	{0x207D, 	"PMERR_INV_HJOURNAL"},
	{0x207E, 	"PMERR_INV_HMF"},
	{0x207F, 	"PMERR_INV_HPS"},
	{0x2080, 	"PMERR_INV_HRGN"},
	{0x2081, 	"PMERR_INV_ID"},
	{0x2082, 	"PMERR_INV_IMAGE_DATA_LENGTH"},
	{0x2083, 	"PMERR_INV_IMAGE_DIMENSION"},
	{0x2084, 	"PMERR_INV_IMAGE_FORMAT"},
	{0x2085, 	"PMERR_INV_IN_AREA"},
	{0x2086, 	"PMERR_INV_IN_CALLED_SEG"},
	{0x2087, 	"PMERR_INV_IN_CURRENT_EDIT_MODE"},
	{0x2088, 	"PMERR_INV_IN_DRAW_MODE"},
	{0x2089, 	"PMERR_INV_IN_ELEMENT"},
	{0x208A, 	"PMERR_INV_IN_IMAGE"},
	{0x208B, 	"PMERR_INV_IN_PATH"},
	{0x208C, 	"PMERR_INV_IN_RETAIN_MODE"},
	{0x208D, 	"PMERR_INV_IN_SEG"},
	{0x208E, 	"PMERR_INV_IN_VECTOR_SYMBOL"},
	{0x208F, 	"PMERR_INV_INFO_TABLE"},
	{0x2090, 	"PMERR_INV_JOURNAL_OPTION"},
	{0x2091, 	"PMERR_INV_KERNING_FLAGS"},
	{0x2092, 	"PMERR_INV_LENGTH_OR_COUNT"},
	{0x2093, 	"PMERR_INV_LINE_END_ATTR"},
	{0x2094, 	"PMERR_INV_LINE_JOIN_ATTR"},
	{0x2095, 	"PMERR_INV_LINE_TYPE_ATTR"},
	{0x2096, 	"PMERR_INV_LINE_WIDTH_ATTR"},
	{0x2097, 	"PMERR_INV_LOGICAL_ADDRESS"},
	{0x2098, 	"PMERR_INV_MARKER_BOX_ATTR"},
	{0x2099, 	"PMERR_INV_MARKER_SET_ATTR"},
	{0x209A, 	"PMERR_INV_MARKER_SYMBOL_ATTR"},
	{0x209B, 	"PMERR_INV_MATRIX_ELEMENT"},
	{0x209C, 	"PMERR_INV_MAX_HITS"},
	{0x209D, 	"PMERR_INV_METAFILE"},
	{0x209E, 	"PMERR_INV_METAFILE_LENGTH"},
	{0x209F, 	"PMERR_INV_METAFILE_OFFSET"},
	{0x20A0, 	"PMERR_INV_MICROPS_DRAW_CONTROL"},
	{0x20A1, 	"PMERR_INV_MICROPS_FUNCTION"},
	{0x20A2, 	"PMERR_INV_MICROPS_ORDER"},
	{0x20A3, 	"PMERR_INV_MIX_ATTR"},
	{0x20A4, 	"PMERR_INV_MODE_FOR_OPEN_DYN"},
	{0x20A5, 	"PMERR_INV_MODE_FOR_REOPEN_SEG"},
	{0x20A6, 	"PMERR_INV_MODIFY_PATH_MODE"},
	{0x20A7, 	"PMERR_INV_MULTIPLIER"},
	{0x20A8, 	"PMERR_INV_NESTED_FIGURES"},
	{0x20A9, 	"PMERR_INV_OR_INCOMPAT_OPTIONS"},
	{0x20AA, 	"PMERR_INV_ORDER_LENGTH"},
	{0x20AB, 	"PMERR_INV_ORDERING_PARM"},
	{0x20AC, 	"PMERR_INV_OUTSIDE_DRAW_MODE"},
	{0x20AD, 	"PMERR_INV_PAGE_VIEWPORT"},
	{0x20AE, 	"PMERR_INV_PATH_ID"},
	{0x20AF, 	"PMERR_INV_PATH_MODE"},
	{0x20B0, 	"PMERR_INV_PATTERN_ATTR"},
	{0x20B1, 	"PMERR_INV_PATTERN_REF_PT_ATTR"},
	{0x20B2, 	"PMERR_INV_PATTERN_SET_ATTR"},
	{0x20B3, 	"PMERR_INV_PATTERN_SET_FONT"},
	{0x20B4, 	"PMERR_INV_PICK_APERTURE_OPTION"},
	{0x20B5, 	"PMERR_INV_PICK_APERTURE_POSN"},
	{0x20B6, 	"PMERR_INV_PICK_APERTURE_SIZE"},
	{0x20B7, 	"PMERR_INV_PICK_NUMBER"},
	{0x20B8, 	"PMERR_INV_PLAY_METAFILE_OPTION"},
	{0x20B9, 	"PMERR_INV_PRIMITIVE_TYPE"},
	{0x20BA, 	"PMERR_INV_PS_SIZE"},
	{0x20BB, 	"PMERR_INV_PUTDATA_FORMAT"},
	{0x20BC, 	"PMERR_INV_QUERY_ELEMENT_NO"},
	{0x20BD, 	"PMERR_INV_RECT"},
	{0x20BE, 	"PMERR_INV_REGION_CONTROL"},
	{0x20BF, 	"PMERR_INV_REGION_MIX_MODE"},
	{0x20C0, 	"PMERR_INV_REPLACE_MODE_FUNC"},
	{0x20C1, 	"PMERR_INV_RESERVED_FIELD"},
	{0x20C2, 	"PMERR_INV_RESET_OPTIONS"},
	{0x20C3, 	"PMERR_INV_RGBCOLOR"},
	{0x20C4, 	"PMERR_INV_SCAN_START"},
	{0x20C5, 	"PMERR_INV_SEG_ATTR"},
	{0x20C6, 	"PMERR_INV_SEG_ATTR_VALUE"},
	{0x20C7, 	"PMERR_INV_SEG_CH_LENGTH"},
	{0x20C8, 	"PMERR_INV_SEG_NAME"},
	{0x20C9, 	"PMERR_INV_SEG_OFFSET"},
	{0x20CA, 	"PMERR_INV_SETID"},
	{0x20CB, 	"PMERR_INV_SETID_TYPE"},
	{0x20CC, 	"PMERR_INV_SET_VIEWPORT_OPTION"},
	{0x20CD, 	"PMERR_INV_SHARPNESS_PARM"},
	{0x20CE, 	"PMERR_INV_SOURCE_OFFSET"},
	{0x20CF, 	"PMERR_INV_STOP_DRAW_VALUE"},
	{0x20D0, 	"PMERR_INV_TRANSFORM_TYPE"},
	{0x20D1, 	"PMERR_INV_USAGE_PARM"},
	{0x20D2, 	"PMERR_INV_VIEWING_LIMITS"},
	{0x20D3, 	"PMERR_JFILE_BUSY"},
	{0x20D4, 	"PMERR_JNL_FUNC_DATA_TOO_LONG"},
	{0x20D5, 	"PMERR_KERNING_NOT_SUPPORTED"},
	{0x20D6, 	"PMERR_LABEL_NOT_FOUND"},
	{0x20D7, 	"PMERR_MATRIX_OVERFLOW"},
	{0x20D8, 	"PMERR_METAFILE_INTERNAL_ERROR"},
	{0x20D9, 	"PMERR_METAFILE_IN_USE"},
	{0x20DA, 	"PMERR_METAFILE_LIMIT_EXCEEDED"},
	{0x20DB, 	"PMERR_NAME_STACK_FULL"},
	{0x20DC, 	"PMERR_NOT_CREATED_BY_DEVOPENDC"},
	{0x20DD, 	"PMERR_NOT_IN_AREA"},
	{0x20DE, 	"PMERR_NOT_IN_DRAW_MODE"},
	{0x20DF, 	"PMERR_NOT_IN_ELEMENT"},
	{0x20E0, 	"PMERR_NOT_IN_IMAGE"},
	{0x20E1, 	"PMERR_NOT_IN_PATH"},
	{0x20E2, 	"PMERR_NOT_IN_RETAIN_MODE"},
	{0x20E3, 	"PMERR_NOT_IN_SEG"},
	{0x20E4, 	"PMERR_NO_BITMAP_SELECTED"},
	{0x20E5, 	"PMERR_NO_CURRENT_ELEMENT"},
	{0x20E6, 	"PMERR_NO_CURRENT_SEG"},
	{0x20E7, 	"PMERR_NO_METAFILE_RECORD_HANDLE"},
	{0x20E8, 	"PMERR_ORDER_TOO_BIG"},
	{0x20E9, 	"PMERR_OTHER_SET_ID_REFS"},
	{0x20EA, 	"PMERR_OVERRAN_SEG"},
	{0x20EB, 	"PMERR_OWN_SET_ID_REFS"},
	{0x20EC, 	"PMERR_PATH_INCOMPLETE"},
	{0x20ED, 	"PMERR_PATH_LIMIT_EXCEEDED"},
	{0x20EE, 	"PMERR_PATH_UNKNOWN"},
	{0x20EF, 	"PMERR_PEL_IS_CLIPPED"},
	{0x20F0, 	"PMERR_PEL_NOT_AVAILABLE"},
	{0x20F1, 	"PMERR_PRIMITIVE_STACK_EMPTY"},
	{0x20F2, 	"PMERR_PROLOG_ERROR"},
	{0x20F3, 	"PMERR_PROLOG_SEG_ATTR_NOT_SET"},
	{0x20F4, 	"PMERR_PS_BUSY"},
	{0x20F5, 	"PMERR_PS_IS_ASSOCIATED"},
	{0x20F6, 	"PMERR_RAM_JNL_FILE_TOO_SMALL"},
	{0x20F7, 	"PMERR_REALIZE_NOT_SUPPORTED"},
	{0x20F8, 	"PMERR_REGION_IS_CLIP_REGION"},
	{0x20F9, 	"PMERR_RESOURCE_DEPLETION"},
	{0x20FA, 	"PMERR_SEG_AND_REFSEG_ARE_SAME"},
	{0x20FB, 	"PMERR_SEG_CALL_RECURSIVE"},
	{0x20FC, 	"PMERR_SEG_CALL_STACK_EMPTY"},
	{0x20FD, 	"PMERR_SEG_CALL_STACK_FULL"},
	{0x20FE, 	"PMERR_SEG_IS_CURRENT"},
	{0x20FF, 	"PMERR_SEG_NOT_CHAINED"},
	{0x2100, 	"PMERR_SEG_NOT_FOUND"},
	{0x2101, 	"PMERR_SEG_STORE_LIMIT_EXCEEDED"},
	{0x2102, 	"PMERR_SETID_IN_USE"},
	{0x2103, 	"PMERR_SETID_NOT_FOUND"},
	{0x2104, 	"PMERR_STARTDOC_NOT_ISSUED"},
	{0x2105, 	"PMERR_STOP_DRAW_OCCURRED"},
	{0x2106, 	"PMERR_TOO_MANY_METAFILES_IN_USE"},
	{0x2107, 	"PMERR_TRUNCATED_ORDER"},
	{0x2108, 	"PMERR_UNCHAINED_SEG_ZERO_INV"},
	{0x2109, 	"PMERR_UNSUPPORTED_ATTR"},
	{0x210A, 	"PMERR_UNSUPPORTED_ATTR_VALUE"},
	{0x210B, 	"PMERR_ENDDOC_NOT_ISSUED"},
	{0x210C, 	"PMERR_PS_NOT_ASSOCIATED"},
	{0x210D, 	"PMERR_INV_FLOOD_FILL_OPTIONS"},
	{0x210E, 	"PMERR_INV_FACENAME"},
	{0x210F, 	"PMERR_PALETTE_SELECTED"},
	{0x2110, 	"PMERR_NO_PALETTE_SELECTED"},
	{0x2111, 	"PMERR_INV_HPAL"},
	{0x2112, 	"PMERR_PALETTE_BUSY"},
	{0x2113, 	"PMERR_START_POINT_CLIPPED"},
	{0x2114, 	"PMERR_NO_FILL"},
	{0x2115, 	"PMERR_INV_FACENAMEDESC"},
	{0x2116, 	"PMERR_INV_BITMAP_DATA"},
	{0x2117, 	"PMERR_INV_CHAR_ALIGN_ATTR"},
	{0x2118, 	"PMERR_INV_HFONT"},
	{0x2119, 	"PMERR_HFONT_IS_SELECTED"},
	{0x2120, 	"PMERR_DRVR_NOT_SUPPORTED"},
	{0x4001, 	"PMERR_SPL_DRIVER_ERROR"},
	{0x4002, 	"PMERR_SPL_DEVICE_ERROR"},
	{0x4003, 	"PMERR_SPL_DEVICE_NOT_INSTALLED"},
	{0x4004, 	"PMERR_SPL_QUEUE_ERROR"},
	{0x4005, 	"PMERR_SPL_INV_HSPL"},
	{0x4006, 	"PMERR_SPL_NO_DISK_SPACE"},
	{0x4007, 	"PMERR_SPL_NO_MEMORY"},
	{0x4008, 	"PMERR_SPL_PRINT_ABORT"},
	{0x4009, 	"PMERR_SPL_SPOOLER_NOT_INSTALLED"},
	{0x400A, 	"PMERR_SPL_INV_FORMS_CODE"},
	{0x400B, 	"PMERR_SPL_INV_PRIORITY"},
	{0x400C, 	"PMERR_SPL_NO_FREE_JOB_ID"},
	{0x400D, 	"PMERR_SPL_NO_DATA"},
	{0x400E, 	"PMERR_SPL_INV_TOKEN"},
	{0x400F, 	"PMERR_SPL_INV_DATATYPE"},
	{0x4010, 	"PMERR_SPL_PROCESSOR_ERROR"},
	{0x4011, 	"PMERR_SPL_INV_JOB_ID"},
	{0x4012, 	"PMERR_SPL_JOB_NOT_PRINTING"},
	{0x4013, 	"PMERR_SPL_JOB_PRINTING"},
	{0x4014, 	"PMERR_SPL_QUEUE_ALREADY_EXISTS"},
	{0x4015, 	"PMERR_SPL_INV_QUEUE_NAME"},
	{0x4016, 	"PMERR_SPL_QUEUE_NOT_EMPTY"},
	{0x4017, 	"PMERR_SPL_DEVICE_ALREADY_EXISTS"},
	{0x4018, 	"PMERR_SPL_DEVICE_LIMIT_REACHED"},
	{0x4019, 	"PMERR_SPL_STATUS_STRING_TRUNC"},
	{0x401A, 	"PMERR_SPL_INV_LENGTH_OR_COUNT"},
	{0x401B, 	"PMERR_SPL_FILE_NOT_FOUND"},
	{0x401C, 	"PMERR_SPL_CANNOT_OPEN_FILE"},
	{0x401D, 	"PMERR_SPL_DRIVER_NOT_INSTALLED"},
	{0x401E, 	"PMERR_SPL_INV_PROCESSOR_DATTYPE"},
	{0x401F, 	"PMERR_SPL_INV_DRIVER_DATATYPE"},
	{0x4020, 	"PMERR_SPL_PROCESSOR_NOT_INST"},
	{0x4021, 	"PMERR_SPL_NO_SUCH_LOG_ADDRESS"},
	{0x4022, 	"PMERR_SPL_PRINTER_NOT_FOUND"},
	{0x4023, 	"PMERR_SPL_DD_NOT_FOUND"},
	{0x4024, 	"PMERR_SPL_QUEUE_NOT_FOUND"},
	{0x4025, 	"PMERR_SPL_MANY_QUEUES_ASSOC"},
	{0x4026, 	"PMERR_SPL_NO_QUEUES_ASSOCIATED"},
	{0x4027, 	"PMERR_SPL_INI_FILE_ERROR"},
	{0x4028, 	"PMERR_SPL_NO_DEFAULT_QUEUE"},
	{0x4029, 	"PMERR_SPL_NO_CURRENT_FORMS_CODE"},
	{0x402A, 	"PMERR_SPL_NOT_AUTHORISED"},
	{0x402B, 	"PMERR_SPL_TEMP_NETWORK_ERROR"},
	{0x402C, 	"PMERR_SPL_HARD_NETWORK_ERROR"},
	{0x402D, 	"PMERR_DEL_NOT_ALLOWED"},
	{0x402E, 	"PMERR_CANNOT_DEL_QP_REF"},
	{0x402F, 	"PMERR_CANNOT_DEL_QNAME_REF"},
	{0x4030, 	"PMERR_CANNOT_DEL_PRINTER_DD_REF"},
	{0x4031, 	"PMERR_CANNOT_DEL_PRN_NAME_REF"},
	{0x4032, 	"PMERR_CANNOT_DEL_PRN_ADDR_REF"},
	{0x4033, 	"PMERR_SPOOLER_QP_NOT_DEFINED"},
	{0x4034, 	"PMERR_PRN_NAME_NOT_DEFINED"},
	{0x4035, 	"PMERR_PRN_ADDR_NOT_DEFINED"},
	{0x4036, 	"PMERR_PRINTER_DD_NOT_DEFINED"},
	{0x4037, 	"PMERR_PRINTER_QUEUE_NOT_DEFINED"},
	{0x4038, 	"PMERR_PRN_ADDR_IN_USE"},
	{0x4039, 	"PMERR_SPL_TOO_MANY_OPEN_FILES"},
	{0x403A, 	"PMERR_SPL_CP_NOT_REQD"},
	{0x4040, 	"PMERR_UNABLE_TO_CLOSE_DEVICE"},
	{0x5001, 	"PMERR_INV_TYPE"},
	{0x5002, 	"PMERR_INV_CONV"},
	{0x5003, 	"PMERR_INV_SEGLEN"},
	{0x5004, 	"PMERR_DUP_SEGNAME"},
	{0x5005, 	"PMERR_INV_XFORM"},
	{0x5006, 	"PMERR_INV_VIEWLIM"},
	{0x5007, 	"PMERR_INV_3DCOORD"},
	{0x5008, 	"PMERR_SMB_OVFLOW"},
	{0x5009, 	"PMERR_SEG_OVFLOW"},
	{0x5010, 	"PMERR_PIC_DUP_FILENAME"},
	{0x8001, 	"PMERR_SPL_ERROR_1"},
	{0x8002, 	"PMERR_SPL_ERROR_2"},
	{0x8003, 	"PMERR_SPL_ERROR_3"},
	{0x8004, 	"PMERR_SPL_ERROR_4"},
	{0x8005, 	"PMERR_SPL_ERROR_5"},
	{0x8006, 	"PMERR_SPL_ERROR_6"},
	{0x8007, 	"PMERR_SPL_ERROR_7"},
	{0x8008, 	"PMERR_SPL_ERROR_8"},
	{0x8009, 	"PMERR_SPL_ERROR_9"},
	{0x8010, 	"PMERR_SPL_ERROR_10"},
	{0x8011, 	"PMERR_SPL_ERROR_11"},
	{0x8012, 	"PMERR_SPL_ERROR_12"},
	{0x8013, 	"PMERR_SPL_ERROR_13"},
	{0x8014, 	"PMERR_SPL_ERROR_14"},
	{0x8015, 	"PMERR_SPL_ERROR_15"},
	{0x8016, 	"PMERR_SPL_ERROR_16"},
	{0x8017, 	"PMERR_SPL_ERROR_17"},
	{0x8018, 	"PMERR_SPL_ERROR_18"},
	{0x8019, 	"PMERR_SPL_ERROR_19"},
	{0x8020, 	"PMERR_SPL_ERROR_20"},
	{0x8021, 	"PMERR_SPL_ERROR_21"},
	{0x8022, 	"PMERR_SPL_ERROR_22"},
	{0x8023, 	"PMERR_SPL_ERROR_23"},
	{0x8024, 	"PMERR_SPL_ERROR_24"},
	{0x8025, 	"PMERR_SPL_ERROR_25"},
	{0x8026, 	"PMERR_SPL_ERROR_26"},
	{0x8027, 	"PMERR_SPL_ERROR_27"},
	{0x8028, 	"PMERR_SPL_ERROR_28"},
	{0x8029, 	"PMERR_SPL_ERROR_29"},
	{0x8030, 	"PMERR_SPL_ERROR_30"},
	{0x8031, 	"PMERR_SPL_ERROR_31"},
	{0x8032, 	"PMERR_SPL_ERROR_32"},
	{0x8033, 	"PMERR_SPL_ERROR_33"},
	{0x8034, 	"PMERR_SPL_ERROR_34"},
	{0x8035, 	"PMERR_SPL_ERROR_35"},
	{0x8036, 	"PMERR_SPL_ERROR_36"},
	{0x8037, 	"PMERR_SPL_ERROR_37"},
	{0x8038, 	"PMERR_SPL_ERROR_38"},
	{0x8039, 	"PMERR_SPL_ERROR_39"},
	{0x8040, 	"PMERR_SPL_ERROR_40"},
	{0x8041, 	"PMERR_SPLMSGBOX_INFO_CAPTION"},
	{0x8042, 	"PMERR_SPLMSGBOX_WARNING_CAPTION"},
	{0x8043, 	"PMERR_SPLMSGBOX_ERROR_CAPTION"},
	{0x8044, 	"PMERR_SPLMSGBOX_SEVERE_CAPTION"},
	{0x8045, 	"PMERR_SPLMSGBOX_JOB_DETAILS"},
	{0x8046, 	"PMERR_SPLMSGBOX_ERROR_ACTION"},
	{0x8047, 	"PMERR_SPLMSGBOX_SEVERE_ACTION"},
	{0x8048, 	"PMERR_SPLMSGBOX_BIT_0_TEXT"},
	{0x8049, 	"PMERR_SPLMSGBOX_BIT_1_TEXT"},
	{0x8050, 	"PMERR_SPLMSGBOX_BIT_2_TEXT"},
	{0x8051, 	"PMERR_SPLMSGBOX_BIT_3_TEXT"},
	{0x8052, 	"PMERR_SPLMSGBOX_BIT_4_TEXT"},
	{0x8053, 	"PMERR_SPLMSGBOX_BIT_5_TEXT"},
	{0x8054, 	"PMERR_SPLMSGBOX_BIT_15_TEXT"},
	{0x8055, 	"PMERR_SPL_NOPATHBUFFER"},
	{0x8093, 	"PMERR_SPL_ALREADY_INITIALISED"},
	{0x8095, 	"PMERR_SPL_ERROR"},
	{0x0000,	NULL}};

char *
getpmerrmsg(void)
{
	PERRINFO  pErrInfoBlk;
	PSZ       pszOffSet;
	static char scratch[256];
	PSZ  	pszErrMsg;
	int	errcode, errcode2;
	int 	i;

		
	errcode = WinGetLastError(wxhAb);
	if ((pErrInfoBlk = WinGetErrorInfo(wxhAb)) != (PERRINFO)NULL) {
		pszOffSet = ((PSZ)pErrInfoBlk) + pErrInfoBlk->offaoffszMsg;
		pszErrMsg = ((PSZ)pErrInfoBlk) + *((PSHORT)pszOffSet);
		errcode2 = (pErrInfoBlk->idError & 0xffff);
		for (i = 0; pmerrtab[i].errmsg != NULL
		     && pmerrtab[i].errcode != errcode2; i++)
			;
		sprintf(scratch, "%s: %s",
			pmerrtab[i].errmsg ? pmerrtab[i].errmsg : "",
			pszErrMsg);
		WinFreeErrorInfo(pErrInfoBlk);
	} else {
		sprintf(scratch, "%x", errcode);
	}
	return scratch;
}


char *
wxMessageStr(int msg)
{
	static char scratch[40];
	switch (msg) {
	case WM_ACTIVATE:	return "WM_ACTIVATE";
	case WM_ADJUSTFRAMEPOS:	return "WM_ADJUSTFRAMEPOS";
	case WM_ADJUSTWINDOWPOS:	return "WM_ADJUSTWINDOWPOS";
	case WM_APPTERMINATENOTIFY:	return "WM_APPTERMINATENOTIFY";
	case WM_BEGINDRAG:	return "WM_BEGINDRAG";
	case WM_BEGINSELECT:	return "WM_BEGINSELECT";
	case WM_BUTTON1CLICK:	return "WM_BUTTON1CLICK";
	case WM_BUTTON1DBLCLK:	return "WM_BUTTON1DBLCLK";
	case WM_BUTTON1DOWN:	return "WM_BUTTON1DOWN";
	case WM_BUTTON1MOTIONEND:	return "WM_BUTTON1MOTIONEND";
	case WM_BUTTON1MOTIONSTART:	return "WM_BUTTON1MOTIONSTART";
	case WM_BUTTON1UP:	return "WM_BUTTON1UP";
	case WM_BUTTON2CLICK:	return "WM_BUTTON2CLICK";
	case WM_BUTTON2DBLCLK:	return "WM_BUTTON2DBLCLK";
	case WM_BUTTON2DOWN:	return "WM_BUTTON2DOWN";
	case WM_BUTTON2MOTIONEND:	return "WM_BUTTON2MOTIONEND";
	case WM_BUTTON2MOTIONSTART:	return "WM_BUTTON2MOTIONSTART";
	case WM_BUTTON2UP:	return "WM_BUTTON2UP";
	case WM_BUTTON3CLICK:	return "WM_BUTTON3CLICK";
	case WM_BUTTON3DBLCLK:	return "WM_BUTTON3DBLCLK";
	case WM_BUTTON3DOWN:	return "WM_BUTTON3DOWN";
	case WM_BUTTON3MOTIONEND:	return "WM_BUTTON3MOTIONEND";
	case WM_BUTTON3MOTIONSTART:	return "WM_BUTTON3MOTIONSTART";
	case WM_BUTTON3UP:	return "WM_BUTTON3UP";
/*	case WM_BUTTONCLICKFIRST:	return "WM_BUTTONCLICKFIRST"; */
/*	case WM_BUTTONCLICKLAST:	return "WM_BUTTONCLICKLAST"; */
	case WM_CALCFRAMERECT:	return "WM_CALCFRAMERECT";
	case WM_CALCVALIDRECTS:	return "WM_CALCVALIDRECTS";
	case WM_CHAR:	return "WM_CHAR";
	case WM_CHORD:	return "WM_CHORD";
	case WM_CLOSE:	return "WM_CLOSE";
	case WM_COMMAND:	return "WM_COMMAND";
	case WM_CONTEXTHELP:	return "WM_CONTEXTHELP";
	case WM_CONTEXTMENU:	return "WM_CONTEXTMENU";
	case WM_CONTROL:	return "WM_CONTROL";
	case WM_CONTROLPOINTER:	return "WM_CONTROLPOINTER";
	case WM_CREATE:	return "WM_CREATE";
/*	case WM_DBCSFIRST:	return "WM_DBCSFIRST"; */
/*	case WM_DBCSLAST:	return "WM_DBCSLAST"; */
/*	case WM_DDE:	return "WM_DDE"; */
	case WM_DESTROY:	return "WM_DESTROY";
	case WM_DESTROYCLIPBOARD:	return "WM_DESTROYCLIPBOARD";
	case WM_DRAGFIRST:	return "WM_DRAGFIRST";
	case WM_DRAGLAST:	return "WM_DRAGLAST";
	case WM_DRAWCLIPBOARD:	return "WM_DRAWCLIPBOARD";
	case WM_DRAWITEM:	return "WM_DRAWITEM";
	case WM_ENABLE:	return "WM_ENABLE";
	case WM_ENDDRAG:	return "WM_ENDDRAG";
	case WM_ENDSELECT:	return "WM_ENDSELECT";
	case WM_ERASEBACKGROUND:	return "WM_ERASEBACKGROUND";
	case WM_ERROR:	return "WM_ERROR";
/*	case WM_EXTMOUSEFIRST:	return "WM_EXTMOUSEFIRST"; */
/*	case WM_EXTMOUSELAST:	return "WM_EXTMOUSELAST";*/
	case WM_FLASHWINDOW:	return "WM_FLASHWINDOW";
	case WM_FOCUSCHANGE:	return "WM_FOCUSCHANGE";
	case WM_FORMATFRAME:	return "WM_FORMATFRAME";
	case WM_HELP:	return "WM_HELP";
	case WM_HELPBASE:	return "WM_HELPBASE";
	case WM_HELPTOP:	return "WM_HELPTOP";
	case WM_HITTEST:	return "WM_HITTEST";
	case WM_HSCROLL:	return "WM_HSCROLL";
	case WM_HSCROLLCLIPBOARD:	return "WM_HSCROLLCLIPBOARD";
	case WM_INITDLG:	return "WM_INITDLG";
	case WM_INITMENU:	return "WM_INITMENU";
	case WM_JOURNALNOTIFY:	return "WM_JOURNALNOTIFY";
	case WM_MATCHMNEMONIC:	return "WM_MATCHMNEMONIC";
	case WM_MEASUREITEM:	return "WM_MEASUREITEM";
	case WM_MENUEND:	return "WM_MENUEND";
	case WM_MENUSELECT:	return "WM_MENUSELECT";
	case WM_MINMAXFRAME:	return "WM_MINMAXFRAME";
	case WM_MMPMFIRST:	return "WM_MMPMFIRST";
	case WM_MMPMLAST:	return "WM_MMPMLAST";
/*	case WM_MOUSEFIRST:	return "WM_MOUSEFIRST"; */
/*	case WM_MOUSELAST:	return "WM_MOUSELAST"; */
	case WM_MOUSEMAP:	return "WM_MOUSEMAP";
	case WM_MOUSEMOVE:	return "WM_MOUSEMOVE";
/*	case WM_MOUSETRANSLATEFIRST:	return "WM_MOUSETRANSLATEFIRST"; */
/*	case WM_MOUSETRANSLATELAST:	return "WM_MOUSETRANSLATELAST"; */
	case WM_MOVE:	return "WM_MOVE";
	case WM_NEXTMENU:	return "WM_NEXTMENU";
	case WM_NULL:	return "WM_NULL";
	case WM_OPEN:	return "WM_OPEN";
	case WM_OWNERPOSCHANGE:	return "WM_OWNERPOSCHANGE";
	case WM_PACTIVATE:	return "WM_PACTIVATE";
	case WM_PAINT:	return "WM_PAINT";
	case WM_PAINTCLIPBOARD:	return "WM_PAINTCLIPBOARD";
	case WM_PCONTROL:	return "WM_PCONTROL";
	case WM_PENFIRST:	return "WM_PENFIRST";
	case WM_PENLAST:	return "WM_PENLAST";
	case WM_PPAINT:	return "WM_PPAINT";
	case WM_PRESPARAMCHANGED:	return "WM_PRESPARAMCHANGED";
	case WM_PSETFOCUS:	return "WM_PSETFOCUS";
	case WM_PSIZE:	return "WM_PSIZE";
	case WM_PSYSCOLORCHANGE:	return "WM_PSYSCOLORCHANGE";
	case WM_QUERYACCELTABLE:	return "WM_QUERYACCELTABLE";
	case WM_QUERYBORDERSIZE:	return "WM_QUERYBORDERSIZE";
	case WM_QUERYCONVERTPOS:	return "WM_QUERYCONVERTPOS";
	case WM_QUERYDLGCODE:	return "WM_QUERYDLGCODE";
	case WM_QUERYFOCUSCHAIN:	return "WM_QUERYFOCUSCHAIN";
	case WM_QUERYFRAMECTLCOUNT:	return "WM_QUERYFRAMECTLCOUNT";
	case WM_QUERYFRAMEINFO:	return "WM_QUERYFRAMEINFO";
	case WM_QUERYHELPINFO:	return "WM_QUERYHELPINFO";
	case WM_QUERYICON:	return "WM_QUERYICON";
	case WM_QUERYTRACKINFO:	return "WM_QUERYTRACKINFO";
	case WM_QUERYWINDOWPARAMS:	return "WM_QUERYWINDOWPARAMS";
	case WM_QUIT:	return "WM_QUIT";
	case WM_REALIZEPALETTE:	return "WM_REALIZEPALETTE";
	case WM_RENDERALLFMTS:	return "WM_RENDERALLFMTS";
	case WM_RENDERFMT:	return "WM_RENDERFMT";
	case WM_SAVEAPPLICATION:	return "WM_SAVEAPPLICATION";
	case WM_SEM1:	return "WM_SEM1";
	case WM_SEM2:	return "WM_SEM2";
	case WM_SEM3:	return "WM_SEM3";
	case WM_SEM4:	return "WM_SEM4";
	case WM_SETACCELTABLE:	return "WM_SETACCELTABLE";
	case WM_SETBORDERSIZE:	return "WM_SETBORDERSIZE";
	case WM_SETFOCUS:	return "WM_SETFOCUS";
	case WM_SETHELPINFO:	return "WM_SETHELPINFO";
	case WM_SETICON:	return "WM_SETICON";
	case WM_SETSELECTION:	return "WM_SETSELECTION";
	case WM_SETWINDOWPARAMS:	return "WM_SETWINDOWPARAMS";
	case WM_SHOW:	return "WM_SHOW";
	case WM_SINGLESELECT:	return "WM_SINGLESELECT";
	case WM_SIZE:	return "WM_SIZE";
	case WM_SIZECLIPBOARD:	return "WM_SIZECLIPBOARD";
	case WM_SUBSTITUTESTRING:	return "WM_SUBSTITUTESTRING";
	case WM_SYSCOLORCHANGE:	return "WM_SYSCOLORCHANGE";
	case WM_SYSCOMMAND:	return "WM_SYSCOMMAND";
	case WM_SYSVALUECHANGED:	return "WM_SYSVALUECHANGED";
	case WM_TEXTEDIT:	return "WM_TEXTEDIT";
	case WM_TIMER:	return "WM_TIMER";
	case WM_TRACKFRAME:	return "WM_TRACKFRAME";
	case WM_TRANSLATEACCEL:	return "WM_TRANSLATEACCEL";
	case WM_UPDATEFRAME:	return "WM_UPDATEFRAME";
	case WM_USER:		return "WM_USER";
	case WM_VIOCHAR:	return "WM_VIOCHAR";
	case WM_VRNDISABLED:	return "WM_VRNDISABLED";
	case WM_VRNENABLED:	return "WM_VRNENABLED";
	case WM_VSCROLL:	return "WM_VSCROLL";
	case WM_VSCROLLCLIPBOARD:	return "WM_VSCROLLCLIPBOARD";
	case WM_WINDOWPOSCHANGED:	return "WM_WINDOWPOSCHANGED";

	case BKM_CALCPAGERECT:		return "BKM_CALCPAGERECT";
	case BKM_DELETEPAGE:		return "BKM_DELETEPAGE";
	case BKM_INSERTPAGE:		return "BKM_INSERTPAGE";
	case BKM_INVALIDATETABS:		return "BKM_INVALIDATETABS";
	case BKM_QUERYPAGECOUNT:		return "BKM_QUERYPAGECOUNT";
	case BKM_QUERYPAGEDATA:		return "BKM_QUERYPAGEDATA";
	case BKM_QUERYPAGEID:		return "BKM_QUERYPAGEID";
	case BKM_QUERYPAGEINFO:		return "BKM_QUERYPAGEINFO";
	case BKM_QUERYPAGESTYLE:		return "BKM_QUERYPAGESTYLE";
	case BKM_QUERYPAGEWINDOWHWND:		return "BKM_QUERYPAGEWINDOWHWND";
	case BKM_QUERYSTATUSLINETEXT:		return "BKM_QUERYSTATUSLINETEXT";
	case BKM_QUERYTABBITMAP:		return "BKM_QUERYTABBITMAP";
	case BKM_QUERYTABTEXT:		return "BKM_QUERYTABTEXT";
	case BKM_SETDIMENSIONS:		return "BKM_SETDIMENSIONS";
	case BKM_SETNOTEBOOKCOLORS:		return "BKM_SETNOTEBOOKCOLORS";
	case BKM_SETPAGEDATA:		return "BKM_SETPAGEDATA";
	case BKM_SETPAGEINFO:		return "BKM_SETPAGEINFO";
	case BKM_SETPAGEWINDOWHWND:		return "BKM_SETPAGEWINDOWHWND";
	case BKM_SETSTATUSLINETEXT:		return "BKM_SETSTATUSLINETEXT";
	case BKM_SETTABBITMAP:		return "BKM_SETTABBITMAP";
	case BKM_SETTABTEXT:		return "BKM_SETTABTEXT";
	case BKM_TURNTOPAGE:		return "BKM_TURNTOPAGE";
	case BM_CLICK:		return "BM_CLICK";
	case BM_QUERYCHECK:		return "BM_QUERYCHECK";
	case BM_QUERYCHECKINDEX:		return "BM_QUERYCHECKINDEX";
	case BM_QUERYHILITE:		return "BM_QUERYHILITE";
	case BM_SETCHECK:		return "BM_SETCHECK";
	case BM_SETDEFAULT:		return "BM_SETDEFAULT";
	case BM_SETHILITE:		return "BM_SETHILITE";
	case CBM_HILITE:		return "CBM_HILITE";
	case CBM_ISLISTSHOWING:		return "CBM_ISLISTSHOWING";
	case CBM_SHOWLIST:		return "CBM_SHOWLIST";
	case CM_ALLOCDETAILFIELDINFO:		return "CM_ALLOCDETAILFIELDINFO";
	case CM_ALLOCRECORD:		return "CM_ALLOCRECORD";
	case CM_ARRANGE:		return "CM_ARRANGE";
	case CM_CLOSEEDIT:		return "CM_CLOSEEDIT";
	case CM_COLLAPSETREE:		return "CM_COLLAPSETREE";
	case CM_ERASERECORD:		return "CM_ERASERECORD";
	case CM_EXPANDTREE:		return "CM_EXPANDTREE";
	case CM_FILTER:		return "CM_FILTER";
	case CM_FREEDETAILFIELDINFO:		return "CM_FREEDETAILFIELDINFO";
	case CM_FREERECORD:		return "CM_FREERECORD";
	case CM_HORZSCROLLSPLITWINDOW:		return "CM_HORZSCROLLSPLITWINDOW";
	case CM_INSERTDETAILFIELDINFO:		return "CM_INSERTDETAILFIELDINFO";
	case CM_INSERTRECORD:		return "CM_INSERTRECORD";
	case CM_INSERTRECORDARRAY:		return "CM_INSERTRECORDARRAY";
	case CM_INVALIDATEDETAILFIELDINFO:		return "CM_INVALIDATEDETAILFIELDINFO";
	case CM_INVALIDATERECORD:		return "CM_INVALIDATERECORD";
	case CM_MOVETREE:		return "CM_MOVETREE";
	case CM_OPENEDIT:		return "CM_OPENEDIT";
	case CM_PAINTBACKGROUND:		return "CM_PAINTBACKGROUND";
	case CM_QUERYCNRINFO:		return "CM_QUERYCNRINFO";
	case CM_QUERYDETAILFIELDINFO:		return "CM_QUERYDETAILFIELDINFO";
	case CM_QUERYDRAGIMAGE:		return "CM_QUERYDRAGIMAGE";
	case CM_QUERYRECORD:		return "CM_QUERYRECORD";
	case CM_QUERYRECORDEMPHASIS:		return "CM_QUERYRECORDEMPHASIS";
	case CM_QUERYRECORDFROMRECT:		return "CM_QUERYRECORDFROMRECT";
	case CM_QUERYRECORDINFO:		return "CM_QUERYRECORDINFO";
	case CM_QUERYRECORDRECT:		return "CM_QUERYRECORDRECT";
	case CM_QUERYVIEWPORTRECT:		return "CM_QUERYVIEWPORTRECT";
	case CM_REMOVEDETAILFIELDINFO:		return "CM_REMOVEDETAILFIELDINFO";
	case CM_REMOVERECORD:		return "CM_REMOVERECORD";
	case CM_SCROLLWINDOW:		return "CM_SCROLLWINDOW";
	case CM_SEARCHSTRING:		return "CM_SEARCHSTRING";
	case CM_SETCNRINFO:		return "CM_SETCNRINFO";
	case CM_SETRECORDEMPHASIS:		return "CM_SETRECORDEMPHASIS";
/*	case CM_SETTEXTVISIBILITY:		return "CM_SETTEXTVISIBILITY"; */
	case CM_SORTRECORD:		return "CM_SORTRECORD";
	case CSM_QUERYINCREMENT:		return "CSM_QUERYINCREMENT";
	case CSM_QUERYRADIUS:		return "CSM_QUERYRADIUS";
	case CSM_QUERYRANGE:		return "CSM_QUERYRANGE";
	case CSM_QUERYVALUE:		return "CSM_QUERYVALUE";
	case CSM_SETBITMAPDATA:		return "CSM_SETBITMAPDATA";
	case CSM_SETINCREMENT:		return "CSM_SETINCREMENT";
	case CSM_SETRANGE:		return "CSM_SETRANGE";
	case CSM_SETVALUE:		return "CSM_SETVALUE";
	case CSN_CHANGED:		return "CSN_CHANGED";
	case CSN_QUERYBACKGROUNDCOLOR:		return "CSN_QUERYBACKGROUNDCOLOR";
	case CSN_SETFOCUS:		return "CSN_SETFOCUS";
	case CSN_TRACKING:		return "CSN_TRACKING";
	case DM_DISCARDOBJECT:		return "DM_DISCARDOBJECT";
	case DM_DRAGERROR:		return "DM_DRAGERROR";
	case DM_DRAGFILECOMPLETE:		return "DM_DRAGFILECOMPLETE";
	case DM_DRAGLEAVE:		return "DM_DRAGLEAVE";
	case DM_DRAGOVER:		return "DM_DRAGOVER";
	case DM_DRAGOVERNOTIFY:		return "DM_DRAGOVERNOTIFY";
/*	case DM_DROP:		return "DM_DROP"; */
	case DM_DROPHELP:		return "DM_DROPHELP";
	case DM_DROPNOTIFY:		return "DM_DROPNOTIFY";
	case DM_EMPHASIZETARGET:		return "DM_EMPHASIZETARGET";
	case DM_ENDCONVERSATION:		return "DM_ENDCONVERSATION";
	case DM_FILERENDERED:		return "DM_FILERENDERED";
	case DM_PRINT:		return "DM_PRINT";
	case DM_PRINTOBJECT:		return "DM_PRINTOBJECT";
	case DM_RENDER:		return "DM_RENDER";
	case DM_RENDERCOMPLETE:		return "DM_RENDERCOMPLETE";
	case DM_RENDERFILE:		return "DM_RENDERFILE";
	case DM_RENDERPREPARE:		return "DM_RENDERPREPARE";
	case EM_CLEAR:		return "EM_CLEAR";
	case EM_COPY:		return "EM_COPY";
	case EM_CUT:		return "EM_CUT";
	case EM_PASTE:		return "EM_PASTE";
	case EM_QUERYCHANGED:		return "EM_QUERYCHANGED";
	case EM_QUERYFIRSTCHAR:		return "EM_QUERYFIRSTCHAR";
	case EM_QUERYREADONLY:		return "EM_QUERYREADONLY";
	case EM_QUERYSEL:		return "EM_QUERYSEL";
	case EM_SETFIRSTCHAR:		return "EM_SETFIRSTCHAR";
	case EM_SETINSERTMODE:		return "EM_SETINSERTMODE";
	case EM_SETREADONLY:		return "EM_SETREADONLY";
	case EM_SETSEL:		return "EM_SETSEL";
	case EM_SETTEXTLIMIT:		return "EM_SETTEXTLIMIT";
	case HM_MSG_BASE:		return "HM_MSG_BASE";
	case HM_DISMISS_WINDOW:		return "HM_DISMISS_WINDOW";
	case HM_DISPLAY_HELP:		return "HM_DISPLAY_HELP";
/*	case HM_EXT_HELP:		return "HM_EXT_HELP"; */
	case HM_GENERAL_HELP:		return "HM_GENERAL_HELP";
	case HM_SET_ACTIVE_WINDOW:		return "HM_SET_ACTIVE_WINDOW";
	case HM_LOAD_HELP_TABLE:		return "HM_LOAD_HELP_TABLE";
	case HM_CREATE_HELP_TABLE:		return "HM_CREATE_HELP_TABLE";
	case HM_SET_HELP_WINDOW_TITLE:		return "HM_SET_HELP_WINDOW_TITLE";
	case HM_SET_SHOW_PANEL_ID:		return "HM_SET_SHOW_PANEL_ID";
/*	case HM_REPLACE_HELP_FOR_HELP:		return "HM_REPLACE_HELP_FOR_HELP"; */
	case HM_REPLACE_USING_HELP:		return "HM_REPLACE_USING_HELP";
	case HM_HELP_INDEX:		return "HM_HELP_INDEX";
	case HM_HELP_CONTENTS:		return "HM_HELP_CONTENTS";
	case HM_KEYS_HELP:		return "HM_KEYS_HELP";
	case HM_SET_HELP_LIBRARY_NAME:		return "HM_SET_HELP_LIBRARY_NAME";
	case HM_ERROR:		return "HM_ERROR";
	case HM_HELPSUBITEM_NOT_FOUND:		return "HM_HELPSUBITEM_NOT_FOUND";
	case HM_QUERY_KEYS_HELP:		return "HM_QUERY_KEYS_HELP";
	case HM_TUTORIAL:		return "HM_TUTORIAL";
/*	case HM_EXT_HELP_UNDEFINED:		return "HM_EXT_HELP_UNDEFINED"; */
	case HM_GENERAL_HELP_UNDEFINED:		return "HM_GENERAL_HELP_UNDEFINED";
	case HM_ACTIONBAR_COMMAND:		return "HM_ACTIONBAR_COMMAND";
	case HM_INFORM:		return "HM_INFORM";
	case HM_SET_OBJCOM_WINDOW:		return "HM_SET_OBJCOM_WINDOW";
	case HM_UPDATE_OBJCOM_WINDOW_CHAIN:		return "HM_UPDATE_OBJCOM_WINDOW_CHAIN";
	case HM_QUERY_DDF_DATA:		return "HM_QUERY_DDF_DATA";
	case HM_INVALIDATE_DDF_DATA:		return "HM_INVALIDATE_DDF_DATA";
	case HM_QUERY:		return "HM_QUERY";
	case HM_SET_COVERPAGE_SIZE:		return "HM_SET_COVERPAGE_SIZE";
	case HM_NOTIFY:		return "HM_NOTIFY";
	case HM_SET_USERDATA:		return "HM_SET_USERDATA";
	case HM_CONTROL:		return "HM_CONTROL";
	case LM_DELETEALL:		return "LM_DELETEALL";
	case LM_DELETEITEM:		return "LM_DELETEITEM";
	case LM_INSERTITEM:		return "LM_INSERTITEM";
	case LM_INSERTMULTITEMS:		return "LM_INSERTMULTITEMS";
	case LM_QUERYITEMCOUNT:		return "LM_QUERYITEMCOUNT";
	case LM_QUERYITEMHANDLE:		return "LM_QUERYITEMHANDLE";
	case LM_QUERYITEMTEXT:		return "LM_QUERYITEMTEXT";
	case LM_QUERYITEMTEXTLENGTH:		return "LM_QUERYITEMTEXTLENGTH";
	case LM_QUERYSELECTION:		return "LM_QUERYSELECTION";
	case LM_QUERYTOPINDEX:		return "LM_QUERYTOPINDEX";
	case LM_SEARCHSTRING:		return "LM_SEARCHSTRING";
	case LM_SELECTITEM:		return "LM_SELECTITEM";
	case LM_SETITEMHANDLE:		return "LM_SETITEMHANDLE";
	case LM_SETITEMHEIGHT:		return "LM_SETITEMHEIGHT";
	case LM_SETITEMTEXT:		return "LM_SETITEMTEXT";
	case LM_SETITEMWIDTH:		return "LM_SETITEMWIDTH";
	case LM_SETTOPINDEX:		return "LM_SETTOPINDEX";
	case MLM_CHARFROMLINE:		return "MLM_CHARFROMLINE";
	case MLM_CLEAR:		return "MLM_CLEAR";
	case MLM_COPY:		return "MLM_COPY";
	case MLM_CUT:		return "MLM_CUT";
	case MLM_DELETE:		return "MLM_DELETE";
	case MLM_DISABLEREFRESH:		return "MLM_DISABLEREFRESH";
	case MLM_ENABLEREFRESH:		return "MLM_ENABLEREFRESH";
	case MLM_EXPORT:		return "MLM_EXPORT";
	case MLM_FORMAT:		return "MLM_FORMAT";
	case MLM_IMPORT:		return "MLM_IMPORT";
	case MLM_INSERT:		return "MLM_INSERT";
	case MLM_LINEFROMCHAR:		return "MLM_LINEFROMCHAR";
	case MLM_PASTE:		return "MLM_PASTE";
	case MLM_QUERYBACKCOLOR:		return "MLM_QUERYBACKCOLOR";
	case MLM_QUERYCHANGED:		return "MLM_QUERYCHANGED";
	case MLM_QUERYFIRSTCHAR:		return "MLM_QUERYFIRSTCHAR";
	case MLM_QUERYFONT:		return "MLM_QUERYFONT";
	case MLM_QUERYFORMATLINELENGTH:		return "MLM_QUERYFORMATLINELENGTH";
	case MLM_QUERYFORMATRECT:		return "MLM_QUERYFORMATRECT";
	case MLM_QUERYFORMATTEXTLENGTH:		return "MLM_QUERYFORMATTEXTLENGTH";
	case MLM_QUERYIMPORTEXPORT:		return "MLM_QUERYIMPORTEXPORT";
	case MLM_QUERYLINECOUNT:		return "MLM_QUERYLINECOUNT";
	case MLM_QUERYLINELENGTH:		return "MLM_QUERYLINELENGTH";
	case MLM_QUERYREADONLY:		return "MLM_QUERYREADONLY";
	case MLM_QUERYSEL:		return "MLM_QUERYSEL";
	case MLM_QUERYSELTEXT:		return "MLM_QUERYSELTEXT";
	case MLM_QUERYTABSTOP:		return "MLM_QUERYTABSTOP";
	case MLM_QUERYTEXTCOLOR:		return "MLM_QUERYTEXTCOLOR";
	case MLM_QUERYTEXTLENGTH:		return "MLM_QUERYTEXTLENGTH";
	case MLM_QUERYTEXTLIMIT:		return "MLM_QUERYTEXTLIMIT";
	case MLM_QUERYUNDO:		return "MLM_QUERYUNDO";
	case MLM_QUERYWRAP:		return "MLM_QUERYWRAP";
	case MLM_RESETUNDO:		return "MLM_RESETUNDO";
	case MLM_SEARCH:		return "MLM_SEARCH";
	case MLM_SETBACKCOLOR:		return "MLM_SETBACKCOLOR";
	case MLM_SETCHANGED:		return "MLM_SETCHANGED";
	case MLM_SETFIRSTCHAR:		return "MLM_SETFIRSTCHAR";
	case MLM_SETFONT:		return "MLM_SETFONT";
	case MLM_SETFORMATRECT:		return "MLM_SETFORMATRECT";
	case MLM_SETIMPORTEXPORT:		return "MLM_SETIMPORTEXPORT";
	case MLM_SETREADONLY:		return "MLM_SETREADONLY";
	case MLM_SETSEL:		return "MLM_SETSEL";
	case MLM_SETTABSTOP:		return "MLM_SETTABSTOP";
	case MLM_SETTEXTCOLOR:		return "MLM_SETTEXTCOLOR";
	case MLM_SETTEXTLIMIT:		return "MLM_SETTEXTLIMIT";
	case MLM_SETWRAP:		return "MLM_SETWRAP";
	case MLM_UNDO:		return "MLM_UNDO";
	case MM_DELETEITEM:		return "MM_DELETEITEM";
	case MM_ENDMENUMODE:		return "MM_ENDMENUMODE";
	case MM_INSERTITEM:		return "MM_INSERTITEM";
	case MM_ISITEMVALID:		return "MM_ISITEMVALID";
	case MM_ITEMIDFROMPOSITION:		return "MM_ITEMIDFROMPOSITION";
	case MM_ITEMPOSITIONFROMID:		return "MM_ITEMPOSITIONFROMID";
	case MM_QUERYDEFAULTITEMID:		return "MM_QUERYDEFAULTITEMID";
	case MM_QUERYITEM:		return "MM_QUERYITEM";
	case MM_QUERYITEMATTR:		return "MM_QUERYITEMATTR";
	case MM_QUERYITEMCOUNT:		return "MM_QUERYITEMCOUNT";
	case MM_QUERYITEMRECT:		return "MM_QUERYITEMRECT";
	case MM_QUERYITEMTEXT:		return "MM_QUERYITEMTEXT";
	case MM_QUERYITEMTEXTLENGTH:		return "MM_QUERYITEMTEXTLENGTH";
	case MM_QUERYSELITEMID:		return "MM_QUERYSELITEMID";
	case MM_REMOVEITEM:		return "MM_REMOVEITEM";
	case MM_SELECTITEM:		return "MM_SELECTITEM";
	case MM_SETDEFAULTITEMID:		return "MM_SETDEFAULTITEMID";
	case MM_SETITEM:		return "MM_SETITEM";
	case MM_SETITEMATTR:		return "MM_SETITEMATTR";
	case MM_SETITEMHANDLE:		return "MM_SETITEMHANDLE";
	case MM_SETITEMTEXT:		return "MM_SETITEMTEXT";
	case MM_STARTMENUMODE:		return "MM_STARTMENUMODE";
	case SLM_ADDDETENT:		return "SLM_ADDDETENT";
	case SLM_QUERYDETENTPOS:		return "SLM_QUERYDETENTPOS";
	case SLM_QUERYSCALETEXT:		return "SLM_QUERYSCALETEXT";
	case SLM_QUERYSLIDERINFO:		return "SLM_QUERYSLIDERINFO";
	case SLM_QUERYTICKPOS:		return "SLM_QUERYTICKPOS";
	case SLM_QUERYTICKSIZE:		return "SLM_QUERYTICKSIZE";
	case SLM_REMOVEDETENT:		return "SLM_REMOVEDETENT";
	case SLM_SETSCALETEXT:		return "SLM_SETSCALETEXT";
	case SLM_SETSLIDERINFO:		return "SLM_SETSLIDERINFO";
	case SLM_SETTICKSIZE:		return "SLM_SETTICKSIZE";
	case SM_QUERYHANDLE:		return "SM_QUERYHANDLE";
	case SM_SETHANDLE:		return "SM_SETHANDLE";
	case SPBM_OVERRIDESETLIMITS:		return "SPBM_OVERRIDESETLIMITS";
	case SPBM_QUERYLIMITS:		return "SPBM_QUERYLIMITS";
	case SPBM_QUERYVALUE:		return "SPBM_QUERYVALUE";
	case SPBM_SETARRAY:		return "SPBM_SETARRAY";
	case SPBM_SETCURRENTVALUE:		return "SPBM_SETCURRENTVALUE";
	case SPBM_SETLIMITS:		return "SPBM_SETLIMITS";
	case SPBM_SETMASTER:		return "SPBM_SETMASTER";
	case SPBM_SETTEXTLIMIT:		return "SPBM_SETTEXTLIMIT";
	case SPBM_SPINDOWN:		return "SPBM_SPINDOWN";
	case SPBM_SPINUP:		return "SPBM_SPINUP";
	case TBM_QUERYHILITE:		return "TBM_QUERYHILITE";
	case TBM_SETHILITE:		return "TBM_SETHILITE";



// MDI Messages
#define	WM_MDIGETACTIVE		(WM_USER+1)
#define	WM_MDICASCADE		(WM_USER+2)
#define	WM_MDITILE		(WM_USER+3)
#define	WM_MDIICONARRANGE	(WM_USER+4)
#define	WM_MDINEXT		(WM_USER+5)
#define	WM_MDISETMENU		(WM_USER+6)
#define	WM_MDIDESTROY		(WM_USER+7)

// wxWindow User defined messages
	case WM_MDIGETACTIVE:	return "WM_MDIGETACTIVE";
	case WM_MDICASCADE:	return "WM_MDICASCADE";
	case WM_MDITILE:	return "WM_MDITILE";
	case WM_MDIICONARRANGE:	return "WM_MDIICONARRANGE";
	case WM_MDINEXT:	return "WM_MDINEXT";
	case WM_MDISETMENU:	return "WM_MDISETMENU";
	case WM_MDIDESTROY:	return "WM_MDIDESTROY";

	default:
		sprintf(scratch, "WM message %#x", msg);
		return scratch;
	}
	return NULL;
}

void
wxTraceEvent(char *rtr, HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2, PVOID wnd)
{
	if (!(msg==WM_HITTEST
//	       ||(msg==WM_TIMER && SHORT1FROMMP(mp1) == TID_CURSOR)
//	       || msg==WM_QUERYFOCUSCHAIN
//	       || msg==WM_CONTROLPOINTER
//	       || msg==WM_REALIZEPALETTE
//	       || msg==WM_FOCUSCHANGE
//	       || msg==WM_MOUSEMOVE
		))
		printf("* %s(%x, msg=%d (%s), mp1=%#x, mp2=%#x), wnd=%#x\n",
		       rtr, (int)hWnd, (int)msg, wxMessageStr((int)msg),
		       (int)mp1, (int)mp2, (int)wnd);
}
