/*
 * File:	wx_buttn.cc
 * Purpose:	Button implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1995, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_buttn.h"
#include "wx_itemp.h"

#define BUTTON_HEIGHT_FACTOR (EDIT_CONTROL_FACTOR * 1.1)

// Buttons

BOOL wxButton::OS2Control(WORD param, WORD id)
{
    TRACE("wxButton::OS2Control(param=%d, id=%d)", (int)param, (int)id);
    if (param == BN_CLICKED) {
	wxCommandEvent event(wxEVENT_TYPE_BUTTON_COMMAND);
	event.eventObject = this;
	ProcessCommand(event);
	return TRUE;
    }
    return FALSE;
}

IMPLEMENT_DYNAMIC_CLASS(wxButton, wxItem)

wxButton::wxButton(void)
{
    TRACE("wxButton::wxButton()");
    buttonBitmap = NULL;
}

wxButton::wxButton(wxPanel *panel, wxFunction Function, const char *label,
                   int x, int y, int width, int height,
                   long style, const char *name):
  wxbButton(panel, Function, label, x, y, width, height, style, name)
{
TRACE("wxButton::wxButton(panel=%#x, Function=%#x, label='%s', x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)panel, (int)Function, label, x, y, width, height, (int)style, name);
  Create(panel, Function, label, x, y, width, height, style, name);
}

wxButton::wxButton(wxPanel *panel, wxFunction Function, wxBitmap *bitmap,
                   int x, int y, int width, int height,
                   long style, const char *name):
  wxbButton(panel, Function, bitmap, x, y, width, height, style, name)
{
TRACE("wxButton::wxButton(panel=%#x, Function=%#x, bitmap=%#x, x=%d, y=%d, w=%d, h=%d, style=%d, name='%s')", (int)panel, (int)Function, (int)bitmap, x, y, width, height, (int)style, name);
  Create(panel, Function, bitmap, x, y, width, height, style, name);
}

/*
 * Create a text button
 */
Bool wxButton::Create(wxPanel *panel, wxFunction Function, const char *label,
                   int x, int y, int width, int height,
                   long style, const char *name)
{
    TRACE("wxButton::Create(panel=%#x, Function=%#x, label='%s', x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)panel, (int)Function, label, x, y, width, height, (int)style, name);
    buttonBitmap = NULL;
    isBitmap = FALSE;
    SetName(name);
  
    if (panel)
	panel->AddChild(this);

    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;

    windowStyle = style;

    assert(panel != NULL);

    panel->GetValidPosition(&x, &y);

    windows_id = NewControlId();

    hwndItem = WinCreateWindow(panel->GetHWND(), WC_BUTTON, (PSZ)label,
				 BS_PUSHBUTTON | WS_TABSTOP, 
				 0, 0, 0, 0, panel->GetHWND(),
				 HWND_TOP, windows_id, NULL, NULL);
    TRACE("wxButton::Create: Window created, parent=%#x, hwndItem=%#x", (int)panel->GetHWND(), (int)hwndItem);

    /*
     * Colours used:
     *	SYSCLR_BUTTONDEFAULT
     *	SYSCLR_BUTTONLIGHT
     *	SYSCLR_BUTTONMIDDLE
     *	SYSCLR_MENUTEXT
     *	SYSCLR_WINDOW
     *	SYSCLR_WINDOWFRAME
     *
     *	PP_BACKGROUNDCOLOR		Button Background colour
     *	PP_BORDERCOLOR			Border around button
     *	PP_DISABLEDFOREGROUNDCOLOR	Disabled Button Text
     *	PP_FOREGROUNDCOLOR		Button Text
     *	PP_HILITEFOREGROUNDCOLOR
     */
    wxSetForegroundColour(hwndItem, buttonColour);
//    wxSetBackgroundColour(hwndItem, &backColour);

    if (buttonFont)
	wxSetWinFont(hwndItem, buttonFont);

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

#if XXX
//  HDC the_dc = GetWindowDC((HWND)ms_handle) ;
    if (buttonFont)
    {
	buttonFont->RealizeResource();

	// This resource won't be deleted during the lifetime of this button,
	// because wxFont::ReleaseResource isn't called until the destructor, and
	// meanwhile, the resource is locked from being cleaned up.
	if (buttonFont->GetResourceHandle())
	    SendMessage((HWND)ms_handle,WM_SETFONT,
			(WPARAM)buttonFont->GetResourceHandle(),0L);
    }
  
//  ReleaseDC((HWND)ms_handle,the_dc) ;
#endif

    SetSize(x, y, width, height);
//  WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);

    Callback(Function);

    panel->AdvanceCursor(this);
    return TRUE;
}

/*
 * Create a bitmap button
 */
Bool wxButton::Create(wxPanel *panel, wxFunction Function, wxBitmap *bitmap,
                   int x, int y, int width, int height,
                   long style, const char *name)
{
    TRACE("wxButton::Create(panel=%#x, Function=%#x, bitmap=%#x, x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)panel, (int)Function, (int)bitmap, x, y, width, height, (int)style, name);
    buttonBitmap = bitmap;
    isBitmap = TRUE;
    SetName(name);

    if (panel)
	panel->AddChild(this);

    buttonFont = panel->buttonFont;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;

    windowStyle = style;

    panel->GetValidPosition(&x, &y);

    windows_id = NewControlId();

    // Button create structure... set Bitmap image
    BTNCDATA bcd;

    bcd.cb = sizeof(BTNCDATA);
    bcd.fsCheckState  = 0;
    bcd.fsHiliteState = 0;
    bcd.hImage        = bitmap->os2_bitmap;

    if (width<0)
	width = bitmap->GetWidth() ;
    if (height<0)
	height = bitmap->GetHeight() ;

    hwndItem = WinCreateWindow(panel->GetHWND(), WC_BUTTON, (PSZ)"test",
				 WS_VISIBLE | BS_PUSHBUTTON | BS_BITMAP | BS_AUTOSIZE | WS_TABSTOP,
				 0, 0, 0, 0, panel->GetHWND(),
				 HWND_TOP, windows_id, &bcd, NULL);
    TRACE("wxButton::Create: Window created, parent=%#x, hwndItem=%#x", (int)panel->GetHWND(), (int)hwndItem);

    /*
     * Colours used:
     *	SYSCLR_BUTTONDEFAULT
     *	SYSCLR_BUTTONLIGHT
     *	SYSCLR_BUTTONMIDDLE
     *	SYSCLR_MENUTEXT
     *	SYSCLR_WINDOW
     *	SYSCLR_WINDOWFRAME
     *
     *	PP_BACKGROUNDCOLOR		Button Background colour
     *	PP_BORDERCOLOR			Border around button
     *	PP_DISABLEDFOREGROUNDCOLOR	Disabled Button Text
     *	PP_FOREGROUNDCOLOR		Button Text
     *	PP_HILITEFOREGROUNDCOLOR
     */
    wxSetForegroundColour(hwndItem, buttonColour);
//    wxSetBackgroundColour(hwndItem, &backColour);

// BS_BITMAP, &bcd

    if (buttonFont)
	wxSetWinFont(hwndItem, buttonFont);

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

#if XXX
//  HDC the_dc = GetWindowDC((HWND)ms_handle) ;

    if (buttonFont) {
	buttonFont->RealizeResource();
  
	if (buttonFont->GetResourceHandle())
	    SendMessage((HWND)ms_handle,WM_SETFONT,
			(WPARAM)buttonFont->GetResourceHandle(),0L);
    }
  
//  ReleaseDC((HWND)ms_handle,the_dc) ;

#endif

    SetSize(x, y, width, height);
//  WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);

    Callback(Function);

    panel->AdvanceCursor(this);
    return TRUE;
}

wxButton::~wxButton(void)
{
    TRACE("wxButton::~wxButton() this=%#x wxWindow=%#x", (int)this, (wxWindow *)this);
}

void wxButton::SetLabel(char *label)
{
    // Should change from bitmap to text form
#if XXX
#endif
    buttonBitmap = NULL;
    isBitmap = FALSE;
    WinSetWindowText(hwndItem, (PSZ)label);
}

void wxButton::SetLabel(wxBitmap *bitmap)
{
    // Should change from text to bitmap form
#if XXX
#endif
    buttonBitmap = bitmap;
    isBitmap = TRUE;
}

char *wxButton::GetLabel(void)
{
    WinQueryWindowText(hwndItem, 300, (PCH)wxBuffer);
    return wxBuffer;
}

/*
 * Get the size of the button when the system
 * automatically sizes it.
 */
void wxButton::GetAutoSize(int *cx, int *cy)
{
    TRACE("wxButton::GetAutoSize(...)");
    char buf[300];
    int tcx, tcy, ccx, ccy;
    WinQueryWindowText(hwndItem, sizeof buf, (PCH)buf);
    wxGetTextExtents(hwndItem, buf, &tcx, &tcy);
    wxGetCharSize(hwndItem, &ccx, &ccy);

    /*
     * Auto-width of button text = text extents + 2 * charwidth
     * + 2 * 4 pels for button borders
     */
    *cx = tcx + 2 * ccx + 2 * 4;

    /*
     * Auto-height of button text is character cell height
     * + 2 * 4 pels for button borders
     */
    *cy = ccy + 2 * 4;
}

void wxButton::SetFocus(void)
{
/*
  wxPanel *panel = (wxPanel *)GetParent();
  if (panel)
  {
    wxWnd *wnd = (wxWnd *)panel->handle;
    WinSendMsg(wnd->hwndItem, DM_SETDEFID, windows_id, 0L);
  }
*/
  wxItem::SetFocus();
}

void wxButton::SetDefault(void)
{
    wxPanel *panel = (wxPanel *)GetParent();
    if (panel)
	panel->defaultItem = this;

    WinSendMsg(hwndItem, BM_SETDEFAULT, 0, 0);
}

void wxButton::SetButtonColour(wxColour& col)
{
    TRACE("wxButton::SetButtonColour(col=%#06x)", (COLOR)col);
    wxItem::SetButtonColour(col);
    if (hwndItem)
	wxSetForegroundColour(hwndItem, buttonColour);
}

