/*
 * File:        wx_canvs.cc
 * Purpose:     wxCanvas implementation (OS/2)
 * Author:      Julian Smart
 * Created:     1993
 * Updated:        August 1994
 * RCS_ID:      $Id: wx_canvs.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:   (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#pragma hdrstop
#include <math.h>

#include "wx_frame.h"
#include "wx_privt.h"
#include "wx_dc.h"
#include "wx_canvs.h"
#include "wx_panel.h"
#include "wx_stdev.h"
#include "wx_utils.h"

#ifdef wx_os2
#define	RGB(R,G,B)	(((R) * 65536) + ((G) * 256) + (B))
#endif

extern char wxCanvasClassName[];

IMPLEMENT_DYNAMIC_CLASS(wxCanvas, wxWindow)

wxCanvas::wxCanvas(void)
{
    TRACE("wxCanvas::wxCanvas()");
    clipping = FALSE;
}

wxCanvas::wxCanvas(wxWindow *parent, int x, int y, int width, int height, long style,
		   const char *name):
    wxbCanvas (parent, x, y, width, height, style, name)
{
    TRACE("wxCanvas::wxCanvas(parent=%#x, x=%d, y=%d, w=%d, height=%d, style=%#x, name='%s')", (int)parent, x, y, width, height, (int)style, name);
    Create(parent, x, y, width, height, style, name);
}

Bool wxCanvas::Create(wxWindow * parent, int x, int y, int width, int height, long style, const char *name)
{
    TRACE("wxCanvas::Create(parent=%#x, x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)parent, x, y, width, height, (int)style, name);
    SetName(name);

    is_retained = ((style & wxRETAINED) == wxRETAINED);

    windowStyle = style;
    is_retained = FALSE;
    clipping = FALSE;
    wxWnd *cparent = NULL;
    if (parent)
	cparent = (wxWnd *)parent->handle;

    if (wxSubType(parent->__type, wxTYPE_PANEL))
	((wxPanel *)parent)->GetValidPosition(&x, &y);

    DWORD os2_flags = WS_VISIBLE;
    if (style & wxBORDER)
	os2_flags |= FS_BORDER;
//os2_flags |= WS_CHILD | WS_VISIBLE;

    wxCanvasWnd *wnd = new wxCanvasWnd(cparent, wxCanvasClassName, this, x, y, width, height, os2_flags);

    handle = (char *)wnd;

    if (parent)
	parent->AddChild (this);
    window_parent = parent;

    horiz_units = 0;
    vert_units = 0;

    wx_dc = new wxCanvasDC(this);

    if (wxSubType(parent->__type, wxTYPE_PANEL))
	((wxPanel *)parent)->AdvanceCursor(this);

    SetBackground(wxWHITE_BRUSH);

    return TRUE;
}

wxCanvas::~wxCanvas (void)
{
    TRACE("wxCanvas::~wxCanvas() this=%#x wxWindow=%#x", (int)this), (wxWindow *)this;
    if (wx_dc)
	delete wx_dc;
}

void wxCanvas::SetColourMap (wxColourMap * cmap)
{
  if (wx_dc)
    wx_dc->SetColourMap(cmap);
}

/*
 * Set canvas size. Also recalculate the scrollbars.,
 */
void wxCanvas::SetSize(int x, int y, int w, int h, int sizeFlags)
{
    TRACE("wxCanvas::SetSize(x=%d, y=%d, w=%d, h=%d, sizeFlags=%#x)", x, y, w, h, sizeFlags);

    if (!handle || !wx_dc)
	return;
    wxCanvasWnd *wnd = (wxCanvasWnd *)handle;

    int currentX, currentY;
    GetPosition (&currentX, &currentY);
    if (x == -1)
	x = currentX;
    if (y == -1)
	y = currentY;

    int ww, hh;
    int vsx, vsy, vsw, vsh;		// Vertical scroll bar position
    int hsx, hsy, hsw, hsh;		// Horizontal scroll bar position
    vsw = WinQuerySysValue(HWND_DESKTOP, SV_CXVSCROLL); // Vertical scroll bar width
    hsh = WinQuerySysValue(HWND_DESKTOP, SV_CYHSCROLL); // Horizontal scroll bar height
    
    GetSize(&ww, &hh);
    if (w == -1)
	w = ww;
    if (h == -1)
	h = hh;

    /*
     * Calculate scroll bar window position(s)
     */
    if (wnd->hwndVertScroll) {
	vsx = x + w - vsw;
	vsy = y;
	vsh = h;
	if (wnd->hwndHorzScroll)
	    vsh -= hsh;
	w -= vsw;
    }
    if (wnd->hwndHorzScroll) {
	hsx = x;
	hsy = y + h - hsh;
	hsw = w;
	if (wnd->hwndVertScroll)
	    hsw -= vsw;
	h -= hsh;
    }

    TRACE("hsh=%d, vsw=%d, x=%d, y=%d, w=%d, h=%d", hsh, vsw, x, y, w, h);

    MoveWindow(x, y, w, h, TRUE);
    if (wnd->hwndVertScroll) {
	wx_MoveWindow(wnd->hwndVertScroll, vsx, vsy, vsw, vsh, TRUE);
	int ymin = (int)wx_dc->min_y;
	int ymax = wxMax((int)wx_dc->max_y, ysize);
	int winheight = h / vert_units;
	int first = ymin / horiz_units;
	int last = ymax / horiz_units - winheight;
	wnd->yscroll_lines_per_page = winheight;
TRACE("ymin=%d, ymax=%d, winheight=%d, first=%d, last=%d", ymin, ymax, winheight, first, last);
	WinSendMsg(wnd->hwndVertScroll, SBM_SETSCROLLBAR,
		   (MPARAM)wnd->xscroll_position, MPFROM2SHORT(first, last));
	WinSendMsg(wnd->hwndVertScroll, SBM_SETTHUMBSIZE,
		   MPFROM2SHORT(winheight, (ymax - ymin) / vert_units + 1), 0);
    }
    if (wnd->hwndHorzScroll) {
	wx_MoveWindow(wnd->hwndHorzScroll, hsx, hsy, hsw, hsh, TRUE);
	int xmin = (int)wx_dc->min_x;
	int xmax = wxMax((int)wx_dc->max_x, xsize);
	int winwidth = w / horiz_units;
	int first = xmin / horiz_units;
	int last = xmax / horiz_units - winwidth;
	wnd->xscroll_lines_per_page = winwidth;
TRACE("xmin=%d, xmax=%d, winwidth=%d, first=%d, last=%d", xmin, xmax, winwidth, first, last);       
	WinSendMsg(wnd->hwndHorzScroll, SBM_SETSCROLLBAR,
		   (MPARAM)wnd->xscroll_position, MPFROM2SHORT(first, last));
	WinSendMsg(wnd->hwndHorzScroll, SBM_SETTHUMBSIZE,
		   MPFROM2SHORT(winwidth, (xmax - xmin) / horiz_units + 1), 0);
    }

    GetEventHandler()->OnSize(w, h);
}

/*
 * horizontal/vertical: number of pixels per unit (e.g. pixels per text line)
 * x/y_length:        : no. units per scrollbar
 * x/y_page:          : no. units per page scrolled
 */
void wxCanvas::SetScrollbars(int horizontal, int vertical,
			     int x_length, int y_length,
			     int x_page, int y_page,
			     int x_pos, int y_pos,
			     Bool setVirtualSize)
{
    TRACE("wxCanvas::SetScrollbars(horizontal=%d, vertical=%d, x_length=%d, y_length=%d, x_page=%d, y_page=%d, x_pos=%d, y_pos=%d, setVirtualSize=%d)", horizontal, vertical, x_length, y_length, x_page, y_page, x_pos, y_pos, setVirtualSize);
    horiz_units = horizontal;		// Width of cell in units
    vert_units = vertical;		// Height of cell in units
    xsize = x_length * horizontal;	// Width in units
    ysize = y_length * vertical;	// Height in units

    wxCanvasWnd *wnd = (wxCanvasWnd *)handle;

    if (wnd) {
	/*
	 * Set up scroll bar parameters
	 */
	wnd->xscroll_pixels_per_line = horizontal;
	wnd->yscroll_pixels_per_line = vertical;
	wnd->xscroll_lines = x_length;
	wnd->yscroll_lines = y_length;
	wnd->xscroll_lines_per_page = x_page;
	wnd->yscroll_lines_per_page = y_page;
	wnd->calcScrolledOffset = setVirtualSize;

	/*
	 * Get scroll bar handles.
	 * If we need a scroll bar, and it does not previously exist
	 * we will create it here.
	 */
	BOOL recalc = FALSE;
	if (wnd->xscroll_lines > 0 && wnd->hwndHorzScroll==NULLHANDLE) {
	    /*
	     * Create horizontal scroll bar as a sibling
	     * Note that the parent window is a client window
	     * and not a frame window.
	     */
	    wnd->hwndHorzScroll = WinCreateWindow(WinQueryWindow(wnd->hwndWnd, QW_PARENT),
						 WC_SCROLLBAR, NULL,
						 SBS_HORZ | WS_VISIBLE, 0, 0, 0, 0, wnd->hwndWnd,
						 HWND_TOP, FID_HORZSCROLL, NULL, NULL);
	    recalc = TRUE;
	} else if (wnd->xscroll_lines <= 0 && wnd->hwndHorzScroll!=NULLHANDLE) {
	    /*
	     * Destroy horizontal scroll bar
	     */
	    WinDestroyWindow(wnd->hwndHorzScroll);
	    wnd->hwndHorzScroll = NULLHANDLE;
	    recalc = TRUE;
	}
	if (wnd->yscroll_lines > 0 && wnd->hwndVertScroll==NULLHANDLE) {
	    /*
	     * Create vertical scroll bar as a sibling
	     * Same comment as above that the parent
	     * window is the a window.
	     */
	    wnd->hwndVertScroll = WinCreateWindow(WinQueryWindow(wnd->hwndWnd, QW_PARENT),
						  WC_SCROLLBAR, NULL,
						  SBS_VERT | WS_VISIBLE, 0, 0, 0, 0, wnd->hwndWnd,
						  HWND_TOP, FID_VERTSCROLL, NULL, NULL);
	    recalc = TRUE;
	} else if (wnd->yscroll_lines <= 0 && wnd->hwndVertScroll != NULLHANDLE) {
	    /*
	     * Destroy Vertical scroll bar
	     */
	    WinDestroyWindow(wnd->hwndVertScroll);
	    wnd->hwndVertScroll = NULLHANDLE;
	    recalc = true;
	}

	if (recalc) {
	    /*
	     * Recalculate size of canvas and set size of scrollbar(s)
	     */
	    SetSize(-1, -1, -1, -1);
	}

	int wnd_width, wnd_height;
	wx_GetWindowSize(wnd->hwndWnd, &wnd_width, &wnd_height);

#if 0
	// Recalculate scroll bar range and position
	if (wnd->hwndHorzScroll) {
	    int Hlast = wxMax(0, wnd->xscroll_lines*wnd->xscroll_pixels_per_line - wnd_width);
	    wnd->xscroll_position = wxMin(x_pos*wnd->xscroll_pixels_per_line, Hlast);

	    WinSendMsg(wnd->hwndVertScroll, SBM_SETSCROLLBAR,
		       (MPARAM)wnd->xscroll_position, MPFROM2SHORT(0, Hlast));
	    WinSendMsg(wnd->hwndVertScroll, SBM_SETTHUMBSIZE,
		       MPFROM2SHORT(wnd_width, wnd->xscroll_lines*wnd->xscroll_pixels_per_line), 0);
	}
	if (wnd->hwndVertScroll) {
	    int Vlast = wxMax(0, wnd->yscroll_lines * wnd->yscroll_pixels_per_line - wnd_height);
	    wnd->yscroll_position = wxMin(y_pos * wnd->yscroll_pixels_per_line, Vlast);

	    WinSendMsg(wnd->hwndVertScroll, SBM_SETSCROLLBAR,
		       (MPARAM)wnd->yscroll_position, MPFROM2SHORT(0, Vlast));
	    WinSendMsg(wnd->hwndVertScroll, SBM_SETTHUMBSIZE,
		       MPFROM2SHORT(wnd_height, wnd->yscroll_lines*wnd->yscroll_pixels_per_line), 0);
        }
#endif	
	WinInvalidateRect(wnd->hwndWnd, NULL, TRUE) ;
	WinUpdateWindow(wnd->hwndWnd) ;
    }
}

void wxCanvas::GetScrollUnitsPerPage (int *x_page, int *y_page)
{
    wxCanvasWnd *wnd = (wxCanvasWnd *) handle;
    if (wnd) {
	*x_page = wnd->xscroll_lines_per_page;
	*y_page = wnd->yscroll_lines_per_page;
    }
}

void wxCanvas::GetScrollPixelsPerUnit(int *x_unit, int *y_unit)
{
    wxCanvasWnd *wnd = (wxCanvasWnd *) handle;
    if (wnd) {
	*x_unit = wnd->xscroll_pixels_per_line;
	*y_unit = wnd->yscroll_pixels_per_line;
    }
}

/*
 * Scroll to given position (scroll position, not pixel position)
 */
void wxCanvas::Scroll(int x_pos, int y_pos)
{
    TRACE("wxCanvas::Scroll(x_pos=%d, y_pos=%d)", x_pos, y_pos);
    int old_x, old_y;
    ViewStart (&old_x, &old_y);
    if (((x_pos == -1) || (x_pos == old_x)) && ((y_pos == -1) || (y_pos == old_y)))
	return;

    wxCanvasWnd *wnd = (wxCanvasWnd *)handle;

    if (wnd) {
	if (x_pos > -1) {
	    wnd->xscroll_position = x_pos;
	    WinSendMsg(wnd->hwndHorzScroll, SBM_SETPOS, MPFROMSHORT(x_pos), (MPARAM)0);
	}
	if (y_pos > -1) {
	    wnd->yscroll_position = y_pos;
	    WinSendMsg(wnd->hwndVertScroll, SBM_SETPOS, MPFROMSHORT(y_pos), (MPARAM)0);
	}
	WinInvalidateRect(wnd->hwndWnd, NULL, TRUE);
	WinUpdateWindow(wnd->hwndWnd);
    }
}

void wxCanvas::EnableScrolling(Bool x_scroll, Bool y_scroll)
{
    TRACE("wxCanvas::EnableScrolling(x_scroll=%d, y_scroll=%d)", x_scroll, y_scroll);
    wxCanvasWnd *wnd = (wxCanvasWnd *) handle;
    wnd->x_scrolling_enabled = x_scroll;
    wnd->y_scrolling_enabled = y_scroll;
}

void wxCanvas::GetVirtualSize (int *x, int *y)
{
    wxCanvasWnd *wnd = (wxCanvasWnd *) handle;
    if (wnd) {
	*x = wnd->xscroll_pixels_per_line * wnd->xscroll_lines;
	*y = wnd->yscroll_pixels_per_line * wnd->yscroll_lines;
    }
}

void wxCanvas::WarpPointer (int x_pos, int y_pos)
{
    TRACE("wxCanvas::WarpPointer(x_pos=%d, y_pos=%d)", x_pos, y_pos);
    // Move the pointer to (x_pos,y_pos) coordinates. They are expressed in
    // pixel coordinates, relatives to the canvas -- So, we first need to
    // substract origin of the window, then convert to screen position

    wxCanvasWnd *wnd = (wxCanvasWnd *) handle;

    if (wnd) {
	x_pos -= wnd->xscroll_position * wnd->xscroll_pixels_per_line;
	y_pos -= wnd->yscroll_position * wnd->yscroll_pixels_per_line;

	RECTL rect;
	wx_WindowRect(wnd->hwndWnd, &rect);
	/*
	 * wx_WindowRect returs in absolute PM coordinates
	 * and y_pos is in relative wx coordinates, hence the subtraction
	 */
	WinSetPointerPos(HWND_DESKTOP, rect.xLeft + x_pos, rect.yTop - y_pos);
    }
}

// Where the current view starts from
void wxCanvas::ViewStart (int *x, int *y)
{
    TRACE("wxCanvas::ViewStart(...)");
    wxCanvasWnd *wnd = (wxCanvasWnd *) handle;
    *x = wnd->xscroll_position;
    *y = wnd->yscroll_position;
}

void wxWnd::DeviceToLogical(float *x, float *y)
{
    if (__type == wxTYPE_CANVAS_WND) {
	wxCanvas *canvas = (wxCanvas *)wx_window;
	if (canvas->wx_dc) {
	    *x = canvas->wx_dc->DeviceToLogicalX((int) *x);
	    *y = canvas->wx_dc->DeviceToLogicalY((int) *y);
	}
    }
}

wxCanvasWnd::wxCanvasWnd(wxWnd *parent, char *winClass, wxWindow * wx_win,
			 int x, int y, int width, int height, DWORD style):
    wxSubWnd(parent, wxCanvasClassName, wx_win, x, y, width, height, style)
{
    __type = wxTYPE_CANVAS_WND;

    TRACE("wxCanvasWnd::wxCanvasWnd");
    unit = 0;
    xmult = ymult = 1.0;
    fxmult = fymult = 1.0;
    xoff = yoff = 0.0;
    hwndVertScroll = hwndHorzScroll = NULLHANDLE;
}


void wxCanvasWnd::OnVScroll(int uscmd, int sslider, int usidentifier)
{
    TRACE("wxCanvasWnd::OnVScroll(uscmd-%d, sslider=%d, usidentifier=%d)", uscmd, sslider, usidentifier);

    int position = (int)WinSendMsg(hwndVertScroll, SBM_QUERYPOS, (MPARAM)NULL, (MPARAM)NULL);
    int nScrollInc;
    switch (uscmd) {
    case SB_LINEUP:
	nScrollInc = -1;
	break;
      
    case SB_LINEDOWN:
	nScrollInc = 1;
	break;
      
    case SB_PAGEUP:
	nScrollInc = -yscroll_lines_per_page;
	break;
      
    case SB_PAGEDOWN:
	nScrollInc = yscroll_lines_per_page;
	break;

      
    case SB_SLIDERTRACK:
	nScrollInc = sslider - position;
	break;
      
    default:
	nScrollInc = 0;
    }

    if (nScrollInc != 0) {
        int new_pos = position + nScrollInc;
	MRESULT range = WinSendMsg(hwndVertScroll, SBM_QUERYRANGE, (MPARAM)NULL, (MPARAM)NULL);
	int sfirst = SHORT1FROMMR(range);
	int slast = SHORT2FROMMR(range);

        if (new_pos < sfirst)
            new_pos = sfirst;
        else if (new_pos > slast)
            new_pos = slast;
	if (new_pos != yscroll_position) {
	    RECTL rectl;
	    WinQueryWindowRect(hwndWnd, &rectl);
	    if (y_scrolling_enabled) {
		RECTL urectl;
		int dy = (new_pos - yscroll_position) * yscroll_pixels_per_line;
		urectl = rectl;
		if (dy > 0)
		    urectl.yTop = urectl.yBottom + dy;
		else
		    urectl.yBottom = urectl.yTop + dy;
		WinScrollWindow(hwndWnd, 0, dy, &rectl, NULL, 0, &urectl, SW_INVALIDATERGN);
	    } else
		WinInvalidateRect(hwndWnd, &rectl, FALSE);
	    yscroll_position = new_pos;
	    WinSendMsg(hwndVertScroll, SBM_SETPOS, MPFROMSHORT(new_pos), (MPARAM)0);
	}
    }
}


void wxCanvasWnd::OnHScroll(int uscmd, int sslider, int usidentifier)
{
    TRACE("wxCanvasWnd::OnHscroll(uscmd-%d, sslider=%d, usidentifier=%d)", uscmd, sslider, usidentifier);
        
    int position = (int)WinSendMsg(hwndHorzScroll, SBM_QUERYPOS, (MPARAM)NULL, (MPARAM)NULL);
    int nScrollInc;
    switch (uscmd) {
    case SB_LINELEFT:
	nScrollInc = -1;
	break;
      
    case SB_LINERIGHT:
	nScrollInc = 1;
	break;
      
    case SB_PAGELEFT:
	nScrollInc = -xscroll_lines_per_page;
	break;
      
    case SB_PAGERIGHT:
	nScrollInc = xscroll_lines_per_page;
	break;
      
    case SB_SLIDERTRACK:
	nScrollInc = sslider - position;
	break;
      
    default:
	nScrollInc = 0;
    }

    if (nScrollInc != 0) {
        int new_pos = position + nScrollInc;
	MRESULT range = WinSendMsg(hwndHorzScroll, SBM_QUERYRANGE, (MPARAM)NULL, (MPARAM)NULL);
	int sfirst = SHORT1FROMMR(range);
	int slast = SHORT2FROMMR(range);

        if (new_pos < sfirst)
            new_pos = sfirst;
        else if (new_pos > slast)
            new_pos = slast;

	if (new_pos != xscroll_position) {
	    RECTL rectl;
	    WinQueryWindowRect(hwndWnd, &rectl);
	    if (x_scrolling_enabled) {
		RECTL urectl;
		int dx = -(new_pos - xscroll_position) * xscroll_pixels_per_line;
		urectl = rectl;
		if (dx > 0)
		    urectl.xRight = urectl.xLeft + dx;
		else
		    urectl.xLeft = urectl.xRight + dx;
		WinScrollWindow(hwndWnd, dx, 0, &rectl, NULL, 0, &urectl, SW_INVALIDATERGN);
	    } else
		WinInvalidateRect(hwndWnd, &rectl, FALSE);
	    xscroll_position = new_pos;
	    WinSendMsg(hwndHorzScroll, SBM_SETPOS, MPFROMSHORT(new_pos), (MPARAM)0);
	}
    }
}


BOOL wxCanvasWnd::OnPaint(HWND hwnd)
{
    TRACE("wxCanvasWnd::OnPaint(hwnd=%#x)", (int)hwnd);

    assert(hpsCanvas);
    OnEraseBkgnd(hpsCanvas);

    if (wx_window)
	wx_window->GetEventHandler()->OnPaint();

    return TRUE;
}

