/*
 * File:	wx_check.cc
 * Purpose:	Check box implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1995, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#pragma hdrstop

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>


#include "wx_panel.h"
#include "wx_check.h"
#include "wx_itemp.h"

BOOL wxCheckBox::OS2Control(WORD param, WORD id)
{
    wxCommandEvent event(wxEVENT_TYPE_CHECKBOX_COMMAND);
    event.commandInt = GetValue();
    event.eventObject = this;
    ProcessCommand(event);
    return TRUE;
}

IMPLEMENT_DYNAMIC_CLASS(wxCheckBox, wxItem)

// Single check box item
wxCheckBox::wxCheckBox(void)
{
}

// Single check box item
wxCheckBox::wxCheckBox(wxPanel *panel, wxFunction func, const char *Title,
                       int x, int y, int width, int height, long style, const char *name):
  wxbCheckBox(panel, func, Title, x, y, width, height, style, name)
{
  Create(panel, func, Title, x, y, width, height, style, name);
}

wxCheckBox::wxCheckBox(wxPanel *panel, wxFunction func, wxBitmap *bitmap,
                       int x, int y, int width, int height, long style, const char *name):
  wxbCheckBox(panel, func, bitmap, x, y, width, height, style, name)
{
  Create(panel, func, bitmap, x, y, width, height, style, name);
}


/*
 * Create a single check-box item
 */
Bool wxCheckBox::Create(wxPanel *panel, wxFunction func, const char *Title,
                       int x, int y, int width, int height, long style, const char *name)
{
    TRACE("wxCheckBox::Create(panel=%#x, func=%#x, Title='%s', x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)panel, (int)func, Title, x, y, width, height, (int)style, name);
    SetName(name);
    if (panel)
	panel->AddChild(this);
//    buttonFont = panel->buttonFont ;

    labelFont = panel->labelFont;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;

    windowStyle = style;

    panel->GetValidPosition(&x, &y);

    windows_id = NewControlId();

    checkWidth = -1 ;
    checkHeight = -1 ;
    hwndItem =  WinCreateWindow(panel->GetHWND(), WC_BUTTON, (PSZ)Title, BS_AUTOCHECKBOX|WS_TABSTOP,
				  0, 0, 0, 0, panel->GetHWND(),
				  HWND_TOP, windows_id, NULL, NULL);
    TRACE("wxCheckBox::Create: Window created, parent=%#x, hwndItem=%#x", (int)panel->GetHWND(), (int)hwndItem);

    /*
     * Colours used:
     *	SYSCLR_BUTTONDEFAULT
     *	SYSCLR_BUTTONLIGHT
     *	SYSCLR_BUTTONMIDDLE
     *	SYSCLR_MENUTEXT
     *	SYSCLR_WINDOW
     *	SYSCLR_WINDOWFRAME
     *
     *	PP_BACKGROUNDCOLOR		Button Background colour
     *	PP_BORDERCOLOR			Border around button
     *	PP_DISABLEDFOREGROUNDCOLOR	Disabled Button Text
     *	PP_FOREGROUNDCOLOR		Button Text
     *	PP_HILITEFOREGROUNDCOLOR
     */
    wxSetForegroundColour(hwndItem, buttonColour);
    wxSetBackgroundColour(hwndItem, backColour);
    
    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

#if XXX
//  HDC the_dc = GetWindowDC((HWND)ms_handle) ;
  if (labelFont) {
    labelFont->RealizeResource();
  
    if (labelFont->GetResourceHandle())
      SendMessage((HWND)ms_handle,WM_SETFONT,
                (WPARAM)labelFont->GetResourceHandle(),0L);
  }
  
//  ReleaseDC((HWND)ms_handle,the_dc) ;
#endif

    if (labelFont)
	wxSetWinFont(hwndItem, labelFont);

    SetSize(x, y, width, height);
//  WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);

    panel->AdvanceCursor(this);
    Callback(func);
    return TRUE;
}

Bool wxCheckBox::Create(wxPanel *panel, wxFunction func, wxBitmap *bitmap,
                       int x, int y, int width, int height, long style, const char *name)
{
    TRACE("wxCheckBox::Create(panel=%#x, func=%#x, bitmap=%#x, x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)panel, (int)func, (int)bitmap, x, y, width, height, (int)style, name);
    SetName(name);
    if (panel) panel->AddChild(this);
//  buttonFont = panel->buttonFont ;

    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;
    windowStyle = style;

    panel->GetValidPosition(&x, &y);

    windows_id = NewControlId();

    checkWidth = -1 ;
    checkHeight = -1 ;
    hwndItem =  WinCreateWindow(panel->GetHWND(), WC_BUTTON, (PSZ)"toggle", BS_AUTOCHECKBOX|WS_TABSTOP|WS_VISIBLE,
				  0, 0, 0, 0, panel->GetHWND(),
				  HWND_TOP, windows_id, NULL, NULL);
    TRACE("wxCheckBox::Create: Window created, parent=%#x, hwndItem=%#x", (int)panel->GetHWND(), (int)hwndItem);

    /*
     * Colours used:
     *	SYSCLR_BUTTONDEFAULT
     *	SYSCLR_BUTTONLIGHT
     *	SYSCLR_BUTTONMIDDLE
     *	SYSCLR_MENUTEXT
     *	SYSCLR_WINDOW
     *	SYSCLR_WINDOWFRAME
     *
     *	PP_BACKGROUNDCOLOR		Button Background colour
     *	PP_BORDERCOLOR			Border around button
     *	PP_DISABLEDFOREGROUNDCOLOR	Disabled Button Text
     *	PP_FOREGROUNDCOLOR		Button Text
     *	PP_HILITEFOREGROUNDCOLOR
     */
    wxSetForegroundColour(hwndItem, buttonColour);
    wxSetBackgroundColour(hwndItem, backColour);

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

#if XXX
//  HDC the_dc = GetWindowDC((HWND)ms_handle) ;

  if (labelFont) {
    labelFont->RealizeResource();
    
    if (labelFont->GetResourceHandle())
      SendMessage((HWND)ms_handle,WM_SETFONT,
                (WPARAM)labelFont->GetResourceHandle(),0L);
  }
//  ReleaseDC((HWND)ms_handle,the_dc) ;
#endif

    if (buttonFont)
	wxSetWinFont(hwndItem, buttonFont);

    SetSize(x, y, width, height);

//  WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);
    panel->AdvanceCursor(this);
    Callback(func);
    return TRUE;
}

wxCheckBox::~wxCheckBox(void)
{
    TRACE("wxCheckBox::~wxCheckBox() this=%#x wxWindow=%#x", this, (wxWindow *)this);
}

void wxCheckBox::SetLabel(char *label)
{
  WinSetWindowText(hwndItem, (PSZ)label);
}

char *wxCheckBox::GetLabel()
{
    char buf[300];
    WinQueryWindowText(hwndItem, 300, (PCH)buf);
    return copystring(buf);
}

void wxCheckBox::SetLabel(wxBitmap *bitmap)
{
    // Will change from text to bitmap form
    // XXX
}

/*
 * Get the size of the button when the system
 * automatically sizes it.
 */
void wxCheckBox::GetAutoSize(int *cx, int *cy)
{
    TRACE("wxCheckBox::GetAutoSize(...)");
    char buf[300];
    int tcx, tcy, ccx, ccy;
    WinQueryWindowText(hwndItem, sizeof buf, (PCH)buf);
    wxGetTextExtents(hwndItem, buf, &tcx, &tcy);
    wxGetCharSize(hwndItem, &ccx, &ccy);

    /*
     * Auto-width of button text = text extents + radio-box-size
     */
    *cx = tcx + RADIO_SIZE;

    /*
     * Auto-height of button text is character cell height
     */
    *cy = ccy;
}

void wxCheckBox::SetValue(Bool val)
{
    WinSendMsg(hwndItem, BM_SETCHECK, MPFROMLONG(val), 0);
}

Bool wxCheckBox::GetValue(void)
{
    return (Bool)WinSendMsg(hwndItem, BM_QUERYCHECK, 0, 0);
}

void wxCheckBox::SetLabelFont(wxFont *font)
{
#if XXX
  // Decrement the usage count of the old label font
  // (we may be able to free it up)
    if (labelFont)
	labelFont->ReleaseResource();
    labelFont = font;
    HWND hWnd = GetHWND();
    if (hWnd != 0) {
	if (font) {
	    font->UseResource();
	    font->RealizeResource();
	    if (font->GetResourceHandle())
		SendMessage(hWnd, WM_SETFONT,
			    (WPARAM)font->GetResourceHandle(),TRUE);
	}
    }
#endif
}

void wxCheckBox::SetButtonColour(wxColour& col)
{
    TRACE("wxCheckBox::SetButtonColour(col=%#06x)", (COLOR)col);
    wxItem::SetButtonColour(col);
    if (hwndItem) {
	wxSetForegroundColour(hwndItem, buttonColour);
    }
}

void wxCheckBox::SetBackgroundColour(wxColour& col)
{
    TRACE("wxCheckBox::SetBackgroundColour(%#06x)", (COLOR)col);
    wxItem::SetBackgroundColour(col);
    if (hwndItem) {
	wxSetBackgroundColour(hwndItem, backColour);
    }
}

