/*
 * File:	wx_choic.cc
 * Purpose:	Choice item implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_itemp.h"
#include "wx_choic.h"

#define	FRAME_MARGIN	6

BOOL wxChoice::OS2Control(WORD param, WORD id)
{
    if (param == CBN_LBSELECT) {
	wxCommandEvent event(wxEVENT_TYPE_CHOICE_COMMAND);
	event.commandInt = GetSelection();
	event.eventObject = this;
	event.commandString = copystring(GetStringSelection());
	ProcessCommand(event);
	delete[] event.commandString;
	return TRUE;
    }
    return FALSE;
}

IMPLEMENT_DYNAMIC_CLASS(wxChoice, wxItem)

wxChoice::wxChoice(void)
{
    no_strings = 0;
}

wxChoice::wxChoice(wxPanel *panel, wxFunction func, const char *Title,
                   int x, int y, int width, int height, int N, char **Choices,
                   long style, const char *name):
  wxbChoice(panel, func, Title, x, y, width, height, N, Choices, style, name)
{
    Create(panel, func, Title, x, y, width, height, N, Choices, style, name);
}

Bool wxChoice::Create(wxPanel *panel, wxFunction func, const char *Title,
                   int x, int y, int width, int height, int N, char **Choices,
                   long style, const char *name)
{
    TRACE("wxChoice::Create(panel=%#x, func=%#x, Title='%s', x=%d, y=%d, w=%d, h=%d, N=%d, Choices=%#x, style=%#x, name='%s')", (int)panel, (int)func, Title, x, y, width, height, N, (int)Choices, (int)style, name);
    SetName(name);
    if (panel)
	panel->AddChild(this);
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;
    no_strings = N;

    windowStyle = style;

//    labelPosition = panel->label_position;
    SetAppropriateLabelPosition();
    panel->GetValidPosition(&x, &y);

    if (Title)
	CreateLabel(panel->GetHWND(), Title, style & wxFIXED_LENGTH);

#if XXX
    if (labelFont) {
	labelFont->RealizeResource();
      
	if (labelFont->GetResourceHandle())
	    SendMessage(labelhWnd,WM_SETFONT,
			(WPARAM)labelFont->GetResourceHandle(),0L);
    }
}
  else
    labelhWnd = NULL;
#endif

  windows_id = (int)NewControlId();

    hwndItem =  WinCreateWindow(panel->GetHWND(), WC_COMBOBOX, NULL, CBS_DROPDOWNLIST | WS_TABSTOP,
				  0, 0, 0, 0, panel->GetHWND(),
				  HWND_TOP, windows_id, NULL, NULL);
    TRACE("wxChoice::Create: Window created, parent=%#x, hwndItem=%#x", (int)panel->GetHWND(), (int)hwndItem);

//    wxSetForegroundColour(hwndItem, &buttonColour);
//    wxSetBackgroundColour(hwndItem, &backColour);

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

#if XXX
  if (buttonFont) {
    buttonFont->RealizeResource();
    if (buttonFont->GetResourceHandle())
      SendMessage((HWND)ms_handle,WM_SETFONT,
                  (WPARAM)panel->buttonFont->GetResourceHandle(),0L);
  }
#endif
  
    if (buttonFont)
	wxSetWinFont(hwndItem, buttonFont);

    int i;
    for (i = 0; i < N; i++)
	(void) WinInsertLboxItem(hwndItem, LIT_END, (PSZ)Choices[i]);
    WinSendMsg(hwndItem, LM_SELECTITEM, 0L, (MPARAM)TRUE);

    SetSize(x, y, width, height);
    Show();
    panel->AdvanceCursor(this);
    Callback(func);

    return TRUE;
}


wxChoice::~wxChoice(void)
{
	TRACE("wxChoice::~wxChoice() this=%#x wxWindow=%#x", this, (wxWindow *)this);
}

void wxChoice::Append(char *Item)
{
    (void) WinInsertLboxItem(hwndItem, LIT_END, (PSZ)Item);
    no_strings ++;
}

// Unfortunately, under XView it doesn't redisplay until user resizes
// window. Any suggestions folks?
void wxChoice::Clear(void)
{
    WinSendMsg(hwndItem, LM_DELETEALL, 0, 0);

    no_strings = 0;
}


int wxChoice::GetSelection(void)
{
    return (int)WinQueryLboxSelectedItem(hwndItem);
}

void wxChoice::SetSelection(int n)
{
    WinSendMsg(hwndItem, LM_SELECTITEM, (MPARAM)n, (MPARAM)TRUE);
}

int wxChoice::FindString(char *s)
{
    TRACE("wxChoice::FindString(s='%s')", s);
    int pos = (int)WinSendMsg(hwndItem, LM_SEARCHSTRING, MPFROM2SHORT(LSS_CASESENSITIVE, LIT_FIRST), s);
    if (pos == LIT_ERROR || pos == LIT_NONE)
	return -1;
    else
	return pos;
}

char *wxChoice::GetString(int n)
{
    TRACE("wxChoice::GetString(n=%d)", n);
    int len = (int)WinQueryLboxItemText(hwndItem, n, wxBuffer, 128);
    wxBuffer[len] = 0;
    TRACE("wxChoice::GetString: len=%d, wxBuffer='%s'", wxBuffer);
    return wxBuffer;
}

/*
 * Get the size of the button when the system
 * automatically sizes it.
 */
void wxChoice::GetAutoSize(int *cx, int *cy)
{
    /*
     * Find the longest string
     */
    int control_width, control_height, i;
    if (no_strings == 0)
	control_width = 100;
    else {
	control_width = 0;
	for (i = 0; i < no_strings; i++) {
	    char *s = GetString(i);
	    int cx = wxGetTextWidth(hwndItem, s);
	    control_width = wxMax(cx, control_width);

	}
	control_width += WinQuerySysValue(HWND_DESKTOP, SV_CXVSCROLL) + FRAME_MARGIN * 2;
    }
    int ccy = wxGetCharHeight(hwndItem);

    /*
     * Choice drop-down list height depends on the number of
     * items up to a maximum of 10.
     */
    if (no_strings == 0)
	control_height = ccy * 10;
    else
	control_height = ccy * wxMin(10, no_strings);
    control_height += FRAME_MARGIN * 2;

    *cx = control_width;
    *cy = control_height;
}




	




