/*
 * Fil:e        wx_dc.cc
 * Purpose:        Device context implementation
 * Author:        Julian Smart
 * Created:        1993
 * Updated:        August 1994
 * RCS_ID:      $Id: wx_dc.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:        (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#ifdef __GNUG__
#pragma implementation
#endif

#include <math.h>
//#include "complex.h"    // CSet++ : for M_PI definition
#include <float.h>      // EMX/GCC : for M_PI definition

#include "wx_panel.h"
#include "wx_privt.h"
#include "wx_dcpan.h"
#include "wx_frame.h"
#include "wx_dc.h"
#include "wx_dccan.h"
#include "wx_dcmem.h"
#include "wx_dcps.h"
#include "wx_utils.h"
#include "wx_canvs.h"
#include "wx_dialg.h"
#include "wx_main.h"

// Declarations local to this file

inline int wxDC::WxToPmX(int x) { return x; }

int wxDC::WxToPmY(int y)
{
    if (canvas) {
	wxCanvasWnd *wnd = (wxCanvasWnd *)canvas->handle;
	return (int)(wnd->win_height / wnd->ymult - y);
    } else if (selected_bitmap)
	return selected_bitmap->GetHeight() - y;
    assert(FALSE);
    return y;				// Don't know how to do it.
}

// #define     wx_round(a)    (int)((a)+.5)

void wxSetupPS(HPS hps);

IMPLEMENT_DYNAMIC_CLASS(wxDC, wxObject)

// Default constructor
wxDC::wxDC(void)
{
    TRACE("wxDC::wxDC");
    __type = wxTYPE_DC;
    filename = NULL;
    selected_bitmap = NULL;
    canvas = NULL;
    cur_ps = NULLHANDLE ;
    cur_bk = 0 ;
    cur_cpen = NULLHANDLE ;
    cur_cbrush = NULLHANDLE ;
    min_x = 0; min_y = 0; max_x = 0; max_y = 0;
    font = NULL;
    logical_origin_x = 0;
    logical_origin_y = 0;
    device_origin_x = 0;
    device_origin_y = 0;
    logical_scale_x = 1.0;
    logical_scale_y = 1.0;
    user_scale_x = 1.0;
    user_scale_y = 1.0;
    system_scale_x = 1.0;
    system_scale_y = 1.0;
    mapping_mode = MM_TEXT;
    title = NULL;
    dont_delete = FALSE;
    hpsDC = NULLHANDLE;
    clipping = FALSE;
    ok = TRUE;
    current_logical_function = -1;
    current_pen = NULL;
    current_brush = NULL;

    TRACE("wxDC::wxDC: current_background_brush set to wxWHITE");
    current_background_brush = wxWHITE_BRUSH;

    current_text_foreground = *wxBLACK;
    current_text_background = *wxWHITE;
    Colour = wxColourDisplay();
}


wxDC::~wxDC(void)
{
    TRACE("wxDC::~wxDC() this=%#x", (int)this);
    if (filename)
	delete[] filename;

    if (hpsDC)
	WinReleasePS(hpsDC);
}

void wxDC::SetClippingRegion(float cx, float cy, float cw, float ch)
{
    TRACE("wxDC::SetClippingRegion(cx=%g, cy=%g, xw=%g, ch=%g)", cx, cy, cw, ch);
    clipping = TRUE;
    clip_x1 = (int)cx;
    clip_y1 = (int)cy;
    clip_x2 = (int)(cx + cw);
    clip_y2 = (int)(cy + ch);

    HPS ps = GetHPS();
    if (ps)
	DoClipping(ps);
}

void wxDC::DoClipping(HPS ps)
{
    TRACE("wxDC::DoClipping(ps=%#x)", (int)ps);

    if (clipping && ps) {
	int x_off = 0;
	int y_off = 0;
	if (canvas) {
	    wxWnd *wnd = (wxWnd *)canvas->handle;
	    wnd->CalcScrolledPosition(0, 0, &x_off, &y_off);
	}
	TRACE("wxDC::DoClipping clip_x1=%d, clip_y1=%d, clip_x2=%d, clip_y2=%d", clip_x1, clip_y1, clip_x2, clip_y2);
	int x1 = clip_x1 + x_off;
	int x2 = WxToPmX(clip_x2 + x_off);
	int y1 = WxToPmY(clip_y1 + y_off);
	int y2 = WxToPmY(clip_y2 + y_off);
	TRACE("wxDC::DoClipping: x1=%d, y1=%d, x2=%d, y2=%d", x1, y1, x2, y2);
	POINTL points[4] = {{x1, y1}, {x2, y1}, {x2, y2}, {x1, y2}};
	GpiBeginPath(ps, 1);
	GpiMove(ps, &points[0]);
	GpiPolyLine(ps, 3L, &points[1]);
	GpiEndPath(ps);
	GpiSetClipPath(ps, 1, SCP_WINDING | SCP_AND);
    }
}

void wxDC::DestroyClippingRegion(void)
{
    HPS ps = GetHPS();
    if (clipping && ps)
	GpiSetClipPath(ps, 0, SCP_RESET);
    clipping = FALSE;
}

Bool wxDC::CanDrawBitmap(void)
{
    return TRUE;
}

Bool wxDC::CanGetTextExtent(void)
{
    HPS ps = GetHPS();
    if (ps) {
	HDC hdc = GpiQueryDevice(ps);
	LONG technology;
	DevQueryCaps(hdc, CAPS_TECHNOLOGY, 1, &technology);
	Bool ok = (technology == CAPS_TECH_RASTER_DISPLAY
		   || technology == CAPS_TECH_RASTER_PRINTER
		   || technology == CAPS_TECH_POSTSCRIPT);
	return ok;
    }
    return FALSE;
}

void wxDC::SetColourMap(wxColourMap *cmap)
{
    HPS ps = GetHPS();
    if (ps) {
	if (!cmap) {
	    // Setting a NULL colourmap is a way of restoring
	    // the original colourmap
	}
	
	if (cmap && cmap->os2_palette) {
	    PULONG pclr;
// XXX
//	HPAL oldPal = ::GpiSelectPalette(ps, cmap->os2_palette);
      
// XXX	    WinRealizePalette(GetHWND(), ps, pclr);
	}
    }
}

void wxDC::Clear(void)
{
    TRACE("wxDC::Clear()");
    HPS ps = GetHPS();
    assert(ps != NULLHANDLE);
    if (ps) {
	RECTL rect;
	if (canvas)
	    WinQueryWindowRect(canvas->GetHWND(), &rect);
	else if (selected_bitmap) {
	    rect.xLeft = rect.yBottom = 0;
	    rect.xRight = selected_bitmap->GetWidth();
	    rect.yTop   = selected_bitmap->GetHeight();
	}
#if 0
	int mode = ::SetMapMode(ps, PU_TEXT);
#endif
//	GpiErase(ps);
	WinFillRect(ps, &rect, current_background_brush->colour);
	TRACE("wxDC::Clear: rect.xLeft=%d, rect.yBottom=%d, rect.xRight=%d, rect.yTop=%d", (int)rect.xLeft, (int)rect.yBottom, (int)rect.xRight, (int)rect.yTop);
    }
}

void wxDC::BeginDrawing(void)
{
    TRACE("wxDC::BeginDrawing(void)");
}

void wxDC::EndDrawing(void)
{
    TRACE("wxDC::EndDrawing(void)");
}

void wxDC::FloodFill(float x, float y, wxColour *col, int style)
{
    int xx = (int)x;
    int yy = (int)y;

    if (current_brush && autoSetting)
        SetBrush(current_brush);
    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x, (int)y, &xx, &yy);
    }

    HPS ps = GetHPS();
    if (ps) {
	POINTL ptl = { WxToPmX(xx), WxToPmY(yy) };
	GpiMove(ps, &ptl);
	GpiFloodFill(ps, style==wxFLOOD_SURFACE ? FF_SURFACE : FF_BOUNDARY, *col);
	CalcBoundingBox((float)WxToPmX((int)x), (float)WxToPmY((int)y));
    }
}

Bool wxDC::GetPixel(float WXUNUSED(x), float WXUNUSED(y), wxColour *WXUNUSED(col))
{
#if 1
    return FALSE ;
#else
    // added by steve 29.12.94 (copied from DrawPoint)
    // returns TRUE for pixels in the color of the current pen
    // and FALSE for all other pixels colors
    // if col is non-NULL return the color of the pixel
    int xx1 = (int)x;
    int yy1 = (int)y;
    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x, (int)y, &xx1, &yy1);
    }

    HPS ps = GetHPS();

    if (ps) {
	// get the color of the pixel
	POINTL ptl = { WxToPmX(xx1), WxToPmY(yy1) };
	LONG lColor = GpiQueryPel(ps, &ptl);

	// get the color of the pen
	COLOR pencolor = 0x00ffffff;
	if (current_pen) {
//	    current_pen->RealizeResource();
	    pencolor = current_pen->GetColour();
	}

	// return the color of the pixel
	if (col)
	    col->Set(GetRValue(pixelcolor),GetGValue(pixelcolor),GetBValue(pixelcolor));
  
	// check, if color of the pixels is the same as the color
	// of the current pen
	return(pixelcolor==pencolor);
    }
    return FALSE;
#endif
}

void wxDC::IntDrawLine(int x1, int y1, int x2, int y2)
{
    TRACE("wxDC::IntDrawLine(x1=%d, y1=%d, x2=%d, y2=%d)", x1, y1, x2, y2);

    if (current_pen && autoSetting)
	SetPen(current_pen);
 
    int xx1 = (int)x1;
    int yy1 = (int)y1;
    int xx2 = (int)x2;
    int yy2 = (int)y2;

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x1, (int)y1, &xx1, &yy1);
	wnd->CalcScrolledPosition((int)x2, (int)y2, &xx2, &yy2);
    }

    HPS ps = GetHPS();
    if (ps) {
	POINTL ptl = { WxToPmX(xx1), WxToPmY(yy1) };
	GpiMove(ps, &ptl);
	ptl.x = WxToPmX(xx2);
	ptl.y = WxToPmY(yy2);
	GpiLine(ps, &ptl);
    }

    CalcBoundingBox((float)WxToPmX(x1), (float)WxToPmY(y1));
    CalcBoundingBox((float)WxToPmX(x2), (float)WxToPmY(y2));
}

void wxDC::CrossHair(int x, int y)
{
    // We suppose that our screen is 2000x2000 max.
    int x1 = x-2000;
    int y1 = y-2000;
    int x2 = x+2000;
    int y2 = y+2000;

    int xx1 = x1 ;
    int yy1 = y1 ;
    int xx2 = x2 ;
    int yy2 = y2 ;
    int xx  = x ;
    int yy  = y ;

    if (current_pen && autoSetting)
        SetPen(current_pen);

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition(x1, y1, &xx1, &yy1);
	wnd->CalcScrolledPosition(x2, y2, &xx2, &yy2);
	wnd->CalcScrolledPosition(x, y, &xx, &yy);
    }

    HPS ps = GetHPS();
    if (ps) {
	POINTL ptl = { WxToPmX(xx1), WxToPmY(yy) };
	GpiMove(ps, &ptl);
	ptl.x = WxToPmX(xx2);
	ptl.y = WxToPmY(yy);
	GpiLine(ps, &ptl);

	ptl.x = WxToPmX(xx);
	ptl.y = WxToPmY(yy1);
	GpiMove(ps, &ptl);
	ptl.x = WxToPmX(xx);
	ptl.y = WxToPmY(yy2);
	GpiLine(ps, &ptl);
    }

    CalcBoundingBox((float)WxToPmX(x1), (float)WxToPmY(y1));
    CalcBoundingBox((float)WxToPmX(x2), (float)WxToPmY(y2));
}

void wxDC::DrawLine(float x1, float y1, float x2, float y2)
{
    TRACE("wxDC::DrawLine(x1=%g, y1=%g, x2=%g, y2=%g)", x1, y1, x2, y2);
    int xx1 = (int)x1;
    int yy1 = (int)y1;
    int xx2 = (int)x2;
    int yy2 = (int)y2;

// BUGBUG - is this necessary? YES YES YES YEs Yes yes ye....
    if (current_pen && autoSetting)
	SetPen(current_pen);

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x1, (int)y1, &xx1, &yy1);
	wnd->CalcScrolledPosition((int)x2, (int)y2, &xx2, &yy2);
    }

    HPS ps = GetHPS();
    if (ps) {
	POINTL ptl = { WxToPmX(xx1), WxToPmY(yy1) };
	GpiMove(ps, &ptl);
	ptl.x = WxToPmX(xx2);
	ptl.y = WxToPmY(yy2);
	GpiLine(ps, &ptl);
    }

    CalcBoundingBox(xx1, yy1);
    CalcBoundingBox(xx2, yy2);
}

void wxDC::DrawArc(float x1, float y1, float x2, float y2, float xc, float yc)
{
    TRACE("wxDC::DrawArc(x1=%g, y1=%g, x2=%g, y2=%g, xc=%g, yc=%g)", x1, y1, x2, y2, xc, yc);
    FIXED stAng, endAng, swAng;

    int xx1 = (int)x1;
    int yy1 = (int)y1;
    int xx2 = (int)x2;
    int yy2 = (int)y2;
    int xxc = (int)xc ;
    int yyc = (int)yc;

    double dx = xc-x1 ;
    double dy = yc-y1 ;
    double radius = (double)sqrt(dx*dx+dy*dy) ;;
    if (xx1==xx2 && xx2==yy2) {
	DrawEllipse(xc,yc,(float)(radius*2.0),(float)(radius*2)) ;
	return ;
    }

// BUGBUG - is this necessary?
    if (current_pen && autoSetting)
	SetPen(current_pen) ;

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x1, (int)y1, &xx1, &yy1);
	wnd->CalcScrolledPosition((int)x2, (int)y2, &xx2, &yy2);
	wnd->CalcScrolledPosition((int)xc, (int)yc, &xxc, &yyc);
    }

    HPS ps = GetHPS();

    if (ps) {
	xx1 = WxToPmX(xx1) ;
	yy1 = WxToPmY(yy1) ;
	xx2 = WxToPmX(xx2) ;
	yy2 = WxToPmY(yy2) ;
	xxc = WxToPmX(xxc) ;
	yyc = WxToPmY(yyc) ;
    /*  
	double ray = sqrt((xxc-xx1)*(xxc-xx1)+(yyc-yy1)*(yyc-yy1)) ;

	(void)MoveToEx(dc, xx1, yy1, NULL);
	int xxx1 = (int)(xxc-ray);
	int yyy1 = (int)(yyc-ray);
	int xxx2 = (int)(xxc+ray);
	int yyy2 = (int)(yyc+ray);
	*/  
	// Do ATAN2 stuff here
	double angle;

	angle = (atan2(yy1-yyc, xx1-xxc)*180.0)/M_PI;
	if (angle < 0) angle += 360;
	stAng = MAKEFIXED((int)angle, 0);

	angle  = (atan2(yy2-yyc, xx2-xxc)*180.0)/M_PI;
	if (angle < 0) angle += 360;
	endAng = MAKEFIXED((int)angle, 0);

	swAng = endAng-stAng;
	if (endAng<stAng) {
	    swAng = MAKEFIXED(360,0) + swAng;
	}

	ARCPARAMS arcp = { (int)radius, (int)radius, 0, 0 };
	POINTL    ptl  = { xxc, yyc };

	GpiSetArcParams(ps, &arcp);
	if (current_brush && current_brush->GetStyle() !=wxTRANSPARENT) {
    // BUGBUG - is this necessary?
	    if (current_brush && current_brush->GetStyle() != wxTRANSPARENT && autoSetting)
		SetBrush(current_brush) ;
    /*      
	    Pie(dc,xxx1,yyy1,xxx2,yyy2,
	    xx1,yy1,xx2,yy2) ;
	    */        
	    GpiBeginPath(ps, 1L);
	    GpiMove(ps, &ptl);
	    GpiPartialArc(ps, &ptl, MAKEFIXED(1, 0), stAng, swAng);
	    GpiLine(ps, &ptl);
	    GpiEndPath(ps);
	    GpiFillPath(ps, 1L, FPATH_ALTERNATE);
	}
	else
	{
    /*  
	Arc(dc,xxx1,yyy1,xxx2,yyy2,
	xx1,yy1,xx2,yy2) ;
	*/        
	    GpiMove(ps, &ptl);
	    GpiPartialArc(ps, &ptl, MAKEFIXED(1, 0), stAng, swAng);
	    GpiLine(ps, &ptl);
	}
    }

    CalcBoundingBox((float)(xc-radius), (float)(yc-radius));
    CalcBoundingBox((float)(xc+radius), (float)(yc+radius));
}

void wxDC::DrawPoint(float x, float y)
{
    TRACE("wxDC::DrawPoint(x=%g, y=%g)", x, y);

// BUGBUG - is this necessary?
    if (current_pen && autoSetting)
	SetPen(current_pen) ;

    int xx1 = (int)x;
    int yy1 = (int)y;

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x, (int)y, &xx1, &yy1);
    }

    HPS ps = GetHPS();
    if (ps) {
	// Plot pixel with current line mix and color
	POINTL ptl = { WxToPmX(xx1), WxToPmY(yy1) };
	GpiSetPel(ps, &ptl);
    }
    CalcBoundingBox(x, y);
}

void wxDC::DrawPolygon(int n, wxPoint points[], float xoffset, float yoffset, int fillStyle)
{
    TRACE("wxDC::DrawPolygon(n=%d, points=..., xoffset=%g, yoffset=%g, fillStyle=%d)", n, xoffset, yoffset, fillStyle);

// BUGBUG - is this necessary?
    if (current_pen && autoSetting)
	SetPen(current_pen) ;

    int xoffset1 = 0;
    int yoffset1 = 0;
    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition(0, 0, &xoffset1, &yoffset1);
    }

    xoffset1 += (int)xoffset; yoffset1 += (int)yoffset;

    HPS ps = GetHPS();
    if (ps) {

	POINTL *cpoints = new POINTL[n];
	int i;
	for (i = 0; i < n; i++) {
	    cpoints[i].x = (int)(WxToPmX((int)(points[i].x + xoffset1)));
	    cpoints[i].y = (int)(WxToPmY((int)(points[i].y + yoffset1)));

	    CalcBoundingBox(points[i].x + xoffset, points[i].y + yoffset);
	}

	GpiBeginPath(ps, 1L);
	GpiMove(ps, &cpoints[0]);
	GpiPolyLine(ps, n-1, &cpoints[1]);
	GpiEndPath(ps);
	GpiFillPath(ps, 1L, (fillStyle==wxODDEVEN_RULE) ? FPATH_ALTERNATE : FPATH_WINDING);
	delete[] cpoints;
    }
}

void wxDC::DrawLines(int n, wxIntPoint points[], int xoffset, int yoffset)
{
    TRACE("wxDC::DrawLines(n=%d, ..., xoffset=%d, yoffset=%d)", n, xoffset, yoffset);
    int xoffset1 = 0;
    int yoffset1 = 0;
 
    if (current_pen && autoSetting)
	SetPen(current_pen) ;
 
    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition(0, 0, &xoffset1, &yoffset1);
    }

    xoffset1 += (int)xoffset;
    yoffset1 += (int)yoffset;
 
    HPS ps = GetHPS();
    if (ps) {

	POINTL *cpoints = new POINTL[n];
	int i;
	for (i = 0; i < n; i++) {
	    int x = points[i].x + xoffset1;
	    int y = points[i].y + yoffset1;
	    cpoints[i].x = WxToPmX(x);
	    cpoints[i].y = WxToPmY(y);
	    CalcBoundingBox((float)x, (float)y);
	}

	(void)GpiMove(ps, &cpoints[0]);
	(void)GpiPolyLine(ps, n-1, &cpoints[1]);
	delete[] cpoints;
    }
}

void wxDC::DrawLines(int n, wxPoint points[], float xoffset, float yoffset)
{
    TRACE("wxDC::DrawLines(n=%d, ..., xoffset=%g, yoffset=%g)", n, xoffset, yoffset);
    int xoffset1 = 0;
    int yoffset1 = 0;

// BUGBUG - is this necessary?
    if (current_pen && autoSetting)
	SetPen(current_pen) ;

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition(0, 0, &xoffset1, &yoffset1);
    }

    xoffset1 += (int)xoffset; yoffset1 += (int)yoffset;

    HPS ps = GetHPS();
    if (ps) {
	POINTL *cpoints = new POINTL[n];
	int i;
	for (i = 0; i < n; i++) {
	    int x = (int)points[i].x + (int)xoffset1;
	    int y = (int)points[i].y + (int)yoffset1;
	    cpoints[i].x = WxToPmX(x);
	    cpoints[i].y = WxToPmY(y);
	    CalcBoundingBox((float)x, (float)y);
	}

	(void)GpiMove(ps, &cpoints[0]);
	(void)GpiPolyLine(ps, n-1, &cpoints[1]);
	delete[] cpoints;
    }
}

void wxDC::DrawRectangle(float x, float y, float width, float height)
{
    TRACE("wxDC::DrawRectangle(x=%g, y=%g, width=%g, height=%g)", x, y, width, height);

    int x1 = (int)x;
    int y1 = (int)y;
    int x2 = (int)(x+width);
    int y2 = (int)(y+height);

// BUGBUG - is this necessary?
    if (current_pen && autoSetting)
	SetPen(current_pen) ;

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x, (int)y, &x1, &y1);
	wnd->CalcScrolledPosition((int)(x+width), (int)(y+height), &x2, &y2);
    }

    HPS ps = GetHPS();
    if (ps) {

	POINTL ptl = { WxToPmX(x1), WxToPmY(y1) };
	GpiMove(ps, &ptl);
	ptl.x = WxToPmX(x2);
	ptl.y = WxToPmY(y2);
	GpiBox(ps, ((current_brush && current_brush->GetStyle() == wxSOLID) ?
		    DRO_OUTLINEFILL : DRO_OUTLINE), &ptl, 0, 0);

	CalcBoundingBox((float)x1, (float)y1);
	CalcBoundingBox((float)x2, (float)y2);
    }
}

void wxDC::DrawRoundedRectangle(float x, float y, float width, float height, float radius)
{
    TRACE("wxDC::DrawRoundedRectangle(x=%g, y=%g, width=%g, height=%g, radius=%g)", x, y, width, height, radius);
    // Now, a negative radius value is interpreted to mean
    // 'the proportion of the smallest X or Y dimension'

    if (radius < 0.0) {
	float smallest = 0.0;
	if (width < height)
	    smallest = width;
	else
	    smallest = height;
	radius = (float)(- radius * smallest);
    }

    int x1 = (int)x;
    int y1 = (int)y;
    int x2 = (int)(x+width);
    int y2 = (int)(y+height);

// BUGBUG - is this necessary?
    if (current_pen && autoSetting)
	SetPen(current_pen) ;

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x, (int)y, &x1, &y1);
	wnd->CalcScrolledPosition((int)(x+width), (int)(y+height), &x2, &y2);
    }

    HPS ps = GetHPS();
    if (ps) {
	POINTL ptl = { WxToPmX(x1), WxToPmY(y1) };
	GpiMove(ps, &ptl);
	ptl.x = WxToPmX(x2);
	ptl.y = WxToPmY(y2);
	GpiBox(ps, ((current_brush && current_brush->GetStyle() == wxSOLID) ?
		    DRO_OUTLINEFILL : DRO_OUTLINE), &ptl, 
	       (int)radius, (int)radius);

	CalcBoundingBox((float)x1, (float)y1);
	CalcBoundingBox((float)x2, (float)y2);
    }
}

void wxDC::DrawEllipse(float x, float y, float width, float height)
{
    TRACE("wxDC::DrawEllipse(x=%g, y=%g, width=%g, height=%g)", x, y, width, height);

    int x1 = (int)x;
    int y1 = (int)y;
    int x2 = (int)(x+width);
    int y2 = (int)(y+height);

// BUGBUG - is this necessary?
    if (current_pen && autoSetting)
	SetPen(current_pen) ;

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x, (int)y, &x1, &y1);
	wnd->CalcScrolledPosition((int)(x+width), (int)(y+height), &x2, &y2);
    }

    HPS ps = GetHPS();
    if (ps) {
	// Set arc parameters
	int xc = (int)(x+width/2.0);
	int yc = (int)(y+height/2.0);
	ARCPARAMS arcp = { (int)(width/2.0), (int)(height/2.0), 0, 0 };
	POINTL    ptl  = { WxToPmX(xc), WxToPmY(yc) };
	GpiSetArcParams(ps, &arcp);
	GpiMove(ps, &ptl);
	GpiFullArc(ps, ((current_brush && current_brush->GetStyle() == wxSOLID) ?
			DRO_OUTLINEFILL : DRO_OUTLINE), MAKEFIXED(1,0));
    }
    CalcBoundingBox((float)x1, (float)y1);
    CalcBoundingBox((float)x2, (float)y2);
}

void wxDC::DrawIcon(wxIcon *icon, float x, float y)
{
    TRACE("wxDC::DrawIcon(icon=%#x, x=%g, y=%g)", (int)icon, x, y);

    int x1 = (int)x;
    int y1 = (int)y;

    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	wnd->CalcScrolledPosition((int)x, (int)y, &x1, &y1);
    }

    HPS ps = GetHPS();
    if (ps) {
	WinDrawPointer(ps, WxToPmX(x1), WxToPmY(y1), icon->os2_icon, DP_NORMAL);
	CalcBoundingBox((float)x1, (float)y1);
    }
}

void wxDC::SetFont(wxFont *the_font)
{
    TRACE("wxDC::SetFont(the_font=%#x)", (int)the_font);
    // Release the current font from servitude (decrements the usage count)

    TRACE("wxDC::SetFont(font=%#x)", font);
    TRACE("wxDC::SetFont: GetFamilyString() => '%s'", the_font->GetFamilyString());
    TRACE("wxDC::SetFont: GetFaceName() => '%s'", the_font->GetFaceName());
    TRACE("wxDC::SetFont: GetStyleString() => '%s'", the_font->GetStyleString());
    TRACE("wxDC::SetFont: GetWeightString() => '%s'", the_font->GetWeightString());

    HPS ps = GetHPS();
    if (ps) {
	font = the_font;
	    
	if (font) {
//	    font->RealizeResource();
// The following line is deleted in wx165
	    font->BuildInternalFont(ps);
	}
	    
	if (font && font->cfont) {
	    TRACE("wxDC::SetFont: Selecting HFONT %#x", font->cfont);
	    if (canvas) {
		wxCanvasWnd *wnd = (wxCanvasWnd *)canvas->handle;
		SIZEF sizef;
		sizef.cx = (long)(((float)font->GetPointSize() * 65536.0) * wnd->fxmult);
		sizef.cy = (long)(((float)font->GetPointSize() * 65536.0) * wnd->fymult);
		GpiSetCharBox(ps, &sizef);
	    }
	    GpiSetCharSet(ps, font->cfont);
	}
    }
}

void wxDC::SetPen(wxPen *pen)
{
    current_pen = pen;

    if (pen) {
	pen->ChangePen();
	HPS hps = GetHPS();
	if (hps)
	    pen->SelectPen(hps);
    } // else should set default pen
}

void wxDC::SetBrush(wxBrush *brush)
{
    current_brush = brush;

    if (brush) {
	brush->ChangeBrush();
	HPS ps = GetHPS();
	if (ps)
	    brush->SelectBrush(ps);
    } // else should set default pen
}

void wxDC::DrawText(const char *text, float x, float y, Bool use16bit)
{
    TRACE("wxDC::DrawText(text='%s', x=%g, y=%g)", text, x, y);

    int xx1 = (int)x;
    int yy1 = (int)y;

    wxCanvasWnd *cwnd = NULL;
    if (canvas) {
	cwnd = (wxCanvasWnd *)canvas->handle;
	cwnd->CalcScrolledPosition((int)x, (int)y, &xx1, &yy1);
    }

    HPS ps = GetHPS();
    if (ps) {
	if (font && font->cfont) {
	    LONG f = GpiQueryCharSet(ps);
	    float pntsize = (float)font->GetPointSize();
	    if (cwnd) {
		SIZEF sizef;
		sizef.cx = (long)(pntsize * 65536.0 * cwnd->fxmult);
		sizef.cy = (long)(pntsize * 65536.0 * cwnd->fymult);
		GpiSetCharBox(ps, &sizef);
	    }
	    GpiSetCharSet(ps, font->cfont);
	    if (cwnd)
		yy1 += (int)(font->cyChar * cwnd->fymult);
	}

	if (current_text_foreground.Ok())
	    GpiSetColor(ps, current_text_foreground);

	DWORD old_background;
	if (current_text_background.Ok()) {
	    old_background = GpiSetBackColor(ps, current_text_background) ;
	}
	LONG curr_mode = GpiQueryBackMix(ps);
	if (current_bk_mode == wxTRANSPARENT)
	    GpiSetBackMix(ps, BM_LEAVEALONE);
	else
	    GpiSetBackMix(ps, BM_OVERPAINT);

	POINTL ptl = { WxToPmX(xx1), WxToPmY(yy1) };
	GpiCharStringAt(ps, &ptl, strlen(text), (PCH)text);

	if (current_text_background.Ok())
	    (void)GpiSetBackColor(ps, old_background);

	// Restore background mix
	GpiSetBackMix(ps, curr_mode);
    }
    CalcBoundingBox((float)xx1, (float)yy1);
    float w, h;
    GetTextExtent(text, &w, &h);
    CalcBoundingBox((float)(xx1 + w), (float)(yy1 + h));
}

void wxDC::SetBackground(wxBrush *brush)
{
    TRACE("wxDC::SetBackground(brush=%#x)", (int)brush);
    current_background_brush = brush;
    if (!brush)
	return;
    brush->ChangeBrush() ;

    if (!canvas)
	return;
    wxWnd *wnd = (wxWnd *)canvas->handle;

    //wnd->background_brush = brush->cbrush;
/*    
      HBRUSH br = (brush->GetStyle()==wxTRANSPARENT) ?
      GetStockObject(NULL_BRUSH) : brush->cbrush;
      */                            
    wxBrush *br = (brush->GetStyle()==wxTRANSPARENT) ? (wxBrush *)NULL : brush;

    // Remember we don't want to delete this brush when we change the
    // background again or delete the window.
    wnd->SetBackgroundColour(br->colour);
//    wnd->SetBackgroundBrush(br, FALSE);
    if (brush->GetStyle()==wxTRANSPARENT)
	wnd->background_transparent = TRUE;
    else {
	wnd->background_transparent = FALSE;
    }

    HPS ps = GetHPS();
    if (ps) {
	if (current_background_brush) {
	    LONG new_color = current_background_brush->colour;
	    if (new_color != cur_bk || ps != cur_ps) {
		(void)GpiSetBackColor(ps, new_color);
		cur_bk = new_color ;
		cur_ps = ps ;
	    }
	}
    }
}

void wxDC::SetBackgroundMode(int mode)
{
    current_bk_mode = mode;
    HPS ps = GetHPS();
    if (ps) {
	if (current_bk_mode == wxTRANSPARENT)
	    GpiSetBackMix(ps, BM_LEAVEALONE);
	else
	    GpiSetBackMix(ps, BM_OVERPAINT);
    }
}

void wxDC::SetLogicalFunction(int function)
{
    current_logical_function = function;

    HPS ps = GetHPS();
    if (ps)
	SetRop(ps);
}

void wxDC::SetRop(HPS ps)
{
    if (!ps || current_logical_function < 0)
	return;

    int c_rop;
    // These may be wrong
    switch (current_logical_function) {
//    case wxXOR: c_rop = R2_XORPEN; break;
    case wxXOR: c_rop = FM_XOR; break;
    case wxINVERT: c_rop = FM_INVERT; break;
    case wxOR_REVERSE: c_rop = FM_MERGESRCNOT; break;
    case wxAND_REVERSE: c_rop = FM_MASKSRCNOT; break;
    case wxCLEAR: c_rop = FM_ONE; break;
    case wxSET: c_rop = FM_ZERO; break;
    case wxSRC_INVERT: c_rop = FM_NOTCOPYSRC; break;
    case wxOR_INVERT: c_rop = FM_MERGENOTSRC; break;
    case wxAND: c_rop = FM_AND; break;
    case wxOR: c_rop = FM_OR; break;
    case wxAND_INVERT: c_rop = FM_NOTMASKSRC; break;
    case wxEQUIV:
    case wxNAND:
    case wxCOPY:
    default:
	c_rop = FM_OVERPAINT; break;
    }
    GpiSetMix(ps, c_rop);
}

Bool wxDC::StartDoc(char *WXUNUSED(message))
{
    if (!wxSubType(__type, wxTYPE_DC_PRINTER))
	return TRUE;
    
    Bool flag = FALSE;

#if 0
    DOCINFO docinfo;
    docinfo.cbSize = sizeof(DOCINFO);
    docinfo.lpszDocName = message;
    docinfo.lpszOutput = filename;
    if (hpsDC) flag = (SP_ERROR != ::StartDoc(hpsDC, &docinfo));
    else flag = FALSE;
#endif
    return flag;
}

void wxDC::EndDoc(void)
{
#if 0
    if (!wxSubType(__type, wxTYPE_DC_PRINTER))
	return;
    if (hpsDC) ::EndDoc(hpsDC);
#endif  
}

void wxDC::StartPage(void)
{
#if 0
    if (!wxSubType(__type, wxTYPE_DC_PRINTER))
	return;
    if (hpsDC)
	::StartPage(hpsDC);
#endif    
}

void wxDC::EndPage(void)
{
#if 0
    if (!wxSubType(__type, wxTYPE_DC_PRINTER))
	return;
    if (hpsDC)
	::EndPage(hpsDC);
#endif    
}

float wxDC::GetCharHeight(void)
{
    FONTMETRICS lpFontMetrics;
    HPS ps = GetHPS();
    if (ps) {
	GpiQueryFontMetrics(ps, sizeof(FONTMETRICS), &lpFontMetrics);
	return (float)(lpFontMetrics.lMaxAscender + lpFontMetrics.lMaxDescender);
    }
    return 0.0;
}

float wxDC::GetCharWidth(void)
{
    FONTMETRICS lpFontMetrics;
    HPS ps = GetHPS();
    if (ps) {
	GpiQueryFontMetrics(ps, sizeof(FONTMETRICS), &lpFontMetrics);
	return (float)lpFontMetrics.lAveCharWidth;
    }
    return 0.0;
}

void wxDC::GetTextExtent(const char *string, float *x, float *y,
                         float *descent, float *externalLeading, wxFont *theFont, Bool use16bit)
{
    wxFont *fontToUse = theFont;
    if (!fontToUse)
	fontToUse = font;

    HPS ps = GetHPS();
    if (ps) {

    //  POINTL startPt = { 0, 0 };
	POINTL sizePt[TXTBOX_COUNT];
	FONTMETRICS fm;

	GpiQueryTextBox(ps, strlen(string), (PCH)string, TXTBOX_COUNT, sizePt);
	GpiQueryFontMetrics(ps, sizeof(FONTMETRICS), &fm);

	*x = (float)sizePt[TXTBOX_TOPRIGHT].x;
	*y = (float)(sizePt[TXTBOX_TOPRIGHT].y - sizePt[TXTBOX_BOTTOMRIGHT].y);
	if (descent)
	    *descent = (float)fm.lMaxDescender;
	if (externalLeading)
	    *externalLeading = (float)fm.lExternalLeading;
    }
}

void wxDC::SetMapMode(int mode)
{
    if (mapping_mode != mode) {
	mapping_mode = mode;
	if (hpsDC)
	    SetupPS(hpsDC);
    }
}

void wxDC::SetUserScale(float x, float y)
{
    if (user_scale_x != x || user_scale_y != y) {
	user_scale_x = x;
	user_scale_y = y;
	if (hpsDC)
	    SetupPS(hpsDC);
    }
}

void wxDC::SetSystemScale(float x, float y)
{
    if (system_scale_x != x || system_scale_y != y) {
	system_scale_x = x;
	system_scale_y = y;
	if (hpsDC)
	    SetupPS(hpsDC);
    }
}

void wxDC::SetLogicalOrigin(float x, float y)
{
    if (logical_origin_x != x || logical_origin_y != y) {
	logical_origin_x = x;
	logical_origin_y = y;
	if (hpsDC)
	    SetupPS(hpsDC);
    }
}

void wxDC::SetDeviceOrigin(float x, float y)
{
    if (device_origin_x != x || device_origin_y != y) {
	device_origin_x = x;
	device_origin_y = y;
	if (hpsDC)
	    SetupPS(hpsDC);
    }
}

float wxDC::DeviceToLogicalX(int x)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnt = {x, 0};
	GpiConvert(ps, CVTC_DEVICE, CVTC_WORLD, 1L, &pnt);
	TRACE("wxDC::DeviceToLogical(x=%d) => %d", x, pnt.x);
	return (float)pnt.x;
    }
    return 0.0;
}

float wxDC::DeviceToLogicalXRel(int x)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnts[2] = {{0, 0}, {x, 0}};
	GpiConvert(ps, CVTC_DEVICE, CVTC_WORLD, 2L, pnts);
	return (float)(pnts[1].x - pnts[0].x);
    }
    return 0.0;
}

float wxDC::DeviceToLogicalY(int y)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnt = {0, y};
	GpiConvert(ps, CVTC_DEVICE, CVTC_WORLD, 1L, &pnt);
	TRACE("wxDC::DeviceToLogical(y=%d) => %d", y, pnt.y);
	return (float)pnt.y;
    }
    return 0.0;
}

#if XXX
void wxDC::DeviceToLogicalXY(int x, int y, int *xx, int *yy)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnt = {x, y};
	GpiConvert(ps, CVTC_DEVICE, CVTC_WORLD, 1L, &pnt);
	*xx = pnt.x;
	*yy = pnt.y;
    }
}
#endif


float wxDC::DeviceToLogicalYRel(int y)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnts[2] = {{0, 0}, {0, y}};
	GpiConvert(ps, CVTC_DEVICE, CVTC_WORLD, 2L, pnts);
	return (float)(pnts[1].y - pnts[0].y);
    }
    return 0.0;
}

int wxDC::LogicalToDeviceX(float x)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnt = {(long)x, 0};
	GpiConvert(ps, CVTC_WORLD, CVTC_DEVICE, 1L, &pnt);
	return pnt.x;
    }
    return 0;
}

int wxDC::LogicalToDeviceXRel(float x)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnts[2] = {{0, 0}, {(long)x, 0}};
	GpiConvert(ps, CVTC_WORLD, CVTC_DEVICE, 2L, pnts);
	return pnts[1].x - pnts[0].x;
    }
    return 0;
}

int wxDC::LogicalToDeviceY(float y)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnt = {0, (long)y};
	GpiConvert(ps, CVTC_WORLD, CVTC_DEVICE, 1L, &pnt);
	return pnt.y;
    }
    return 0;
}

int wxDC::LogicalToDeviceYRel(float y)
{
    HPS ps = GetHPS();
    if (ps) {
	POINTL pnts[2] = {{0, 0}, {0, (long)y}};
	GpiConvert(ps, CVTC_WORLD, CVTC_DEVICE, 2L, pnts);
	return pnts[1].y - pnts[0].y;
    }
    return 0;
}

Bool wxDC::Blit(float xdest, float ydest, float width, float height,
                wxCanvasDC *source, float xsrc, float ysrc, int rop)
{
    TRACE("wxDC::Blit(xdest=%g, ydest=%g, width=%g, height=%g, source=%#x, xsrc=%g, ysrc=%g, rop=%d)", xdest, ydest, width, height, source, xsrc, ysrc, rop);

    wxCanvasWnd *wnd = canvas ? wnd = (wxCanvasWnd *)canvas->handle : (wxCanvasWnd *)NULL;
    wxCanvasWnd *wnd_src = source->canvas ? (wxCanvasWnd *)source->canvas->handle : (wxCanvasWnd *)NULL;

    HPS ps = GetHPS();
    assert(ps != NULLHANDLE);
    TRACE("ps=%#x", ps);
    if (!ps)
	return FALSE;

    HPS ps_src = source->GetHPS();
    assert(ps_src != NULLHANDLE);
    TRACE("ps_src=%#x", ps_src);
    if (!ps_src)
	return FALSE;

    int xdest1 = (int)xdest;
    int ydest1 = (int)ydest;
    int xsrc1 = (int)xsrc;
    int ysrc1 = (int)ysrc;

    if (wnd)
	wnd->CalcScrolledPosition((int)xdest, (int)ydest, &xdest1, &ydest1);
    if (wnd_src)
	wnd_src->CalcScrolledPosition((int)xsrc, (int)ysrc, &xsrc1, &ysrc1);

    POINTL ptls[] = {{xdest1,ydest1},
		     {xdest1+(int)width,ydest1+(int)height},
		     {xsrc1,ysrc1}};
#if 1
    Bool success = GpiBitBlt(ps, ps_src, 3L, ptls, ROP_NOTSRCCOPY, BBO_IGNORE);
#else
    Bool success = GpiBitBlt(ps, ps_src, 3L, ptls, 
			     rop == wxCOPY ?	ROP_SRCCOPY :
			     rop == wxCLEAR ? 	ROP_ONE :
			     rop == wxSET ? 	ROP_ZERO :
			     rop == wxINVERT ? 	ROP_DSTINVERT :
			     rop == wxAND ? 	ROP_MERGECOPY :
			     rop == wxOR ? 	ROP_MERGEPAINT :
			     rop == wxSRC_INVERT ? ROP_NOTSRCCOPY :
			     rop == wxXOR ? 	ROP_SRCINVERT :
			     rop == wxOR_REVERSE ? ROP_MERGEPAINT :
			     rop == wxAND_REVERSE ? ROP_SRCERASE :
			                        ROP_SRCCOPY,
			     BBO_IGNORE);
#endif
    TRACE("calling GpiBitBlt(ps=%#x, ps_src=%#x, 3L, ..., ...) success=%d", ps, ps_src, success);
    return success != 0;
}

void wxDC::GetSize(float *width, float *height)
{
    HPS ps = GetHPS();
    if (ps) {
	LONG w, h;
	HDC hdc = GpiQueryDevice(ps);
	DevQueryCaps(hdc, CAPS_WIDTH, 1L, &w);
	DevQueryCaps(hdc, CAPS_HEIGHT, 1L, &h);
	*width = (float)w;
	*height = (float)h;
    }
    TRACE("wxDC::GetSize() returns width=%d, height=%d", (int)*width, (int)*height);
}

void wxDC::GetSizeMM(float *width, float *height)
{
    HPS ps = GetHPS();
    if (ps) {
	LONG	xDeviceRes, yDeviceRes;
	HDC hdc = GpiQueryDevice(ps);
	LONG w, h;
	DevQueryCaps(hdc, CAPS_WIDTH, 1L, &w);		// Width in PELS
	DevQueryCaps(hdc, CAPS_HEIGHT, 1L, &h);		// Height in PELS
	DevQueryCaps(hdc, CAPS_HORIZONTAL_RESOLUTION, 1L, &xDeviceRes);	// PELS per meter
	DevQueryCaps(hdc, CAPS_VERTICAL_RESOLUTION,   1L, &yDeviceRes);
	*width = (float)w/(float)xDeviceRes*1000.0;
	*height = (float)h/(float)yDeviceRes*1000.0;	
    }
    TRACE("wxDC::GetSizeMM() returns width=%d, height=%d", (int)*width, (int)*height);
}

/*
 * Shortcut to get the current Presentation Space.
 * Returns NULL if we have none (is it really supposed to be possible?)
 */
HPS
wxDC::GetHPS(void)
{
    if (hpsDC)
	return hpsDC;
    assert(0);				// Should not happen -- I think.
    return NULLHANDLE;
}

void
wxDC::FreeHPS(void)
{
}

IMPLEMENT_DYNAMIC_CLASS(wxCanvasDC, wxDC)

wxCanvasDC::wxCanvasDC(void)
{
    TRACE("wxCanvasDC::wxCanvasDC()");
    __type = wxTYPE_DC_CANVAS;
    device = wxDEVICE_WINDOWS;
}

wxCanvasDC::wxCanvasDC(wxCanvas *the_canvas) : wxbCanvasDC(the_canvas)
{
    TRACE("wxCanvasDC::wxCanvasDC(the_canvas=%#x)", the_canvas);
    __type = wxTYPE_DC_CANVAS;
    canvas = the_canvas;
    device = wxDEVICE_WINDOWS;

    /*
     * Create a permanent PS for this canvas.
     * This is to simplify and speed up code
     * since it is too common to draw outside
     * of a Onpaint() or BeginDrawing()
     */
    if (!canvas)
	return;
    wxCanvasWnd *wnd = (wxCanvasWnd *)canvas->handle;
    assert(wnd->__type == wxTYPE_CANVAS_WND);

    if (wnd) {
	HDC hdc = WinOpenWindowDC(wnd->hwndWnd);
	SIZEL sizlWork;
	sizlWork.cx = WinQuerySysValue(HWND_DESKTOP, SV_CXSCREEN);
	sizlWork.cy = WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN);
	hpsDC = GpiCreatePS(wxhAb, hdc, &sizlWork, GPIT_NORMAL | GPIA_ASSOC | PU_PELS);
	wnd->hpsCanvas = hpsDC;
	TRACE("wxCanvasDC::wxCanvasDC: this=%#x, hpsDC=%#x", this, hpsDC);

	/*
	 * Increase the page size by 1000, to get higher resolution
	 * when zooming.,
	 */
	SIZEL sizlPage = {0, 0};
	GpiSetPS(hpsDC, &sizlPage, PU_PELS);
	GpiQueryPS(hpsDC, &sizlPage);
	sizlPage.cx *= 1000;
	sizlPage.cy *= 1000;
	GpiSetPS(hpsDC, &sizlPage, PU_PELS);
	GpiQueryPS(hpsDC, &sizlPage);
    }
    SetMapMode(MM_TEXT);
    wxSetupPS(hpsDC);

    SetBrush(wxWHITE_BRUSH);
    SetPen(wxBLACK_PEN);
}

wxCanvasDC::~wxCanvasDC(void)
{
    TRACE("wxCanvasDC::~wxCanvasDC() this=%#x wxDC=%#x", (int)this, (wxDC *)this);
    if (hpsDC)
	GpiDestroyPS(hpsDC);
    hpsDC = NULLHANDLE;
}

void wxCanvasDC::GetClippingBox(float *x, float *y, float *w, float *h)
{
    if (clipping) {
	*x = (float)clip_x1 ;
	*y = (float)clip_y1 ;
	*w = (float)(clip_x2 - clip_x1) ;
	*h = (float)(clip_y2 - clip_y1) ;
    } else
	*x = *y = *w = *h = 0 ;
    TRACE("wxCanvasDC::GetClippingBox(x=%g, y=%g, w=%g, h=%g)", x, y, w, h);
}

/*
 * Shortcut to get the current Presentation Space.
 * Returns NULL if we have none (is it really supposed to be possible?)
 */
HPS
wxCanvasDC::GetHPS(void)
{
    if (canvas) {
	wxWnd *wnd = (wxWnd *)canvas->handle;
	if (wnd->hpsUpdate)
	    return wnd->hpsUpdate;
	if (hpsDC)
	    return hpsDC;
    } 
    if (hpsDC)
	return hpsDC;
    assert(0);				// Should not happen -- I think.
    return NULLHANDLE;
}

void
wxCanvasDC::FreeHPS(void)
{
}

IMPLEMENT_CLASS(wxPrinterDC, wxDC)

wxPrinterDC::wxPrinterDC(char *driver_name, char *device_name, char *file, Bool interactive)
{
    TRACE("wxPrinterDC::wxPrinterDC(driver_name='%s', device_name='%s', file='%s', interactive=%d)", driver_name, device_name, file, interactive);
    __type = wxTYPE_DC_PRINTER;
    wx_interactive = interactive;
    device = wxDEVICE_WINDOWS;
    if (file)
	filename = copystring(file);
    else filename = NULL;

#if 0
#if USE_COMMON_DIALOGS
    if (interactive) {
	PRINTDLG pd;
        
	pd.lStructSize = sizeof( PRINTDLG );
	pd.hwndOwner=NULL;
	pd.hDevMode=(HANDLE)NULL;
	pd.hDevNames=(HANDLE)NULL;
	pd.Flags=PD_RETURNDC | PD_NOSELECTION | PD_NOPAGENUMS;
	pd.nFromPage=0;
	pd.nToPage=0;
	pd.nMinPage=0;
	pd.nMaxPage=0;
	pd.nCopies=1;
	pd.hInstance=(HINSTANCE)NULL;

	if ( PrintDlg( &pd ) != 0 ) {
	    hpsDC = pd.HPS;
	    ok = TRUE;
	} else {
	    ok = FALSE;
	    return;
	}

	dont_delete = TRUE; // ??? WHY???
    } else
#endif
	if (driver_name && device_name && file) {
	    hpsDC = CreateDC(driver_name, device_name, file, NULL);
	    ok = hpsDC ? TRUE: FALSE;
	} else {
	    hpsDC = wxGetPrinterDC();
	    ok = hpsDC ? TRUE: FALSE;
	}

    if (hpsDC) {
//     int width = GetDeviceCaps(hpsDC, VERTRES);
//     int height = GetDeviceCaps(hpsDC, HORZRES);
	SetMapMode(MM_TEXT);
    }
    SetBrush(wxBLACK_BRUSH);
    SetPen(wxBLACK_PEN);
#endif
}

wxPrinterDC::wxPrinterDC(HPS theDC)
{
    TRACE("wxPrinterDC::wxPrinterDC(theDC=%#x)", theDC);
    __type = wxTYPE_DC_PRINTER;
    wx_interactive = FALSE;
    device = wxDEVICE_WINDOWS;

    filename = NULL;
 
    hpsDC = theDC;
    ok = TRUE;
    if (hpsDC) {
//     int width = GetDeviceCaps(hpsDC, VERTRES);
//     int height = GetDeviceCaps(hpsDC, HORZRES);
	SetMapMode(MM_TEXT);
    }
    SetBrush(wxBLACK_BRUSH);
    SetPen(wxBLACK_PEN);
}

wxPrinterDC::~wxPrinterDC(void)
{
    TRACE("wxPrinterDC::~wxPrinterDC() this=%#x wxDC=%#x", this, (wxDC *)this);
}

#if 1
void
wxDC::SetupPS(HPS hps)
{
    TRACE("wxDC::SetupPS(hps=%#x)", (int)hps);

    if (canvas) {
	float xmult, ymult;					// Multipliers pixels -> units
	float fxmult, fymult;
	HDC hdc = GpiQueryDevice(hps);
	LONG	xDeviceRes, yDeviceRes, xFontRes, yFontRes;
	DevQueryCaps(hdc, CAPS_HORIZONTAL_RESOLUTION, 1L, &xDeviceRes); // x PELS per meter
	DevQueryCaps(hdc, CAPS_VERTICAL_RESOLUTION,   1L, &yDeviceRes); // y PELS per meter
	DevQueryCaps(hdc, CAPS_HORIZONTAL_FONT_RES, 1L, &xFontRes); // x PELS per inch
	DevQueryCaps(hdc, CAPS_VERTICAL_FONT_RES,   1L, &yFontRes); // y PELS per inch
	xFontRes = (xFontRes * 10000)/254;	// Font resolution PELS per meter
	yFontRes = (yFontRes * 10000)/254;

	switch (mapping_mode) {
	default:
	case MM_ISOTROPIC:
	case MM_ANISOTROPIC:
	case MM_TEXT:
	    xmult = fxmult = 1.0;
	    ymult = fymult = 1.0;
	    break;
	  
	case MM_METRIC:
	    // 1mm units = 1000 mm/meter
	    xmult = (float)xDeviceRes/1000.0;
	    ymult = (float)yDeviceRes/1000.0;
	    fxmult = (float)xFontRes/1000.0;
	    fymult = (float)yFontRes/1000.0;
	    break;

	    case MM_LOMETRIC:
		// .1 mm units = 10000 mm/meter
		xmult = (float)xDeviceRes/10000.0;
		ymult = (float)yDeviceRes/10000.0;
		fxmult = (float)xFontRes/10000.0;
		fymult = (float)yFontRes/10000.0;
		break;

	    case MM_HIMETRIC:
		// .01 mm units = 100000 mm/meter
		xmult = (float)xDeviceRes/100000.0;
		ymult = (float)yDeviceRes/100000.0;
		fxmult = (float)xFontRes/100000.0;
		fymult = (float)yFontRes/100000.0;
		break;

	    case MM_TWIPS:
		// 1/1440 of an inch
		xmult = (float)xDeviceRes/(1000.0/25.4*1440.0);
		ymult = (float)yDeviceRes/(1000.0/25.4*1440.0);
		fxmult = (float)xFontRes/(1000.0/25.4*1440.0);
		fymult = (float)yFontRes/(1000.0/25.4*1440.0);
		break;

	    case MM_POINTS:
		// 1/72 of an inch
		xmult = (float)xDeviceRes/(1000.0/25.4*72.0);
		ymult = (float)yDeviceRes/(1000.0/25.4*72.0);
		fxmult = (float)xFontRes/(1000.0/25.4*72.0);
		fymult = (float)yFontRes/(1000.0/25.4*72.0);
		break;
	    }

	SIZEL sizel;
	GpiQueryPS(hpsDC, &sizel);

	/*
	 * Set up the viewport including all the scaling
	 * and also scrolling.
	 */
	RECTL rectl;
	int x0, y0;

	wxCanvasWnd *wnd = (wxCanvasWnd *)canvas->handle;
	wnd->CalcScrolledPosition(0, 0, &x0, &y0);
	wnd->xmult = xmult;
	wnd->ymult = ymult;
	wnd->fxmult = fxmult;
	wnd->fymult = fymult;
	
	rectl.xLeft = (int)((xmult * logical_scale_x * system_scale_x * user_scale_x)
			    * (x0 + device_origin_x) + logical_origin_x);
	rectl.xRight = (int)((xmult * logical_scale_x * system_scale_x * user_scale_x)
			     * (sizel.cx + x0 + device_origin_x) + logical_origin_x);
	
	rectl.yBottom = (int)((ymult * logical_scale_y * system_scale_y * user_scale_y)
			      * (y0 + device_origin_y) + logical_origin_y);
	rectl.yTop = (int)((ymult * logical_scale_y * system_scale_y * user_scale_y)
			   * (sizel.cy + y0 + device_origin_y) + logical_origin_y);
	GpiSetPageViewport(hps, &rectl);
    }

    if (current_bk_mode == wxTRANSPARENT)
	GpiSetBackMix(hps, BM_LEAVEALONE);
    else
	GpiSetBackMix(hps, BM_OVERPAINT);
#if XXX
    Colour ;
    int current_logical_function ;
    wxPen current_pen ;
    wxBrush current_brush ;
    wxBrush current_background_brush ;
    wxColour current_text_foreground ;
    wxColour current_text_background ;
    font ;
#endif
}

#else
void
wxDC::SetupPS(HPS hps)
{
    TRACE("wxDC::SetupPS(hps=%#x)", (int)hps);
    SIZEL sizlPage = {0, 0};
    int unit, multip;

    /*
     * First set up mapping/scaling
     */
    switch (mapping_mode) {
    default:
    case MM_ISOTROPIC:
    case MM_ANISOTROPIC:
    case MM_TEXT:
	unit = PU_PELS;
	multip = 1;
	break;
	  
    case MM_METRIC:
	unit = PU_LOMETRIC;
	multip = 10;
		
    case MM_LOMETRIC:
	unit = PU_LOMETRIC;
	multip = 1;
	break;

    case MM_HIMETRIC:
	unit = PU_HIMETRIC;
	multip = 1;
	break;

    case MM_TWIPS:
	unit = PU_TWIPS;
	multip = 1;
	break;
		
    case MM_POINTS:
	unit = PU_TWIPS;
	multip = 20;
	break;
    }

    if (!canvas)
	return;
    wxCanvasWnd *wnd = (wxCanvasWnd *)canvas->handle;
    if (wnd) {
	/*
	 * Calculate coordinate transformation and
	 * store away for later use.
	 * Need it when processing a OnPaint()
	 */
	wnd->unit = unit;
	wnd->xmult = multip * logical_scale_x * system_scale_x * user_scale_x;
	wnd->ymult = multip * logical_scale_y * system_scale_y * user_scale_y;
	wnd->xoff = -logical_origin_x * wnd->xmult + device_origin_x;
	wnd->yoff = -logical_origin_y * wnd->ymult + device_origin_y;
    
	// Note: An identical copy of this code is in wxCanvasWnd::OnPaint
	GpiSetPS(hps, &sizlPage, wnd->unit);
	RECTL rcl;
	GpiQueryPageViewport(hps, &rcl);
	rcl.xRight = (ULONG)(((float)rcl.xRight * wnd->xmult) + wnd->xoff);
	rcl.xLeft =  (ULONG)(((float)rcl.xLeft * wnd->xmult) + wnd->xoff);
	rcl.yTop = (ULONG)(((float)rcl.yTop * wnd->ymult) + wnd->yoff);
	rcl.yBottom = (ULONG)(((float)rcl.yBottom + wnd->ymult) + wnd->yoff);
	GpiSetPageViewport(hps, &rcl);

	wxSetupPS(hps);			// Since GpiSetPS() resetted everything
    }

//	current_pen
//	current_brush
//	current_background_brush
//	current_text_foreground
//	current_text_background
//	font

    // Set RGB mode
    // GpiCreateLogColorTable(hps, 0, LCOLF_RGB, 0, 0, NULL);

    if (current_background_brush)
	(void)GpiSetBackColor(hps, current_background_brush->colour);
    if (current_bk_mode == wxTRANSPARENT)
	GpiSetBackMix(hps, BM_LEAVEALONE);
    else
	GpiSetBackMix(hps, BM_OVERPAINT);
    if (current_text_foreground.Ok())
	    GpiSetColor(hps, current_text_foreground);
    if (current_text_background.Ok())
	GpiSetBackColor(hps, current_text_background) ;
}
#endif

/*
 * Panel device context
 */
 
IMPLEMENT_DYNAMIC_CLASS(wxPanelDC, wxCanvasDC)

wxPanelDC::wxPanelDC(void)
{
    TRACE("wxPanelDC::wxPanelDC()");
    __type = wxTYPE_DC_PANEL;
    panel = NULL;
    hpsTemp = NULLHANDLE;
}

wxPanelDC::wxPanelDC(wxPanel *the_panel)
{
    TRACE("wxPanelDC::wxPanelDC(the_panel=%#x)", the_panel);
    __type = wxTYPE_DC_PANEL;
    panel = the_panel;
    hpsTemp = NULLHANDLE;
}

wxPanelDC::~wxPanelDC(void)
{
    TRACE("wxPanelDC::~wxPanelDC() this=%#x wxDC=%#x", (int)this, (wxDC *)this);
}

HPS
wxPanelDC::GetHPS(void)
{
    wxPanelWnd *wnd = NULLHANDLE;
    if (panel) {
	wnd = (wxPanelWnd *)panel->handle;
	if (wnd->hpsUpdate)
	    return wnd->hpsUpdate;
    }

    if (hpsDC)
	return hpsDC;

    if (hpsTemp == NULLHANDLE && wnd) {
	hpsTemp = WinGetPS(wnd->hwndWnd);
	return hpsTemp;
    } 
    assert(0);				// Should never happen -- I think.
    return NULLHANDLE;
}

void
wxPanelDC::FreeHPS(void)
{
    if (hpsTemp) {
	WinReleasePS(hpsTemp);
	hpsTemp = NULLHANDLE;
    }
}


// Create a PS representing the whole screen
IMPLEMENT_DYNAMIC_CLASS(wxScreenDC, wxCanvasDC)

wxScreenDC::wxScreenDC(void)
{
    TRACE("wxScreenDC::wxScreenDC(void)");
    __type = wxTYPE_DC_CANVAS; // ?
    if (!canvas)
	return;
    wxCanvasWnd *wnd = (wxCanvasWnd *)canvas->handle;
    if (wnd)
	hpsDC = WinGetPS(wnd->hwndWnd);
    TRACE("wxScreenDC::wxScreenDC: hpsDC set to %#x", (int)hpsDC);
    wxSetupPS(hpsDC);
}

wxScreenDC::~wxScreenDC(void)
{
    TRACE("wxScreenDC::~wxScreenDC() this=%#x wxDC=%#x", this, (wxDC *)this);
    WinReleasePS(hpsDC);
}

HDC wxGetPrinterDC(void)
{
    TRACE("wxGetPrinterDC()");
#if USE_COMMON_DIALOGSXXX
    HDC         hDC;
    LPDEVMODE   lpDevMode = NULL;
    LPDEVNAMES  lpDevNames;
    LPSTR       lpszDriverName;
    LPSTR       lpszDeviceName;
    LPSTR       lpszPortName;

    PRINTDLG    pd;
    pd.lStructSize    = sizeof(PRINTDLG);
    pd.hwndOwner      = (HWND)NULL;
    pd.hDevMode       = NULL;
    pd.hDevNames      = NULL;
    pd.Flags          = PD_RETURNDEFAULT;
    pd.nCopies        = 1;

    if (!PrintDlg((LPPRINTDLG)&pd))
        return(NULL);

    if (!pd.hDevNames)
        return(NULL);

    lpDevNames = (LPDEVNAMES)GlobalLock(pd.hDevNames);
    lpszDriverName = (LPSTR)lpDevNames + lpDevNames->wDriverOffset;
    lpszDeviceName = (LPSTR)lpDevNames + lpDevNames->wDeviceOffset;
    lpszPortName   = (LPSTR)lpDevNames + lpDevNames->wOutputOffset;
    GlobalUnlock(pd.hDevNames);

    if (pd.hDevMode)
        lpDevMode = (LPDEVMODE)GlobalLock(pd.hDevMode);

#ifdef WIN32
    hDC = CreateDC(lpszDriverName, lpszDeviceName, lpszPortName, (DEVMODE *)lpDevMode);
#else
    hDC = CreateDC(lpszDriverName, lpszDeviceName, lpszPortName, (LPSTR)lpDevMode);
#endif

    if (pd.hDevMode && lpDevMode)
        GlobalUnlock(pd.hDevMode);

    if (pd.hDevNames) {
	GlobalFree(pd.hDevNames);
	pd.hDevNames=NULL;
    }
    if (pd.hDevMode) {
	GlobalFree(pd.hDevMode);
	pd.hDevMode=NULL;
    }
    return(hDC);
#else
    return 0;
#endif
}

/*
 * Memory DC
 *
 */

IMPLEMENT_DYNAMIC_CLASS(wxMemoryDC, wxCanvasDC)

wxMemoryDC::wxMemoryDC(void)
{
    TRACE("wxMemoryDC::wxMemoryDC");
    __type = wxTYPE_DC_MEMORY;
    device = wxDEVICE_WINDOWS;

    // CreateCompatibleDC
    SIZEL sizl = {0, 0};
#if 0
    DEVOPENSTRUC dop = {0L, (PSZ)"DISPLAY", NULL, 0L, 0L, 0L, 0L, 0L, 0L };
    HDC hdc = DevOpenDC(wxhAb, OD_MEMORY, (PSZ)"*", 5L, (PDEVOPENDATA)&dop, NULLHANDLE);
#else
    HDC hdc = DevOpenDC(wxhAb, OD_MEMORY, (PSZ)"*", 0, NULL, NULLHANDLE);
#endif
    hpsDC = GpiCreatePS(wxhAb, hdc, &sizl, PU_PELS | GPIA_ASSOC);
    TRACE("wxMemoryDC::wxMemoryDC: this=%#x, hpsDC=%#x", this, hpsDC);
    ok = (hpsDC != GPI_ERROR);
    if (ok)
	wxSetupPS(hpsDC);
    DevCloseDC(hdc);

    SetBrush(wxWHITE_BRUSH);
    SetPen(wxBLACK_PEN);
}

wxMemoryDC::wxMemoryDC(wxCanvasDC *old_dc) : wxbMemoryDC(old_dc)
{
    TRACE("wxMemoryDC::wxMemoryDC(old_dc=%#x)", old_dc);
    __type = wxTYPE_DC_MEMORY;
    device = wxDEVICE_WINDOWS;

    HPS ps = old_dc->GetHPS();
    if (ps) {
	// CreateCompatibleDC
	SIZEL sizl = {0, 0};
#if 0
	DEVOPENSTRUC dop = {0L, (PSZ)"DISPLAY", NULL, 0L, 0L, 0L, 0L, 0L, 0L };
	HDC hdc = DevOpenDC(wxhAb, OD_MEMORY, (PSZ)"*", 5L, (PDEVOPENDATA)&dop, NULLHANDLE);
#else
	HDC hdc = DevOpenDC(wxhAb, OD_MEMORY, (PSZ)"*", 0, NULL, NULLHANDLE);
#endif
	hpsDC = GpiCreatePS(wxhAb, hdc, &sizl, PU_PELS | GPIA_ASSOC);
	TRACE("wxMemoryDC::wxMemoryDC: this=%#x, hpsDC=%#x", this, hpsDC);
	
	ok = (hpsDC != GPI_ERROR);
	if (ok)
	    wxSetupPS(hpsDC);
	DevCloseDC(hdc);
    }

    SetBrush(wxWHITE_BRUSH);
    SetPen(wxBLACK_PEN);
}

wxMemoryDC::~wxMemoryDC(void)
{
    TRACE("wxMemoryDC::~wxMemoryDC() this=%#x wxDC=%#x", this, (wxDC *)this);
    GpiDestroyPS(hpsDC);
    hpsDC = NULLHANDLE;
}

void wxMemoryDC::SelectObject(wxBitmap *bitmap)
{
    TRACE("wxMemoryDC::SelectObject(bitmap=%#x)", bitmap);

    if (bitmap) {
	// Do own check for whether the bitmap is already selected into
	// a device context
	if (bitmap->selectedInto && (bitmap->selectedInto != this)) {
	    wxFatalError("Error in wxMemoryDC::SelectObject\nBitmap is selected in another wxMemoryDC.\nDelete the first wxMemoryDC or use SelectObject(NULL)");
	    return;
	}
  
	HBITMAP bm = GpiSetBitmap(hpsDC, bitmap->os2_bitmap);
	if (bm == PMERR_BITMAP_IN_USE) {
	    wxFatalError("Error in wxMemoryDC::SelectObject\nBitmap may not be loaded, or may be selected in another wxMemoryDC.\nDelete the first wxMemoryDC to deselect bitmap.");
	}
	bitmap->selectedInto = this;
	selected_bitmap = bitmap;
    } else {
	GpiSetBitmap(hpsDC, 0);		// Deselect bitmap
	selected_bitmap = NULL;
    }
}

void wxMemoryDC::GetSize(float *width, float *height)
{
    TRACE("wxMemoryDC::GetSize(width=%g, height=%g)", width, height);
    if (!selected_bitmap) {
	*width = 0.0; *height = 0.0;
	return;
    }
    *width = selected_bitmap->GetWidth();
    *height = selected_bitmap->GetHeight();
    TRACE("wxMemoryDC::GetSize() returns width=%d, height=%d", (int)*width, (int)*height);
}
 
