/*
 * File:        wx_dialg.cc
 * Purpose:        wxDialogBox and miscellaneous dialog functions
 * Author:        Julian Smart
 * Created:        1993
 * Updated:        August 1994
 * RCS_ID:      $Id: wx_dialg.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:        (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#ifdef __WATCOMC__
#define INCLUDE_COMMDLG_H        1
#endif

#include "wx_dialg.h"
#include "wx_utils.h"
#include "wx_frame.h"
#include "wx_main.h"

#include "wx_privt.h"

extern char wxDialogClassName[];

#define wxDIALOG_DEFAULT_X 300
#define wxDIALOG_DEFAULT_Y 300

// Lists to keep track of windows, so we can disable/enable them
// for modal dialogs
wxList wxModalDialogs;
wxList wxModelessWindows;  // Frames and modeless dialogs

wxDialogWnd::wxDialogWnd(wxWnd *parent, wxWindow *wx_win,
              int x, int y, int width, int height,
              char *dialog_template):
    wxSubWnd(parent, wxDialogClassName, wx_win, x, y, width, height, 0, dialog_template)
{
    __type = wxTYPE_DIALOG_WND;

    TRACE("wxDialogWnd::wxDialogWnd(parent=%#x, wx_win=%#x, x=%d, y=%d, w=%d, h=%d, dialog_template='%s')", (int)parent, (int)wx_win, x, y, width, height, dialog_template);
}

wxDialogWnd::~wxDialogWnd()
{
    TRACE("wxDialogWnd::~wxDialogWnd() this=%#x, wxWnd=%#x", this, (wxWnd *)this);
    WinDestroyWindow(hwndWnd);
    hwndWnd = NULLHANDLE;
}

MRESULT wxDialogWnd::DefWindowProc(ULONG nMsg, MPARAM mp1, MPARAM mp2)
{
    return WinDefWindowProc(hwndWnd, nMsg, mp1, mp2);
}

BOOL wxDialogWnd::ProcessMessage(QMSG* WXUNUSED(pMsg))
{
#if 0
  return ::IsDialogMessage(hwndWnd, pMsg);
#endif
  return FALSE;
}

BOOL wxDialogWnd::OnClose(void)
{
  if (wx_window)
  {
    if (wx_window->GetEventHandler()->OnClose())
    {
      ((wxDialogBox *)wx_window)->Show(FALSE);
      delete wx_window;
      return TRUE;
    } else return FALSE;
  }
  return FALSE;
}


IMPLEMENT_DYNAMIC_CLASS(wxDialogBox, wxPanel)

wxDialogBox::wxDialogBox(void)
{
    TRACE("wxDialogBox:wxDialogBox()");
    modal_showing = FALSE;
}

// Dialog box - like panel but doesn't need a frame, and is modal or
// non-modal
wxDialogBox::wxDialogBox(wxWindow *Parent, const char *Title, Bool Modal, 
               int x, int y, int width, int height, long style, const char *name):
  wxbDialogBox(Parent, Title, Modal, x, y, width, height, style, name)
{
    TRACE("wxDialogBox:wxDialogBox(Parent=%#x, Title='%s, Modal=%d, x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)Parent, Title, (int)Modal, x, y, width, height, (int)style, name);
    modal_showing = FALSE;
    Create(Parent, Title, Modal, x, y, width, height, style, name);
}
  
Bool wxDialogBox::Create(wxWindow *Parent, const char *Title, Bool Modal, 
                         int x, int y, int width, int height, long style, const char *name)
{
    TRACE("wxDialogBox::Create(Parent=%#x, Title='%s', Modal=%d, x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s'", (int)Parent, Title, (int)Modal, x, y, width, height, (int)style, name);
    SetName(name);

    // Do anything that needs to be done in the generic base class
    wxbDialogBox::Create(Parent, Title, Modal, x, y, width, height, style, name);

    if (Parent) Parent->AddChild(this);
    window_parent = Parent;

    labelFont = wxTheFontList->FindOrCreateFont(12, wxSWISS, wxNORMAL, wxBOLD);
    buttonFont = wxTheFontList->FindOrCreateFont(12, wxSWISS, wxNORMAL, wxBOLD);

    if (x < 0) x = wxDIALOG_DEFAULT_X;
    if (y < 0) y = wxDIALOG_DEFAULT_Y;

    windowStyle = style;
    wxWnd *cparent = NULL;
    if (Parent)
	cparent = (wxWnd *)Parent->handle;

    if (width < 0)
	width = 500;
    if (height < 0)
	height = 500;
#if 0
    // Allows creation of dialogs with & without captions under MSWindows
    wxDialogWnd *wnd;
    if(style & wxCAPTION){
	wnd = new wxDialogWnd(cparent, this, x, y, width, height, "wxCaptionDialog");
    } else{
	wnd = new wxDialogWnd(cparent, this, x, y, width, height, "wxNoCaptionDialog");
    }
#endif
    wxDialogWnd *wnd = new wxDialogWnd(cparent, this, x, y, width, height, "wxDummyDialog");

    handle = (char *)wnd;

	    HWND hwnd1 = wnd->GetFrameHWND();
	    HWND hwnd2 = WinQueryWindow(wnd->hwndWnd, QW_PARENT);
	    assert(hwnd1 == hwnd2);

#if 0
    WinSetWindowText(WinQueryWindow(wnd->hwndWnd, QW_PARENT), (PSZ)Title);
#else
    WinSetWindowText(wnd->hwndWnd, (PSZ)Title);
#endif

    if (!Modal)
	wxModelessWindows.Append(this);

    wx_dc = new wxPanelDC(this);

    modal = Modal;
    return TRUE;
}

void wxDialogBox::SetModal(Bool flag)
{
    modal = flag;
  
    wxModelessWindows.DeleteObject(this);
    if (!flag)
	wxModelessWindows.Append(this);
}

wxDialogBox::~wxDialogBox()
{
    TRACE("wxDialogBox::~wxDialogBox() this=%#x wxWindow=%#x", (int)this, (wxWindow *)this);
    // Destroy Frame window. Child window is destroyed by wxWindow::~wxWindow

#if 0
	    HWND hwnd1 = GetFrameHWND();
	    HWND hwnd2 = WinQueryWindow(GetHWND(), QW_PARENT);
	    assert(hwnd1 == hwnd2);

    WinDestroyWindow(WinQueryWindow(GetHWND(), QW_PARENT));
#endif
}

#if XXX
// By default, pressing escape quits the dialog
Bool wxDialogBox::OnCharHook(wxKeyEvent& event)
{
    if (handle) {
	if (event.keyCode == WXK_ESCAPE) {
	    if (OnClose()) {
		Show(FALSE);
		delete this;
	    }
	    return TRUE;
	}
    }
    return FALSE;
}
#endif

void wxDialogBox::Fit(void)
{
    TRACE("wxDialogBox::Fit()");
    wxPanel::Fit();
}

void wxDialogBox::Iconize(Bool iconize)
{
TRACE("wxDialogBox::Iconize(iconize==%d)", (int)iconize);
  // Windows dialog boxes can't be iconized
}

Bool wxDialogBox::Iconized(void)
{
  return FALSE;
}

/*
 * Set size of Dialog (frame) window
 */
void wxDialogBox::SetSize(int x, int y, int width, int height, int sizeFlags)
{
    TRACE("wxDialogBox::SetSize(x=%d, y=%d, w=%d, h=%d, sizeFlags=%d)", x, y, width, height, sizeFlags);
    
    wxWindow::SetSize(x, y, width, height);

    // Set Frame window visible

    wxWnd *wnd = (wxWnd *)handle;
    HWND hwnd1 = wnd->GetFrameHWND();
    HWND hwnd2 = WinQueryWindow(GetHWND(), QW_PARENT);
    assert(hwnd1 == hwnd2);

    WinShowWindow(WinQueryWindow(GetHWND(), QW_PARENT), TRUE);
}

/*
 * Set size of Client window.
 *
 * To convert from client window size to frame window size
 * we just add the current size difference.
 */
void wxDialogBox::SetClientSize(int width, int height)
{
    TRACE("wxDialogBox::SetClientSize(w=%d, h=%d)", width, height);
#if 1
    wxWindow::SetClientSize(width, height);
#else 
    wxWnd *wnd = (wxWnd *)handle;
    HWND hwnd1 = wnd->GetFrameHWND();
    HWND hwnd2 = WinQueryWindow(GetHWND(), QW_PARENT);

    assert(hwnd1 == hwnd2);

    HWND hwndClient = GetHWND();
    HWND hwndFrame = WinQueryWindow(hwndClient, QW_PARENT);
    RECTL rcl = {0, 0, width, height};
    WinCalcFrameRect(hwndFrame, &rcl, FALSE);
    WinSetWindowPos(hwndFrame, HWND_TOP, 0, 0, swp.cx, swp.cy, SWP_SIZE);

    // This appears to be necessary
    WinInvalidateRect(hwndClient, NULL, TRUE) ;

    GetEventHandler()->OnSize(width, height);
#endif
}

void wxDialogBox::GetPosition(int *x, int *y)
{
    HWND hWnd = GetHWND();
    wxRECTL rect;
    wx_ClientRect(hWnd, &rect);

    *x = rect.xLeft;
    *y = rect.yTop;
    TRACE("wxDialogBox::GetPosition returns x=%d, y=%d", *x, *y);
}

Bool wxDialogBox::IsShown(void)
{
    return is_show;
}

Bool wxDialogBox::Show(Bool show)
{
    TRACE("wxDialogBox::Show(show=%d) called", (int)show);

    is_show = show;
    wxWnd *dialog = (wxWnd *)handle;
    if (!dialog)
	return FALSE;
#if 1
    HWND hwndParent = GetParent() ? GetParent()->GetHWND() : NULLHANDLE;
    if (show) {
	if (hwndParent)
	    WinSetWindowPos(hwndParent, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
	if (modal) {
	    WinProcessDlg(dialog->hwndWnd);
	    modal_showing = TRUE;
	}
    } else {
	if (hwndParent)
	    WinShowWindow(hwndParent, FALSE);
	if (modal) {
	    WinDismissDlg(dialog->hwndWnd, TRUE);
	    modal_showing = FALSE;
	}
    }
#else
    if (modal) {
	if (show) {
	    if (wxModalDialogs.Member(this) && modal_showing) {
		WinSetWindowPos(dialog->hwndWnd, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
		return TRUE;
	    }
      
	    modal_showing = TRUE;
	    wxNode *node = wxModalDialogs.First();
	    while (node) {
		wxDialogBox *box = (wxDialogBox *)node->Data();
		wxWnd *the_dialog = (wxWnd *)box->handle;
		if (box != this)
		    TRACE("wxDialogBox::Show: Wanting to disable window %#x", the_dialog->hwndWnd);
//		    WinEnableWindow(the_dialog->hwndWnd, FALSE);
		node = node->Next();
	    }
	    node = wxModelessWindows.First();
	    while (node) {
		wxWindow *win = (wxWindow *)node->Data();
		wxWnd *wnd = (wxWnd *)win->handle;
		    TRACE("wxDialogBox::Show: Wanting to disable window %#x", wnd->hwndWnd);
//		WinEnableWindow(wnd->hwndWnd, FALSE);
		node = node->Next();
	    }

	    WinEnableWindow(dialog->hwndWnd, TRUE);
	    WinSetWindowPos(dialog->hwndWnd, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);

TRACE("wxDialogBox::Show: Calling WinProcessDlg(%#x)", dialog->hwndWnd);
	    ULONG reply = WinProcessDlg(dialog->hwndWnd);
TRACE("wxDialogBox::Show: Return from WinProcessDlg");
	} else {
TRACE("wxDialogBox::Show: Calling WinDismissDlg(%#x)", dialog->hwndWnd);
	    WinDismissDlg(dialog->hwndWnd, FALSE);

	    wxModalDialogs.DeleteObject(this);

	    wxNode *first = wxModalDialogs.First();

	    // If there's still a modal dialog active, we
	    // enable it, else we enable all modeless windows
	    if (first) {
		wxDialogBox *box = (wxDialogBox *)first->Data();
		HWND hwnd = box->GetHWND();
		if (box->winEnabled)
		    WinEnableWindow(hwnd, TRUE);
		WinSetWindowPos(dialog->hwndWnd, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
	    } else {
		wxNode *node = wxModelessWindows.First();
		while (node) {
		    wxWindow *win = (wxWindow *)node->Data();
		    HWND hwnd = win->GetHWND();
		    // Only enable again if not user-disabled.
		    if (win->winEnabled)
			WinEnableWindow(hwnd, TRUE);
		    node = node->Next();
		}
	    }
	    // Try to highlight the correct window (the parent)
	    HWND hWndParent = 0;
	    if (GetParent()) {
		hWndParent = GetParent()->GetHWND();
		if (hWndParent)
		    WinSetWindowPos(dialog->hwndWnd, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
	    }
	    WinShowWindow(dialog->hwndWnd, FALSE);
	    modal_showing = FALSE;
	}
    } else { // Not modal
	if (show) {
	    WinSetWindowPos(dialog->hwndWnd, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ZORDER);
	} else {
	    // Try to highlight the correct window (the parent)
	    HWND hWndParent = 0;
	    if (GetParent()) {
		hWndParent = GetParent()->GetHWND();
		if (hWndParent)
		    WinSetWindowPos(hWndParent, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ZORDER);
	    }
	    WinShowWindow(dialog->hwndWnd, FALSE);
	}
    }
#endif
    return TRUE;
}

void wxDialogBox::SetTitle(const char *title)
{
    TRACE("wxDialogBox::SetTitle(title='%s')", title);
    wxWnd *wnd = (wxWnd *)handle;

    HWND hwnd1 = wnd->GetFrameHWND();
    HWND hwnd2 = WinQueryWindow(wnd->hwndWnd, QW_PARENT);
    assert(hwnd1 == hwnd2);

#if 0
    WinSetWindowText(WinQueryWindow(wnd->hwndWnd, QW_PARENT), (PSZ)title);
#else
    WinSetWindowText(wnd->hwndWnd, (PSZ)title);
#endif
}

char *wxDialogBox::GetTitle(void)
{
TRACE("wxDialogBox::GetTitle()");
  wxWnd *wnd = (wxWnd *)handle;

	    HWND hwnd1 = wnd->GetFrameHWND();
	    HWND hwnd2 = WinQueryWindow(wnd->hwndWnd, QW_PARENT);
	    assert(hwnd1 == hwnd2);

#if 0
  WinQueryWindowText(WinQueryWindow(wnd->hwndWnd, QW_PARENT), 1000, (PCH)wxBuffer);
#else
  WinQueryWindowText(wnd->hwndWnd, 1000, (PCH)wxBuffer);
#endif
  return wxBuffer;
}

HWND wxDialogBox::GetHWND(void)
{
    /*
     * In dialog boxes, hwndWnd points at the
     * frame window, not the client window
     */
    HWND hwnd = wxWindow::GetHWND();
    return (HWND)WinWindowFromID(hwnd, FID_CLIENT);
}

