/*
 * File:        wx_frame.cc
 * Purpose:        wxFrame implementation
 * Author:        Julian Smart
 * Created:        1993
 * Updated:        August 1994
 * RCS_ID:      $Id: wx_frame.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:        (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_privt.h"
#include "wx_frame.h"
#include "wx_menu.h"
#include "wx_gdi.h"
#include "wx_main.h"
#include "wx_utils.h"
#include "wx_rsrc.h"


extern wxList wxModelessWindows;

wxPen *wxStatusGreyPen = NULL;
wxPen *wxStatusWhitePen = NULL;

// This range gives a maximum of 500
// MDI children. Should be enough :-)
#define wxFIRST_MDI_CHILD 4100
#define wxLAST_MDI_CHILD 4600

// Status border dimensions
#define         wxTHICK_LINE_BORDER 3
#define         wxTHICK_LINE_WIDTH  1

#define	FID_STATUSLINE	0x8801

extern char wxFrameClientClassName[];
extern char wxMDIClientClassName[];
extern char wxMDIChildClientClassName[];
extern char wxPanelClassName[];
extern char wxCanvasClassName[];
extern char wxDialogClassName[];
extern char wxStatusLineClassName[];

extern "C" { int printf (__const__ char *format, ...); }
MRESULT EXPENTRY wxFrameProc(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2);

// Simulates GetClientRect for OS/2.. if FID_CLIENT exists, it returns
// its window rectangle, otherwise, returns window rectangle
void wx_ClientRect(HWND hWnd, PRECTL pRect)
{
#undef WinWindowFromID
	HWND hWndClient = WinWindowFromID(hWnd, FID_CLIENT);

	if (hWndClient == NULLHANDLE)
		hWndClient = hWnd;

	WinQueryWindowRect(hWndClient, pRect);
//
// PGD: Why?
//
	pRect->xRight  = rcWidth(pRect);
	pRect->xLeft   = 0;
	pRect->yTop    = rcHeight(pRect);
	pRect->yBottom = 0;
TRACE("wx_Clientrect returns xLeft=%d, yBottom=%d, xRight=%d, yTop=%d", (int)pRect->xLeft, (int)pRect->yBottom, (int)pRect->xRight, (int)pRect->yTop);
}

void wx_ClientRect(HWND hWnd, wxRECTL *pRect)
{
#undef WinWindowFromID
	HWND hWndClient = WinWindowFromID(hWnd, FID_CLIENT);

	if (hWndClient == NULLHANDLE)
		hWndClient = hWnd;

	RECTL rect;
	WinQueryWindowRect(hWndClient, &rect);
//
// PGD: Why?
//
	pRect->xRight  = rcWidth(rect);
	pRect->xLeft   = 0;
	pRect->yTop    = 0;
	pRect->yBottom = rcHeight(rect);
TRACE("wx_Clientrect returns xLeft=%d, yBottom=%d, xRight=%d, yTop=%d", (int)pRect->xLeft, (int)pRect->yBottom, (int)pRect->xRight, (int)pRect->yTop);
}


/*
 * Return the coordinates of a window relative to the specified
 * window, or to the desktop.
 */
void wx_WindowRect(HWND hWnd, PRECTL pRect, HWND hwndParent)
{
    TRACE("wx_WindowRect(hWnd=%#x, ..., hwndParent=%#x)", (int)hWnd, (int)hwndParent);
	if (hwndParent == NULLHANDLE)
		hwndParent = HWND_DESKTOP;
	WinQueryWindowRect(hWnd, pRect);
	TRACE("\thwnd rect: xLeft=%d, yBottom=%d, xRight=%d, yTop=%d", (int)pRect->xLeft, (int)pRect->yBottom, (int)pRect->xRight, (int)pRect->yTop);
	WinMapWindowPoints(hWnd, hwndParent, (PPOINTL)pRect, 2L);
	TRACE("\tafter map: xLeft=%d, yBottom=%d, xRight=%d, yTop=%d", (int)pRect->xLeft, (int)pRect->yBottom, (int)pRect->xRight, (int)pRect->yTop);
}

/*
 * Same as above, but using wxWindows coordinates
 */
void wx_WindowRect(HWND hWnd, wxRECTL *pRect, HWND hwndParent)
{
TRACE("wx_WindowRect(hWnd=%#x, ..., hwndParent=%#x)", (int)hWnd, (int)hwndParent);
	if (hwndParent == NULLHANDLE)
		hwndParent = HWND_DESKTOP;
	RECTL rect, prect;
	WinQueryWindowRect(hWnd, &rect);
TRACE("\tWinQueryWindowRect(hWnd=%#x, ...) returns {xLeft=%d, yBottom=%d, xRight=%d, yTop=%d}", (int)hWnd, (int)rect.xLeft, (int)rect.yBottom, (int)rect.xRight, (int)rect.yTop);
	WinMapWindowPoints(hWnd, hwndParent, (PPOINTL)&rect, 2L);
TRACE("\tWinMapWindowPoints(hWnd=%#x, hwndParent=%#x, ...) returns {xLeft=%d, yBottom=%d, xRight=%d, yTop=%d}", (int)hWnd, (int)hwndParent, (int)rect.xLeft, (int)rect.yBottom, (int)rect.xRight, (int)rect.yTop);

	/*
	 * Now convert to wxWindows coordinates
	 */
	WinQueryWindowRect(hwndParent, &prect);
TRACE("\tWinQueryWindowRect(%#x, ...) returns {xLeft=%d, yBottom=%d, xRight=%d, yTop=%d}", (int)hwndParent, (int)prect.xLeft, (int)prect.yBottom, (int)prect.xRight, (int)prect.yTop);
	pRect->xLeft = rect.xLeft;
	pRect->xRight = rect.xRight;
	pRect->yTop = (prect.yTop - prect.yBottom) - rect.yTop;
	pRect->yBottom = (prect.yTop - prect.yBottom) - rect.yBottom;
TRACE("\tfinal result {xLeft=%d, yBottom=%d, xRight=%d, yTop=%d}", (int)pRect->xLeft, (int)pRect->yBottom, (int)pRect->xRight, (int)pRect->yTop);
}

#if 0
/*
 * Convert from pm coordinates to wx coordinates
 */
void wx_pmCoord(HWND hwnd, wxPOINTL *wxPoint, POINTL *pmPoint, int ncoords)
{
	for (int i = 0; i < ncoords; i++) {
		pmPoint->x = wxPoint->x;
		pmPoint->y = win_height - wxPoint->y;
		wxPoint++;
		pmPoint++;
	}
}

/*
 * Convert from wx coordinates to pm coordinates
 */
void wx_wxCoord(HWND hwnd, POINTL *pmPoint, wxPOINTL *wxPoint, int ncoords)
{
	for (int i = 0; i < ncoords; i++) {
		wxPoint->x = pmPoint->x;
		wxPoint->y = win_height - pmPoint->y;
		pmPoint++;
		wxPoint++;
	}
}
#endif

/*
 * Return the size of a window
 */
void wx_GetWindowSize(HWND hWnd, int *cx, int *cy)
{
    RECTL rect;
    WinQueryWindowRect(hWnd, &rect);
    if (cx)
	*cx = rcWidth(rect);
    if (cy)
	*cy = rcHeight(rect);
    TRACE("wx_GetWindowSize(hWnd=%#x): x=%d, y=%d", hWnd, cx ? *cx : 0, cy ? *cy : 0);
}

// Simulates WinSetWindowPos, but with wxWindows coordinates
void wx_MoveWindow(HWND hWnd, int x, int y, int w, int h, Bool show)
{
    TRACE("wx_MoveWindow(hWnd=%#x, x=%d, y=%d, w=%d, h=%d, show=%d)", (int)hWnd, x, y, w, h, (int)show);

    HWND hwndParent = WinQueryWindow(hWnd, QW_PARENT);
    assert(hwndParent);
    RECTL pRect;
    WinQueryWindowRect(hwndParent, &pRect);
    TRACE("\tParent hwnd=%#x, xLeft=%d, yBottom=%d, xRight=%d, yTop=%d", (int)hwndParent, (int)pRect.xLeft, (int)pRect.yBottom, (int)pRect.xRight, (int)pRect.yTop);
    if (y > rcHeight(pRect))
	printf("!!! WARNING WinSetWindowPos args out of range, y>rcHeight(pRect)\n");
//assert(y <= rcHeight(pRect));
    WinSetWindowPos(hWnd, HWND_TOP, x, rcHeight(pRect) - (y + h), w, h,
		    SWP_MOVE | SWP_SIZE | (show ? SWP_SHOW : 0) | SWP_ZORDER);
}

/*
 * Convert from screen coordinates to client window coordinates
 * using wxWindows coordinate system
 */
void wx_ScreenToClient(HWND hwnd, wxPOINTL *ppoint)
{
TRACE("wx_ScreenToClient(hwnd=%#x, ppoint={%d, %d}", (int)hwnd, (int)ppoint->x, (int)ppoint->y);

	RECTL rect1;
	WinQueryWindowRect(hwnd, &rect1);

	/*
	 * Figure out if we have a client window
	 * and use it if so.
	 */
	HWND hwndClient = WinWindowFromID(hwnd, FID_CLIENT);
	if (hwndClient == NULLHANDLE)
		hwndClient = hwnd;

	/*
	 * Get client window rectangle and
	 * map it to screen coordinates
	 */
	RECTL cRect;
	WinQueryWindowRect(hwndClient, &cRect);
	WinMapWindowPoints(hwndClient, HWND_DESKTOP, (PPOINTL)&cRect, 2L);

	/*
	 * Convert to client window coordinates
	 */
	RECTL sRect;
	WinQueryWindowRect(HWND_DESKTOP, &sRect);
	ppoint->x -= cRect.xLeft;
	ppoint->y -= sRect.yTop - cRect.yTop;

//	assert(ppoint->x >= -1 && ppoint->y >= -1);
}

/*
 * Convert from client coordinates to screen coordinates
 * use wxWindows coordinate system
 */
void wx_ClientToScreen(HWND hwnd, wxPOINTL *ppoint)
{
	printf("wx_ClientToScreen(hwnd=%#x, ppoint={%d, %d}\n", (int)hwnd, (int)ppoint->x, (int)ppoint->y);

	/*
	 * Figure out if we have a client window
	 * and use it if so.
	 */
	HWND hwndClient = WinWindowFromID(hwnd, FID_CLIENT);
	if (hwndClient == NULLHANDLE)
		hwndClient = hwnd;

	/*
	 * Get client window rectangle and
	 * map it to display coordinates
	 */
	RECTL cRect;
	WinQueryWindowRect(hwndClient, &cRect);
	WinMapWindowPoints(hwndClient, HWND_DESKTOP, (PPOINTL)&cRect, 2L);
	
	/*
	 * Convert to display coordinates
	 */
	RECTL sRect;
	WinQueryWindowRect(HWND_DESKTOP, &sRect);
	ppoint->x += cRect.xLeft;
	ppoint->y += sRect.yTop - cRect.yTop;

TRACE("\tScreen coords = {%d, %d}", (int)ppoint->x, (int)ppoint->y);
	assert(ppoint->x >= 0 && ppoint->y >= 0);
}

IMPLEMENT_DYNAMIC_CLASS(wxFrame, wxWindow)

wxFrame::wxFrame(void)
{
    wx_iconized = FALSE;
}

wxFrame::wxFrame(wxFrame *parent, const char *title, int x, int y,
                 int width, int height, long style, const char *name)
    : wxbFrame(parent, title, x, y, width, height, style, name)
{
TRACE("wxFrame::wxFrame(parent=%#x, title='%s', x=%d, y=%d, width=%d, height=%d, style=%#x, name='%s')", (int)parent, title, x, y, width, height, (int)style, name);
    Create(parent, title, x, y, width, height, style, name);
}

Bool wxFrame::Create(wxFrame *Parent, const char *title, int x, int y,
		     int width, int height, long style, const char *name)
{
TRACE("wxFrame::Create(Parent=%#x, title='%s', x=%d, y=%d, width=%d, height=%d, style=%#x, name='%s')", (int)Parent, title, x, y, width, height, (int)style, name);
    wxbFrame::Create(Parent, title, x, y, width, height, style, name);
  
    SetName(name);
    frame_type = style & (wxSDI | wxMDI_PARENT | wxMDI_CHILD);
    windowStyle = style;
  
    if (Parent) Parent->AddChild(this);
    window_parent = Parent;
    wx_iconized = FALSE;
    wxWnd *cparent = NULL;
    if (Parent)
	cparent = (wxWnd *)Parent->handle;

    switch (frame_type) {
    case wxMDI_PARENT:
	handle = (char *)new wxMDIFrameWnd(NULL, this, title, x, y, width, height, style);
	break;

    case wxMDI_CHILD:
	handle = (char *)new wxMDIChildWnd((wxMDIFrameWnd *)cparent, this, title, x, y, width, height, style);
	break;

    default:
    case wxSDI:
	handle = (char *)new wxFrameWnd(cparent, this, title, x, y, width, height, style);
	break;
    }

    wxModelessWindows.Append(this);
    return TRUE;
}

wxFrame::~wxFrame(void)
{
    TRACE("wxFrame::~wxFrame() this=%#x wxWindow=%#x", this, (wxWindow *)this);
    if (wx_menu_bar)
	delete wx_menu_bar;

    if (icon)
	delete icon;

    if (this == wxTheApp->wx_frame) {
	wxTheApp->wx_frame = NULL;
	//      PostQuitMessage(0);
    }

    wxModelessWindows.DeleteObject(this);
}

HWND wxFrame::GetWinMenu(void)
{
    if (handle)
	return ((wxWnd *)handle)->hwndMenu;
    else return 0;
}
  

// Get size *available for subwindows* i.e. excluding menu bar etc.
// For XView, this is the same as GetSize
void wxFrame::GetClientSize(int *cx, int *cy)
{
    int cx1, cx2, cy1, cy2;

    wxFrameWnd *wnd = (wxFrameWnd *)handle;
    assert(wnd);
    assert(wnd->__type == wxTYPE_FRAME_WND);

/*
 * this is the correct way to calculate window
 * size.
 */
    cx1 = wnd->swpClient.cx;
    cy1 = wnd->swpClient.cy;

/*
 * ...But since the swpClient variables are not
 * always set properly, we also get the window
 * size from wx_ClientRect() and compare the values.
 */
    wxRECTL rect;
    wx_ClientRect(GetHWND(), &rect);
    cx2 = rcWidth(rect);
    cy2 = rcHeight(rect);

/*
 * ...Both methods should return the same window
 * size. If not we have a bug somewhere, so
 * we better abort right here, and find out why
 * swpClient was not correctly set.
 * Other code assume that swpClient contains the
 * correct values.
 */
printf("cx1=%d, cx2=%d, cy1=%d, cy2=%d\n", cx1, cx2, cy1, cy2);
    assert(cx1 == cx2 && cy1 == cy2);
    *cx = cx1;
    *cy = cy1;

    TRACE("wxFrame::GetClientSize() returns x=%d, y=%d", (int)*cx, (int)*cy);
}

/*
 * Set size of Client window.
 */
void wxFrame::SetClientSize(int width, int height)
{
    TRACE("wxFrame::SetClientSize(w=%d, h=%d)", width, height);
#if 1
    wxWindow::SetClientSize(width, height);
#else
	HWND hwndFrame = GetHWND();
	HWND hwndClient = (HWND)WinWindowFromID(hwndFrame, FID_CLIENT);
	RECTL rcl = {0, 0, width, height};
	WinCalcFrameRect(hwndFrame, &rcl, FALSE);
	WinSetWindowPos(hwndFrame, HWND_TOP, 0, 0, swp.cx, swp.cy, SWP_SIZE);

	// This appears to be necessary
	WinInvalidateRect(hwndClient, NULL, TRUE) ;

	GetEventHandler()->OnSize(width, height);
#endif
}

void wxFrame::GetSize(int *width, int *height)
{
	wx_GetWindowSize(GetHWND(), width, height);
TRACE("wxFrame::GetSize() returns width=%d, height=%d", *width, *height);
}

void wxFrame::GetPosition(int *x, int *y)
{
    wxWnd *wnd = (wxWnd *)handle;
    wxWindow *parent = GetParent();
    wxRECTL rect;
    wx_WindowRect(wnd->hwndWnd, &rect);
    wxPOINTL point = { rect.xLeft, rect.yTop };

    // Since we now have the absolute screen coords,
    // if there's a parent we must subtract its top left corner
    if (parent && __type==wxTYPE_MDICHILD_WND) {
	wxWnd *pwnd = (wxWnd *)parent->handle;
	wx_ScreenToClient(pwnd->GetClientHWND(), &point);
    }
    *x = point.x;
    *y = point.y;
  TRACE("wxFrame::GetPosition returns x=%d, y=%d", *x, *y);
}

void wxFrame::SetSize(int x, int y, int width, int height, int sizeFlags)
{
    TRACE("wxFrame::SetSize(x=%d, y=%d, w=%d, h=%d, sizeFlags=%#x)", x, y, width, height, sizeFlags);
    MoveWindow(x, y, width, height);
    GetEventHandler()->OnSize(width, height);
}

Bool wxFrame::Show(Bool show)
{
    wxWnd *wnd = (wxWnd *)handle;

    if (!show) {
	// Try to highlight the correct window (the parent)
	HWND hWndParent = 0;
	if (GetParent()) {
	    hWndParent = GetParent()->GetHWND();
	    if (hWndParent)
		WinSetWindowPos(hWndParent, HWND_TOP, 0, 0, 0, 0, SWP_ACTIVATE);
	}
    }
	
    WinShowWindow(wnd->hwndWnd, show);
    if (show) {
#if 0  
	WinSetWindowPos(wnd->hwndWnd, HWND_TOP, 0, 0, 0, 0, SWP_ACTIVATE);
#endif    
	OnActivate(TRUE);
    }
    return TRUE;
}

void wxFrame::Iconize(Bool iconize)
{
    if (!iconize)
	Show(TRUE);

    wxWnd *wnd = (wxWnd *)handle;
    WinSetWindowPos(wnd->hwndWnd, HWND_TOP, 0, 0, 0, 0, (iconize) ? SWP_MINIMIZE : SWP_RESTORE);
    wx_iconized = iconize;
}

// Equivalent to maximize/restore in Windows
void wxFrame::Maximize(Bool maximize)
{
    Show(TRUE);
    wxWnd *wnd = (wxWnd *)handle;
    WinSetWindowPos(wnd->hwndWnd, HWND_TOP, 0, 0, 0, 0, (maximize) ? SWP_MAXIMIZE : SWP_RESTORE);
    wx_iconized = FALSE;
}

Bool wxFrame::Iconized(void)
{
    wxWnd *wnd = (wxWnd *)handle;
    wx_iconized = !!(WinQueryWindowULong(wnd->GetFrameHWND(), QWL_STYLE) & WS_MINIMIZED);
    return wx_iconized;
}

void wxFrame::SetTitle(const char *title)
{
    wxWnd *wnd = (wxWnd *)handle;
    WinSetWindowText(wnd->hwndWnd, (PSZ)title);
}

char *wxFrame::GetTitle(void)
{
  wxWnd *wnd = (wxWnd *)handle;
  WinQueryWindowText(wnd->hwndWnd, 1000, (PCH)wxBuffer);
  return wxBuffer;
}

void wxFrame::SetIcon(wxIcon *wx_icon)
{
    icon = wx_icon;

    wxFrameWnd *wnd = (wxFrameWnd *)handle;
    if (!wx_icon)
	wnd->icon = 0;
    else
	wnd->icon = wx_icon->os2_icon;
}


void wxFrame::CreateStatusLine(int number, Constdata char *name)
{
    wxFrameWnd *wnd = (wxFrameWnd *)handle;

TRACE("wxFrame::CreateStatusLine(number=%d, name='%s')", number, name);
    if (status_line_exists)
	return;

    /*
     * Create status line window, child of frame window
     */
    ULONG ctldata[3];
    ctldata[0] = sizeof ctldata;
    ctldata[1] = (int)wnd;
    ctldata[2] = 0;
    wnd->hwndStatusLine = WinCreateWindow(wnd->hwndWnd, (PSZ)wxStatusLineClassName,
				       NULL, WS_VISIBLE,
				       0, 0, 0, 0,
				       wnd->hwndWnd,
				       HWND_TOP, FID_STATUSLINE, (PVOID)ctldata, NULL);

    for (int i = 0; i < number; i++) {
	wnd->status_item[i] = WinCreateWindow(wnd->hwndStatusLine, WC_STATIC,
					      NULL, SS_TEXT | DT_VCENTER | WS_VISIBLE,
					      0, 0, 0, 0,
					      wnd->hwndStatusLine,
					      HWND_TOP, (int)NewControlId(), NULL, NULL);
	// To be fixed
////	wxSetBackgroundColour(wnd->status_item[i], CLR_PALEGRAY);
////	wxSetWinColour(wnd->status_item[i], PP_FOREGROUNDCOLORINDEX, CLR_BLACK);
    }

    wnd->nitems = number;

    /*
     * Send a FORMATFRAME message
     * to resize the format subwindows.
     * This will position the status line at the bottom of the frame window,
     * and set the size of it.
     */
    status_line_exists = TRUE;

    /*
     * Reformat the frame window.
     */
    int ccount = (int)WinSendMsg(wnd->hwndWnd, WM_QUERYFRAMECTLCOUNT, 0, 0);
    PSWP pswp = new SWP[ccount];
printf("wxFrame::CreateStatusLine: swpClient.x=%ld, swpClient.y=%ld, swpClient.cx=%ld, swpClient.cy=%ld\n", wnd->swpClient.x, wnd->swpClient.y, wnd->swpClient.cx, wnd->swpClient.cy);
    WinSendMsg(wnd->hwndWnd, WM_FORMATFRAME, (MPARAM)pswp, &wnd->swpClient);
printf("wxFrame::CreateStatusLine: swpClient.x=%ld, swpClient.y=%ld, swpClient.cx=%ld, swpClient.cy=%ld\n", wnd->swpClient.x, wnd->swpClient.y, wnd->swpClient.cx, wnd->swpClient.cy);
    delete[] pswp;

    nb_status = number;

#if XXX
    wxFrameWnd *cframe = (wxFrameWnd *)handle;
  
    HPS ps = WinGetPS(cframe->hwndWnd);
#if 0
    GpiSetCharSet(ps, wxSTATUS_LINE_FONT);
#endif
    WinReleasePS(ps);

    if (!wxStatusGreyPen) {
	wxStatusGreyPen = new wxPen("GREY", wxTHICK_LINE_WIDTH, wxSOLID);
	wxStatusWhitePen = new wxPen("WHITE", wxTHICK_LINE_WIDTH, wxSOLID);
    }
#endif
}

void wxFrame::SetStatusText(char *text, int number)
{
TRACE("wxFrame::SetStatusText(text='%s', number=%d)", text, number);
    if (!status_line_exists)
	return;

    if ((number < 0) || (number >= nb_status))
	return;

    wxFrameWnd *wnd = (wxFrameWnd *)handle;
TRACE("wxFrame::SetStatusText: wnd->status_item[%d] = %#x, wnd=%#x", number, (int)wnd->status_item[number], (int)wnd);
    WinSetWindowText(wnd->status_item[number], (PSZ)text);
}

#if 0
void wxFrame::PositionStatusWindow(void)
{
return;
    // We will assume that in a multi status line, all fields have the
    //  same width.
    wxFrameWnd *cframe = (wxFrameWnd *)handle;
    RECTL rect;
    wx_ClientRect(cframe->hwndWnd, &rect);
    int cwidth = rcWidth(rect);
    int cheight = rcHeight(rect);
    for (int i = 0; i < nb_status; i++) {
	int real_width = (int)(cwidth/nb_status);
	WinSetWindowPos(status_window[i]->hwndWnd, HWND_TOP, i*real_width, 0, real_width,
			status_window[0]->height, SWP_MOVE | SWP_SIZE);
    }
}
#endif

void wxFrame::LoadAccelerators(ULONG resourceId)
{
    TRACE("LoadAccelerators(resourceId=%ld)", resourceId);
    wxFrameWnd *cframe = (wxFrameWnd *)handle;
    if (cframe) {
	cframe->accelerator_table = WinLoadAccelTable(wxhAb, NULLHANDLE, resourceId);
	TRACE("cframe->hwndWnd=%#x", cframe->hwndWnd);
	WinSetAccelTable(wxhAb, cframe->accelerator_table, cframe->hwndWnd);
    }
}

void wxFrame::Fit(void)
{
    // Work out max. size
    wxNode *node = children->First();
    int max_width = 0;
    int max_height = 0;
    while (node) {
	// Find a child that's a subwindow, but not a dialog box.
	wxWindow *win = (wxWindow *)node->Data();

	if ((wxSubType(win->__type, wxTYPE_PANEL) &&
	     !wxSubType(win->__type, wxTYPE_DIALOG_BOX)) ||
	    wxSubType(win->__type, wxTYPE_TEXT_WINDOW) ||
	    wxSubType(win->__type, wxTYPE_CANVAS)) {
	    int width, height;
	    int x, y;
	    win->GetSize(&width, &height);
	    win->GetPosition(&x, &y);

	    if ((x + width) > max_width)
		max_width = x + width;
	    if ((y + height) > max_height)
		max_height = y + height;
	}
	node = node->Next();
    }
    SetClientSize(max_width, max_height);
}


// Returns the active MDI child window
wxFrame *wxFrame::GetActiveChild(void)
{
    if (windowStyle & wxMDI_PARENT != wxMDI_PARENT)
	return NULL;
    
    wxMDIFrameWnd *wnd = (wxMDIFrameWnd *)handle;
    if (!wnd)
	return NULL;
    
    HWND hWnd = (HWND)WinSendMsg(wnd->hwndClient, WM_MDIGETACTIVE, 0, 0L);
    if (hWnd == 0)
	return NULL;
    else {
	wxWnd *wnd = (wxWnd *)WinQueryWindowPtr(hWnd, QWL_USER);
	if (wnd)
	    return (wxFrame *)wnd->wx_window;
    }
    return NULL;
}

/*
 * Frame window
 *
 */

PFNWP wxFrameWnd::wxFrameProc = NULL;

wxFrameWnd::wxFrameWnd(void)
{
    __type = wxTYPE_FRAME_WND;
    hwndClient = NULLHANDLE;
    hwndStatusLine = NULLHANDLE;
    nitems = NULLHANDLE;
    swpClient.x = swpClient.y = swpClient.cx = swpClient.cy = 0;
printf("wxFrameWnd::wxFrameWnd: swpClient.x=%ld, swpClient.y=%ld, swpClient.cx=%ld, swpClient.cy=%ld\n", swpClient.x, swpClient.y, swpClient.cx, swpClient.cy);
}
		   
wxFrameWnd::wxFrameWnd(wxWnd *Parent, wxWindow *wx_win, const char *title,
                   int x, int y, int width, int height, long style)
{
    TRACE("wxFrameWnd::wxFrameWnd(Parent=%#x, wx_win=%#x, title='%s', x=%d, y=%d, width=%d, height=%d, style=%#x", Parent, wx_win, title, x, y, width, height, style);
    __type = wxTYPE_FRAME_WND;
    defaultIcon = (wxSTD_FRAME_ICON ? wxSTD_FRAME_ICON : wxDEFAULT_FRAME_ICON);
    Create(Parent, wxFrameClientClassName, wx_win, title, x, y, width, height, style);
}

void
wxFrameWnd::Create(wxWnd *parent, char *wclass, wxWindow *wx_win, const char *title,
                   int x, int y, int width, int height, long style)
{
    TRACE("wxFrameWnd::Create(parent=%#x, wclass='%s', wx_win=%#x, title='%s', x=%d, y=%d, width=%d, height=%d, style=%#x", parent, wclass, wx_win, title, x, y, width, height, style);
    __type = wxTYPE_FRAME_WND;

    DWORD os2_flags = FCF_NOBYTEALIGN | FCF_TASKLIST | FCF_TITLEBAR;
    if (style & wxMINIMIZE_BOX)
	os2_flags |= FCF_MINBUTTON;
    if (style & wxMAXIMIZE_BOX)
	os2_flags |= FCF_MAXBUTTON;
    if (style & wxTHICK_FRAME)
	os2_flags |= FCF_SIZEBORDER;
    if (style & wxSYSTEM_MENU)
	os2_flags |= FCF_SYSMENU;
    // Use shell positioning if the window is a top-level window without
    // x- and y- position given
    if ((x == 0 && y == 0) && parent == NULL)
	os2_flags |= FCF_SHELLPOSITION;
#if 0
    if (style & wxSTAY_ON_TOP)
	extendedStyle |= WS_EX_TOPMOST;
#endif
    icon = NULLHANDLE;
    iconized = FALSE;
    hwndClient = NULLHANDLE;
    hwndStatusLine = NULLHANDLE;

    wx_window = wx_win;
    HWND hwndParent = HWND_DESKTOP;
    if (parent)
	hwndParent = parent->GetClientHWND();

    //
    // We have to convert the coordinates from wxWindows origin
    // to PM origin
    //
    if (y > 0) {
	RECTL prcl;
	WinQueryWindowRect(hwndParent, &prcl);
	y = rcHeight(prcl) - y - height;
    }

    /*
     * Create frame window
     *
     * The frame data structure is needed
     * as parameters to WC_FRAME
     */
    FRAMECDATA fcd;
    fcd.cb = sizeof(FRAMECDATA);
    fcd.flCreateFlags = os2_flags;
    fcd.hmodResources = NULLHANDLE;
    fcd.idResources   = NULLHANDLE;

    ULONG ctldata[3];
    ctldata[0] = sizeof ctldata;
    ctldata[1] = (int)this;
    ctldata[2] = (int)&fcd;

    TRACE("Creating frame window, parent=%#x", (int)hwndParent);
    hwndWnd = WinCreateWindow(hwndParent, WC_FRAME, (PSZ)title,
				 0,
				 0, 0, 0, 0,
				 NULLHANDLE,
				 HWND_TOP, NewControlId(), &fcd, NULL);
    TRACE("Frame window handle=%#x", hwndWnd);

    /*
     * Setup the QWL_USER windows word to point to
     * the C++ wxFrameWnd class.
     */
    WinSetWindowPtr(hwndWnd, QWL_USER, this);

    /*
     * Subclass the frame
     */
    wxFrameProc = WinSubclassWindow(hwndWnd, ::wxFrameProc);

    /*
     * Now create Client window
     */
    TRACE("Creating client window, parent=%#x", (int)hwndWnd);
    hwndClient = WinCreateWindow(hwndWnd, (PSZ)wclass, NULL,
				  WS_VISIBLE,
				  0, 0, 0, 0, hwndWnd, 
				  HWND_BOTTOM, FID_CLIENT, (PVOID)ctldata, NULL);
    TRACE("Client window handle=%#x", (int)hwndClient);

    /*
     * Pick up default presentation parameters
     */
    background_colour = wxGetBackgroundColour(hwndClient);
    if (background_colour == -1)
	background_colour = wxGetSysColour(SYSCLR_BACKGROUND);
    TRACE("wxFrameWnd::Create: background_colour set to %#06x", background_colour);

    /*
     * Make the whole thing visible.
     * Before this point all setups should have been done.
     */
    if (os2_flags & FCF_SHELLPOSITION) {
	if (width != 0 && height != 0 && FALSE) {
	    /*
	     * Shell positioning but user sizing.
	     * Since we want preserve the frame top-left position, we have to relocate
	     * the y-position in addition to changing the height and width.
	     */
	    SWP swp;
	    WinQueryWindowPos(hwndWnd, &swp);
TRACE("swp.x=%d, swp.y=%d, swp.cx=%d, swp.cy=%d, width=%d, height=%d", swp.x, swp.y, swp.cx, swp.cy, width, height);
	    WinSetWindowPos(hwndWnd, HWND_TOP, swp.x, swp.y /* + swp.cy - height */, width, height,
			    SWP_MOVE | SWP_SIZE | SWP_ACTIVATE | SWP_SHOW);
	} else {
	    SWP swp;
	    WinQueryWindowPos(hwndWnd, &swp);
TRACE("(1)swp.x=%d, swp.y=%d, swp.cx=%d, swp.cy=%d, width=%d, height=%d", swp.x, swp.y, swp.cx, swp.cy, width, height);
	    /*
	     * The window is completely positioned and sized by the shell.
	     */
	    WinShowWindow(hwndWnd, TRUE);

	    WinQueryWindowPos(hwndWnd, &swp);
	    TRACE("(2) swp.x=%d, swp.y=%d, swp.cx=%d, swp.cy=%d, width=%d, height=%d", swp.x, swp.y, swp.cx, swp.cy, width, height);
	    // Hack to set the correct window size.
	    // Until I can figure out a way to do it before the WinShowWindow()
	    if (width != 0 && height != 0)
		WinSetWindowPos(hwndWnd, HWND_TOP, swp.x, swp.y - (height - swp.cy), width, height,
				SWP_MOVE | SWP_SIZE | SWP_ACTIVATE | SWP_SHOW);
	}
    } else {
	/*
	 * No shell positioning. A valid window position and size
	 * better be valid.
	 */
	WinSetWindowPos(hwndWnd, HWND_TOP, x, y, width, height, SWP_SHOW | SWP_ACTIVATE | SWP_SIZE | SWP_MOVE);
    }

    TRACE("hwndWnd=%#x, hwndClient=%#x", (int)hwndWnd, (int)hwndClient);

#if 0         
    // Seems to be necessary if we use WS_POPUP
    // style instead of WS_OVERLAPPED
    if (width > -1 && height > -1)
	::PostMessage(hwndWnd, WM_SIZE, SIZE_RESTORED, MAKELPARAM(width, height));
#endif    

    /*
     * Reformat the frame window.
     */
    int ccount = (int)WinSendMsg(hwndWnd, WM_QUERYFRAMECTLCOUNT, 0, 0);
    PSWP pswp = new SWP[ccount];
printf("wxFrameWnd::Create: swpClient.x=%ld, swpClient.y=%ld, swpClient.cx=%ld, swpClient.cy=%ld\n", swpClient.x, swpClient.y, swpClient.cx, swpClient.cy);
    WinSendMsg(hwndWnd, WM_FORMATFRAME, (MPARAM)pswp, &swpClient);
    TRACE("wxFrameWnd::Create Setting swpClient, x=%d, y=%d, cx=%d, cy=%d", swpClient.x, swpClient.y, swpClient.cx, swpClient.cy);
    delete[] pswp;

    SWP swp;
    // Set up swpClient to contain client window size and position
WinQueryWindowPos(hwndClient, &swpClient);
WinQueryWindowPos(hwndClient, &swp);
printf("wxFrameWnd::Create: (swpClient) swp.x=%ld, swp.y=%ld, swp.cx=%ld, swp.cy=%ld\n", swp.x, swp.y, swp.cx, swp.cy);
}

wxFrameWnd::~wxFrameWnd(void)
{
    TRACE("wxFrameWnd::~wxFramewnd() this=%#x, wxWnd=%#x", this, (wxWnd *)this);
    if (hwndStatusLine) {
	for (int i = 0; i < nitems; i++)
	    WinDestroyWindow(status_item[i]);
	WinDestroyWindow(hwndStatusLine);
    }
    WinDestroyWindow(hwndWnd);		// Will destroy client window also
    hwndWnd = NULLHANDLE;		// Stop looping
}

BOOL wxFrameWnd::OnPaint(HWND hwnd)
{
    printf("wxFrameWnd::OnPaint(hwnd=%#x)\n", (int)hwnd);

#if XXX
    if (iconized) {
	HPOINTER the_icon = icon;
	if (the_icon == 0)
	    the_icon = defaultIcon;

	// Erase background before painting or we get white background
	this->DefWindowProc(WM_ICONERASEBKGND,(WORD)ps.hps,0L);
      
	if (the_icon) {
	    RECTL rect;
	    wx_ClientRect(hwndWnd,&rect);

	    int icon_width = 32;
	    int icon_height = 32;
	    int icon_x = (int)((rect.xRight - icon_width)/2);
	    int icon_y = (int)((rect.yBottom - icon_height)/2);
	    WinDrawPointer(hpsUpdate, icon_x, icon_y, the_icon, DP_NORMAL);
	}
    }
#endif

    if (!iconized && wx_window)
	wx_window->GetEventHandler()->OnPaint();

    return 0;
}

HPOINTER wxFrameWnd::OnQueryDragIcon(void)
{
  if (icon != 0)
    return icon;
  else
    return defaultIcon;
}

void wxFrameWnd::OnSize(int x, int y, UINT id)
{
    TRACE("wxFrameWnd::OnSize(x=%d, y=%d, id=%d) (hwndWnd=%#x)", x, y, (int)id, hwndWnd);

#if 0
    switch (id) {
    case SIZEFULLSCREEN:
    case SIZENORMAL:
	iconized = FALSE;
	break;
    case SIZEICONIC:
	iconized = TRUE;
	break;
    }
#endif

    TRACE("iconized = %d", iconized);
    if (!iconized) {
	wxFrame *frame = (wxFrame *)wx_window;
#if 0
/* Should go away completely */
	if (frame && frame->status_window[0])
	    frame->PositionStatusWindow();
#endif
	TRACE("wx_window=%#x", wx_window);
	if (wx_window && wx_window->handle) {
	    TRACE("wx_window->handle=%#x", wx_window->handle);
	    wx_window->GetEventHandler()->OnSize(x, y);
	}
    }
}

BOOL wxFrameWnd::OnClose(void)
{
    TRACE("wxFrameWnd::OnClose()");
    if (wx_window) {
	if (wx_window->OnClose()) {
	    delete wx_window;
	    return TRUE;
	}
	else return FALSE;
    }
    return FALSE;
}

BOOL wxFrameWnd::OS2Command(int cmd, int source, int pointer)
{
    TRACE("wxFrameWnd::OS2Command(cmd=%#x, source=%d, pointer=%d)", cmd, source, pointer);
    if (source == CMDSRC_MENU || source == CMDSRC_ACCELERATOR) {
	wxFrame *frame = (wxFrame *)wx_window;
	if (frame->GetMenuBar() && frame->GetMenuBar()->FindItemForId(cmd)) {
	    ((wxFrame *)wx_window)->Command(cmd);
	    return TRUE;
	}
    }
    return FALSE;
}

void wxFrameWnd::OnMenuSelect(WORD nItem, WORD nFlags, HWND hSysMenu)
{
    TRACE("wxFrameWnd::OnMenuSelect(nItem=%d, nFlags=%#x, hSysMenu=%#x)", (int)nItem, nFlags, hSysMenu);
    wxFrame *frame = (wxFrame *)wx_window;
    if (nFlags == 0xFFFF && hSysMenu == NULLHANDLE)
	frame->OnMenuSelect(-1);
    else if (nFlags != MIS_SEPARATOR)
	frame->OnMenuSelect(nItem);
}

BOOL wxFrameWnd::ProcessMessage(QMSG* pMsg)
{
    TRACE("wxFrameWnd::ProcessMessage(pMsg=%#x)", pMsg);
#if 0        
  if (accelerator_table != NULL &&
          ::TranslateAccelerator(hwndWnd, accelerator_table, pMsg))
    return TRUE;
#endif        
  return FALSE;
}

MRESULT wxFrameWnd::DefWindowProc(ULONG msg, MPARAM mp1, MPARAM mp2)
{
    return WinDefWindowProc(hwndWnd, msg, mp1, mp2);
}

/*
 * Windows MDI stuff
 */

wxMDIFrameWnd::wxMDIFrameWnd(wxWnd *Parent, wxWindow *wx_win, const char *title,
		       int x, int y, int width, int height, long style)
{
    TRACE("wxMDIFrameWnd::wxMDIFrameWnd(Parent=%#x, wx_win=%#x, title='%s', x=%d, y=%d, width=%d, height=%d, style=%#x", Parent, wx_win, title, x, y, width, height, style);
    __type = wxTYPE_MDIFRAME_WND;
    defaultIcon = (wxSTD_MDIPARENTFRAME_ICON ? wxSTD_MDIPARENTFRAME_ICON : wxDEFAULT_MDIPARENTFRAME_ICON);
    parent_frame_active = TRUE;
    current_child = NULL;

    Create(Parent, wxMDIClientClassName, wx_win, title, x, y, width, height, style);

    window_menu = wxMakeWindowMenu(hwndWnd);
}

wxMDIFrameWnd::~wxMDIFrameWnd(void)
{
    TRACE("wxMDIFrameWnd::~wxMDIFrameWnd() this=%#x, wxWnd=%#x", this, (wxWnd *)this);
    WinDestroyWindow(hwndWnd);		// Will destroy client window also
    hwndWnd = NULLHANDLE;		// Stop looping

    TRACE("wxMDIFrameWnd::~wxMDIFrameWnd");
}

BOOL wxMDIFrameWnd::OnDestroy(void)
{
    TRACE("wxMDIFrameWnd::OnDestroy()");
    return FALSE;
}

void wxMDIFrameWnd::OnCreate(PCREATESTRUCT cs)
{
    TRACE("wxMDIFrameWnd::OnCreate(cs=%#x)", (int)cs);
#if 0
  CLIENTCREATESTRUCT ccs;
        
  ccs.hWindowMenu = window_menu;
  ccs.idFirstChild = wxFIRST_MDI_CHILD;

  hwndClient = ::CreateWindowEx(0, "mdiclient", NULL,
                WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN, 0, 0, 0, 0, hwndWnd, NULL,
                wxhAb, (LPSTR)(LPCLIENTCREATESTRUCT)&ccs);
#endif                
}

void wxMDIFrameWnd::OnSize(int x, int y, UINT id)
{
    TRACE("wxMDIFrameWnd::OnSize(x=%d, y=%d, id=%d)", x, y, (int)id);
#if 0
    switch (id)
    {
    case SIZEFULLSCREEN:
    case SIZENORMAL:
	iconized = FALSE;
	break;
    case SIZEICONIC:
	iconized = TRUE;
	break;
    }
#endif
    if (!iconized)
    {
	wxFrame *frame = (wxFrame *)wx_window;

#if 0
	if (frame && (frame->status_window[0] || frame->frameToolBar)) {
	    RECTL rect;

	    wx_ClientRect(hwndWnd, &rect);
	    int cwidth  = rcWidth(rect);
	    int cheight = rcHeight(rect);
	    int ctop = 0;
	    int tw, th;

	    if (frame->frameToolBar) {
		frame->frameToolBar->GetSize(&tw, &th);
		ctop = th;
		cheight -= th;
	    }

	    if (frame->status_window[0])
		cheight -= frame->status_window[0]->height;

	    WinSetWindowPos(hwndClient, HWND_TOP, 0, frame->status_window[0]->height, cwidth,
			    cheight - frame->status_window[0]->height, SWP_MOVE | SWP_SIZE);

	    if (frame->frameToolBar)
		frame->frameToolBar->SetSize(0, 0, cwidth, th);
#if 0
/* Should go away completely */
	    if (frame->status_window[0])
		frame->PositionStatusWindow();
#endif
	}
	else
#endif
	    (void)DefWindowProc(last_msg, last_mp1, last_mp2);

	if (wx_window && wx_window->handle)
	    wx_window->GetEventHandler()->OnSize(x, y);
    }
}

BOOL wxMDIFrameWnd::OS2Command(int cmd, int source, int pointer)
{
    TRACE("wxMDIFrameWnd::OS2Command(cmd=%#x. source=%d, pointer=%d)", cmd, source, pointer);

    if (source == CMDSRC_MENU) {
	switch (cmd) {
	case IDM_WINDOWCASCADE:
	    WinSendMsg(hwndClient, WM_MDICASCADE, 0 /*MDITILE_SKIPDISABLED*/, 0);
	    return TRUE;
	case IDM_WINDOWTILE:
	    WinSendMsg(hwndClient, WM_MDITILE, 0 /*MDITILE_HORIZONTAL*/, 0);
	    return TRUE;
	case IDM_WINDOWICONS:
	    WinSendMsg(hwndClient, WM_MDIICONARRANGE, 0, 0);
	    return TRUE;
	case IDM_WINDOWNEXT:
//        SendMsg(hwndClient, WM_MDINEXT, current_child->hwndWnd, 0);
	    WinSendMsg(hwndClient, WM_MDINEXT, 0, 0);
	    return TRUE;
	default:
	    break;
	}
	if (cmd >= 0xF000) {
	    return FALSE; // Get WndProc to call default proc
	}
    
	if (parent_frame_active && (cmd < wxFIRST_MDI_CHILD || cmd > wxLAST_MDI_CHILD)) {
	    ((wxFrame *)wx_window)->Command(cmd);
	    return TRUE;
	} else if (current_child && (cmd < wxFIRST_MDI_CHILD || cmd > wxLAST_MDI_CHILD)) {
/*
  ((wxFrame *)(current_child->wx_window))->Command(cmd);
  return TRUE;
  */
	    return current_child->OS2Command(cmd, source, pointer);
	}
    }
    return FALSE;
}

void wxMDIFrameWnd::OnMenuSelect(WORD nItem, WORD nFlags, HWND hSysMenu)
{
    if (parent_frame_active) {
	wxFrame *frame = (wxFrame *)wx_window;
	if (nFlags == 0xFFFF && hSysMenu == NULLHANDLE)
	    frame->GetEventHandler()->OnMenuSelect(-1);
	else if (nFlags != MIS_SEPARATOR)
	    frame->GetEventHandler()->OnMenuSelect(nItem);
    } else if (current_child) {
	current_child->OnMenuSelect(nItem, nFlags, hSysMenu);
    }
}

BOOL wxMDIFrameWnd::ProcessMessage(QMSG* pMsg)
{
    TRACE("wxMDIFrameWnd::ProcessMessage(pMsg=%#x)", pMsg);
    if ((current_child != NULL) && (current_child->hwndWnd != NULLHANDLE) && current_child->ProcessMessage(pMsg))
	return TRUE;
        
#if 0
  if (accelerator_table != NULL &&
          ::TranslateAccelerator(hwndWnd, accelerator_table, pMsg))
    return TRUE;
#endif    
      
#if 0  
  if (pMsg->msg == WM_KEYDOWN || pMsg->msg == WM_SYSKEYDOWN) {
    if (::TranslateMDISysAccel(hwndClient, pMsg))
      return TRUE;
  }
#endif

  return FALSE;
}

#if 0
// pgd: NOW in wxwnd::OnEraseBkgnd
BOOL wxMDIFrameWnd::OnEraseBkgnd(HPS ps, PRECTL pprcPaint)
{
    TRACE("wxMDIFrameWnd::OnEraseBkgnd(ps=%#x)", (int)ps);
    return FALSE;
}
#endif

wxMDIChildWnd::wxMDIChildWnd(wxMDIFrameWnd *Parent, wxWindow *wx_win, const char *title,
                   int x, int y, int width, int height, long style)
{
    TRACE("wxMDIChildWnd::wxMDIChildWnd(Parent=%#x, wx_win=%d, title='%s', x=%d, y=%d, width=%d, height=%d, style=%#x", Parent, wx_win, title, x, y, width, height, style);
    __type = wxTYPE_MDICHILD_WND;
    defaultIcon = (wxSTD_MDICHILDFRAME_ICON ? wxSTD_MDICHILDFRAME_ICON : wxDEFAULT_MDICHILDFRAME_ICON);

    active = FALSE;
    Create(Parent, wxMDIChildClientClassName, wx_win, title, x, y, width, height, style);

#if 0
    MDICREATESTRUCT mcs;
        
    mcs.szClass = wxMDIChildWndFrameClassName;
    mcs.szTitle = title;
    mcs.hOwner = wxhAb;
    if (x > -1) mcs.x = x;
    else mcs.x = CW_USEDEFAULT;

    if (y > -1) mcs.y = y;
    else mcs.y = CW_USEDEFAULT;

    if (width > -1) mcs.cx = width;
    else mcs.cx = CW_USEDEFAULT;

    if (height > -1) mcs.cy = height;
    else mcs.cy = CW_USEDEFAULT;
#endif
}

static HWND invalidHandle = 0;
void wxMDIChildWnd::OnSize(int x, int y, UINT id)
{
    TRACE("wxMDIChildWnd::Onsize(x=%d, y=%d, id=%#x", x, y, (int)id);
    if (!hwndWnd)
	return;

    if (invalidHandle == hwndWnd) {
	return;
    }
  
    (void)DefWindowProc(last_msg, last_mp1, last_mp2);
  
#if 0
    switch (id)
    {
    case SIZEFULLSCREEN:
    case SIZENORMAL:
	iconized = FALSE;
	break;
    case SIZEICONIC:
	iconized = TRUE;
	break;
    }
#endif
    if (!iconized)
    {
#if 0
	wxFrame *frame = (wxFrame *)wx_window;
/* Should go away completely */
	if (frame && frame->status_window[0])
	    frame->PositionStatusWindow();
#endif

	if (wx_window && wx_window->handle)
	    wx_window->GetEventHandler()->OnSize(x, y);

    }
}

BOOL wxMDIChildWnd::OnClose(void)
{
    TRACE("wxMDIChildWnd::OnClose()");
    if (wx_window && hwndWnd) {
	if (wx_window->GetEventHandler()->OnClose()) {
	    delete wx_window;
	    return TRUE;
	} else return FALSE;
    }
    return FALSE;
}

BOOL wxMDIChildWnd::OS2Command(int id, int cmd, int pointer)
{
    TRACE("wxMDIChildWnd::OS2Command(id=%#x, cmd=%d, pointer=%d)", id, cmd, pointer);
//  (void)DefWindowProc(last_msg, last_mp1, last_mp2);
    if ((cmd == CMDSRC_MENU) && hwndWnd) {
	((wxFrame *)wx_window)->Command(id);
	return TRUE;
    }
    return FALSE;
}

BOOL wxMDIChildWnd::ProcessMessage(QMSG *msg)
{
    TRACE("wxMDIChildWnd::ProcessMessage(msg=%#x)", (int)msg);
#if 0
    if (accelerator_table && handle) {
	wxFrame *parent = (wxFrame *)wx_window->GetParent();
	HWND parent_hwnd = ((wxWnd *)parent->handle)->hwndWnd;
	return ::TranslateAccelerator(parent_hwnd, accelerator_table, msg);
    }
#endif
    return FALSE;
}

BOOL wxMDIChildWnd::OnMDIActivate(BOOL bActivate, HWND one, HWND two)
{
    TRACE("wxMDIChildWnd::OnMDIActivate(bActivate=%d, one=%#x, two=%#x)", (int)bActivate, one, two);

    wxFrame *parent = (wxFrame *)wx_window->GetParent();
    wxFrame *child = (wxFrame *)wx_window;
    HWND parent_menu = parent->GetWinMenu();
    HWND child_menu = child->GetWinMenu();

    wxMDIFrameWnd *cparent = (wxMDIFrameWnd *)parent->handle;
    if (bActivate) {
	active = TRUE;
	cparent->current_child = this;
	if (child_menu)
	{
	    cparent->parent_frame_active = FALSE;
// XXX	    HWND subMenu = GetSubMenu(cparent->window_menu, 0);
      HWND subMenu = 0;
	    WinSendMsg(cparent->hwndClient, WM_MDISETMENU, (MPARAM)child_menu, (MPARAM)subMenu);
//	    ::DrawMenuBar(cparent->hwndWnd);
	}
	if (child)
	    child->GetEventHandler()->OnActivate(TRUE);
    } else {
	if (cparent->current_child == this)
	    cparent->current_child = NULL;
	if (child)
	    child->GetEventHandler()->OnActivate(FALSE);

	active = FALSE;
	if (parent_menu) {
	    cparent->parent_frame_active = TRUE;
// XXX	    HWND subMenu = GetSubMenu(cparent->window_menu, 0);
      HWND subMenu = 0;
	    WinSendMsg(cparent->hwndClient, WM_MDISETMENU, (MPARAM)parent_menu, (MPARAM)subMenu);
//	    ::DrawMenuBar(cparent->hwndWnd);
	}
    }
    wx_window->OnActivate(bActivate);

    return 0;
}

wxMDIChildWnd::~wxMDIChildWnd(void)
{
    TRACE("wxMDIChildWnd::~wxMDIChildWnd() this=%#x, wxWnd=%#x", this, (wxWnd *)this);
}

void wxMDIChildWnd::DestroyWindow(void)
{
    TRACE("wxMDIChildWnd::DestroyWindow()");

//    DetachWindowMenu();
//    invalidHandle = handle;

    wxFrame *parent = (wxFrame *)wx_window->GetParent();
    wxMDIFrameWnd *cparent = (wxMDIFrameWnd *)parent->handle;

    // Must make sure this handle is invalidated (set to NULL)
    // since all sorts of things could happen after the
    // child client is destroyed, but before the wxFrame is
    // destroyed.

// XXX    HWND oldHandle = (HWND)handle;
    HWND oldHandle = NULLHANDLE;
    WinSendMsg(cparent->hwndClient, WM_MDIDESTROY, (MPARAM)oldHandle, (MPARAM)0);
    invalidHandle = 0;

    if (hwndMenu) {
//	::DestroyMenu(hwndMenu);
	hwndMenu = 0;
    }
}


MRESULT EXPENTRY
wxStatusLineWndProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxFrameWnd *wnd;
    if (msg == WM_CREATE) {
	ULONG *ctlptr = (ULONG *)PVOIDFROMMP(mp1);
	wnd = (wxFrameWnd *)ctlptr[1];
TRACE("wxStatusLineWndProc: WM_CREATE, hWnd=%#x, wnd=%#x", (int)hwnd, (int)wnd);
	WinSetWindowPtr(hwnd, QWL_USER, wnd);
    } else
	wnd = (wxFrameWnd *)WinQueryWindowPtr(hwnd, QWL_USER);
#ifdef DEBUGPM
    wxTraceEvent("wxStatusLineWndProc", hwnd, msg, mp1, mp2, (PVOID)wnd);
#endif

    assert(wnd != NULL && hwnd != NULLHANDLE);

TRACE("wnd->__type=%s", wxGetTypeName(wnd->__type));

    switch (msg) {
    case WM_CREATE:
	return (MRESULT)0;

    case WM_PAINT: {
	RECTL rect;

	HPS hps = WinBeginPaint(hwnd, NULLHANDLE, &rect);

	WinFillRect(hps, &rect, CLR_PALEGRAY); // Paint background colour

	WinQueryWindowRect(hwnd, &rect);

	/*
	 * Draw border around each subwindow in the status area
	 */
	int i;
	TRACE("nitems of window %#x is %d hwnd=%#x", wnd, wnd->nitems, hwnd);
	for (i = 0; i < wnd->nitems; i++) {
	    SWP swp;
	    WinQueryWindowPos(wnd->status_item[i], &swp);
	    int x1 = swp.x - wxTHICK_LINE_WIDTH;
	    int y1 = swp.y - wxTHICK_LINE_WIDTH;
	    int x2 = swp.x + swp.cx;
	    int y2 = swp.y + swp.cy;

TRACE("Status item %d: x1=%d, y1=%d, x2=%d, y2=%d", i, x1, y1, x2, y2);

	    // Right and bottom white lines
	    GpiSetColor(hps, CLR_WHITE);
	    POINTL pnt;
	    // Bottom white line
	    pnt.x = x1;
	    pnt.y = y1;
	    GpiMove(hps, &pnt);
	    pnt.x = x2;
	    GpiLine(hps, &pnt);
	    // Right white line
	    pnt.y = y2;
	    GpiLine(hps, &pnt);

	    // Left and top grey lines
	    GpiSetColor(hps, CLR_DARKGRAY);
	    // Top grey line
	    pnt.x = x1;
	    GpiLine(hps, &pnt);
	    // Left grey line
	    pnt.y = y1 + wxTHICK_LINE_WIDTH;
	    GpiLine(hps, &pnt);
	}

	WinEndPaint(hps);
	return 0;
    }

    case WM_ERASEBACKGROUND:
	return (MRESULT)TRUE;
    }
    return WinDefWindowProc(hwnd, msg, mp1, mp2);
}
