/*
 * File:	wx_gauge.cc
 * Purpose:	Gauge implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1995, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_gauge.h"
#include "wx_itemp.h"


/*
 * Gauge
 */

#if USE_GAUGE 

IMPLEMENT_DYNAMIC_CLASS(wxGauge, wxItem)

wxGauge::wxGauge(void)
{
}

wxGauge::wxGauge(wxPanel *panel, const char *label,
                   int range, int x, int y, int width, int height,
                   long style, const char *name):
  wxbGauge(panel, label, range, x, y, width, height, style, name)
{
    Create(panel, label, range, x, y, width, height, style, name);
}

Bool wxGauge::Create(wxPanel *panel, const char *label,
                   int range, int x, int y, int width, int height,
                   long style, const char *name)
{
    TRACE("wxGauge::Create(panel=%#x, label='%s', range=%d, x=%d, y=%d, width=%d, height=%d, style=%ld, name='%s'", panel, label, range, x, y, width, height, style, name);
    SetName(name);
    if (panel) panel->AddChild(this);
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;

    windowStyle = style;

    SetAppropriateLabelPosition();
    panel->GetValidPosition(&x, &y);

    const char *cf;
    if (label)
	CreateLabel(panel->GetHWND(), label);

    ULONG sstyle = SLS_RIBBONSTRIP | SLS_READONLY | SLS_CENTER;
    if (windowStyle & wxHORIZONTAL)
	sstyle = SLS_HORIZONTAL | SLS_HOMELEFT;
    else
	sstyle = SLS_VERTICAL | SLS_HOMEBOTTOM;
  
    SLDCDATA sldcData;
    sldcData.cbSize = sizeof(SLDCDATA);
    sldcData.usScale1Increments = range;
    sldcData.usScale1Spacing = 0;
    sldcData.usScale2Increments = 0;
    sldcData.usScale2Spacing = 0;

    windows_id = (int)NewControlId();
    hwndItem = WinCreateWindow(panel->GetHWND(), WC_SLIDER, NULL,
				 sstyle, 0, 0, 0, 0, panel->GetHWND(),
				 HWND_TOP, windows_id, &sldcData, NULL);
    TRACE("wxGauge::Create: Window created, parent=%#x, handle=%#x", panel->GetHWND(), hwndItem);

    wxSetBackgroundColour(hwndItem, backColour);

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

    if (width == -1)
	width = DEFAULT_ITEM_WIDTH /*50*/;
    if (height == -1)
	height = DEFAULT_ITEM_HEIGHT /*50*/;
    SetSize(x, y, width, height);	// Will also default, so is above needed?
    Show();

    panel->AdvanceCursor(this);
    return TRUE;
}

wxGauge::~wxGauge(void)
{
    TRACE("wxGauge::~wxGauge() this=%#x wxWindow=%#x", this, (wxWindow *)this);
//    WinDestroyWindow(hwndItem);
}

void wxGauge::SetShadowWidth(int w)
{
    //SendMessage(hwndItem, ZYZG_SETWIDTH3D, w, 0);
}

void wxGauge::SetBezelFace(int w)
{
    //SendMessage(hwndItem, ZYZG_SETBEZELFACE, w, 0);
}

void wxGauge::SetRange(int r)
{
    //SendMessage(hwndItem, ZYZG_SETRANGE, r, 0);
}

void wxGauge::SetValue(int pos)
{
    WinSendMsg(hwndItem, SLM_SETSLIDERINFO,
	       MPFROM2SHORT(SMA_SLIDERARMPOSITION, SMA_INCREMENTVALUE),
	       MPFROMSHORT(pos));
}

int wxGauge::GetShadowWidth(void)
{
    // XXX return (int) SendMessage(hwndItem, ZYZG_GETWIDTH3D, 0, 0);
    return 0;
}

int wxGauge::GetBezelFace(void)
{
    // XXX return (int) SendMessage(hwndItem, ZYZG_GETBEZELFACE, 0, 0);
    return 0;
}

int wxGauge::GetRange(void)
{
    // XXX return (int) SendMessage(hwndItem, ZYZG_GETRANGE, 0, 0);
    return 0;
}

int wxGauge::GetValue(void)
{
    ULONG ulValue = (ULONG)WinSendMsg(
	hwndItem, SLM_QUERYSLIDERINFO,
	MPFROM2SHORT(SMA_SLIDERARMPOSITION, SMA_INCREMENTVALUE),
	NULL);
    return (int)ulValue;
}

void wxGauge::SetBackgroundColour(wxColour& col)
{
    TRACE("wxGauge::SetBackgroundColour(%#06x)", (COLOR)col);
    wxItem::SetBackgroundColour(col);
    if (hwndItem) {
	wxSetBackgroundColour(hwndItem, backColour);
    }
}

#endif
