/*
 * File:	wx_group.cc
 * Purpose:	Group box implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1995, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */


#include "wx_group.h"
#include "wx_itemp.h"

/*
 * Group box
 */
 
IMPLEMENT_DYNAMIC_CLASS(wxGroupBox, wxItem)

wxGroupBox::wxGroupBox(void)
{
}

wxGroupBox::wxGroupBox(wxPanel *panel, const char *label,
                   int x, int y, int width, int height,
                   long style, const char *name):
  wxbGroupBox(panel, label, x, y, width, height, style, name)
{
  Create(panel, label, x, y, width, height, style, name);
}

Bool wxGroupBox::Create(wxPanel *panel, const char *label,
                   int x, int y, int width, int height,
                   long style, const char *name)
{
    TRACE("wxGroupBox::Create(panel=%#x, label='%s', x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)panel, label, x, y, width, height, (int)style, name);
    SetName(name);
    if (panel) panel->AddChild(this);
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;

    windowStyle = style;

    panel->GetValidPosition(&x, &y);

    windows_id = (int)NewControlId();

    hwndItem = WinCreateWindow(panel->GetHWND(), WC_BUTTON, (PSZ)label, SS_GROUPBOX,
				 0, 0, 0, 0, panel->GetHWND(),
				 HWND_TOP, windows_id, NULL, NULL);
    TRACE("wxGroupBox::Create: Window created, parent=%#x, handle=%#x", (int)panel->GetHWND(), (int)hwndItem);

//    wxSetForegroundColour(hwndItem, &buttonColour);
//    wxSetBackgroundColour(hwndItem, &backColour);

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

    if (labelFont)
	wxSetWinFont(hwndItem, labelFont);

    SetSize(x, y, width, height);
//  WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);

//  Callback(Function);

    panel->AdvanceCursor(this);
    return TRUE;
}

wxGroupBox::~wxGroupBox(void)
{
    TRACE("wxGroupBox::~wxGroupBox() this=%#x wxWindow=%#x", this, (wxWindow *)this);
}

void wxGroupBox::SetLabel(char *label)
{
  WinSetWindowText(hwndItem, (PSZ)label);
}

char *wxGroupBox::GetLabel(void)
{
  WinQueryWindowText(hwndItem, 300, (PCH)wxBuffer);
  return wxBuffer;
}

void wxGroupBox::SetSize(int x, int y, int width, int height, int sizeFlags)
{
    TRACE("wxGroupBox::SetSize(x=%d, y=%d, w=%d, h=%d, sizeFlags=%#x)", x, y, width, height, sizeFlags);

    // If we're prepared to use the existing size, then...
    if (x == -1)
	x = currentX;
    if (y == -1)
	y = currentY;
	
    if (width == -1 && !(sizeFlags && wxSIZE_AUTO_WIDTH))
	width = currentW;
    if (height == -1 && !(sizeFlags & wxSIZE_AUTO_HEIGHT))
	height = currentH;

    char buf[300];

    float current_width;

    float cyf;

    WinQueryWindowText(hwndItem, 300, (PCH)buf);
    GetTextExtent(buf, &current_width, &cyf, NULL, NULL, buttonFont);
    if (width < 0)
	width = (int)(current_width + 3 * wxGetCharWidth(hwndItem)) ;
    if (height<0)
	height = (int)(cyf*EDIT_CONTROL_FACTOR) ;

    currentX = x;
    currentY = y;
    currentW = width;
    currentH = height;

    MoveWindow(x, y, width, height, TRUE);

    GetEventHandler()->OnSize(width, height);
}
