/*
 * File:        wx_item.cc
 * Purpose:        Panel item implementation
 * Author:        Julian Smart
 * Created:        1993
 * Updated:        August 1994
 * RCS_ID:      $Id: wx_item.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:        (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_itemp.h"
#include "wx_txt.h"

MRESULT EXPENTRY wxSubclassedGenericControlProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);

IMPLEMENT_ABSTRACT_CLASS(wxItem, wxWindow)

// Item members
wxItem::wxItem(void)
{
    TRACE("wxItem::wxItem()");
    oldWndProc = NULL;
    os2LastXPos = 0;
    os2LastYPos = 0;
    os2LastEvent = 0;
    hwndItem = NULLHANDLE;
// XXX    backGroundBrush = NULL;
    currentX = currentY = currentW = currentH = 0;
    buttonstate = 0L;

    hwndLabel = NULLHANDLE;
    label_width = label_height = 0;
    label_x = label_y = 0;

    labelColour = *wxBLACK;
    buttonColour = *wxBLACK;
    fixed_length_label_size = 0;
}

wxItem::~wxItem(void)
{
    TRACE("wxITem::~wxItem() this=%#x wxWindow=%#x", this, (wxWindow *)this);

#if XXX
  if (backgroundBrush) {
    ::DeleteObject(backgroundBrush);
    backgroundBrush = 0;
  }
#endif

    // item may be a menu, so check.
    wxObject *obj = (wxObject *)GetParent();
    if (obj && wxSubType(obj->__type, wxTYPE_PANEL)) {

	    // If we delete an item, we should initialize the parent panel,
	    // because it could now be invalid.
	    wxPanel *panel = (wxPanel *)GetParent();
	    if (panel) {
		panel->last_created = NULL;
		panel->cursor_x = PANEL_LEFT_MARGIN;
		panel->cursor_y = PANEL_TOP_MARGIN;
		panel->max_height = 0;
		panel->max_line_height = 0;
		panel->max_width = 0;
		panel->hSpacing = PANEL_HSPACING;
		panel->vSpacing = PANEL_VSPACING;
		panel->initial_hspacing = panel->hSpacing ;
		panel->initial_vspacing = panel->vSpacing ;
		panel->current_hspacing = panel->hSpacing ;
		panel->current_vspacing = panel->vSpacing ;

		panel->new_line = FALSE;
		panel->label_position = wxHORIZONTAL;
		panel->has_child = FALSE ;
		panel->last_created = 0 ;
	    }
	//printf("wxItem: GetHWND()=>%#x\n", (int)GetHWND());
    }

    WinDestroyWindow(hwndItem);
    TRACE("wxItem::~wxItem: Destroying window %#x", (int)hwndItem);
}

void wxItem::GetSize(int *width, int *height)
{
    TRACE("wxItem::GetSize(&width=%#x, &height=%#x)", (int)width, (int)height);
    wxRECTL rect = {-1, -1, -1, -1};

    wxFindMaxSize(hwndItem, &rect);
    if (hwndLabel)
	wxFindMaxSize(hwndLabel, &rect);
    *width  = rcWidth(rect);
    *height = rcHeight(rect);
    TRACE("wxItem::GetSize() returns width=%d, height=%d", *width, *height);
}

void wxItem::GetPosition(int *x, int *y)
{
    TRACE("wxItem::GetPosition(&x=%#x, &y=%#x)", (int)x, (int)y);

    wxWindow *parent = GetParent();

// Two different ways of calculating the same thing...
#if 0
    int x1, y1;
    RECTL prect1;
    SWP swp;
    WinQueryWindowPos(hwndItem, &swp);
    WinQueryWindowRect(WinQueryWindow(hwndItem, QW_PARENT), &prect1);
    *x = swp.x;
    *y = rcHeight(prect1) - (swp.y + swp.cy);
    wxWnd *xparent = (wxWnd *)parent->handle;
    TRACE("Window=%#x, Parent window=%#x, Client window=%#x", (int)hwndItem, (int)WinQueryWindow(hwndItem, QW_PARENT), (int)xparent->GetClientHWND());

    x1 = *x;
    y1 = *y;
#else
    wxRECTL rect = {-1, -1, -1, -1};

    wxFindMaxSize(hwndItem, &rect);

    if (hwndLabel)
	wxFindMaxSize(hwndLabel, &rect);

    // Since we now have the absolute screen coords,
    // if there's a parent we must subtract its top left corner
    wxPOINTL point = { rect.xLeft, rect.yTop };

    if (parent) {
	wxWnd *cparent = (wxWnd *)parent->handle;
	TRACE("\tCalling wxScreenToClient");
	wx_ScreenToClient(cparent->GetClientHWND(), &point);
    }
    *x = point.x;
    *y = point.y;
#endif
// If you like:
//    assert(*x == x1 && *y == y1);
    TRACE("wxItem::GetPosition returns x=%d, y=%d", *x, *y);
}

void wxItem::SetSize(int x, int y, int width, int height, int sizeFlags)
{
    TRACE("wxItem::SetSize(x=%d, y=%d, w=%d, h=%d, sizeFlags=%#x)", x, y, width, height, sizeFlags);

    int item_width = width, item_height = height;

    if (x == -1)
	x = currentX;
    if (y == -1)
	y = currentY;
    /*
     * Check for autosizing the item
     */
    Bool autosize = FALSE;
    if ((item_width == -1 || item_height == -1) && (sizeFlags && wxSIZE_AUTO))
	autosize = TRUE;
    else {
	/*
	 * Else use previous height and width
	 */
	if (item_width == -1)
	    item_width = currentW;
	if (item_height == -1)
	    item_height = currentH;
	if (item_width == -1 || item_height == -1)
	    autosize = TRUE;
    }

    if (autosize) {
	int icx, icy;
	GetAutoSize(&icx, &icy);
	if (item_width == -1 && (sizeFlags && wxSIZE_AUTO_WIDTH))
	    item_width = icx;
	if (item_height == -1 && (sizeFlags && wxSIZE_AUTO_HEIGHT))
	    item_height = icy;
	/*
	 * If we have a label, we have to reserve
	 * space for it also.
	 */
	if (hwndLabel) {
	    label_height = wxGetCharHeight(hwndLabel);
	    if (fixed_length_label_size)
		label_width = wxGetCharWidth(hwndLabel) * fixed_length_label_size;
	    else
		label_width = wxGetTextWidth(hwndLabel);
	    if (labelPosition == wxHORIZONTAL)
		label_width += wxGetCharWidth(hwndLabel);
	    
	    /*
	     * Now when we have label size, add it to item size
	     */
	    if (labelPosition == wxHORIZONTAL) {
		item_width += label_width;
		item_height = wxMax(item_height, label_height);
	    } else if (labelPosition == wxVERTICAL) {
		item_height += label_height;
		item_width = wxMax(item_width, label_width);
	    }
	}
    }

    /*
     * Save the new coordinates and size
     */
    currentX = x;
    currentY = y;
    int oldW = currentW, oldH = currentH;
    currentW = item_width;
    currentH = item_height;

    PositionItem(x, y, item_width, item_height);

    if (oldW != currentW || oldH != currentH)
	GetEventHandler()->OnSize(item_width, item_height);
}

/*
 * Position the item
 */
void wxItem::PositionItem(int x, int y, int width, int height)
{
    if (hwndLabel)
	AdjustLabelPos(&x, &y, &width, &height);
    MoveWindow(x, y, width, height, TRUE);
}

void wxItem::GetAutoSize(int *cx, int *cy)
{
    TRACE("wxItem::GetAutoSize()");
    *cx = DEFAULT_ITEM_WIDTH;
    *cy = DEFAULT_ITEM_HEIGHT;
}


void wxItem::SetClientSize(int width, int height)
{
    TRACE("wxItem::SetClientSize(w=%d, h=%d)", width, height);
    SetSize(-1, -1, width, height);
}

void wxItem::SetLabel(char *label)
{
    if (hwndLabel) {
	RefreshLabelSize(label);
	wx_MoveWindow(hwndLabel, label_x, label_y, label_width, label_height);
	WinSetWindowText(hwndLabel, (PSZ)label);
    } else if (hwndItem)
	WinSetWindowText(hwndItem, (PSZ)label);
}

char *wxItem::GetLabel(void)
{
    if (hwndLabel)
	WinQueryWindowText(hwndLabel, 300, (PCH)wxBuffer);
    else if (hwndItem)
	WinQueryWindowText(hwndItem, 1000, (PCH)wxBuffer);
    else
	return NULL;
    return wxBuffer;
}

void wxItem::SetFocus(void)
{
    wxWindow::SetFocus();
}

Bool wxItem::Show(Bool show)
{
    TRACE("wxItem::Show(show=%d)", show);
    if (show)
	WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE | SWP_ZORDER);
    else
	WinShowWindow(hwndItem, FALSE);
    if (hwndLabel)
	WinShowWindow(hwndLabel, show);
    return TRUE;
}

float wxItem::GetCharHeight(void)
{
    return (float)wxGetCharHeight(hwndItem);
}

float wxItem::GetCharWidth(void)
{
    return (float)wxGetCharWidth(hwndItem);
}

void wxItem::GetTextExtent(char *string, float *x, float *y,
                           float *descent, float *externalLeading,
                           wxFont *the_font)
{
    HPS ps = WinGetPS(hwndItem);
// XXX
#if 0
    HFONT fnt = 0;
    HFONT was = 0;

    if (the_font && (fnt=the_font->GetInternalFont(ps)))
	was = SelectObject(ps,fnt) ;
#endif
    POINTL      sizePt[TXTBOX_COUNT];
    FONTMETRICS fm;
    GpiQueryFontMetrics(ps, sizeof(FONTMETRICS), &fm);
    GpiQueryTextBox(ps, strlen(string), (PCH)string, TXTBOX_COUNT, sizePt);

#if 0
    if (the_font && fnt && was)
	SelectObject(ps,was) ;
#endif
    WinReleasePS(ps);

    *x = (float)(sizePt[TXTBOX_TOPRIGHT].x - sizePt[TXTBOX_TOPLEFT].x);
    *y = (float)(sizePt[TXTBOX_TOPRIGHT].y - sizePt[TXTBOX_BOTTOMRIGHT].y);
    if (descent) *descent = (float)fm.lMaxDescender;
    if (externalLeading) *externalLeading = (float)fm.lExternalLeading;
}

void wxItem::SetLabelFont(wxFont *font)
{
    TRACE("wxItem::SetLabelFont(font=%#x)", font);
    TRACE("wxItem::SetLabelFont: GetFamilyString() => '%s'", font->GetFamilyString());
    TRACE("wxItem::SetLabelFont: GetFaceName() => '%s'", font->GetFaceName());
    TRACE("wxItem::SetLabelFont: GetStyleString() => '%s'", font->GetStyleString());
    TRACE("wxItem::SetLabelFont: GetWeightString() => '%s'", font->GetWeightString());
    labelFont = font;
    
#if XXX
    HWND hWnd = GetLabelHWND();
    if (hWnd != 0) {
	if (font) {
	    if (font->GetResourceHandle())
		SendMessage(hWnd, WM_SETFONT,
			    (WPARAM)font->GetResourceHandle(),TRUE);
	}
    }
#endif
}

void wxItem::SetButtonFont(wxFont *font)
{
    TRACE("wxItem::SetButtonFont(font=%#x)", font);
    buttonFont = font;

    TRACE("wxItem::SetButtonFont(font=%#x)", font);
    TRACE("wxItem::SetButtonFont: GetFamilyString() => '%s'", font->GetFamilyString());
    TRACE("wxItem::SetButtonFont: GetFaceName() => '%s'", font->GetFaceName());
    TRACE("wxItem::SetButtonFont: GetStyleString() => '%s'", font->GetStyleString());
    TRACE("wxItem::SetButtonFont: GetWeightString() => '%s'", font->GetWeightString());

#if XXX
    HWND hWnd = GetHWND();
    if (hWnd != 0) {
	if (font)
	{
	    font->UseResource();
	    font->RealizeResource();
      
	    if (font->GetResourceHandle())
		SendMessage(hWnd, WM_SETFONT,
			    (WPARAM)font->GetResourceHandle(),TRUE);
	}
    }
#endif
}

void wxItem::Enable(Bool enable)
{
    winEnabled = enable;
    HWND hWnd = GetHWND();
    if (hWnd)
	WinEnableWindow(hWnd, (BOOL)enable);

    if (GetLabelHWND())
	WinEnableWindow(GetLabelHWND(), (BOOL)enable);
}

void wxItem::SubclassControl(HWND hwnd)
{
    // Subclass again for purposes of dialog editing mode
//    wxAddControlHandle(hWnd, this);
    WinSetWindowPtr(hwnd, QWL_USER, this);
    oldWndProc = WinSubclassWindow(hwnd, wxSubclassedGenericControlProc);
// wxEditControlSubClassProc
}

// Call this repeatedly for several wnds to find the overall size
// of the widget.
// Call it initially with -1 for all values in rect.
// Keep calling for other widgets, and rectl will be modified
// to calculate largest bounding rectangle.
void wxFindMaxSize(HWND wnd, wxRECTL *prect)
{
TRACE("wxFindMaxSize(wnd=%#x, {xLeft=%d, yBottom=%d, xRight=%d, yTop=%d})", (int)wnd, (int)prect->xLeft, (int)prect->yBottom, (int)prect->xRight, (int)prect->yTop);
	wxRECTL rectItem;
	wx_WindowRect(wnd, &rectItem);
	if (prect->xLeft == -1 || rectItem.xLeft < prect->xLeft)
		prect->xLeft = rectItem.xLeft;
	if (rectItem.xRight > prect->xRight)
		prect->xRight = rectItem.xRight;
	if (rectItem.yBottom > prect->yBottom)
		prect->yBottom = rectItem.yBottom;
	if (prect->yTop == -1 || rectItem.yTop < prect->yTop)
		prect->yTop = rectItem.yTop;
TRACE("wxFindMaxSize: xLeft=%d, yBottom=%d, xRight=%d, yTop=%d", (int)prect->xLeft, (int)prect->yBottom, (int)prect->xRight, (int)prect->yTop);
}

/*
// Not currently used
void wxConvertDialogToPixels(wxWindow *control, int *x, int *y)
{
  if (control->window_parent && control->window_parent->__type == wxTYPE_DIALOG_WND) {
    DWORD word = GetDialogBaseUnits();
    int xs = LOWORD(word);
    int ys = HIWORD(word);
    *x = (int)(*x * xs/4);
    *y = (int)(*y * ys/8);
  }
  else
  {
    *x = *x;
    *y = *y;
  }
}
*/

void wxItem::SetBackgroundColour(wxColour& col)
{
    TRACE("wxItem::SetBackgroundColour(%#06lx)", (LONG)col);
    wxItem::SetBackgroundColour(col);
    if (hwndLabel)
	wxSetBackgroundColour(hwndLabel, backColour);
}

void wxItem::SetButtonColour(wxColour& col)
{
    TRACE("wxItem::SetButtonColour(col=%#06lx)", (LONG)col);
    buttonColour = col;
}

void wxItem::SetLabelColour(wxColour& col) 
{
    TRACE("wxItem::SetLabelColour(col=%#06lx)", (LONG)col);
    labelColour = col;
    if (hwndLabel)
	wxSetForegroundColour(hwndLabel, col);
}

// Menus

// Sub-classed generic control proc
MRESULT EXPENTRY wxSubclassedGenericControlProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
//  wxItem *item = wxFindControlFromHandle(hwnd);
    wxItem *item = (wxItem *)WinQueryWindowPtr(hwnd, QWL_USER);

#ifdef DEBUGPM
    wxTraceEvent("wxSubclassedGenericControlProc", hwnd, msg, mp1, mp2, (PVOID)item);
#endif

    if (!item) {
	wxDebugMsg("Panic! Cannot find wxItem for this HWND in wxSubclassedGenericControlProc.\n");
	return (*item->oldWndProc)(hwnd, msg, mp1, mp2);
    }

    // If not in edit mode (or has been removed from parent), call the default proc.
    wxPanel *panel = (wxPanel *)item->GetParent();

    if (!panel)
	return (*item->oldWndProc)(hwnd, msg, mp1, mp2);
   
    // Special edit control processing
    if (!panel->GetUserEditMode() && (item->__type == wxTYPE_TEXT)) {
	switch (msg) {

#if XXX
	case WM_GETDLGCODE: {
	    if (item->GetWindowStyleFlag() & wxPROCESS_ENTER)
		return DLGC_WANTALLKEYS;
	    break;
	}
#endif

	case WM_CHAR: { // Always an ASCII character
	    if ((CHARMSG(&mp1)->fs & KC_VIRTUALKEY) && (CHARMSG(&mp1)->vkey == VK_NEWLINE)) {
		wxCommandEvent event(wxEVENT_TYPE_TEXT_ENTER_COMMAND);
		event.commandString = ((wxText *)item)->GetValue();
		event.eventObject = item;
		item->ProcessCommand(event);
		return FALSE;
	    }
	}
	break;

	case WM_SETFOCUS:
	    HWND hwnd = (HWND)mp1;
	    if (SHORT1FROMMP(mp2))
		item->GetEventHandler()->OnSetFocus();
	    else
		item->GetEventHandler()->OnKillFocus();
	    return (*item->oldWndProc)(hwnd, msg, mp1, mp2);
	}

    }

    if (!panel || !panel->GetUserEditMode())
	return (*item->oldWndProc)(hwnd, msg, mp1, mp2);

#if 0
    if (panel && panel->handle) {
	wxWnd *wnd = (wxWnd *)panel->handle;
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
    } else {
	int x = SHORT1FROMMP(mp1);
	int y = SHORT2FROMMP(mp2);
	HWND pwnd = WinQueryWindow(hwndItem, QW_PARENT);
	RECTL rcl;
	WinQueryWindowRect(hWnd, &rcl);
	y = rcHeight(rcl) - y;
    }
#endif

    int x = SHORT1FROMMP(mp1);
    int y = SHORT2FROMMP(mp2);
    int flags = SHORT2FROMMP(mp2);

    // If a mouse message, must convert X and Y to item coordinates
    // from HWND coordinates (the HWND may be part of the composite wxItem)
    if ((msg == WM_BUTTON1DOWN) || (msg == WM_BUTTON1UP) || (msg == WM_BUTTON1DBLCLK) || 
	(msg == WM_BUTTON2DOWN) || (msg == WM_BUTTON2UP) || (msg == WM_BUTTON2DBLCLK) || 
	(msg == WM_BUTTON3DOWN) || (msg == WM_BUTTON3UP) || (msg == WM_BUTTON3DBLCLK) || 
	(msg == WM_MOUSEMOVE)) {
	int ix, iy;
	item->GetPosition(&ix, &iy);
	RECTL rect;
	POINTL ptl = {x, y};
	HWND hwndItem = item->GetHWND();
	WinMapWindowPoints(hwnd, hwndItem, &ptl, 2L);
	WinQueryWindowRect(hwndItem, &rect);
	x = ptl.x;
	y = rcHeight(rect) - ptl.y;
    }

    switch (msg) {

    case WM_SETFOCUS:
	if (SHORT1FROMMP(mp2))
	    item->GetEventHandler()->OnSetFocus();
	else
	    item->GetEventHandler()->OnKillFocus();
	break;

    case WM_BUTTON2DOWN: {
	wxMouseEvent event(wxEVENT_TYPE_RIGHT_DOWN);

	item->buttonstate |= BUTTON2;

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX check out MSW version about faking events.
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_RIGHT_DOWN;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }
    case WM_BUTTON2UP: {
	wxMouseEvent event(wxEVENT_TYPE_RIGHT_UP);

	item->buttonstate &= ~BUTTON2;

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX Check out MSW version about faking events
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_RIGHT_UP;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }
    case WM_BUTTON2DBLCLK: {
	wxMouseEvent event(wxEVENT_TYPE_RIGHT_DCLICK);

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX Check out MSW version about faking events
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_RIGHT_DCLICK;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }

    case WM_BUTTON3DOWN: {
	wxMouseEvent event(wxEVENT_TYPE_MIDDLE_DOWN);

	item->buttonstate |= BUTTON3;
	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX Check out MSW version about faking events
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_MIDDLE_DOWN;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }

    case WM_BUTTON3UP: {
	wxMouseEvent event(wxEVENT_TYPE_MIDDLE_UP);

	item->buttonstate &= ~BUTTON3;

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX Check out MSW version about faking events
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_MIDDLE_UP;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }
    case WM_BUTTON3DBLCLK: {
	wxMouseEvent event(wxEVENT_TYPE_MIDDLE_DCLICK);

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX Check out MSW version about faking events
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_MIDDLE_DCLICK;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }
    case WM_BUTTON1DOWN: {
	wxMouseEvent event(wxEVENT_TYPE_LEFT_DOWN);

	item->buttonstate |= BUTTON1;

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX Check out MSW version about faking events
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_LEFT_DOWN;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }
    case WM_BUTTON1UP: {
	wxMouseEvent event(wxEVENT_TYPE_LEFT_UP);

	item->buttonstate &= ~BUTTON1;

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX Check out MSW version about faking events
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_LEFT_UP;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }
    case WM_BUTTON1DBLCLK: {
	wxMouseEvent event(wxEVENT_TYPE_LEFT_DCLICK);

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);

	event.x = x; event.y = y;

// XXX Check out MSW version about faking events
	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_LEFT_DCLICK;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }
    case WM_MOUSEMOVE: {
	wxMouseEvent event(wxEVENT_TYPE_MOTION);

	event.shiftDown = (flags & KC_SHIFT);
	event.controlDown = (flags & KC_CTRL);
	event.altDown = (flags & KC_ALT);
	event.leftDown = !!(item->buttonstate & BUTTON1);
	event.middleDown = !!(item->buttonstate & BUTTON3);
	event.rightDown = !!(item->buttonstate & BUTTON2);
//	event.leftDown = WinGetKeyState(HWND_DESKTOP, VK_BUTTON1);
//	event.middleDown = WinGetKeyState(HWND_DESKTOP, VK_BUTTON3);
//	event.rightDown = WinGetKeyState(HWND_DESKTOP, VK_BUTTON2);

	event.x = x; event.y = y;

	item->os2LastXPos = x; item->os2LastYPos = y;
	item->os2LastEvent = wxEVENT_TYPE_MOTION;
	item->GetEventHandler()->OnEvent(event);
	return (MPARAM)TRUE;
    }

    // Ensure that static items get messages
//    case WM_NCHITTEST:
//	return (long)HTCLIENT;

    default:
	return (*item->oldWndProc)(hwnd, msg, mp1, mp2);

    }
    return (*item->oldWndProc)(hwnd, msg, mp1, mp2);
}

/*
 * Create the a label to the item
 */
void
wxItem::CreateLabel(HWND hwndParent, const char *text, const Bool isFixedLength)
{
    TRACE("wxItem::CreateLabel(hwndParent=%#x, text='%s', isFixedLength=%d)", hwndParent, text, isFixedLength);
    if (text && isFixedLength)
	fixed_length_label_size = strlen(text);
    else
	fixed_length_label_size = 0;

    hwndLabel = WinCreateWindow(hwndParent, WC_STATIC, (PSZ)text, SS_TEXT,
				0, 0, 0, 0, hwndParent,
				HWND_TOP, NewControlId(), NULL, NULL);
    TRACE("wxItem::CreateLabel: Window created, parent=%#x, handle=%#x", (int)hwndParent, (int)hwndLabel);

    if (labelFont)
	wxSetWinFont(hwndLabel, labelFont);

TRACE("wxItem::CreateLabel: backColour=%#06x", (LONG)backColour);
    wxSetBackgroundColour(hwndLabel, backColour);
    wxSetForegroundColour(hwndLabel, labelColour);
//    wxSetBackgroundColour(hwndLabel, wxRED);
//    wxSetForegroundColour(hwndLabel, wxBLUE);
    RefreshLabelSize();
}


void
wxItem::RefreshLabelSize(const char *text = NULL)
{
    TRACE("wxItem::RefreshLabelSize('%s')", text);

    HPS ps = WinGetPS(hwndLabel);
    FONTMETRICS fm;
    GpiQueryFontMetrics(ps, sizeof(FONTMETRICS), &fm);
    label_height = fm.lMaxAscender + fm.lMaxDescender;
    if (fixed_length_label_size)
	label_width = fm.lAveCharWidth * fixed_length_label_size;
    else {
	char buf[300];
	if (text == NULL) {
	    WinQueryWindowText(hwndLabel, sizeof buf, (PCH)buf);
	    text = buf;
	}
	POINTL aptl[TXTBOX_TOPRIGHT+1];
	GpiQueryTextBox(ps, strlen(text), (PCH)text, TXTBOX_TOPRIGHT+1, aptl);
	label_width = aptl[TXTBOX_TOPRIGHT].x - aptl[TXTBOX_TOPLEFT].x;
    }
    if (labelPosition == wxHORIZONTAL)
	label_width += fm.lAveCharWidth;
    WinReleasePS(ps);
    TRACE("wxItem::RefreshLabelSize, label_height=%d, label_width=%d", label_height, label_width);
}

void
wxItem::AdjustLabelPos(int *x, int *y, int *width, int *height)
{
    TRACE("wxItem::AdjustLabelPos(x=%d, y=%d, width=%d, height=%d)", *x, *y, *width, *height);
    int cy = wxGetCharHeight(hwndLabel);
    if (hwndLabel) {
	if (labelPosition == wxHORIZONTAL) {
	    if (*height < 0)
		*height = wxMax(cy, label_height) * 15 / 10;
	    label_x = *x;
	    label_y = *y; // + (height - label_height)/2 // Center vertically
	    *x = label_x + label_width + 10;
	    if (*width >= 0)
		*width -= (*x - label_x);
	} else { // wxVERTICAL
	    label_x = *x;
	    label_y = *y;
	    *y = label_y + label_height; // +3 Allow for 3D border
	    if (*height >= label_height)
		*height -= label_height;
//	    if (*height <= 0)
//		*height = cy * 15 / 10;
//	    else
//		*height -= label_height + 3;
	}
	wx_MoveWindow(hwndLabel, label_x, label_y, label_width, label_height);
    }
}

/*
 * Allocates control IDs within the appropriate range
 */
int NewControlId(void)
{
    static int controlId = 0;
    controlId++;
    return controlId;
}
