/*
 * File:	wx_lbox.cc
 * Purpose:	List box implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_lbox.h"
#include "wx_itemp.h"

// Listbox item
BOOL wxListBox::OS2Control(WORD param, WORD WXUNUSED(id))
{
    wxCommandEvent event(wxEVENT_TYPE_LISTBOX_COMMAND);
/*
  if (param == LBN_SELCANCEL)
  {
  event.extraLong = FALSE;
  }
  */
    if (param == LN_SELECT)
    {
/*
  if (multiple != wxMULTIPLE) {
  event.commandInt = GetSelection();
  event.clientData = GetClientData(event.commandInt);
  event.commandString = copystring(GetStringSelection());
  }
  */
	int *liste = NULL;
	int count = GetSelections(&liste) ;
	if (count && liste) {
	    event.commandInt = liste[0] ;
	    event.clientData = GetClientData(event.commandInt);
	    event.commandString = copystring(GetString(event.commandInt));
	    char *s = GetString(event.commandInt);
	    if (s)
		event.commandString = copystring(s);
	} else {
	    event.commandInt = -1 ;
	    event.commandString = copystring("") ;
	}

	event.eventObject = this;
	ProcessCommand(event);
	if (event.commandString)
	    delete event.commandString ;
/*
  if (multiple != wxMULTIPLE)
  delete[] event.commandString;
  */
	return TRUE;
    } else if (param == LN_ENTER) {
	wxPanel *parent = (wxPanel *)GetParent();
	if (parent)
	    parent->GetEventHandler()->OnDefaultAction(this);
	return TRUE;
    }
    return FALSE;
}

IMPLEMENT_DYNAMIC_CLASS(wxListBox, wxItem)

wxListBox::wxListBox(void)
{
    no_items = 0;
}

wxListBox::wxListBox(wxPanel *panel, wxFunction func,
                       const char *Title, Bool Multiple,
                       int x, int y, int width, int height,
                       int N, char **Choices, long style, const char *name):
  wxbListBox(panel, func, Title, Multiple, x, y, width, height, N, Choices,
             style, name)
{
    Create(panel, func, Title, Multiple, x, y, width, height, N, Choices, style, name);
}

Bool wxListBox::Create(wxPanel *panel, wxFunction func,
                       const char *Title, Bool Multiple,
                       int x, int y, int width, int height,
                       int N, char **Choices, long style, const char *name)
{
    TRACE("wxListBox::Create(panel=%#x, func=%#x, Title='%s', Multiple=%d, x=%d, y=%d, w=%d, h=%d, N=%d, Choices=%#x, style=%#x, name='%s')", (int)panel, (int)func, Title, (int)Multiple, x, y, width, height, N, (int)Choices, (int)style, name);

    SetName(name);

    if (panel) panel->AddChild(this);

    /*
     * Default colours and font is
     * inherited from the panel
     */
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;

    multiple = Multiple & wxMULTIPLE_MASK;
    windowStyle = style;

//    labelPosition = panel->label_position;
    SetAppropriateLabelPosition();
    panel->GetValidPosition(&x, &y);

    // If label exists, create a static control for it.
    if (Title)
	CreateLabel(panel->GetHWND(), Title, style & wxFIXED_LENGTH);

    DWORD wstyle = 0;
    if (multiple == wxMULTIPLE)
	wstyle = LS_MULTIPLESEL | WS_TABSTOP;
    else if (multiple == wxEXTENDED)
	wstyle = LS_EXTENDEDSEL | WS_TABSTOP ;
    else
	wstyle = WS_TABSTOP;
#if 0
    if (multiple == wxMULTIPLE)
	wstyle = WS_VSCROLL | WS_BORDER | LBS_MULTIPLESEL | LBS_NOTIFY | WS_TABSTOP;
    else if (multiple == wxEXTENDED)
	wstyle = WS_VSCROLL | WS_BORDER | LBS_EXTENDEDSEL | LBS_NOTIFY | WS_TABSTOP ;
    else
	wstyle = WS_VSCROLL | WS_BORDER | LBS_NOTIFY | WS_TABSTOP;
    if ((Multiple&wxALWAYS_SB) || (style & wxALWAYS_SB))
	wstyle |= LBS_DISABLENOSCROLL ;
    if (style & wxHSCROLL)
	wstyle |= WS_HSCROLL;
#endif
    windows_id = (int)NewControlId();

    hwndItem =  WinCreateWindow(panel->GetHWND(), WC_LISTBOX, NULL, wstyle, 
				  0, 0, 0, 0, panel->GetHWND(),
				  HWND_TOP, windows_id, NULL, NULL);
    TRACE("wxListBox::Create: Window created, parent=%#x, handle=%#x", (int)panel->GetHWND(), (int)hwndItem);

//    wxSetBackgroundColour(hwndItem, &backColour);

    // Subclass for purposes of dialog editing mode
    SubclassControl(hwndItem);

    /*
     * Insert the listbox items into the newly created listbox
     */
    for (int i = 0; i < N; i++)
	(void)WinInsertLboxItem(hwndItem, LIT_END, (PSZ)Choices[i]);

    /*
     * Unless we have multiple selection mode,
     * select the first item as default
     */
    if (!Multiple)
	WinSendMsg(hwndItem, LM_SELECTITEM, 0L, (MPARAM)TRUE);

    no_items = N;

    if (buttonFont)
	wxSetWinFont(hwndItem, buttonFont);

    SetSize(x, y, width, height);
    Show();
    panel->AdvanceCursor(this);
    Callback(func);

    WinShowWindow(hwndItem, TRUE);
    return TRUE;
}

wxListBox::~wxListBox(void)
{
    TRACE("wxListBox::~wxListBox() this=%#x wxWindow=%#x", this, (wxWindow *)this);
    if (selections)
	delete[] selections;
}


void wxListBox::SetFirstItem(int N)
{
    WinSendMsg(hwndItem, LM_SETTOPINDEX, (MPARAM)N, (MPARAM)0) ;
}

void wxListBox::SetFirstItem(char *s)
{
    int N = FindString(s) ;

    if (N>=0)
	SetFirstItem(N) ;
}

void wxListBox::Delete(int N)
{
    (void)WinDeleteLboxItem(hwndItem, N);
    no_items --;
}

void wxListBox::Append(char *Item)
{
    (void)WinInsertLboxItem(hwndItem, LIT_END, (PSZ)Item);
    no_items ++;
}

void wxListBox::Append(char *Item, char *Client_data)
{
    TRACE("wxListBox::Append(Item='%s', Client_data=%#x)", Item, Client_data);
    int index = (int)WinInsertLboxItem(hwndItem, LIT_END, Item);
    WinSendMsg(hwndItem, LM_SETITEMHANDLE, MPFROMLONG(index), MPFROMLONG(Client_data));
    no_items++;
}

void wxListBox::Set(int n, char *choices[])
{
    WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_HIDE);
    for (int i = 0; i < n; i++)
	(void)WinInsertLboxItem(hwndItem, LIT_END, choices[i]);
    no_items = n;
    WinShowWindow(hwndItem, TRUE);
//  WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);
}

int wxListBox::FindString(char *s)
{
    TRACE("wxListBox::FindString(s='%s')", s);
    int pos = (int)WinSendMsg(hwndItem, LM_SEARCHSTRING, MPFROM2SHORT(LSS_CASESENSITIVE, LIT_FIRST), s);
    if (pos == LIT_ERROR || pos == LIT_NONE)
	return -1;
    else
	return pos;
}

void wxListBox::Clear(void)
{
    TRACE("wxListBox::Clear()");
    WinSendMsg(hwndItem, LM_DELETEALL, 0, 0);
    no_items = 0;
}

void wxListBox::SetSelection(int N, Bool select)
{
    TRACE("wxListBox::SetSelection(N=%d, select=%d)", N, (int)select);
    WinSendMsg(hwndItem, LM_SELECTITEM, (MPARAM)N, (MPARAM)select);
}

Bool wxListBox::Selected(int N)
{
    return WinQueryLboxSelectedItem(hwndItem) == N;

}

void wxListBox::Deselect(int N)
{
    WinSendMsg(hwndItem, LM_SELECTITEM, (MPARAM)N, (MPARAM)FALSE);
}

char *wxListBox::GetClientData(int N)
{
    TRACE("wxListBox::GetClientData(N=%d)", N);
    return (char *)WinSendMsg(hwndItem, LM_QUERYITEMHANDLE, MPFROMLONG(N), 0);
}

void wxListBox::SetClientData(int N, char *client_data)
{
    TRACE("wxListBox::SetClientData(N=%d, client_data='%s')", N, client_data);
    WinSendMsg(hwndItem, LM_SETITEMHANDLE, MPFROMLONG(N), MPFROMLONG(client_data));
}

// Return number of selections and an array of selected integers
// Use selections field to store data, which will be cleaned up
// by destructor if necessary.
int wxListBox::GetSelections(int **list_selections)
{
    TRACE("wxListBox::GetSelections(...)");

    if (selections) {
	delete[] selections; selections = NULL;
    };
    if (multiple == wxSINGLE) {
	int sel = (int)WinQueryLboxSelectedItem(hwndItem);
	if (sel == LIT_NONE)
	    return 0;
	selections = new int[1];
	selections[0] = sel;
	*list_selections = selections;
	return 1;
    } else {
	int nitems = WinQueryLboxCount(hwndItem);
	int *tmpselections = new int[nitems];
	int nsels = 0;
	int lastsel = 0;
	for (;;) {
	    int thissel = (int)WinSendMsg(hwndItem, LM_QUERYSELECTION, (MPARAM)lastsel, 0);
	    if (thissel == LIT_NONE)
		break;
	    tmpselections[nsels++] = thissel;
	}
	if (nsels > 0) {
	    selections = new int[nsels];
	    for (int i = 0; i < nsels; i++)
		selections[i] = tmpselections[i];
	}
	delete[] tmpselections;
	*list_selections = selections;
	return nsels;
    }
    return 0;
}

// Get single selection, for single choice list items
int wxListBox::GetSelection(void)
{
    if (selections) {
	delete[] selections; selections = NULL;
    };
    if (multiple == wxSINGLE) {
	int sel = (int)WinQueryLboxSelectedItem(hwndItem);
	if (sel == LIT_NONE)
	    return -1;
	else
	    return sel;
    }
    return -1;
}

// Find string for position
char *wxListBox::GetString(int N)
{
    if (N < 0 || N > no_items)
	return NULL;
    int len = (int)WinQueryLboxItemText(hwndItem, N, wxBuffer, 128);
    wxBuffer[len] = 0;
    return wxBuffer;
}

void wxListBox::InsertItems(int nItems, char **Items, int pos)
{
    for (int i = 0; i < nItems; i++) {
	(void)WinInsertLboxItem(hwndItem, pos + i, (PSZ)Items[i]);
	no_items++;
    }
}


void wxListBox::SetString(int N, char *s)
{
    char *oldData = (char *)wxListBox::GetClientData(N);
  
    WinSetLboxItemText(hwndItem, N, s);
    if (oldData)
	wxListBox::SetClientData(N, oldData);
}

