/*
 * File:        wx_main.cc
 * Purpose:        wxApp implementation
 * Author:        Julian Smart
 * Created:        1993
 * Updated:        August 1994
 * RCS_ID:      $Id: wx_main.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:        (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_mgstr.h"
#include "wx_frame.h"
#include "wx_main.h"
#include "wx_utils.h"
#include "wx_gdi.h"
#include "wx_dc.h"
#include "wx_dialg.h"
#include "wx_cmdlg.h"
#include "wx_rsrc.h"

#include "wx_privt.h"

#include <string.h>
#include <stdio.h>

// Status border dimensions
#define         wxTHICK_LINE_BORDER 3
#define         wxTHICK_LINE_WIDTH  1

#define	FID_STATUSLINE	0x8801

HAB wxhAb = 0;
extern PFNWP wxEditControlSubClassProc;

#if XXX
extern void wxSetKeyboardHook(Bool doIt);
#endif

char wxFrameClientClassName[]   = "wxFrameClientClass";
char wxMDIClientClassName[]     = "wxMDIClientClass";
char wxMDIChildClientClassName[]= "wxMDIChildClientClass";
char wxPanelClassName[]         = "wxPanelClass";
char wxCanvasClassName[]        = "wxCanvasClass";
char wxDialogClassName[]        = "wxDialogClass";
char wxStatusLineClassName[]    = "wxStatusLineClass";
char wxTimerClassName[]		= "wxTimerClass";

extern wxWnd *wxWndHook;

LHANDLE hPenOS2 = NULLHANDLE;
HPOINTER wxSTD_FRAME_ICON = NULLHANDLE;
HPOINTER wxSTD_MDICHILDFRAME_ICON = NULLHANDLE;
HPOINTER wxSTD_MDIPARENTFRAME_ICON = NULLHANDLE;

HPOINTER wxDEFAULT_FRAME_ICON = NULLHANDLE;
HPOINTER wxDEFAULT_MDICHILDFRAME_ICON = NULLHANDLE;
HPOINTER wxDEFAULT_MDIPARENTFRAME_ICON = NULLHANDLE;

FATTRS wxSTATUS_LINE_FONT;
HWND wxDesktopHandle = NULLHANDLE;

MRESULT EXPENTRY wxWndProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY wxDlgProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY wxStatusLineWndProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY wxTimerProc(HWND, ULONG, MPARAM, MPARAM);

//=======================================================================
// Window procedure for wxMDIClientClass
// ************ No Window Handle? *************
//=======================================================================
MRESULT EXPENTRY wxMDIClientProc(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxWnd *wnd;
    int width, height;

    if (msg == WM_CREATE) {
	ULONG *ctlptr = (ULONG *)PVOIDFROMMP(mp1);
	wnd = (wxWnd *)ctlptr[1];
	TRACE("wxMDIClientProc: WM_CREATE, hWnd=%#x, wnd=%#x", (int)hWnd, (int)wnd);
	WinSetWindowPtr(hWnd, QWL_USER, wnd);
	assert(wnd->hwndWnd != NULLHANDLE);
    } else
	wnd = (wxWnd *)WinQueryWindowPtr(hWnd, QWL_USER);

#ifdef DEBUGPM
    wxTraceEvent("wmMDIClientProc", hWnd, msg, mp1, mp2, (PVOID)wnd); 
#endif

    assert(wnd != NULL && hWnd != NULLHANDLE);

    wnd->last_msg = msg;
    wnd->last_mp1 = mp1;
    wnd->last_mp2 = mp2;

    //======================
    // Do message
    switch(msg) {
    case WM_ERASEBACKGROUND:
	return (MRESULT)TRUE;

    case WM_PAINT: {
	RECTL rectl;
	HPS hps = WinBeginPaint(hWnd, NULLHANDLE, 0);
	wxSetupPS(hps);
	WinQueryWindowRect(hWnd, &rectl);
	WinFillRect(hps, &rectl, *wxBLUE);
	WinEndPaint(hps);
	return 0L;
    }

    case WM_COMMAND: {
	USHORT uscmd = SHORT1FROMMP(mp1);
	USHORT ussource = SHORT1FROMMP(mp2);
	USHORT uspointer = SHORT1FROMMP(mp2);

	if (!wnd->OS2Command(uscmd, ussource, uspointer))
	    return WinDefWindowProc(hWnd, msg, mp1, mp2);
	return (MRESULT)0L;
    }

    case WM_MDICASCADE:
//	CascadeChildWindows(hWnd, mp1);
	break;
    case WM_MDITILE:
//	TileChildWindows(hWnd, mp1);
	break;
    case WM_MDIICONARRANGE:
//	ArrangeIcons(hWnd);
	break;
    case WM_MDINEXT:
//	SwitchToNextChildWindow();
	break;

    case WM_SIZE: {
	/*
	 * It appears that this message is never received,
	 * instead WM_WINDOWPOSCHANGED is received.
	 * I dunno.
	 */
//	assert(FALSE);
//	ULONG style = WinQueryWindowULong(hWnd, QWL_STYLE);
	int oldwidth = SHORT1FROMMP(mp1);
	int oldheight = SHORT2FROMMP(mp1);
	width  = SHORT1FROMMP(mp2);
	height = SHORT2FROMMP(mp2);
	if (width != oldwidth || height != oldheight)
	    goto sizechange;
//	wnd->OnSize(width, height, style);
	break;
    }

    case WM_WINDOWPOSCHANGED: {
	PSWP pswp = (PSWP)PVOIDFROMMP(mp1);
	if ((pswp->fl & SWP_SIZE) == 0)
	    break;
	width  = pswp->cx;
	height = pswp->cy;
	goto sizechange;
    }

    sizechange: {
	ULONG style = WinQueryWindowULong(hWnd, QWL_STYLE);
	if (height != wnd->win_height) {
	    /*
	     * The height of the window has changed.
	     * Since the wxWindows and PM coordinate systems
	     * are different, we have to move all child
	     * windows to the correct y-position since when resizing
	     * the child windows will retain the position from the top of
	     * the frame window.
	     */
	    int dh = height - wnd->win_height;
	    HENUM henum = WinBeginEnumWindows(hWnd);
	    while (HWND hwnd = WinGetNextWindow(henum)) {
		SWP swp;
		WinQueryWindowPos(hwnd, &swp);
		WinSetWindowPos(hwnd, 0, swp.x, swp.y + dh, 0, 0, SWP_MOVE);
	    }
	}
	wnd->win_height = height;
	wnd->win_width = width;
	TRACE("wxChildProc: win_height=%d, win_width=%d", wnd->win_height, wnd->win_width);
	
	RECTL rectlFrame;
	WinQueryWindowRect(wnd->hwndWnd, &rectlFrame);
	wnd->OnSize(rcWidth(rectlFrame), rcHeight(rectlFrame), style);
	break;
    }

    default:
	return WinDefWindowProc(hWnd, msg, mp1, mp2);
    };
    return 0L;
};

//=======================================================================
// Window procedure for wxFrameClientClass, wxMDIChildClientClass
//=======================================================================
MRESULT EXPENTRY wxChildProc(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxWnd *wnd;
    int width, height;
    if (msg == WM_CREATE) {
	ULONG *ctlptr = (ULONG *)PVOIDFROMMP(mp1);
	wnd = (wxWnd *)ctlptr[1];
	TRACE("wxChildProc: WM_CREATE, hWnd=%#x, wnd=%#x", (int)hWnd, (int)wnd);
	WinSetWindowPtr(hWnd, QWL_USER, wnd);
	assert(wnd->hwndWnd != NULLHANDLE);
    } else
	wnd = (wxWnd *)WinQueryWindowPtr(hWnd, QWL_USER);

#ifdef DEBUGPM
    wxTraceEvent("wxChildProc", hWnd, msg, mp1, mp2, (PVOID)wnd);
#endif

    assert(wnd != NULL && hWnd != NULLHANDLE);

    wnd->last_msg = msg;
    wnd->last_mp1 = mp1;
    wnd->last_mp2 = mp2;

    //======================
    // Do message

    switch(msg) {
    case WM_ERASEBACKGROUND:
	return (MRESULT)TRUE;

    case WM_COMMAND: {
	USHORT uscmd = SHORT1FROMMP(mp1);
	USHORT ussource = SHORT1FROMMP(mp2);
	USHORT uspointer = SHORT1FROMMP(mp2);

	if (!wnd->OS2Command(uscmd, ussource, uspointer))
	    return WinDefWindowProc(hWnd, msg, mp1, mp2);
	return (MRESULT)0L;
    }

    case WM_PAINT: {
	RECTL rectl;
	HPS hps = WinBeginPaint(hWnd, NULLHANDLE, 0);
	wxSetupPS(hps);
	WinQueryWindowRect(hWnd, &rectl);
	WinFillRect(hps, &rectl, *wxLIGHT_GREY);
	WinEndPaint(hps);
	return 0L;
    }

    case WM_SIZE: {
	/*
	 * It appears that this message is never received,
	 * instead WM_WINDOWPOSCHANGED is received.
	 * I dunno.
	 */
//	assert(FALSE);
//	ULONG style = WinQueryWindowULong(hWnd, QWL_STYLE);
	int oldwidth = SHORT1FROMMP(mp1);
	int oldheight = SHORT2FROMMP(mp1);
	width  = SHORT1FROMMP(mp2);
	height = SHORT2FROMMP(mp2);
	if (width != oldwidth || height != oldheight)
	    goto sizechange;
//	wnd->OnSize(width, height, style);
	break;
    }

    case WM_WINDOWPOSCHANGED: {
	PSWP pswp = (PSWP)PVOIDFROMMP(mp1);
	if ((pswp->fl & SWP_SIZE) == 0)
	    break;
	width  = pswp->cx;
	height = pswp->cy;
	goto sizechange;
    }

    sizechange: {
	ULONG style = WinQueryWindowULong(hWnd, QWL_STYLE);
	if (height != wnd->win_height) {
	    /*
	     * The height of the window has changed.
	     * Since the wxWindows and PM coordinate systems
	     * are different, we have to move all child
	     * windows to the correct y-position since when resizing
	     * the child windows will retain the position from the top of
	     * the frame window.
	     */
	    int dh = height - wnd->win_height;
	    HENUM henum = WinBeginEnumWindows(hWnd);
	    while (HWND hwnd = WinGetNextWindow(henum)) {
		SWP swp;
		WinQueryWindowPos(hwnd, &swp);
		WinSetWindowPos(hwnd, 0, swp.x, swp.y + dh, 0, 0, SWP_MOVE);
	    }
	}
	wnd->win_height = height;
	wnd->win_width = width;
	TRACE("wxChildProc: win_height=%d, win_width=%d", wnd->win_height, wnd->win_width);
	
	RECTL rectlFrame;
	WinQueryWindowRect(wnd->hwndWnd, &rectlFrame);
	wnd->OnSize(rcWidth(rectlFrame), rcHeight(rectlFrame), style);
	break;
    }

    case WM_PRESPARAMCHANGED: {
	ULONG idAttrType = LONGFROMMP(mp1);

	switch (idAttrType) {

	case PP_BACKGROUNDCOLOR:
	    wnd->OS2BackgroundColourChanged();
	    break;

	case PP_FONTNAMESIZE:
	    wnd->OS2FontChanged();
	    break;
	}
	return 0;
    }
	
    default:
	return WinDefWindowProc(hWnd, msg, mp1, mp2);
    };
    return 0L;
};


//=======================================================================
// Window procedure for wxPanelClass
//=======================================================================
#if 0
MRESULT EXPENTRY wxPanelProc(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxWnd *wnd = (wxWnd *)WinQueryWindowPtr(hWnd, QWL_USER);
    assert(wnd != NULL);

    if (wnd && !wnd->hwndWnd) {
	wnd->hwndWnd = hWnd;
	MRESULT res = wnd->DefWindowProc(msg, mp1, mp2);
	wnd->hwndWnd = NULLHANDLE;
	return res;
    }

    if (wnd) {
	wnd->last_msg    = msg;
	wnd->last_mp1 = mp1;
	wnd->last_mp2 = mp2;
    }
    else {
	return WinDefWindowProc(hWnd, msg, mp1, mp2);
    }

    //======================
    // Do message
    switch(msg) {

    case WM_COMMAND: {
	USHORT uscmd = SHORT1FROMMP(mp1);
	USHORT ussource = SHORT1FROMMP(mp2);
	USSHORT uspointer = SHORT1FROMMP(mp2);

	if (!wnd->OS2Command(uscmd, ussource, uspointer))
	    return WinDefWindowProc(hWnd, msg, mp1, mp2);
	return (MRESULT)0L;
    }

    case WM_PAINT:
	WinBeginPaint( etc, etc );
	if (wnd->OnPaint(hWnd))
	    return 0L;
	return WinDefWindowProc(hWnd, msg, mp1, mp2);
	  
    default:
	return WinDefWindowProc(hWnd, msg, mp1, mp2);
    };
};
#endif


//=======================================================================
// Window procedure for wxCanvasClass (subclassed Frame class)
// ************FRAME*********
//=======================================================================
#if 0
MRESULT EXPENTRY wxCanvasProc(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxWnd *wnd = (wxWnd *)WinQueryWindowPtr(hWnd, QWL_USER);
    assert(wnd != NULL);

    if (wnd && !wnd->hwndWnd) {
	wnd->hwndWnd = hWnd;
	MRESULT res = wnd->DefWindowProc(msg, mp1, mp2);
	wnd->hwndWnd = NULLHANDLE;
	return res;
    }

    if (wnd) {
	wnd->last_msg    = msg;
	wnd->last_mp1 = mp1;
	wnd->last_mp2 = mp2;
    } else
	return wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2);

    //======================
    // Do message
    switch(msg) {
    case WM_ERASEBACKGROUND:
	return (MRESULT)TRUE;

    case WM_COMMAND: {
	USHORT uscmd = SHORT1FROMMP(mp1);
	USHORT ussource = SHORT1FROMMP(mp2);
	USSHORT uspointer = SHORT1FROMMP(mp2);

	if (!wnd->OS2Command(uscmd, ussource, uspointer))
	    return wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2);
	return (MRESULT)0L;
    }
    case WM_PAINT: {
	WinBeginPaint(etc, etc);
	if (wnd->OnPaint(hWnd))
	    return 0L;
	return wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2);
    }
    default:
	return wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2);
    };
};
#endif

//=======================================================================
// Window procedure for wxFrameClass
// ************FRAME*********
//=======================================================================
MRESULT EXPENTRY wxFrameProc(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxFrameWnd *wnd = (wxFrameWnd *)WinQueryWindowPtr(hWnd, QWL_USER);

#ifdef DEBUGPM
    wxTraceEvent("wxFrameProc", hWnd, msg, mp1, mp2, (PVOID)wnd);
#endif

    assert(wnd != NULL && hWnd != NULLHANDLE);

    wnd->last_msg    = msg;
    wnd->last_mp1 = mp1;
    wnd->last_mp2 = mp2;

    //======================
    // Do message
    switch(msg) {
    case WM_WINDOWPOSCHANGED: {
	MRESULT rc = wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2);

	PSWP pSwp = (PSWP)mp1;

	if (pSwp->fl & SWP_SIZE) {
	    wnd->OnSize(pSwp->cx, pSwp->cy, 0);
	}
	return rc;
    }

#if 0
    case WM_COMMAND:
    {
	USHORT uscmd = SHORT1FROMMP(mp1);
	USHORT ussource = SHORT1FROMMP(mp2);
	USSHORT uspointer = SHORT1FROMMP(mp2);

	if (!wnd->OS2Command(uscmd, ussource, uspointer))
	    return wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2);
	return (MRESULT)0L;
    }
#endif

    case WM_QUERYFRAMECTLCOUNT:
	return wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2) + 1;
	break;

    case WM_FORMATFRAME: {
	PSWP	pswp = (PSWP)PVOIDFROMMP(mp1);
	PRECTL	pprectl = (PRECTL)PVOIDFROMMP(mp2);
	int i;

	/*
	 * Call WC_FRAME class WM_FORMATFRAME message handler
	 */
	MRESULT rc = wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2);
	int 	ccount = SHORT1FROMMR(rc);

	/*
	 * If we have a status line, we have to rearrange the
	 * client window to leave place for the status line
	 */
	if (wnd->hwndStatusLine && !(WinQueryWindowULong(wnd->GetFrameHWND(), QWL_STYLE) & WS_MINIMIZED)) {
	    SWP	swpStatusLine;		/* Position/size of Info Status Line */

	    /*
	     * Find the swp entry for the client window.
	     */
	    int clienti = -1;
	    for (i = ccount-1; i >= 0; i--) {
		if (WinQueryWindowUShort(pswp[i].hwnd, QWS_ID) == FID_CLIENT) {
		    clienti = i;
		    break;
		}
	    }

	    /*
	     * As a first approximation, move up the client window,
	     * and assign space for the info area window below
	     * the client window.
	     */
	    assert(clienti != -1);
	    WinQueryWindowPos(wnd->hwndStatusLine, &swpStatusLine);
	    int cymenu = WinQuerySysValue(HWND_DESKTOP, SV_CYMENU) + 2*wxTHICK_LINE_BORDER;
	    printf("cymenu=%d\n", cymenu);
	    BOOL changed = swpStatusLine.cy != cymenu || swpStatusLine.cx != pswp[clienti].cx;
	    swpStatusLine = pswp[clienti];
	    swpStatusLine.hwnd = wnd->hwndStatusLine;
	    swpStatusLine.cy = cymenu;
	    pswp[clienti].y += cymenu;
	    pswp[clienti].cy -= cymenu;
	    if (pswp[clienti].cy < 0)
		pswp[clienti].cy = 0;

	    /*
	     * Move up the client window
	     * to make place for the Status Line window, and
	     * move up all secondary frame windows located
	     * below the client window (the horizontal
	     * scroll bar).
	     */
	    for (i = 0; i < ccount-1; i++) {
		if (pswp[i].y < pswp[clienti].y && pswp[i].x < pswp[clienti].x + pswp[clienti].cx) {
		    if (pswp[i].y < swpStatusLine.y)
			swpStatusLine.y = pswp[i].y;
		    pswp[i].y += cymenu;
		}
	    }

	    /*
	     * Now we have the final position and size of the
	     * client window, and the final y position of the
	     * Status Line window, see if any
	     * secondary frame window extends below and to the
	     * right of the client window (the vertical
	     * scroll bar), and extend the info area window if so.
	     */
	    for (i = 0; i < ccount-1; i++) {
		if (pswp[i].y < pswp[clienti].y && pswp[i].x >= pswp[clienti].x + pswp[clienti].cx) {
		    if (swpStatusLine.x + swpStatusLine.cx < pswp[i].x + pswp[i].cx)
			swpStatusLine.cx = pswp[i].x + pswp[i].cx - swpStatusLine.x;
		    pswp[i].y += cymenu;
		    pswp[i].cy -= cymenu;
		    if (pswp[i].cy < 0)
			pswp[i].cy = 0;
		}
	    }

	    /*
	     * If the size has changed, position the sub-windows of status-line also.
	     */
	    if (changed) {
//		int borderwidth = WinQuerySysValue(HWND_DESKTOP, SV_CXBORDER);
//		int borderheight = WinQuerySysValue(HWND_DESKTOP, SV_CYBORDER);
		int borderwidthLR = wxTHICK_LINE_BORDER * 2 + wxTHICK_LINE_WIDTH;
		int borderheight = wxTHICK_LINE_BORDER + wxTHICK_LINE_WIDTH;
		int borderwidthM = wxTHICK_LINE_BORDER + wxTHICK_LINE_WIDTH * 2;
		int indent = 0; /* wxTHICK_LINE_BORDER * 2 */;
		int subwidth = (swpStatusLine.cx - borderwidthLR * 2 - indent) / wnd->nitems;
		for (i = 0; i < wnd->nitems; i++) {
		    WinSetWindowPos(wnd->status_item[i], NULLHANDLE,
				    borderwidthLR + indent + i * (subwidth + borderwidthM),
				    borderheight,
				    subwidth - borderwidthM,
				    swpStatusLine.cy - 2*borderheight,
				    SWP_SIZE | SWP_MOVE);
		}

		/*
		 * Set up the SWP entry for the Status Line
		 */
		pswp[ccount++] = swpStatusLine;

		/*
		 * Remember size of client space/window
		 */
		if (clienti >= 0) {
		    wnd->swpClient = pswp[clienti];
		    TRACE("wxFrameProc: Setting swpClient, x=%d, y=%d, cx=%d, cy=%d", wnd->swpClient.x, wnd->swpClient.y, wnd->swpClient.cx, wnd->swpClient.cy);
		}
	    }
	}
	return MPFROMLONG(ccount);
    }

    case WM_ADJUSTWINDOWPOS: {
	PSWP pswp = (PSWP)mp1;
	TRACE("WM_ADJUSTWINDOWPOS: pswp->fl=%#x pswp->hwnd=%#x pswp->x=%d pswp->y=%d pswp->cx=%d pswp->cy=%d", pswp->fl, pswp->hwnd, pswp->x, pswp->y, pswp->cx, pswp->cy);
	goto deflt;
    }

    case WM_ADJUSTFRAMEPOS: {
	PSWP pswp = (PSWP)mp1;
	TRACE("WM_ADJUSTFRAMEPOS: pswp->fl=%#x pswp->hwnd=%#x pswp->x=%d pswp->y=%d pswp->cx=%d pswp->cy=%d", pswp->fl, pswp->hwnd, pswp->x, pswp->y, pswp->cx, pswp->cy);
	goto deflt;
    }

    /*
     * Called to get the associated icon for a frame window.
     */
    case WM_QUERYICON:
	return (HPOINTER)wnd->icon;

    default:
    deflt:
	return wxFrameWnd::wxFrameProc(hWnd, msg, mp1, mp2);
    };
    return 0L;
};


void wxInitialize(HAB hInstance)
{
    wxCommonInit();

    wxDesktopHandle = WinQueryDesktopWindow(wxhAb, NULLHANDLE);

    wxSTD_MDICHILDFRAME_ICON  = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, _wxSTD_MDICHILDFRAME);
    wxSTD_MDIPARENTFRAME_ICON = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, _wxSTD_MDIPARENTFRAME);
    wxDEFAULT_FRAME_ICON          = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, _wxDEFAULT_FRAME);
    wxDEFAULT_MDIPARENTFRAME_ICON = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, _wxDEFAULT_MDIPARENTFRAME);
    wxDEFAULT_MDICHILDFRAME_ICON  = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, _wxDEFAULT_MDICHILDFRAME);

    strcpy(wxSTATUS_LINE_FONT.szFacename, "Helv");
//  wxSTATUS_LINE_FONT.szFacename      = "Helv";
    wxSTATUS_LINE_FONT.usRecordLength  = sizeof(FATTRS);
    wxSTATUS_LINE_FONT.fsSelection     = 0;
    wxSTATUS_LINE_FONT.lMatch          = 0L;
    wxSTATUS_LINE_FONT.idRegistry      = 0;
    wxSTATUS_LINE_FONT.usCodePage      = 850;
    wxSTATUS_LINE_FONT.lMaxBaselineExt = 16;
    wxSTATUS_LINE_FONT.lAveCharWidth   = 12;
    wxSTATUS_LINE_FONT.fsType          = 0;
    wxSTATUS_LINE_FONT.fsFontUse       = FATTR_FONTUSE_NOMIX;

#if 0
    wxSTATUS_LINE_FONT = CreateFont(16, 0, 0, 0, FW_NORMAL, 0, 0, 0,
				    ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
				    PROOF_QUALITY, DEFAULT_PITCH | FF_SWISS,
				    "Arial");
#endif                    

    CLASSINFO cInfo;

    //===============================================
    // Register the frame window class.
    WinQueryClassInfo(hInstance, WC_FRAME, &cInfo);

    // Save old frame window procedure
    wxFrameWnd::wxFrameProc = cInfo.pfnWindowProc;

    // Register the frame client window class
    WinRegisterClass(hInstance, (PSZ)wxFrameClientClassName, wxChildProc, CS_SIZEREDRAW, 4);

///////////////////////////////////////////////////////////////////////
    // Register the MDI client window class
    WinRegisterClass(hInstance, (PSZ)wxMDIClientClassName, wxMDIClientProc, CS_SIZEREDRAW, 4);

///////////////////////////////////////////////////////////////////////
    // Register the MDI child client window class (same as FrameClient?)
    WinRegisterClass(hInstance, (PSZ)wxMDIChildClientClassName, wxChildProc, CS_SIZEREDRAW, 4);

///////////////////////////////////////////////////////////////////////
// Register the panel window class.
    WinRegisterClass (hInstance, (PCSZ)wxPanelClassName, (PFNWP)wxWndProc, CS_SIZEREDRAW, 4L);

///////////////////////////////////////////////////////////////////////
// Register the canvas and textsubwindow class name
    WinRegisterClass (hInstance, (PCSZ)wxCanvasClassName, (PFNWP)wxWndProc, CS_SIZEREDRAW, 4L);

///////////////////////////////////////////////////////////////////////
// Register the dialog class name
    WinRegisterClass (hInstance, (PCSZ)wxDialogClassName, (PFNWP)wxDlgProc, CS_SIZEREDRAW, 4L);

///////////////////////////////////////////////////////////////////////
// Register the status area class name
    WinRegisterClass(hInstance, (PCSZ)wxStatusLineClassName, (PFNWP)wxStatusLineWndProc, CS_SIZEREDRAW, 4L);

///////////////////////////////////////////////////////////////////////
// Register the timer class
    WinRegisterClass(hInstance, (PCSZ)wxTimerClassName, (PFNWP)wxTimerProc, 0, 4L);



#ifdef XXX
    wxSetKeyboardHook(TRUE);
#endif
}


// Cleans up any wxWindows internal structures left lying around
void wxCleanUp(void)
{
#ifdef XXX
    wxSetKeyboardHook(FALSE);
#endif
    wxCommonCleanUp();

    if (wxSTD_FRAME_ICON)
	WinDestroyPointer(wxSTD_FRAME_ICON);
    if (wxSTD_MDICHILDFRAME_ICON)
	WinDestroyPointer(wxSTD_MDICHILDFRAME_ICON);
    if (wxDEFAULT_FRAME_ICON)
	WinDestroyPointer(wxDEFAULT_FRAME_ICON);
    if (wxDEFAULT_MDICHILDFRAME_ICON)
	WinDestroyPointer(wxDEFAULT_MDICHILDFRAME_ICON);
    if (wxDEFAULT_MDIPARENTFRAME_ICON)
	WinDestroyPointer(wxDEFAULT_MDIPARENTFRAME_ICON);
#if 0
    if (wxSTATUS_LINE_FONT)
	DeleteObject(wxSTATUS_LINE_FONT);
#endif
}

#ifndef _WINDLL

// Main windows entry point

static HMQ wxhMq;


int main(int argc, char* argv[])
{
    if ((wxhAb = WinInitialize(0)) == NULLHANDLE) {
	return 0;
    }

    wxInitialize(wxhAb);

    if (!wxTheApp) {
	wxMessageBox("wxWindows error: You have to define an instance of wxApp!\n");
	return 0;
    }

#if 1
    wxTheApp->argc = argc;
    wxTheApp->argv = argv;
#else

    // Why all this? PGD
    // It is broken anyways
    //

    // Split command line into tokens, as in usual main(argc, argv)
    // form
    char **command = new char*[50];
    int count   = 0;
    char *token = 0;
    char *buf   = new char[500];	// Hangs around until end of app. in case
    // user carries pointers to the tokens

    int i     = 0;
    buf[0]    = 0;

    // Append args to buffer string, fill in command structure
    for(i = 0, token = buf; i < argc; i++) {
	sprintf(token, "%s ", argv[i]);
	command[count++] = token;

	token += strlen(token);
    }
    wxTheApp->argc = count;
    wxTheApp->argv = command;
#endif

    if ((wxhMq = WinCreateMsgQueue(wxhAb, 0)) == NULLHANDLE) {
	wxCleanUp();
	WinTerminate(wxhAb);
	return 0;
    }
    wxTheApp->hInstance = wxhAb;

#if XXX
    // Get application name
    {
	char name[200];
	::GetModuleFileName(hInstance, name, 199);
	command[count++] = copystring(name);

	strcpy(name, wxFileNameFromPath(name));
	wxStripExtension(name);
	wxTheApp->SetAppName(name);
    }
#endif

    wxTheApp->wx_frame = wxTheApp->OnInit();
 
    int retValue = 1;
    if (wxTheApp->wx_frame && wxTheApp->wx_frame->handle) {
	// show the toplevel frame, only if we are not iconized (from MS-Windows)
#if XXX
	if(wxTheApp->GetShowFrameOnInit() && (nCmdShow!=SW_HIDE)) wxTheApp->wx_frame->Show(TRUE);
#endif
	retValue = wxTheApp->MainLoop();
    }

    if (wxTheApp->wx_frame) {
	wxTheApp->wx_frame->GetEventHandler()->OnClose();
	delete wxTheApp->wx_frame;
    }
  
    wxCleanUp();

    WinDestroyMsgQueue(wxhMq);
    WinTerminate(wxhAb);

//    delete [] buf ;
//    delete [] command[0] ;
//    delete [] command ;
    return retValue;
}

#else /*  _WINDLL  */

extern "C"
int _dllmain(HINSTANCE hInstance,
        WORD wDataSegment, WORD wHeapSize, LPSTR lpszCmdLine)
{
  wxhAb = hInstance;
  wxInitialize(hInstance);

  wxTheApp->argc = 0;
  wxTheApp->argv = NULL;
  wxTheApp->hInstance = hInstance;

  wxTheApp->wx_frame = wxTheApp->OnInit();
  if (wxTheApp->wx_frame && wxTheApp->wx_frame->handle) {
    wxTheApp->wx_frame->Show(TRUE);
  }

  return 1;
}
#endif // _WINDLL

IMPLEMENT_DYNAMIC_CLASS(wxApp, wxObject)

wxApp::wxApp(wxlanguage_t language):wxbApp(language)
{
  wx_frame = NULL;
  wxTheApp = this;
  death_processed = FALSE;
  work_proc = NULL ;
  wx_class = NULL;
  wxSetLanguage(language);
}

wxApp::~wxApp(void)
{
    TRACE("wxApp::~wxApp(this=%#x)", (int)this);
}

Bool wxApp::Initialized(void)
{
#ifndef _WINDLL
  if (wx_frame)
    return TRUE;
  else
    return FALSE;
#endif
#ifdef _WINDLL // Assume initialized if DLL (no way of telling)
  return TRUE;
#endif
}

/*
 * Get and process a message, returning FALSE if WM_QUIT
 * received.
 *
 */
BOOL wxApp::DoMessage(void)
{
    if (!::WinGetMsg(wxhAb, &current_msg, NULLHANDLE, 0, 0)) {
	return FALSE;
    }
    
    // Process the message
//  if (!ProcessMessage(&current_msg))
// XXX
//  last_msg_time = current_msg.time;	// MATTHEW: timeStamp impl.
    {
	::WinDispatchMsg(wxhAb, &current_msg);
    }
    return TRUE;
/*
 * This method is correct to make the work_proc function,
 * but eats up CPU time doing nothing when executing OnIdle()!
 * So probably it's best to use timers to implement background
 * processing.
 if (::WinPeekMsg(wxhAb, &current_msg, NULLHANDLE, 0, 0, PM_REMOVE))
 {
 if (current_msg.msg == WM_QUIT)
 return FALSE;
 else
 {
 // Process the message
 if (!ProcessMessage(&current_msg))
 {
 ::WinDispatchMsg(wxhAb, &current_msg);
 }
 return TRUE;
 }
 }
 return TRUE;
 */
}

/*
 * Keep trying to process messages until WM_QUIT
 * received
 */
int wxApp::MainLoop(void)
{
    TRACE("wxApp::MainLoop()");
    keep_going = TRUE;
    while (keep_going) {
#if 0
	while (!::WinPeekMsg(wxhAb, &current_msg, NULLHANDLE, 0, 0, PM_NOREMOVE) &&
	       OnIdle()) {}
	if (!DoMessage())
	    keep_going = FALSE;

//    while (Pending())
//      Dispatch();
#else
	if (!DoMessage())
	    keep_going = FALSE;
#endif
    }

    return SHORT1FROMMP(current_msg.mp1);
}

void wxApp::ExitMainLoop(void)
{
    TRACE("wxApp::ExitMainLoop()");
    keep_going = FALSE;
}

Bool wxApp::Pending(void)
{
    TRACE("wxApp::Pending()");
    OnIdle() ;
    return (::WinPeekMsg(wxhAb, &current_msg, NULLHANDLE, 0, 0, PM_NOREMOVE)) ;
}

void wxApp::Dispatch(void)
{
    TRACE("wxApp::Dispatch()");
    if (!DoMessage())
	keep_going = FALSE;
}

/*
 * Give all windows a chance to preprocess
 * the message. Some may have accelerator tables, or have
 * MDI complications.
 */
BOOL wxApp::ProcessMessage(QMSG *msg)
{
    TRACE("wxApp::ProcessMessage(msg=%#x)", msg);
    HWND hWnd;

    /*
     * Anyone for a message? Try youngest descendants first.
     */
    for (hWnd = msg->hwnd; hWnd != wxDesktopHandle; hWnd = WinQueryWindow(hWnd, QW_PARENT)) {
	wxWnd *wnd = (wxWnd *)WinQueryWindowPtr(hWnd, QWL_USER);
	if (wnd) {
	    if (wnd->ProcessMessage(msg))
		return TRUE;
			
	    // STOP if we've reached the top of the hierarchy!
	    if (wx_frame && (wnd == (wxWnd *)wx_frame->handle))
		return FALSE;
	}
    }

    if (wx_frame && ((wxWnd *)wx_frame->handle)->ProcessMessage(msg))
	return TRUE;
    else
	return FALSE;
}

BOOL wxApp::OnIdle(void)
{
    TRACE("wxApp:OnIdle()");
    if (work_proc)
	(*work_proc)(this) ;
    return FALSE;
}

#if XXX
// Windows specific. Intercept keyboard input: by default,
// route it to the active frame or dialog box.
Bool wxApp::OnCharHook(wxKeyEvent& event)
{
    wxWindow *win = wxGetActiveWindow();
    if (win)
	return win->GetEventHandler()->OnCharHook(event);
    else
	return FALSE;
}
#endif

void wxExit(void)
{
    TRACE("wxExit()");
    if (wxTheApp)
	(void)wxTheApp->OnExit();
    wxCleanUp();
    WinDestroyMsgQueue(wxhMq);
    WinTerminate(wxhAb);
    exit(0);
    //FatalAppExit(0, "Fatal error: exiting");
}

// Yield to incoming messages
Bool wxYield(void)
{
    TRACE("wxYield()");
    QMSG msg;
    // We want to go back to the main message loop
    // if we see a WM_QUIT. (?)
    while (WinPeekMsg(wxhAb, &msg, NULLHANDLE, 0, 0, PM_NOREMOVE) && msg.msg != WM_QUIT) {
	if (!wxTheApp->DoMessage())
	    break;
    }

    return TRUE;
}
