/*
 * File:        wx_dc.cc
 * Purpose:        Device context implementation
 * Author:        Julian Smart
 * Created:        1993
 * Updated:        August 1994
 * RCS_ID:      $Id: wx_dc.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:        (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include <math.h>
//#include "complex.h"    // CSet++ : for M_PI definition
#include <float.h>      // EMX/GCC : for M_PI definition

#include "common.h"
#include "wx_frame.h"
#include "wx_dc.h"
#include "wx_dccan.h"
#include "wx_dcmem.h"
#include "wx_dcps.h"
#include "wx_utils.h"
#include "wx_canvs.h"
#include "wx_dialg.h"
#include "wx_main.h"
#include "wx_privt.h"

HDC wxGetPrinterDC(void)
{
#if USE_COMMON_DIALOGSXXX
    HDC         hDC;
    LPDEVMODE   lpDevMode = NULL;
    LPDEVNAMES  lpDevNames;
    LPSTR       lpszDriverName;
    LPSTR       lpszDeviceName;
    LPSTR       lpszPortName;

    PRINTDLG    pd;
    pd.lStructSize    = sizeof(PRINTDLG);
    pd.hwndOwner      = (HWND)NULL;
    pd.hDevMode       = NULL;
    pd.hDevNames      = NULL;
    pd.Flags          = PD_RETURNDEFAULT;
    pd.nCopies        = 1;

    if (!PrintDlg((LPPRINTDLG)&pd))
        return(NULL);

    if (!pd.hDevNames)
        return(NULL);

    lpDevNames = (LPDEVNAMES)GlobalLock(pd.hDevNames);
    lpszDriverName = (LPSTR)lpDevNames + lpDevNames->wDriverOffset;
    lpszDeviceName = (LPSTR)lpDevNames + lpDevNames->wDeviceOffset;
    lpszPortName   = (LPSTR)lpDevNames + lpDevNames->wOutputOffset;
    GlobalUnlock(pd.hDevNames);

    if (pd.hDevMode)
        lpDevMode = (LPDEVMODE)GlobalLock(pd.hDevMode);

    hDC = CreateDC(lpszDriverName, lpszDeviceName, lpszPortName, (LPSTR)lpDevMode);

    if (pd.hDevMode && lpDevMode)
        GlobalUnlock(pd.hDevMode);

    if (pd.hDevNames)
    {
	GlobalFree(pd.hDevNames);
	pd.hDevNames=NULL;
    }
    if (pd.hDevMode)
    {
       GlobalFree(pd.hDevMode);
       pd.hDevMode=NULL;
    }
    return(hDC);
#else
  return 0;
#endif
}

/*
 * Memory DC
 *
 */

wxMemoryDC::wxMemoryDC(void)
{
    __type = wxTYPE_DC_MEMORY;
    device = wxDEVICE_WINDOWS;

    // CreateCompatibleDC
    SIZEL sizl = {0, 0};
#if 0
    DEVOPENSTRUC dop = {0L, (PSZ)"DISPLAY", NULL, 0L, 0L, 0L, 0L, 0L, 0L };
    HDC hdc = DevOpenDC(wxhAb, OD_MEMORY, (PSZ)"*", 5L, (PDEVOPENDATA)&dop, NULLHANDLE);
#else
    HDC hdc = DevOpenDC(wxhAb, OD_MEMORY, (PSZ)"*", 0L, NULL, NULLHANDLE);
#endif
    hpsDC = GpiCreatePS(wxhAb, hdc, &sizl, PU_PELS | GPIA_ASSOC);
  
    ok = (hpsDC != GPI_ERROR);
    if (ok)
	wxSetupPS(hpsDC);
    DevCloseDC(hdc);

    SetBrush(wxWHITE_BRUSH);
    SetPen(wxBLACK_PEN);
}

wxMemoryDC::wxMemoryDC(wxCanvasDC *old_dc) : wxbMemoryDC(old_dc)
{
    __type = wxTYPE_DC_MEMORY;
    device = wxDEVICE_WINDOWS;

    wxWnd *wnd = NULL;
    if (old_dc->canvas)
	wnd = (wxWnd *)old_dc->canvas->handle;

    HPS ps = old_dc->GetHPS();
    if (ps) {
	// CreateCompatibleDC
	SIZEL sizl = {0, 0};
#if 0
	DEVOPENSTRUC dop = {0L, (PSZ)"DISPLAY", NULL, 0L, 0L, 0L, 0L, 0L, 0L };
	HDC hdc = DevOpenDC(wxhAb, OD_MEMORY, (PSZ)"*", 5L, (PDEVOPENDATA)&dop, NULLHANDLE);
#else
	HDC hdc = DevOpenDC(wxhAb, OD_MEMORY, (PSZ)"*", 0L, NULL, NULLHANDLE);
#endif
	hpsDC = GpiCreatePS(wxhAb, hdc, &sizl, PU_PELS | GPIA_ASSOC);
      
	ok = (hpsDC != GPI_ERROR);
	if (ok)
	    wxSetupPS(hpsDC);
	DevCloseDC(hdc);
      
	SetBrush(wxWHITE_BRUSH);
	SetPen(wxBLACK_PEN);
    }
}

wxMemoryDC::~wxMemoryDC(void)
{
    TRACE("wxMemoryDC::~wxMemoryDC() this=%#x wxDC=%#x", this, (wxDC *)this);
    if (hpsDC)
	GpiDestroyPS(hpsDC);
    hpsDC = NULLHANDLE;

}

void wxMemoryDC::SelectObject(wxBitmap *bitmap)
{
    if (!bitmap) {
	// XXX
	// Selecting NULL bitmap should?
	// 
    // Do own check for whether the bitmap is already selected into
    // a device context
    if (bitmap->selectedInto && (bitmap->selectedInto != this)) {
	wxFatalError("Error in wxMemoryDC::SelectObject\nBitmap is selected in another wxMemoryDC.\nDelete the first wxMemoryDC or use SelectObject(NULL)");
	return;
    }
  
    selected_bitmap = bitmap;
    bitmap->selectedInto = this;
#if DEBUGxx
    wxDebugMsg("wxMemoryDC::SelectObject: Selecting HBITMAP %X\n", bitmap->ms_bitmap);
#endif
    HBITMAP bm = GpiSetBitmap(hpsDC, bitmap->os2_bitmap);
    if (bm == PMERR_BITMAP_IN_USE) {
	wxFatalError("Error in wxMemoryDC::SelectObject\nBitmap may not be loaded, or may be selected in another wxMemoryDC.\nDelete the first wxMemoryDC to deselect bitmap.");
    }
}

void wxMemoryDC::GetSize(float *width, float *height)
{
  if (!selected_bitmap)
  {
    *width = 0.0; *height = 0.0;
    return;
  }
  *width = selected_bitmap->GetWidth();
  *height = selected_bitmap->GetHeight();
TRACE("wxMemoryDC::GetSize() returns width=%d, height=%d", (int)*width, (int)*height);
}
 
