/*
 * File:	wx_menu.cc
 * Purpose:	Menu implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_frame.h"
#include "wx_menu.h"
#include "wx_itemp.h"

extern void
debugfatal(const char *fcn, const char *file, int line);

/*
 * Return the menu position given the menu item id
 */
SHORT
MenuItemIdFromPosition(HWND hwnd, int ItemIndex)
{
    SHORT sIdentity = (SHORT)WinSendMsg(hwnd, MM_ITEMIDFROMPOSITION, (MPARAM)ItemIndex, 0);
    return sIdentity;
}

/*
 * Set the text of a menu item
 */
static BOOL
MenuSetItemText(HWND hwnd, USHORT id, char *szItemText)
{
    BOOL res = WinSetMenuItemText(hwnd, id, (PSZ)szItemText);
    if (res == 0)
	debugfatal("MenuSetItemText", __FILE__, __LINE__);
    return res;
}

static BOOL
MenuSetItemTextByPos(HWND hwnd, int pos, char *pszItemText)
{
    SHORT id = MenuItemIdFromPosition(hwnd, pos);
    BOOL res = WinSetMenuItemText(hwnd, id, (PSZ)pszItemText);
    if (res == 0)
	debugfatal("MenuSetItemTextByPos", __FILE__, __LINE__);
    return res;
}

/*
 * Return the number of items in the menu
 */
static SHORT
MenuQueryItemCount(HWND hwnd)
{
    return (SHORT)WinSendMsg(hwnd, MM_QUERYITEMCOUNT, 0, 0);
}

/*
 *  Return the text of the specified menu item. 
 */
static SHORT
MenuQueryItemText(HWND hwnd, USHORT usitem, SHORT smaxcount, PSZ pszItemText)
{
    return (SHORT)WinSendMsg(hwnd, MM_QUERYITEMTEXT, MPFROM2SHORT(usitem, smaxcount), (MPARAM)pszItemText);
}

/*
 * Insert a menu item into a menu.
 */
static int
MenuInsertItem(HWND hwnd, char *ItemText, int iPosition, int afStyle,
	       int afAttribute = 0, int id = -1, HWND hwndSubMenu = NULLHANDLE, ULONG hItem = 0)
{
    TRACE("MenuInsertItem(hwnd=%#x, ItemText='%s', iPosition=%d, afStyle=%d, afAttribute=%d, id=%d, hwndSubMenu=%#x, hItem=%ld)", hwnd, ItemText, iPosition, afStyle, afAttribute, id, hwndSubMenu, hItem);
    MENUITEM mi;

    mi.iPosition = iPosition;
    mi.afStyle = afStyle;
    mi.afAttribute = 0;
    mi.id = id;
    mi.hwndSubMenu = hwndSubMenu;
    mi.hItem = hItem;
    SHORT res = (SHORT)WinSendMsg(hwnd, MM_INSERTITEM, &mi, (PSZ)ItemText);
    if (res == MIT_MEMERROR || res == MIT_ERROR)
	debugfatal("MenuInsertItem", __FILE__, __LINE__);
    return res;

}

void
MenuCheckItem(HWND hwnd, int id, BOOL fcheck)
{
    BOOL res = WinCheckMenuItem(hwnd, id, fcheck);
    if (res == 0)
	debugfatal("MenuCheckItem", __FILE__, __LINE__);
}

			 
void
MenuEnableItem(HWND hwnd, int id, BOOL flag)
{
    TRACE("MenuEnableItem(hwnc=%#x, id=%d, flag=%d)", hwnd, id, flag);
    BOOL res = WinEnableMenuItem(hwnd, id, flag);
    if (res == 0)
	debugfatal("MenuEnableItem", __FILE__, __LINE__);
}

			 
void
MenuEnableItemByPos(HWND hwnd, int pos, BOOL flag)
{
    SHORT id = MenuItemIdFromPosition(hwnd, pos);
    BOOL res = WinEnableMenuItem(hwnd, id, flag);
    if (res == 0)
	debugfatal("MenuEnableItemByPos", __FILE__, __LINE__);
}

BOOL
MenuIsItemChecked(HWND hwnd, int id)
{
    BOOL res = WinIsMenuItemChecked(hwnd, id);
    return res;
}

			 
BOOL
MenuIsItemEnabled(HWND hwnd, int id)
{
    BOOL res = WinIsMenuItemEnabled(hwnd, id);
    return res;
}

			 
BOOL
MenuIsItemValid(HWND hwnd, int id)
{
    BOOL res = WinIsMenuItemValid(hwnd, id);
    return res;
}


SHORT
MenuDeleteItemByPos(HWND hwnd, int pos)
{
    SHORT id = MenuItemIdFromPosition(hwnd, pos);
    SHORT sItemsLeft = (SHORT)WinSendMsg(hwnd, MM_DELETEITEM, MPFROMSHORT(id), NULL);
    return sItemsLeft;
}


BOOL wxMenu::OS2Control(WORD param, WORD id)
{
    TRACE("wxMenu::OS2Control(param=%d, id=%d)", (int)param, (int)id);
    wxCommandEvent event(wxEVENT_TYPE_MENU_COMMAND);
    event.eventObject = this;
    event.commandInt = param;
    ProcessCommand(event);
    return TRUE;
}

/*
 * This routine will convert the &'s in a Windows
 * menu line to the ~'s required for OS/2
 */
void wxConvertMenuTitle(char *s)
{
    if (s) {
	int len = strlen(s);
	for(int i=0; i<len; i++) {
	    if (s[i] == '&') {
		if ((i+1)<=len && s[i+1]=='&')
		    i++;
		else
		    s[i] = '~';
	    }
	}
    }
}

extern wxMenu *wxCurrentPopupMenu;

Bool wxWindow::PopupMenu(wxMenu *menu, float x, float y)
{
    TRACE("wxWindow::PopupMenu(menu=%#x, x=%g, y=%g", menu, x, y);
    HWND hWnd = GetHWND();
    HWND hwndMenu = menu->GetHWND();
    POINTL point = {(long)x, (long)y};
    wxWnd *wnd = (wxWnd *)handle;
    wxCurrentPopupMenu = menu;
    WinPopupMenu(hWnd, hWnd, hwndMenu, point.x, wnd->WxToPmY(point.y), 0,
		 PU_HCONSTRAIN | PU_VCONSTRAIN | PU_KEYBOARD | PU_MOUSEBUTTON1 | PU_MOUSEBUTTON2 | PU_MOUSEBUTTON2DOWN);
    return TRUE;
}

// Menu Bar
IMPLEMENT_DYNAMIC_CLASS(wxMenuBar, wxItem)

wxMenuBar::wxMenuBar(void)
{
    n = 0;
    menus = NULL;
    titles = NULL;
    menu_bar_frame = NULL;
}

wxMenuBar::wxMenuBar(int N, wxMenu *Menus[], char *Titles[]):wxbMenuBar(N, Menus, Titles)
{
}

wxMenuBar::~wxMenuBar(void)
{
    TRACE("wxMenuBar::~wxMenuBar() this=%#x wxWindow=%#x", this, (wxWindow *)this);
#if 0
//  HWND menu = hwndItem;


  // In fact, don't want menu to be destroyed before MDI
  // shuffling has taken place. Let it be destroyed
  // automatically when the window is destroyed.

//  DestroyMenu(menu);
//  hwndItem = NULLHANDLE;

  int i;
/*
  // See remarks in ::~wxMenu() method
  // BEWARE - this may interfere with MDI fixes, so
  // may need to remove
  int N = 0 ;

  if (menu_bar_frame && (menu_bar_frame->frame_type == wxSDI))
  {
    if (menu)
      N = GetMenuItemCount(menu) ;
    for (i = N-1; i >= 0; i--)
      RemoveMenu(menu, i, MF_BYPOSITION);
  }
*/
  for (i = 0; i < n; i++)
  {
    delete menus[i];
    delete[] titles[i];
  }
  delete[] menus;
  delete[] titles;

/* Don't destroy menu here, in case we're MDI and
   need to do some shuffling with VALID menu handles.
  if (menu)
    DestroyMenu(menu);
  hwndItem = NULLHANDLE;
*/
#endif
}

/* MATTHEW: [6] */
Bool wxMenuBar::OnDelete(wxMenu *a_menu, int pos)
{
    TRACE("wxMenuBar::OnDelete(a_menu=%#x, pos=%d)", a_menu, pos);
    if (!menu_bar_frame)
	return TRUE;

    MenuDeleteItemByPos(hwndItem, pos);
    menus[pos]->hwndItem = menus[pos]->save_hwndItem;
    menus[pos]->save_hwndItem = NULLHANDLE;

#if 0
    if (menu_bar_frame) {
	wxWnd *cframe = (wxWnd*)menu_bar_frame->handle ;
	HWND hand = (HWND)cframe->handle ;
	DrawMenuBar(hand) ;
    }
#endif

    return TRUE;

//    return FALSE;
}

/* MATTHEW: [6] */
Bool wxMenuBar::OnAppend(wxMenu *a_menu, char *title)
{
    TRACE("wxMenuBar::OnAppend(a_menu=%#x, title='%s')", a_menu, title);
    if (!a_menu->hwndItem)
	return FALSE;

    if (!menu_bar_frame)
	return TRUE;

#if XXX
    a_menu->save_hwndItem = a_menu->hwndItem;
    a_menu->hwndItem = NULL;

    AppendMenu(hwndItem, MF_POPUP | MF_STRING, (UINT)a_menu->save_hwndItem, title);

    wxWnd *cframe = (wxWnd*)menu_bar_frame->handle;
    HWND hand = (HWND)cframe->handle;
    DrawMenuBar(hand);

    return TRUE;
#else
    return FALSE;
#endif
}


// Not any more, use onappend and ondelete

void wxMenuBar::Append(wxMenu *menu, char *title)
{
    TRACE("wxMenuBar::Append(menu=%#x, title='%s')", menu, title);
#if 1
    wxbMenuBar::Append(menu, title);
#else
  n ++;
  wxMenu **new_menus = new wxMenu *[n];
  char **new_titles = new char *[n];

  int i;
  for (i = 0; i < n - 1; i++)
    {
      new_menus[i] = menus[i];
      menus[i] = NULL;
      new_titles[i] = titles[i];
      titles[i] = NULL;
    }
  if (menus)
   {
     delete[] menus;
     delete[] titles;
   }
  menus = new_menus;
  titles = new_titles;

  menus[n-1] = menu;
  titles[n-1] = copystring(title);
  wxConvertMenuTitle(titles[n-1]);

  menu->menu_bar = this;
#endif
}


// Must only be used AFTER menu has been attached to frame,
// otherwise use individual menus to enable/disable items
void wxMenuBar::Enable(int Id, Bool Flag)
{
    TRACE("wxMenuBar::Enable(iD=%d, Flag=%d)", Id, Flag);
    wxMenu *itemMenu = NULL;
    wxMenuItem *item = FindItemForId(Id, &itemMenu) ;
    if (!item)
	return;

    if (itemMenu->hwndItem)
	MenuEnableItem(itemMenu->hwndItem, Id, Flag);
    else if (itemMenu->save_hwndItem)
	MenuEnableItem(itemMenu->save_hwndItem, Id, Flag);
}

void wxMenuBar::EnableTop(int pos, Bool flag)
{
    MenuEnableItemByPos(hwndItem, pos, flag);
#if 0
    wxWnd *cframe = (wxWnd*)menu_bar_frame->handle ;
    HWND hand = (HWND)cframe->hwndItem ;
    DrawMenuBar(hand) ;
#endif
}

// Must only be used AFTER menu has been attached to frame,
// otherwise use individual menus
void wxMenuBar::Check(int Id, Bool Flag)
{
    TRACE("wxMenuBar::Check(iD=%d, Flag=%d)", Id, Flag);
    wxMenu *itemMenu = NULL;
    wxMenuItem *item = FindItemForId(Id, &itemMenu) ;
    if (!item)
	return;
    
    if (!item->checkable)
	return;
    
    if (itemMenu->hwndItem)
	MenuCheckItem(itemMenu->hwndItem, Id, Flag);
    else if (itemMenu->save_hwndItem)
	MenuCheckItem(itemMenu->save_hwndItem, Id, Flag);
}

Bool wxMenuBar::Checked(int Id)
{
    wxMenu *itemMenu = NULL;
    wxMenuItem *item = FindItemForId(Id, &itemMenu) ;
    if (!item)
	return FALSE;

    if (itemMenu->hwndItem)
	return MenuIsItemChecked(itemMenu->hwndItem, Id);
    if (itemMenu->save_hwndItem)
	return MenuIsItemChecked(itemMenu->save_hwndItem, Id);
    return FALSE;
}

void wxMenuBar::SetLabel(int Id, char *label)
{
    TRACE("wxMenuBar::SetLabel(Id=%d, label='%s'", Id, label);
    label = copystring(label);
    wxConvertMenuTitle(label);

    wxMenu *itemMenu = NULL;
    wxMenuItem *item = FindItemForId(Id, &itemMenu) ;
    if (!item)
	return;

    if (itemMenu->hwndItem)
	MenuSetItemText(itemMenu->hwndItem, Id, label);
    else if (itemMenu->save_hwndItem)
	MenuSetItemText(itemMenu->save_hwndItem, Id, label);
    delete label;
}

char *wxMenuBar::GetLabel(int Id)
{
    wxMenu *itemMenu = NULL;
    wxMenuItem *item = FindItemForId(Id, &itemMenu) ;

    if (!item)
	return NULL;
    static char tmp[128] ;
    int len = 0;
    if (itemMenu->hwndItem)
	len = MenuQueryItemText(itemMenu->hwndItem, Id, sizeof tmp, (PSZ)tmp);
    else if (itemMenu->save_hwndItem)
	len = MenuQueryItemText(itemMenu->save_hwndItem, Id, sizeof tmp, (PSZ)tmp);
    char *p = copystring(tmp) ;
    return(p) ;
}

void wxMenuBar::SetLabelTop(int pos, char *label)
{
    TRACE("wxMenuBar::SetLabelTop(Id=%d, label='%s'", pos, label);
    label = copystring(label);
    wxConvertMenuTitle(label);
    MenuSetItemTextByPos(hwndItem, pos, label);
    delete label;
}

char *wxMenuBar::GetLabelTop(int pos)
{
    char tmp[128];
    int len = MenuQueryItemText(hwndItem, pos, sizeof tmp - 1, (PSZ)tmp);
    tmp[len] = '\0' ;
    char *p = copystring(tmp) ;
    return p;
}

void wxFrame::SetMenuBar(wxMenuBar *menu_bar)
{
    TRACE("wxFrame::SetMenuBar(menu_bar=%#x)", menu_bar);
    int      i;
    HWND hwndMenu;

    if (menu_bar->menu_bar_frame)
	return;

    wxWnd *cframe = (wxWnd *)handle;
    assert(cframe != NULL);

    hwndMenu = WinCreateMenu(cframe->hwndWnd, NULLHANDLE);

    /*
     * Insert all the top level menu items
     */
    for (i = 0; i < menu_bar->n; i ++) {
	HWND hwndSubMenu = (HWND)menu_bar->menus[i]->hwndItem;
	//
	// After looking Bounds Checker result, it seems that all
	// menus must be individually destroyed. So, don't reset hwndItem,
	// to  allow ~wxMenu to do the job.
	//
	menu_bar->menus[i]->save_hwndItem = hwndSubMenu;
	// Uncommenting for the moment... JACS

	menu_bar->menus[i]->hwndItem = NULLHANDLE;
    
	MenuInsertItem(hwndMenu, menu_bar->titles[i], i, MIS_TEXT | MIS_SUBMENU, 0, i, hwndSubMenu);
    }
    WinSendMsg(cframe->hwndWnd, WM_UPDATEFRAME, MPFROMSHORT(FCF_MENU), 0);

    menu_bar->hwndItem = hwndMenu;
    if (wx_menu_bar)
	delete wx_menu_bar;

    cframe->hwndMenu = hwndMenu;

    switch (frame_type) {
    case wxMDI_PARENT: {
TRACE("frame_type = wxMDI_PARENT");
	wxMDIFrameWnd *mdi_frame = (wxMDIFrameWnd *)cframe;
	HWND hwndWindowMenu = mdi_frame->window_menu;

	/*
	 * Try to insert Window menu in front of Help, otherwise append it.
	 */
	int n = MenuQueryItemCount(hwndMenu);
	Bool success = FALSE;
	for (i = 0; i < n; i++) {
	    char buf[100];
	    int chars = MenuQueryItemText(hwndMenu, i, sizeof buf, (PSZ)buf);
TRACE("Menu item %d is '%s' chars=%d", i, buf, chars);
	    if (chars > 0 && (stricmp(buf, "~Help") == 0 || stricmp(buf, "Help") == 0)) {
		success = TRUE;
		break;
	    }
	}
	
	MenuInsertItem(hwndMenu, "~Window", success ? i : MIT_END, MIS_TEXT | MIS_SUBMENU, 0, i, hwndWindowMenu);
	mdi_frame->parent_frame_active = TRUE;
	break;
    }

    case wxMDI_CHILD: {
TRACE("frame_type = wxMDI_CHILD");
	wxMDIFrameWnd *parent = (wxMDIFrameWnd *)GetParent()->handle;
	parent->parent_frame_active = FALSE;
	HWND hwndWindowMenu = parent->window_menu;

	/*
	 * Try to insert Window menu in front of Help, otherwise append it.
	 */
	int n = MenuQueryItemCount(hwndMenu);
	Bool success = FALSE;
	for (i = 0; i < n; i++) {
	    char buf[100];
	    int chars = MenuQueryItemText(hwndMenu, i, sizeof buf, (PSZ)buf);
	    if (chars > 0 && (stricmp(buf, "~Help") == 0 || stricmp(buf, "Help") == 0)) {
		success = TRUE;
		break;
	    }
	}
	
	MenuInsertItem(hwndMenu, "~Window", success ? i : MIT_END, MIS_TEXT | MIS_SUBMENU, 0, i, hwndWindowMenu);
	break;
    }
    default:
TRACE("frame_type = %#x", frame_type);

    }

    TRACE("wx_menu_bar set to %#x", menu_bar);
    wx_menu_bar = menu_bar;
    menu_bar->menu_bar_frame = this;
}

char *wxMenu::GetLabel(int Id)
{
    static char tmp[128];
    int len = 0;

    wxMenu *itemMenu = NULL;
    wxMenuItem *item = FindItemForId(Id, &itemMenu) ;
    
    if (item) {
	if (itemMenu->hwndItem)
	    len = MenuQueryItemText(itemMenu->hwndItem, Id, sizeof tmp - 1, (PSZ)tmp);
	else if (itemMenu->save_hwndItem)
	    len = MenuQueryItemText(itemMenu->save_hwndItem, Id, sizeof tmp - 1, (PSZ)tmp);
    }
    tmp[len] = '\0';
    return tmp;
}

// Construct a menu with optional title (then use append)
IMPLEMENT_DYNAMIC_CLASS(wxMenu, wxItem)
IMPLEMENT_DYNAMIC_CLASS(wxMenuItem, wxObject)

/*
 * The first argument is presently ignored. The second argument is 
 * a callback function if the menu is used as a popup using wxWindow::PopupMenu.
 */
wxMenu::wxMenu(char *Title, wxFunction func) : wxbMenu(Title, func)
{
    TRACE("wxMenu:wxMenu(Title='%s', func=%#x)", Title, func);
    wxConvertMenuTitle(title);

    mustBeBreaked = FALSE ;
    no_items = 0;
    menu_bar = NULL;

    hwndItem = WinCreateMenu(HWND_OBJECT, NULLHANDLE);
    save_hwndItem = NULLHANDLE;
    top_level_menu = this;
#if XXX
    if (title) {
	Append(-2,title) ;
	AppendSeparator() ;
    }
#endif

    Callback(func);
}

// The wxWindow destructor will take care of deleting the submenus.
wxMenu::~wxMenu(void)
{
    TRACE("wxMenu::~wxMenu() this=%#x wxWindow=%#x", this, (wxWindow *)this);
//    if (hwndItem) {
//	WinDestroyWindow(hwndItem);
//	TRACE("wxMenu::~wxMenu: Destroying window, this=%#x, hwndItem=%#x", (int)this, (int)hwndItem);
//	hwndItem = NULLHANDLE;
//    }

    // Windows seems really bad on Menu de-allocation...
    // After many try, here is what I do: RemoveMenu() will ensure
    // that popup are "disconnected" from their parent; then call
    // delete method on each child (which in turn do a recursive job),
    // and finally, DestroyMenu()
    //
    // With that, BoundCheckers is happy, and no complaints...
/*  
    int N = 0 ;
    if (hwndItem)
    N = GetMenuItemCount(hwndItem);
    for (int i = N-1; i >= 0; i--)
    RemoveMenu(hwndItem, i, MF_BYPOSITION);
    */
    wxNode *node = menuItems.First();
    while (node)
    {
	wxMenuItem *item = (wxMenuItem *)node->Data();
	item->menuBar = NULL;
/*
  // Delete child menus.
  // Beware: they must not be appended to children list!!!
  // (because order of delete is significant)
  if (item->subMenu)
  delete item->subMenu ;
  item->subMenu = NULL ;
  */
	wxNode *next = node->Next();
	delete item;
	delete node;
	node = next;
    }
/*
  if (hwndItem)
  DestroyMenu(hwndItem);
  hwndItem = NULLHANDLE;
  */
}

void wxMenu::Break(void)
{
    TRACE("wxMenu::Break()");
    mustBeBreaked = TRUE ;
}

// Ordinary menu item
void wxMenu::Append(int Id, char *Label, char *helpString, Bool checkable)
{
    TRACE("wxMenu::Append(Id=%d, Label='%s', helpString='%s', checkable=%d)", Id, Label, helpString, checkable);
#if XXX
    if (!SubMenu->hwndItem)
	return;
#endif

    // 'checkable' parameter is useless for Windows.
    wxMenuItem *item = new wxMenuItem;
    item->checkable = checkable ;
    item->itemId = Id;
    item->itemName = copystring(Label);
    wxConvertMenuTitle(item->itemName);
    item->subMenu = NULL;
    if (helpString)
	item->helpString = copystring(helpString);

    menuItems.Append(item);

    int os2_flags = mustBeBreaked? MIS_BREAK : 0 ;
    mustBeBreaked = FALSE ;

    if (hwndItem)
	MenuInsertItem(hwndItem, item->itemName, MIT_END, MIS_TEXT | os2_flags, 0, Id);
    else if (save_hwndItem)  // For Dynamic Manu Append, Thx!
	MenuInsertItem(save_hwndItem, item->itemName, MIT_END, MIS_TEXT | os2_flags, 0, Id);

#if XXX
// What is this supposed to do?
    if (Id==-2) {
	if (hwndItem)
	    MenuEnableItemByPos(hwndItem, no_items, FALSE);
	else if (save_hwndItem) // For Dynamic Menu Append, Thx!!
	    MenuEnableItemByPos((HWND)save_hwndItem, no_items, FALSE);
    }
#endif

    no_items ++;
}

void wxMenu::AppendSeparator(void)
{
    TRACE("wxMenu::AppendSeparator()");
    int os2_flags = mustBeBreaked? MIS_BREAK : 0 ;
    mustBeBreaked = FALSE ;

    if (hwndItem)
	MenuInsertItem(hwndItem, NULL, MIT_END, MIS_SEPARATOR | os2_flags);
    else if (save_hwndItem)  // For Dynamic Manu Append, Thx!
	MenuInsertItem(hwndItem, NULL, MIT_END, MIS_SEPARATOR | os2_flags);

    wxMenuItem *item = new wxMenuItem;
    item->checkable = FALSE ;
    item->itemId = -1;
    menuItems.Append(item);
    no_items ++;
}

// Pullright item
void wxMenu::Append(int Id, char *Label, wxMenu *SubMenu, char *helpString)
{
    TRACE("wxMenu::Append(Id=%d, Label='%s', SubMenu=%#x, helpString='%s'", Id, Label, SubMenu, helpString);

    SubMenu->top_level_menu = top_level_menu;
    SubMenu->window_parent = this;
    // To do a clean delete, don't append to children list (see ::~wxMenu)
    // Yes you do for my code says JACS..., I'm uncommenting it.
    children->Append(SubMenu);            // Store submenu for later deletion

    wxMenuItem *item = new wxMenuItem;
    item->checkable = FALSE ;
    item->itemId = Id;
    item->itemName = copystring(Label);
    wxConvertMenuTitle(item->itemName);
    if (helpString)
	item->helpString = copystring(helpString);
    item->subMenu = SubMenu;

    menuItems.Append(item);

//  int os2_flags = mustBeBreaked? MIS_BREAK : 0 ;
    mustBeBreaked = FALSE ;

//  HWND menu = hwndItem;
    HWND child = (HWND)SubMenu->hwndItem;
    //
    // After looking Bounds Checker result, it seems that all
    // menus must be individually destroyed. So, don't reset hwndItem,
    // to  allow ~wxMenu to do the job.
    // NO, my code should work I THINK - JACS
    //
    SubMenu->save_hwndItem = child;
    SubMenu->hwndItem = NULLHANDLE;

    MenuInsertItem(hwndItem, item->itemName, MIT_END, MIS_SUBMENU, 0, Id, child);

    no_items ++;
}

void wxMenu::Delete(int id)
{
    TRACE("wxMenu::Delete(id=%d)", id);
    wxNode *node;
    wxMenuItem *item;
    int pos;
    HWND hwndItem;

    for (pos = 0, node = menuItems.First(); node; node = node->Next(), pos++) {
	item = (wxMenuItem *)node->Data();
	if (item->itemId == id)
	    break;
    }

    if (!node)
	return;

    hwndItem = hwndItem ? hwndItem : save_hwndItem;
    assert(hwndItem);

    if (item->subMenu) {
	MenuDeleteItemByPos(hwndItem, pos);
	item->subMenu->hwndItem = item->subMenu->save_hwndItem;
	item->subMenu->save_hwndItem = NULLHANDLE;
	item->subMenu->window_parent = NULL;
	item->subMenu->top_level_menu = NULL;
    } else
        MenuDeleteItemByPos(hwndItem, pos);

    menuItems.DeleteNode(node);
    delete item;
}

void wxMenu::Enable(int Id, Bool Flag)
{
    TRACE("wxMenu::Enable(Id=%d, Flag=%d)", Id, (int)Flag);
    wxMenuItem *item = FindItemForId(Id) ;
    if (item==NULL)
	return;

    if (item->subMenu == NULL) {
	// Because you reset hwndItem, that cannot works in cascaded menus, no???
	if (hwndItem)
	    MenuEnableItem(hwndItem, Id, Flag);
	else if (save_hwndItem)
	    MenuEnableItem(hwndItem, Id, Flag);
    } else {
	wxMenu *father = item->subMenu->top_level_menu ;
	wxNode *node = father->menuItems.First() ;
	int i=0 ;
	while (node) {
	    wxMenuItem *matched = (wxMenuItem*)node->Data() ;
	    if (matched==item)
		break ;
	    i++ ;
	    node = node->Next() ;
	}
	MenuEnableItem(father->save_hwndItem, i, Flag);
    }
}

void wxMenu::Check(int Id, Bool Flag)
{
    TRACE("wxMenu::Check(Id=%d, Flag=%d)", Id, Flag);
    wxMenuItem *item = FindItemForId(Id);
    if (item->checkable) {
	if (hwndItem)
	    MenuCheckItem(save_hwndItem, Id, Flag);
	else if (save_hwndItem)
	    MenuCheckItem(save_hwndItem, Id, Flag);
    }
}

Bool wxMenu::Checked(int Id)
{
    if (hwndItem)
	return MenuIsItemChecked(hwndItem, Id);
    if (save_hwndItem)
	return MenuIsItemChecked(save_hwndItem, Id);
    return FALSE ;
}

void wxMenu::SetTitle(const char *label)
{
    TRACE("wxMenu::SetTitle(label='%s')", label);
#if XXX
// What is this functions actually supposed to do?
    if (title)
	delete[] title ;
    if (label) {
	title = copystring(label) ;
	wxConvertMenuTitle(title);
    } else
	title = copystring(" ") ;
    if (hwndItem)
	MenuSetItemText(hwndItem, 0, title);
    else if (save_hwndItem)
	MenuSetItemText(save_hwndItem, 0, title);
#endif
}

char *wxMenu::GetTitle()
{
    return title;
}

void wxMenu::SetLabel(int Id, char *label)
{
    TRACE("wxMenu::SetLabel(Id=%d, label='%s')", Id, label);
    label = copystring(label);
    wxConvertMenuTitle(label);

    wxMenuItem *item = FindItemForId(Id) ;
    if (item == NULL)
	return;

    if (item->subMenu==NULL) {
	if (hwndItem)
	    MenuSetItemText(hwndItem, Id, label);
	else if (save_hwndItem)
	    MenuSetItemText(save_hwndItem, Id, label);
    } else {
	wxMenu *father = item->subMenu->top_level_menu ;
	wxNode *node = father->menuItems.First() ;
	int i = 0 ;
	while (node) {
	    wxMenuItem *matched = (wxMenuItem*)node->Data() ;
	    if (matched == item)
		break ;
	    i++ ;
	    node = node->Next() ;
	}
	MenuSetItemTextByPos(father->save_hwndItem, i, label);
    }
    delete label;
}

HWND
wxMakeWindowMenu(HWND hwndOwner)
{
    HWND hwndMenu = NULLHANDLE;
    HWND hwndWindowMenu = NULLHANDLE;
//    if (hwndWindowMenu == NULLHANDLE) {
	hwndMenu = WinCreateMenu(HWND_OBJECT /* hwndOwner */, NULLHANDLE);
	MenuInsertItem(hwndMenu, "~Cascade", MIT_END, MIS_TEXT, 0, IDM_WINDOWCASCADE);
	MenuInsertItem(hwndMenu, "~Tile", MIT_END, MIS_TEXT, 0, IDM_WINDOWTILE);
	MenuInsertItem(hwndMenu, "~Arrange icons", MIT_END, MIS_TEXT, 0, IDM_WINDOWICONS);
	MenuInsertItem(hwndMenu, "~Next", MIT_END, MIS_TEXT, 0, IDM_WINDOWNEXT);
//    }
    return hwndMenu;
}
