/*
 * File:	wx_messg.cc
 * Purpose:	Message item implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_messg.h"
#include "wx_itemp.h"

// Message
IMPLEMENT_DYNAMIC_CLASS(wxMessage, wxItem)

wxMessage::wxMessage(void)
{
    messageBitmap = NULL;
}

wxMessage::wxMessage(wxPanel *panel, const char *label, int x, int y, long style, const char *name):
    wxbMessage(panel, label, x, y, style, name)
{
    Create(panel, label, x, y, style, name);
}

#if USE_BITMAP_MESSAGE
wxMessage::wxMessage(wxPanel *panel, wxBitmap *image, int x, int y, long style, const char *name):
  wxbMessage(panel, image, x, y, style, name)
{
    Create(panel, image, x, y, style, name);
}
#endif
  
Bool wxMessage::Create(wxPanel *panel, const char *label, int x, int y, long style, const char *name)
{
    TRACE("wxMessage::Create(panel=%#x, label='%s', x=%d, y=%d, style=%#x, name='%s')", (int)panel, label, x, y, (int)style, name);
    messageBitmap = NULL;
    SetName(name);
    if (panel) panel->AddChild(this);
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;
    windowStyle = style;

    hwndItem = WinCreateWindow(panel->GetHWND(), WC_STATIC, (PSZ)label, SS_TEXT,
			       0, 0, 0, 0, panel->GetHWND(),
			       HWND_TOP, NewControlId(), NULL, NULL);
    TRACE("wxMessage::Create: Window created, parent=%#x, handle=%#x", (int)panel->GetHWND(), (int)hwndItem);

    // Subclass for purposes of dialog editing mode
    SubclassControl(hwndItem);

    if (labelFont)
	wxSetWinFont(hwndItem, labelFont);

    wxSetBackgroundColour(hwndItem, backColour);
    wxSetForegroundColour(hwndItem, labelColour);

    panel->GetValidPosition(&x, &y);

    SetSize(x, y, -1, -1);
    panel->AdvanceCursor(this);
    return TRUE;
}

#if USE_BITMAP_MESSAGE
Bool wxMessage::Create(wxPanel *panel, wxBitmap *image, int x, int y, long style, const char *name)
{
    TRACE("wxMessage::Create(panel=%#x, image=%#x, x=%d, y=%d, style=%#x, name='%s')", (int)panel, (int)image, x, y, (int)style, name);
    messageBitmap = image;
    SetName(name);
    if (panel) panel->AddChild(this);
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;
    windowStyle = style;

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

#if XXX
    if (image)
	WinSendMsg(hwndItem,WM_CHANGEBITMAP,
		   (MPARAM)((image->GetHeight()<<8)+image->GetWidth()),
		   (MPARAM)image->os2_bitmap);
#endif  
/*
  HPS the_ps = GetWindowPS(hwndItem) ;
  if (labelFont && labelFont->GetInternalFont(the_ps))
  SendMessage(hwndItem,WM_SETFONT,
  (MPARAM)labelFont->GetInternalFont(the_ps),0L);
  ReleasePS(hwndItem,the_ps) ;
  */
    panel->GetValidPosition(&x, &y);

    SetSize(x, y, image ? image->GetWidth() : 0, image ? image->GetHeight() : 0);
    panel->AdvanceCursor(this);
    return TRUE;
}
#endif

wxMessage::~wxMessage(void)
{
    TRACE("wxMessage::~wxMessage() this=%#x wxWindow=%#x", this, (wxWindow *)this);
//printf("wxMessage: GetHWND()=>%#x\n", GetHWND());
}

void wxMessage::GetAutoSize(int *cx, int *cy)
{
    TRACE("wxMessage::GetAutoSize(...)");
    *cx = wxGetTextWidth(hwndItem);
    *cy = wxGetCharHeight(hwndItem);
}

void wxMessage::SetLabel(char *label)
{
    messageBitmap = NULL;
    isBitmap = FALSE;
    float w, h;
    GetTextExtent(label, &w, &h, NULL, NULL, labelFont);
    WinSetWindowPos(hwndItem, 0, 0, 0, (LONG)w, (LONG)h, SWP_SIZE);
    WinSetWindowText(hwndItem, (PSZ)label);
}

#if USE_BITMAP_MESSAGE
void wxMessage::SetLabel(wxBitmap *bitmap)
{
    messageBitmap = bitmap;
    isBitmap = TRUE;
#if XXX
// To be implemented
#endif
}
#endif


void wxMessage::SetLabelFont(wxFont *font)
{
    labelFont = font;
    HWND hWnd = GetHWND();
    if (hWnd != 0) {
	if (font) {
#if XXX
	    if (font->GetResourceHandle())
		SendMessage(hWnd, WM_SETFONT,
			    (WPARAM)font->GetResourceHandle(),TRUE);
#endif
	}
    }
}


void wxMessage::SetLabelColour(wxColour& col) 
{
    TRACE("wxMessage::SetLabelColour(col=%#06lx)", (LONG)col);
    wxItem::SetLabelColour(col);
    if (hwndLabel)
	wxSetForegroundColour(hwndLabel, col);
}

void wxMessage::SetBackgroundColour(wxColour& col)
{
    TRACE("wxMessage::SetBackgroundColour(col=%#06lx)", (LONG)col);
    wxItem::SetBackgroundColour(col);
    if (hwndItem)
	wxSetBackgroundColour(hwndItem, backColour);
}

