/*
 * File:	wx_mtxt.cc
 * Purpose:	Multi-line text item implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1995, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_mtxt.h"
#include "wx_itemp.h"

// Multi-line Text item
IMPLEMENT_DYNAMIC_CLASS(wxMultiText, wxText)

wxMultiText::wxMultiText(void)
{
}

wxMultiText::wxMultiText(wxPanel *panel, wxFunction Function, const char *label, const char *value,
               int x, int y, int width, int height, long style, const char *name):
  wxbMultiText(panel, Function, label, value, x, y, width, height, style, name)
{
  Create(panel, Function, label, value, x, y, width, height, style, name);
}

Bool wxMultiText::Create(wxPanel *panel, wxFunction Function, const char *label, const char *value,
               int x, int y, int width, int height, long style, const char *name)
{
    TRACE("wxMultiText::Create(panel=%#x, Function=%#x, label='%s', value=%#x, x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)panel, (int)Function, label, (int)value, x, y, width, height, (int)style, name);
    SetName(name);
    if (panel) panel->AddChild(this);
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    windowStyle = style;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;
    if (height == -1) height = 50;

    window_parent = panel;
//  labelPosition = panel->label_position;
    SetAppropriateLabelPosition();
    panel->GetValidPosition(&x, &y);

    // If label exists, create a static control for it.
    if (label)
	CreateLabel(panel->GetHWND(), label, style & wxFIXED_LENGTH);

    windows_id = (int)NewControlId();
    long os2Style = ES_LEFT | MLS_VSCROLL | MLS_BORDER | WS_TABSTOP;

    if (windowStyle & wxREADONLY)
	os2Style |= MLS_READONLY;

    if (windowStyle & wxHSCROLL)
	os2Style |= (MLS_HSCROLL | ES_AUTOSCROLL) ;

#if XXX
    if (windowStyle & wxPASSWORD) // hidden input
	msStyle |= ES_PASSWORD;
#endif

    hwndItem =  WinCreateWindow(panel->GetHWND(), WC_MLE, (PSZ)label, os2Style,
				  0, 0, 0, 0, panel->GetHWND(),
				  HWND_TOP, windows_id, NULL, NULL);
    TRACE("wxMultiText::Create: Window created, parent=%#x, handle=%#x", (int)panel->GetHWND(), (int)hwndItem);

//    wxSetBackgroundColour(hwndItem, &backColour);

    // Subclass for purposes of dialog editing mode
    SubclassControl(hwndItem);

    if (buttonFont)
	wxSetWinFont(hwndItem, buttonFont);

    SetSize(x, y, width, height);
    Show();

    if (value)
	WinSetWindowText(hwndItem, (PSZ)value);

    panel->AdvanceCursor(this);
    Callback(Function);

    return TRUE;

}


char *wxMultiText::GetValue(void)
{
#if XXX
  // Fix by /Roffe for Borland 4.0.
  int TL = GetWindowTextLength(hwndItem);
  if (TL > 999)
    TL = 999;

  // Calling WinSendMsg this way is equivalent with a GetWindowText call.
  // Note that (some of) the parameters have to be casted appropriately...
#if 0
  WinSendMsg(hwndItem, WM_GETTEXT, TL, (MPARAM)((PSZ) wxBuffer));
#endif

  if(TL == 999)
    wxBuffer[999] = '\0';
#endif
  return wxBuffer;

/* Not sure if this is still needed.

// Julian! I can't see why this job below should have to be done -
// the edit control has the es_wantreturn attribute byte set,
// and will thus have the cr/lf characters inserted appropriately... /Roffe

  int buf_len = 1000;
  int no_chars = 0;
  int no_lines = (int)SendMessage(hwndItem, EM_GETLINECOUNT, 0, 0L);
  for (int i = 0; i < no_lines; i++)
  {
    *(LPINT)(wxBuffer+no_chars) = buf_len;
    int n = (int)SendMessage(hwndItem, EM_GETLINE, i,
                         (LONG)(wxBuffer+no_chars));
    no_chars += n;
    buf_len -= (n + 2);
    if (i < (no_lines - 1))
    {
      wxBuffer[no_chars] = 13;
      no_chars ++;
      wxBuffer[no_chars] = 10;
      no_chars ++;
    }
  }
  wxBuffer[no_chars] = 0;
  return wxBuffer;
*/
}

void wxMultiText::GetValue(char *buffer, int maxSize)
{
#if XXX
  // Fix by /Roffe for Borland 4.0.
  int TL = GetWindowTextLength(hwndItem);

  // We still want to keep within the limits for an edit control, won't we?
  if (maxSize > 32766)
    maxSize = 32766;

  if (TL > maxSize)
    TL = maxSize;

  // Here we use GetWindowText instead.
  WinQueryWindowText(hwndItem, TL, (PCH)buffer);

  if(TL == maxSize)
    buffer[maxSize] = '\0';

/*
  buffer[0] = 0;
  int no_chars = 0;
  int no_lines = (int)SendMessage(hwndItem, EM_GETLINECOUNT, 0, 0L);
  for (int i = 0; i < no_lines; i++)
  {
    *(LPINT)(buffer+no_chars) = maxSize;
    int n = (int)SendMessage(hwndItem, EM_GETLINE, i,
                         (LONG)(buffer+no_chars));
    no_chars += n;
    maxSize -= (n + 2);
    if (i < (no_lines - 1))
    {
      buffer[no_chars] = 13;
      no_chars ++;
      buffer[no_chars] = 10;
      no_chars ++;
    }
  }
  buffer[no_chars] = 0;
*/
#endif
}

/*
 * ADDED BY JACS Nov 1995
 */
 
/*
 * Some of the following functions are yet to be implemented
 *
 */
 
int wxMultiText::GetNumberOfLines(void)
{
#if XXX
  return (int)SendMessage(GetHWND(), EM_GETLINECOUNT, (WPARAM)0, (LPARAM)0);
#else
  return 0;
#endif
}

long wxMultiText::XYToPosition(long x, long y)
{
#if XXX
    HWND hWnd = GetHWND();
    
    // This gets the char index for the _beginning_ of this line
    int charIndex = (int)SendMessage(hWnd, EM_LINEINDEX, (WPARAM)y, (LPARAM)0);
    return (long)(x + charIndex);
#else
    return 0;
#endif
}

void wxMultiText::PositionToXY(long pos, long *x, long *y)
{
#if XXX
    HWND hWnd = GetHWND();

    // This gets the line number containing the character
    int lineNo = (int)SendMessage(hWnd, EM_LINEFROMCHAR, (WPARAM)pos, (LPARAM)0);
    // This gets the char index for the _beginning_ of this line
    int charIndex = (int)SendMessage(hWnd, EM_LINEINDEX, (WPARAM)lineNo, (LPARAM)0);
    // The X position must therefore be the different between pos and charIndex
    *x = (long)(pos - charIndex);
    *y = (long)lineNo;
#endif
}

void wxMultiText::ShowPosition(long pos)
{
#if XXX
    HWND hWnd = GetHWND();
    
    // To scroll to a position, we pass the number of lines and characters
    // to scroll *by*. This means that we need to:
    // (1) Find the line position of the current line.
    // (2) Find the line position of pos.
    // (3) Scroll by (pos - current).
    // For now, ignore the horizontal scrolling.

    // Is this where scrolling is relative to - the line containing the caret?
    // Or is the first visible line??? Try first visible line.
    int currentLineLineNo1 = (int)SendMessage(hWnd, EM_LINEFROMCHAR, (WPARAM)-1, (LPARAM)0L);

    int currentLineLineNo = (int)SendMessage(hWnd, EM_GETFIRSTVISIBLELINE, (WPARAM)0, (LPARAM)0L);

    int specifiedLineLineNo = (int)SendMessage(hWnd, EM_LINEFROMCHAR, (WPARAM)pos, (LPARAM)0L);
    
    int linesToScroll = specifiedLineLineNo - currentLineLineNo;

/*
    wxDebugMsg("Caret line: %d; Current visible line: %d; Specified line: %d; lines to scroll: %d\n",
      currentLineLineNo1, currentLineLineNo, specifiedLineLineNo, linesToScroll);
*/

    if (linesToScroll != 0)
      (void)SendMessage(hWnd, EM_LINESCROLL, (WPARAM)0, (LPARAM)MAKELPARAM(linesToScroll, 0));
#endif
}

int wxMultiText::GetLineLength(long lineNo)
{
#if XXX
    long charIndex = XYToPosition(0, lineNo);
    HWND hWnd = GetHWND();
    int len = (int)SendMessage(hWnd, EM_LINELENGTH, (WPARAM)charIndex, (LPARAM)0);
    return len;
#else
    return 0;
#endif
}

int wxMultiText::GetLineText(long lineNo, char *buf)
{
#if XXX
    HWND hWnd = GetHWND();
    *(WORD *)buf = 128;
    int noChars = (int)SendMessage(hWnd, EM_GETLINE, (WPARAM)lineNo, (LPARAM)buf);
    buf[noChars] = 0;
    return noChars;
#else
    return 0;
#endif
}

// Copy selection to clipboard
void wxMultiText::Copy(void)
{
#if XXX
    HWND hWnd = GetHWND();
    SendMessage(hWnd, WM_COPY, 0, 0L);
#endif    
}

// Paste clipboard into text window
void wxMultiText::Paste(void)
{
#if XXX
    HWND hWnd = GetHWND();
    SendMessage(hWnd, WM_PASTE, 0, 0L);
#endif
}

// Copy selection to clipboard, then remove selection.
void wxMultiText::Cut(void)
{
#if XXX
    HWND hWnd = GetHWND();
    SendMessage(hWnd, WM_CUT, 0, 0L);
#endif
}

void wxMultiText::WriteText(char *text)
{
#if XXX
    // Covert \n to \r\n
    int len = strlen(text);
    char *newtext = new char[(len*2)+1];
    int i = 0;
    int j = 0;
    while (i < len)
    {
      if (text[i] == '\n')
      {
        newtext[j] = '\r';
        j ++;
      }
      newtext[j] = text[i];
      i ++;
      j ++;
    }
    newtext[j] = 0;
    SendMessage(GetHWND(), EM_REPLACESEL, 0, (LPARAM)newtext);
    delete[] newtext;
#endif
}

