/*
 * File:        wx_panel.cc
 * Purpose:     wxPanel class implementation
 * Author:      Julian Smart
 * Created:     1993
 * Updated:     August 1994
 * RCS_ID:      $Id: wx_panel.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:   (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_item.h"
#include "wx_frame.h"
#include "wx_utils.h"

#include "wx_privt.h"
#include "wx_dcpan.h"

#if USE_EXTENDED_STATICS
#include "wx_stat.h"
#endif

extern char wxPanelClassName[];
extern char wxCanvasClassName[];
COLOR wxFindColor(ULONG rgb_color);
extern HAB wxhAb;

wxPanelWnd::wxPanelWnd(wxWnd *parent, char *winClass, wxWindow *wx_win,
              int x, int y, int width, int height, DWORD flags):
  wxSubWnd(parent, winClass, wx_win, x, y, width, height, flags)
{
    __type = wxTYPE_PANEL_WND;
}

MRESULT wxPanelWnd::DefWindowProc(ULONG nMsg, MPARAM mp1, MPARAM mp2)
{
  return ::WinDefWindowProc(hwndWnd, nMsg, mp1, mp2);
}

BOOL wxPanelWnd::ProcessMessage(QMSG* pMsg)
{
    TRACE("wxPanelWnd::ProcessMessage(pMsg=%#x)", (pMsg));
    // XXX
#if 0
  return ::IsDialogMessage(hwndWnd, pMsg);
#endif
  return FALSE;			// Was TRUE
}


IMPLEMENT_DYNAMIC_CLASS(wxPanel, wxCanvas)

wxPanel::wxPanel(void)
{
    TRACE("wxPanel::wxPanel()");
//  tempPS = 0;
    cursor_x = PANEL_LEFT_MARGIN;
    cursor_y = PANEL_TOP_MARGIN;
    max_height = 0;
    max_line_height = 0;
    max_width = 0;
    hSpacing = PANEL_HSPACING;
    vSpacing = PANEL_VSPACING;
    initial_hspacing = hSpacing ;
    initial_vspacing = vSpacing ;
    current_hspacing = hSpacing ;
    current_vspacing = vSpacing ;

    last_created = 0 ;
    backBrush = 0;		// pgd
    backColour = *wxWHITE ;
    labelColour = *wxBLACK ;
    buttonColour = *wxBLACK ;
}

// Constructor
wxPanel::wxPanel(wxWindow *parent, int x, int y, int width, int height,
                 long style, const char *name):
  wxbPanel(parent, x, y, width, height, style, name)
{
    TRACE("wxPanel::wxPanel(parent=%#x, x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)parent, x, y, width, height, (int)style, name);
    wx_dc = NULL;
    Create(parent, x, y, width, height, style, name);
}

// Constructor
Bool wxPanel::Create(wxWindow *parent, int x, int y, int width, int height, long style,
                     const char *name)
{
    TRACE("wxPanel::Create(parent=%#x, x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)parent, x, y, width, height, (int)style, name);

    if (!parent)
	return FALSE;

    SetName(name);

    is_retained = ((style & wxRETAINED) == wxRETAINED);
    Bool have_colours = FALSE;
    if (wxSubType(parent->__type, wxTYPE_PANEL)) {
	TRACE("wxPanel::Create: Inheriting parameters/colours from parent panel");
	/*
	 * Inherit parameters from parent panel
	 */
	have_colours = TRUE;
	wxPanel *parentPanel = (wxPanel *)parent;
	parentPanel->GetValidPosition(&x,&y);
	labelFont = parentPanel->labelFont;
	buttonFont = parentPanel->buttonFont;
	backColour = parentPanel->backColour;
	labelColour = parentPanel->labelColour;
	buttonColour = parentPanel->buttonColour;
    }

    windowStyle = style;

    if (windowStyle & wxVERTICAL_LABEL)
	SetLabelPosition(wxVERTICAL);
    else if (windowStyle & wxHORIZONTAL_LABEL)
	SetLabelPosition(wxHORIZONTAL);
    else
	SetLabelPosition(wxHORIZONTAL);


//  tempPS = 0;
    cursor_x = PANEL_LEFT_MARGIN;
    cursor_y = PANEL_TOP_MARGIN;
    max_height = 0;
    max_line_height = 0;
    max_width = 0;
    hSpacing = PANEL_HSPACING;
    vSpacing = PANEL_VSPACING;
    initial_hspacing = hSpacing ;
    initial_vspacing = vSpacing ;
    current_hspacing = hSpacing ;
    current_vspacing = vSpacing ;

    last_created = 0 ;
    backBrush = 0;		// pgd

    window_parent = parent;

    wxWnd *cparent = NULL;
    if (parent)
	cparent = (wxWnd *)parent->handle;

    DWORD os2flags = WS_VISIBLE;
#if 0  
    if (style & wxBORDER)
	os2flags |= WS_BORDER;
    os2flags |= WS_CHILD | WS_VISIBLE;
#endif  

    wxPanelWnd *wnd = new wxPanelWnd(cparent, wxCanvasClassName, this, x, y, width, height, os2flags);

    handle = (char *)wnd;
    if (parent)
	parent->AddChild(this);

    if (!have_colours) {
	/*
	 * Query system colours.
	 * They are set up at window creation time
	 * so we just query what we got.
	 */
	TRACE("wxPanel::Create: Getting System colours");
	backColour = wxGetSysColour(SYSCLR_DIALOGBACKGROUND);
	buttonColour = wxGetSysColour(SYSCLR_MENUTEXT);
	labelColour = wxGetSysColour(SYSCLR_WINDOWTEXT);
    }
    TRACE("wxPanel::Create: BackColour=%#06lx", (LONG)backColour);

    wx_dc = new wxPanelDC(this);

    wxSetBackgroundColour(GetHWND(), backColour);
    
    if (wxSubType(parent->__type, wxTYPE_PANEL))
	((wxPanel *)parent)->AdvanceCursor(this) ;

    return TRUE;
}

wxPanel::~wxPanel(void)
{
    TRACE("wxPanel::~wxPanel() this=%#x wxWindow=%#x", (int)this, (wxWindow *)this);
}

void wxPanel::SetLabelPosition(int pos)  // wxHORIZONTAL or wxVERTICAL
{
    label_position = pos;
}

void wxPanel::SetSize(int x, int y, int w, int h, int sizeFlags)
{
    TRACE("wxPanel::SetSize(x=%d, y=%d, w=%d, h=%d, sizeFlags=%#x)", x, y, w, h, sizeFlags);

    int currentX, currentY;
    GetPosition(&currentX, &currentY);
    if (x == -1)
	x = currentX;
    if (y == -1)
	y = currentY;

    int old_width,old_height;
    GetSize(&old_width, &old_height);
    if (w == -1)
	w = old_width ;
    if (h == -1)
	h = old_height ;

    MoveWindow(x, y, w, h);

    // Cheat.. OS/2 windows are upside down so we have to move all the children objects
    int dh = h-old_height;
    if (dh) {
	HENUM henum = WinBeginEnumWindows(GetHWND());
	while (HWND hwnd = WinGetNextWindow(henum)) {
	    SWP swp;
	    WinQueryWindowPos(hwnd, &swp);
	    WinSetWindowPos(hwnd, 0, swp.x, swp.y + dh, 0, 0, SWP_MOVE);
	}
    }

#if XXX
    wxNode   *cNode;
    wxObject *cObj;
    wxItem   *cItem;
    int       delta = (h-old_height);

    for(cNode=children->First(); cNode; cNode=cNode->Next()) {
	cObj = (wxObject *)cNode->Data();
	if (wxSubType(cObj->__type, wxTYPE_ITEM)) {
	    cItem = (wxItem *)cObj;
	    cItem->GetPosition(&currentX, &currentY);
	    cItem->SetSize(-1, (currentY - delta), -1, -1);
	}
    }
#endif

    GetEventHandler()->OnSize(w, h);
}

/*****************************************************************
 * ITEM PLACEMENT FUNCTIONS
 *****************************************************************/

 // Start a new line
void wxPanel::RealNewLine(void)
{
TRACE("wxPanel::RealNewLine");
  //cursor_x = PANEL_LEFT_MARGIN;
TRACE("\told cursor_x=%d, old cursor_y=%d", cursor_x, cursor_y);
  cursor_x = initial_hspacing;
  if (max_line_height == 0)
      cursor_y += current_vspacing;
  else
      cursor_y += current_vspacing + max_line_height;
TRACE("\tnew cursor_x=%d, nex cursor_y=%d", cursor_x, cursor_y);
  max_line_height = 0;
  new_line = FALSE ;
}
  
void wxPanel::NewLine(int pixels)
{
  if (new_line)
    current_vspacing += pixels ;
  else
    current_vspacing = pixels ;
  new_line = TRUE ;
}

void wxPanel::NewLine(void)
{
  if (new_line)
    current_vspacing += vSpacing ;
  else
    current_vspacing = vSpacing ;
  new_line = TRUE ;
}

/*
// Start a new line, OLD VERSION
void wxPanel::NewLine(void)
{
  cursor_x = PANEL_LEFT_MARGIN;
  if (max_line_height == 0)
  {
    cursor_y += vspacing;
  }
  else
    cursor_y += vspacing + max_line_height;
  max_line_height = 0;
}
*/

void wxPanel::Tab(void)
{
  cursor_x += hSpacing;
  if (cursor_x > max_width)
    max_width = cursor_x;
TRACE("wxPanel::Tab(), new cursor_x=%d, new max_width=%d", cursor_x, max_width);
}

void wxPanel::Tab(int pixels)
{
  cursor_x += pixels;
  if (cursor_x > max_width)
    max_width = cursor_x;
TRACE("wxPanel::Tab(pixels=%d) new cursor_x=%d, new max_width=%d", pixels, cursor_x, max_width);
}

void wxPanel::GetCursor(int *x, int *y)
{
  *x = cursor_x;
  *y = cursor_y;
TRACE("wxPanel::GetCursor() returns x=%d, y=%d", *x, *y);
}

// Set/get horizontal spacing
void wxPanel::SetHorizontalSpacing(int sp)
{
TRACE("wxPanel::SetHorizontalSpacing(sp=%d)", sp);
  hSpacing = sp;
  current_hspacing = sp ;
}

int wxPanel::GetHorizontalSpacing(void)
{
TRACE("wxPanel::GetHorizontalSpacing() returns %d", hSpacing);
  return hSpacing;
}

// Set/get vertical spacing
void wxPanel::SetVerticalSpacing(int sp)
{
TRACE("wxPanel::SetVerticalSpacing(sp=%d)", sp);
  vSpacing = sp;
  current_vspacing = sp ;
}

int wxPanel::GetVerticalSpacing(void)
{
TRACE("wxPanel::GetVerticalSpacing() returns %d", vSpacing);
  return vSpacing;
}

// Fits the panel around the items
void wxPanel::Fit(void)
{
    TRACE("wxPanel::Fit()");
    RealAdvanceCursor() ;
    TRACE("max_width=%d, initial_hspacing=%d, max_height=%d, initial_vspacing=%d", max_width, max_height, initial_hspacing, initial_vspacing);
    SetClientSize(max_width + initial_hspacing, max_height + initial_vspacing);
/*
  SetClientSize(max_width + PANEL_HSPACING, max_height + PANEL_VSPACING);
  */
}

// Update next cursor position
void wxPanel::RealAdvanceCursor(void)
{
TRACE("wxPanel::RealAdvanceCursor(), last_created=%#x", (int)last_created);
  wxWindow *item = last_created ;
  if (item)
  {
    int width, height;
    int x, y;
    item->GetSize(&width, &height);
    item->GetPosition(&x, &y);
TRACE("\tlast_created: x=%d, y=%d, width=%d, height=%d", x, y, width, height);

    if ((x + width) > max_width)
      max_width = x + width;
    if ((y + height) > max_height)
      max_height = y + height;
    if (height > max_line_height)
      max_line_height = height;

    cursor_x = x + width + current_hspacing;
    cursor_y = y;
TRACE("\tmax_width=%d, max_height=%d, max_line_height=%d, cursor_x=%d, cursor_y=%d", max_width, max_height, max_line_height, cursor_x, cursor_y);
    last_created = NULL ;
  }
  if (new_line)
    RealNewLine() ;
}


// Update next cursor position
void wxPanel::AdvanceCursor(wxWindow *item)
{
TRACE("wxPanel::AdvanceCursor(item=%#x)", (int)item);
/*
  int width, height;
  int x, y;
  item->GetSize(&width, &height);
  item->GetPosition(&x, &y);

  if ((x + width) > max_width)
    max_width = x + width;
  if ((y + height) > max_height)
    max_height = y + height;
  if (height > max_line_height)
    max_line_height = height;

  cursor_x = x + width + hspacing;
  cursor_y = y;
*/
   last_created = item ;
}

// If x or y are not specified (i.e. < 0), supply
// values based on left to right, top to bottom layout.
// Internal use only.
void wxPanel::GetValidPosition(int *x, int *y)
{
  if (*x < 0)
    *x = cursor_x;

  if (*y < 0)
    *y = cursor_y;
TRACE("wxPanel::GetValidPosition() return x=%d, y=%d", *x, *y);
}

void wxPanel::Centre(int direction)
{
TRACE("wxPanel::Centre(direction=%d)", direction);
  int x, y, width, height, panel_width, panel_height, new_x, new_y;

  wxPanel *father = (wxPanel *)GetParent();
  if (!father)
    return;

  father->GetClientSize(&panel_width, &panel_height);
  GetSize(&width, &height);
  GetPosition(&x, &y);

  new_x = -1;
  new_y = -1;

  if (direction & wxHORIZONTAL)
    new_x = (int)((panel_width - width)/2);

  if (direction & wxVERTICAL)
    new_y = (int)((panel_height - height)/2);

  SetSize(new_x, new_y, -1, -1);
}

void wxPanel::Enable(Bool Flag)
{
  wxWnd *sub = (wxWnd*)handle ;
  WinEnableWindow(sub->hwndWnd,(Bool)Flag) ;
}

void wxPanel::AddChild(wxObject *child)
{
TRACE("wxPanel::AddChild(child=%#x)", (int)child);
  if (!has_child)
  {
    initial_hspacing = hSpacing ;
    initial_vspacing = vSpacing ;
  }
  has_child = TRUE ;

  cursor_x = hSpacing ;
  cursor_y = vSpacing ;
  RealAdvanceCursor() ;
  current_hspacing = hSpacing ;
  current_vspacing = vSpacing ;

  children->Append(child) ;
}

void wxPanel::SetBackgroundColour(wxColour& col)
{
    TRACE("wxPanel::SetBackgroundColour(...)");
    wxWnd *wnd = (wxWnd *)handle;
    if (wnd)
	wxSetBackgroundColour(wnd->hwndWnd, col);
}

void wxPanel::SetLabelColour(wxColour& col) 
{
    labelColour = col;
    wxWnd *wnd = (wxWnd *)handle;
    if (wnd)
	wxSetForegroundColour(wnd->hwndWnd, col);
}

void wxPanel::SetButtonColour(wxColour& col)
{
    buttonColour = col;
    // XXX
}

#if USE_EXTENDED_STATICS

#define LOH_MAX(a,b) ((a > b) ? a : b)
#define LOH_MIN(a,b) ((a < b) ? a : b)

Bool IsBoxCovered(RECTL r1, RECTL r2)
   {
     int x_min, x_max, y_min, y_max, w_max, h_max;

     x_min = LOH_MIN(r1.left  ,r2.left);
     x_max = LOH_MAX(r1.right ,r2.right);
     y_min = LOH_MIN(r1.top   ,r2.top);
     y_max = LOH_MAX(r1.bottom,r2.bottom);

     w_max = r1.right - r1.left + r2.right - r2.left;
     h_max = r1.bottom - r1.top + r2.bottom - r2.top;

     if (((x_max - x_min) <= (w_max)) && ((y_max - y_min) <= (h_max)))
          return TRUE;
     else return FALSE;
   }

void wxPanel::DrawAllStaticItems(PAINTSTRUCT *ps)
{
  int x,y,w,h;
  RECTL r1;
  if (staticItems.Number() > 0)
  {
    wxDC *dc = GetDC();
    dc->BeginDrawing();
    wxNode *node = staticItems.First();
    wxStaticItem *item;
    while(node)
    {
       item = (wxStaticItem *)node -> Data();
       item -> GetDrawingSize(&x,&y,&w,&h);
       r1.left   = x;
       r1.top    = y;
       r1.right  = x + w;
       r1.bottom = y + h;

       if (!ps || (IsBoxCovered(ps -> rcPaint,r1) && item -> IsShow()))
       {
         item -> Draw(ps);
       }
       node = node -> Next();
    }
    dc->EndDrawing();
  }
}
#endif

void wxPanel::OnPaint(void)
{
#if USE_EXTENDED_STATICS
  DrawAllStaticItems(tempPS);
#endif
  PaintSelectionHandles();
}

