/*
 * File:	wx_rbox.cc
 * Purpose:	Radio box item implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1995, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_rbox.h"
#include "wx_itemp.h"

#define	FRAME_MARGIN	6
#define	LINE_SPACING	4
#define	COLUMN_SPACING	8

// Radio box item
BOOL wxRadioBox::OS2Control(WORD notifycode, WORD id)
{
    TRACE("wxRadioBox::OS2Control(notifycode=%d, id=%d)", notifycode, id);
    if (notifycode == BN_CLICKED) {
	for (int i = 0; i < no_items; i++)
	    if (id == WinQueryWindowUShort(hwndButtons[i], QWS_ID)) {
		selected = i;
		break;
	    }

	wxCommandEvent event(wxEVENT_TYPE_RADIOBOX_COMMAND);
	event.commandInt = selected;
	event.eventObject = this;
	ProcessCommand(event);
	return TRUE;
    }
    return FALSE;
}

IMPLEMENT_DYNAMIC_CLASS(wxRadioBox, wxItem)

wxRadioBox::wxRadioBox(void)
{
    no_items = 0;
    hwndButtons = NULL;
    majorDim = 0 ;
    selected = -1;
    radioWidth = NULL ;
    radioHeight = NULL ;
}

wxRadioBox::wxRadioBox(wxPanel *panel, wxFunction func,
                       const char *Title,
                       int x, int y, int width, int height,
                       int N, char **Choices,
                       int MajorDim,long style, const char *name):
  wxbRadioBox(panel, func, Title, x, y, width, height, N, Choices,
              MajorDim, style, name)
{
  Create(panel, func, Title, x, y, width, height, N, Choices, MajorDim, style, name);
}

/*
 * Causes problems for the Turbo C++ for Windows linker
 *
 */

wxRadioBox::wxRadioBox(wxPanel *panel, wxFunction func,
                       const char *Title,
                       int x, int y, int width, int height,
                       int N, wxBitmap **Choices,
                       int MajorDim,long style, const char *name):
  wxbRadioBox(panel, func, Title, x, y, width, height, N, Choices,
              MajorDim, style, name)
{
  Create(panel, func, Title, x, y, width, height, N, Choices, MajorDim, style, name);
}


Bool wxRadioBox::Create(wxPanel *panel, wxFunction func,
                       const char *Title,
                       int x, int y, int width, int height,
                       int N, char **Choices,
                       int MajorDim, long style, const char *name)
{
    TRACE("wxRadioBox::Create(panel=%#x, func=%#x, Title='%s', x=%d, y=%d, w=%d, h=%d, N=%d, Choices=%#x, MajorDim=%d, style=%#x, name='%s')", (int)panel, (int)func, Title, x, y, width, height, N, (int)Choices, MajorDim, (int)style, name);
    SetName(name);
    if (panel) panel->AddChild(this);

    /*
     * Inherit Fonts and Colours from the panel
     */
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;

    noRowsOrCols = MajorDim;
    if (MajorDim==0)
	MajorDim = N ;
    majorDim = MajorDim ;
    windowStyle = style ;
    selected = -1;

    SetAppropriateLabelPosition();
    panel->GetValidPosition(&x, &y);

    char *the_label = NULL ;

    if (Title) {
	the_label = new char[strlen(Title)+1] ;
	if (style & wxFIXED_LENGTH) {
	    for (int i = 0; i < (int)strlen(Title); i++)
	        the_label[i] = MEANING_CHARACTER;
	} else
	    strcpy(the_label, Title) ;
	the_label[strlen(Title)] = '\0' ;
    } else
	the_label = copystring("") ;
    
    /*
     * Create the OS/2 window
     */
    HWND the_handle = panel->GetHWND();

    hwndItem = WinCreateWindow(the_handle, WC_STATIC, (PSZ)Title,
			       SS_GROUPBOX, 0, 0, 0, 0, the_handle,
			       HWND_TOP, NewControlId(), NULL, NULL);
    TRACE("wxRadioBox::Create: Window created, parent=%#x, handle=%#x", (int)the_handle, (int)hwndItem);

    /*
     * Set the correct colours as inherited from
     * the panel.
     */
    wxSetBackgroundColour(hwndItem, backColour);
    wxSetForegroundColour(hwndItem, buttonColour);

    /*
     * Subclass for purposes of dialog editing mode
     */
    SubclassControl(hwndItem);

    if (labelFont)
	wxSetWinFont(hwndItem, labelFont);

    // Some radio boxes test consecutive id.
    (void)NewControlId() ;
    hwndButtons = new HWND[N];
    radioWidth = new int[N] ;
    radioHeight = new int[N] ;
    for (int i = 0; i < N; i++) {
	radioWidth[i] = radioHeight[i] = -1;
	long groupStyle = 0;
	if (i == 0 && style==0)
	    groupStyle = WS_GROUP;
	long NewId = NewControlId();
	hwndButtons[i] = WinCreateWindow(the_handle, WC_BUTTON, (PSZ)Choices[i],
					 groupStyle | BS_AUTORADIOBUTTON, 0, 0, 0, 0, the_handle,
					 HWND_TOP, NewId, NULL, NULL);
	if (buttonFont)
	    wxSetWinFont(hwndButtons[i], buttonFont);
	wxSetBackgroundColour(hwndButtons[i], backColour);
	wxSetForegroundColour(hwndButtons[i], buttonColour);

	subControls.Append((wxObject *)NewId);
    }
    (void)NewControlId() ;

    no_items = N;
    SetSelection(0);

    SetSize(x, y, width, height);
    panel->AdvanceCursor(this);
    Callback(func);

    if (Title) {
	if (style & wxFIXED_LENGTH)
	    SetLabel((char *)Title) ;
    }
    if (the_label)
	delete [] the_label ;

    return TRUE;
}


Bool wxRadioBox::Create(wxPanel *panel, wxFunction func,
                       const char *Title,
                       int x, int y, int width, int height,
                       int N, wxBitmap **Choices,
                       int MajorDim, long style, const char *name)
{
    TRACE("wxRadioBox::Create(panel=%#x, func=%#x, Title='%s', x=%d, y=%d, w=%d, h=%d, N=%d, Choices=%#x, MajorDim=%d, style=%#x, name='%s')", (int)panel, (int)func, Title, x, y, width, height, N, (int)Choices, MajorDim, (int)style, name);
    SetName(name);
    if (panel) panel->AddChild(this);
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
TRACE("buttonColour=%#06lx", (LONG)buttonColour);
    buttonColour = panel->buttonColour ;
    noRowsOrCols = MajorDim;
    if (MajorDim==0)
	MajorDim = N ;
    majorDim = MajorDim ;
    windowStyle = style ;
    selected = -1;

    panel->GetValidPosition(&x, &y);

    char *the_label = NULL ;

    if (Title) {
	the_label = new char[strlen(Title)+1] ;
	if (style & wxFIXED_LENGTH)
	    for (int i = 0; i < (int)strlen(Title); i++)
		the_label[i] = MEANING_CHARACTER;
	else
	    strcpy(the_label,Title) ;
	the_label[strlen(Title)] = '\0' ;
    } else
	the_label = copystring("") ;

    HWND the_handle = panel->GetHWND();

    hwndItem = WinCreateWindow(the_handle, WC_BUTTON, (PSZ)Title,
			       0, 0, 0, 0, 0, the_handle,
			       HWND_TOP, NewControlId(), NULL, NULL);
    TRACE("wxRadioBox::Create: Window created, parent=%#x, handle=%#x", (int)the_handle, (int)hwndItem);

    wxSetForegroundColour(hwndItem, buttonColour);
    wxSetBackgroundColour(hwndItem, backColour);

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

    if (labelFont)
	wxSetWinFont(hwndItem, labelFont);

    (void)NewControlId() ;
    hwndButtons = new HWND[N];
    radioWidth = new int[N] ;
    radioHeight = new int[N] ;
    for (int i = 0; i < N; i++) {
	long groupStyle = 0;
	if (i == 0 && style == 0)
	    groupStyle = WS_GROUP;
	long newId = NewControlId();
	radioWidth[i]  = Choices[i]->GetWidth();
	radioHeight[i] = Choices[i]->GetHeight();
	char tmp[32] ;
	sprintf(tmp,"Toggle %d",i) ;
	hwndButtons[i] = WinCreateWindow(the_handle, WC_BUTTON, (PSZ)tmp,
					  groupStyle | BS_AUTORADIOBUTTON, 0, 0, 0, 0, the_handle,
					  HWND_TOP, newId, NULL, NULL);
	if (buttonFont)
	    wxSetWinFont(hwndButtons[i], buttonFont);

	wxSetBackgroundColour(hwndButtons[i], backColour);
	wxSetForegroundColour(hwndButtons[i], buttonColour);

	subControls.Append((wxObject *)newId);

    }
    (void)NewControlId() ;

    no_items = N;
    SetSelection(0);

    SetSize(x, y, width, height);
//  WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);
    panel->AdvanceCursor(this);
    Callback(func);

    if (Title)
    {
	if (style&wxFIXED_LENGTH)
	    SetLabel((char *)Title) ;
	if (the_label)
	    delete [] the_label ;
    }
//  if (the_label)
//    delete [] the_label ;

    return TRUE;
}


wxRadioBox::~wxRadioBox(void)
{
    TRACE("wxRadioBox::~wxRadioBox() this=%#x wxWindow=%#x", this, (wxWindow *)this);
    if (hwndButtons) {
	for (int i = 0; i < no_items; i++)
	    WinDestroyWindow(hwndButtons[i]);
	delete[] hwndButtons;
    }
    if (radioWidth)
	delete[] radioWidth ;
    if (radioHeight)
	delete[] radioHeight ;
}

char *wxRadioBox::GetLabel(int item)
{
    char buf[300];
    WinQueryWindowText((HWND)hwndButtons[item], 300, (PCH)buf);
    return copystring(buf);
}

void wxRadioBox::SetLabel(int item, const char *label)
{
  radioWidth[item] = radioHeight[item] = -1 ;
  WinSetWindowText((HWND)hwndButtons[item], (PSZ)label);
}

void wxRadioBox::SetLabel(int item, wxBitmap *bitmap)
{
    // Switch from Bitmap style to text style, if needed
    // XXX
}

int wxRadioBox::FindString(char *s)
{
    for (int i = 0; i < no_items; i++) {
	WinQueryWindowText(hwndButtons[i], 1000, (PCH)wxBuffer);
	if (strcmp(wxBuffer, s) == 0)
	    return i;
    }
    return -1;
}

void wxRadioBox::SetSelection(int N)
{
    if ((N < 0) || (N >= no_items))
	return;

    WinSendMsg(hwndButtons[N], BM_SETCHECK, (MPARAM)1, 0L);

    selected = N;
}

// Get single selection, for single choice list items
int wxRadioBox::GetSelection(void)
{
    return selected;
}

// Find string for position
char *wxRadioBox::GetString(int N)
{
    WinQueryWindowText(hwndButtons[N], 1000, (PCH)wxBuffer);
    return wxBuffer;
}

void wxRadioBox::GetAutoSize(int *cx, int *cy)
{
    TRACE("wxRadioBox::GetAutoSize()");

    int ccx, ccy;
    wxGetCharSize(hwndItem, &ccx, &ccy);
    int maxWidth = -1;
    int maxHeight = -1;

    int i;
    for (i = 0 ; i < no_items; i++) {
	int eachWidth, eachHeight ;
	if (radioWidth[i] < 0) {
	    int tcx = wxGetTextWidth(hwndButtons[i]);
	    eachWidth = tcx + RADIO_SIZE;
	    eachHeight = ccy;
	} else {
	    eachWidth = radioWidth[i];
	    eachHeight = radioHeight[i];
	}
	maxWidth = wxMax(maxWidth, eachWidth);
	maxHeight = wxMax(maxHeight, eachHeight);
    }

    int nrows, ncols;
    if (windowStyle & wxVERTICAL) {
	nrows = majorDim;
	ncols = (no_items + majorDim - 1) / majorDim;
    } else {
	ncols = majorDim;
	nrows = (no_items + majorDim - 1) / majorDim;
    }
    
    int frame_width = wxGetTextWidth(hwndItem) + 2 * FRAME_MARGIN + 2 * ccx / 2;
    TRACE("ncols=%d, nrows=%d, frame_width=%d, maxWidth=%d, maxHeight=%d", ncols, nrows, frame_width, maxWidth, maxHeight);
    *cy = nrows * maxHeight + (nrows - 1) * LINE_SPACING + 2 * FRAME_MARGIN + ccy / 2;
    *cx = ncols * maxWidth + (ncols - 1) * COLUMN_SPACING + 2 * FRAME_MARGIN;
    if (frame_width > *cx)
	*cx = frame_width;
}


/*
 * Position the item within the given coordinates
 */
void wxRadioBox::PositionItem(int x, int y, int width, int height)
{
    TRACE("wxRadioBox::PositionItem(x=%d, y=%d, width=%d, height=%d", x, y, width, height);
    int ccx, ccy;
    wxGetCharSize(hwndItem, &ccx, &ccy);

    /*
     * First position the group box
     */
    MoveWindow(x, y, width, height, TRUE);

    /*
     * Position the radio box items to fit in the given space
     */
    int nrows, ncols;
    if (windowStyle & wxVERTICAL) {
	nrows = majorDim;
	ncols = (no_items + majorDim - 1) / majorDim;
    } else {
	ncols = majorDim;
	nrows = (no_items + majorDim - 1) / majorDim;
    }

    TRACE("ncols=%d, nrows=%d", ncols, nrows);
    int i;
    /*
     * (x0, y0) is the position of the upper left item
     */
    int x0 = x + FRAME_MARGIN;
    int y0 = y + ccy / 2  + FRAME_MARGIN;
    int xx = (width - FRAME_MARGIN * 2) / ncols;
    int yy = (height - ccy / 2 - 2 * FRAME_MARGIN) / nrows;
    TRACE("x0=%d, y0=%d, xx=%d, yy=%d", x0, y0, xx, yy);
    for (i = 0; i < no_items; i++)
	wx_MoveWindow(hwndButtons[i], x0 + xx * (i % ncols), y0 + yy * (i / ncols), xx, yy, TRUE);
}

void wxRadioBox::GetSize(int *width, int *height)
{
    wxRECTL rect = {-1, -1, -1, -1};

    if (hwndItem)
	wxFindMaxSize(hwndItem, &rect);

    for (int i = 0; i < no_items; i++)
	wxFindMaxSize(hwndButtons[i], &rect);

    *width  = rcWidth(rect);
    *height = rcHeight(rect);
    TRACE("wxRadioBox::GetSize() returns width=%d, height=%d", *width, *height);
}

void wxRadioBox::GetPosition(int *x, int *y)
{
    wxWindow *parent = GetParent();
    wxRECTL rect = {-1, -1, -1, -1};

    for (int i = 0; i < no_items; i++)
	wxFindMaxSize(hwndButtons[i], &rect);

    if (hwndItem)
	wxFindMaxSize(hwndItem, &rect);

    // Since we now have the absolute screen coords,
    // if there's a parent we must subtract its top left corner
    wxPOINTL point = { rect.xLeft, rect.yTop };
    if (parent) {
	wxWnd *cparent = (wxWnd *)(parent->handle);
	wx_ScreenToClient(cparent->GetClientHWND(), &point);
    }

    *x = point.x;
    *y = point.y;
    TRACE("wxRadioBox::GetPosition returns x=%d, y=%d", *x, *y);
}

char *wxRadioBox::GetLabel(void)
{
    WinQueryWindowText(hwndItem, 300, (PCH)wxBuffer);
    return wxBuffer;
}

void wxRadioBox::SetLabel(char *label)
{
	float w, h;
	GetTextExtent(label, &w, &h, NULL, NULL, labelFont);
	WinSetWindowPos(hwndItem, 0, 0, 0, (int)(w+10), (int)h, SWP_SIZE);
	WinSetWindowText(hwndItem, (PSZ)label);
}

void wxRadioBox::SetFocus(void)
{
  if (no_items > 0)
   ::WinSetFocus(HWND_DESKTOP,hwndButtons[0]);
}

// Enable all controls
void wxRadioBox::Enable(Bool enable)
{
    wxItem::Enable(enable);
  
    int i;
    for (i = 0; i < no_items; i++)
	WinEnableWindow(hwndButtons[i], enable);
}

Bool wxRadioBox::Show(Bool show)
{
    if (show) {
	WinSetWindowPos(hwndItem, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE | SWP_ZORDER);
	for (int i = 0; i < no_items; i++)
	    WinShowWindow(hwndButtons[i], TRUE);
//          WinSetWindowPos(hwndButtons[i], HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);
    } else
	WinShowWindow(hwndItem, FALSE);
    return TRUE;
}

// Enable a specific button
void wxRadioBox::Enable(int item, Bool enable)
{
  if (item<0)
    wxWindow::Enable(enable) ;
  else if (item < no_items)
    ::WinEnableWindow(hwndButtons[item], enable);
}

// Show a specific button
void wxRadioBox::Show(int item, Bool show)
{
	if (item<0)
		wxRadioBox::Show(show);
	else if (item < no_items)
//    WinSetWindowPos(hwndButtons[item], HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);
		WinShowWindow(hwndButtons[item], TRUE);
}

void wxRadioBox::SetLabelFont(wxFont *font)
{
    labelFont = font;
    HWND hWnd = GetHWND();
#if XXX
    if (hWnd != 0) {
	if (font) {
	    if (font->GetResourceHandle())
		SendMessage(hWnd, WM_SETFONT,
			    (WPARAM)font->GetResourceHandle(),TRUE);
	}
    }
#endif
}

void wxRadioBox::SetButtonFont(wxFont *font)
{
    buttonFont = font;
    if (font) {
	int i;
#if XXX
	for (i = 0; i < no_items; i++) {
	    if (font->GetResourceHandle()) {
		SendMessage(hwndButtons[i], WM_SETFONT,
			    (WPARAM)font->GetResourceHandle(),TRUE);
	    }
	}
#endif
    }
}

void wxRadioBox::SetButtonColour(wxColour& col)
{
    TRACE("wxRadioBox::SetButtonColour(col=%#06lx)", (LONG)col);
    wxItem::SetButtonColour(col);
    if (hwndItem) {
	wxSetForegroundColour(hwndItem, buttonColour);
	for (int i = 0; i < no_items; i++)
	    wxSetForegroundColour(hwndButtons[i], buttonColour);
    }
}

void wxRadioBox::SetBackgroundColour(wxColour& col)
{
    TRACE("wxRadioBox::SetBackgroundColour(%#06lx)", (LONG)col);
    wxItem::SetBackgroundColour(col);
    if (hwndItem) {
	wxSetBackgroundColour(hwndItem, backColour);
	for (int i = 0; i < no_items; i++)
	    wxSetBackgroundColour(hwndButtons[i], backColour);
    }
}

