/*
 * File:        wx_scrol.cc
 * Purpose:     Scrolbar items implementation (OS/2 version)
 * Author:    Sergey Krasnov (ksa@orgland.ru)
 * Created:   1995
 * Updated:
 * Copyright:
 */

static const char sccsid[] = "%W% %G%";

#if !defined(_MSC_VER) && !defined(wx_wxh)
#define wx_wxh
#endif

#include "common.h"

#include "wx.h"
#pragma hdrstop

#if USE_SCROLLBAR

#include "wx_scrol.h"
#include "wx_privt.h"
#include "wx_itemp.h"
#include "wx_utils.h"

extern wxList wxScrollBarList;
extern void wxFindMaxSize(HWND hwnd, RECTL *rect);

// Scrollbar
IMPLEMENT_DYNAMIC_CLASS(wxScrollBar, wxItem)

wxScrollBar::wxScrollBar(void)
{
    pageSize = 0;
    viewSize = 0;
}

wxScrollBar::wxScrollBar(wxPanel *panel, wxFunction func,
			 int x, int y, int width, int height, long style, const char *name):
    wxbScrollBar(panel, func, x, y, width, height, style, name)
{
    Create(panel, func, x, y, width, height, style, name);
}

Bool wxScrollBar::Create(wxPanel *panel, wxFunction func,
                int x, int y, int width, int height,
                long style, const char *name)
{
    if (!panel)
        return FALSE;
    panel->AddChild(this);
    SetName(name);

    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;
    windowStyle = style;
    wxWnd *cparent = NULL;
    cparent = (wxWnd *)(panel->handle);
    window_parent = panel;
    labelPosition = panel->label_position;
    panel->GetValidPosition(&x, &y);

    // Now create scrollbar
    windows_id = (int)NewControlId();
    ULONG flStyle = (style & wxHORIZONTAL) ? SBS_HORZ: SBS_VERT;

    hwndItem = WinCreateWindow(panel->GetHWND(), WC_SCROLLBAR, NULL,
			       flStyle, 0, 0, 0, 0, panel->GetHWND(),
			       HWND_TOP, windows_id, NULL, NULL);

    // Subclass again for purposes of dialog editing mode
    SubclassControl(hwndItem);

    wxScrollBarList.Append((long)hwndItem, this);
    pageSize = 1;
    viewSize = 1;

    WinSendMsg(hwndItem, SBM_SETSCROLLBAR, (MPARAM)0, MPFROM2SHORT(0, 1));

    WinShowWindow(hwndItem, TRUE);

    SetSize(x, y, width, height);

    panel->AdvanceCursor(this);
    Callback(func);

    return TRUE;
}

wxScrollBar::~wxScrollBar(void)
{
    TRACE("wxScrollBar::~wxScrollBar() this=%#x wxWindow=%#x", this, (wxWindow *)this);
    wxScrollBarList.DeleteObject(this);
}

// Called from wx_win.cc: wxWnd::OnHScroll, wxWnd::OnVScroll
void wxScrollBarEvent(HWND hbar, int wParam, int pos)
{
    TRACE("wxScrollBarEvent(hbar=%#x, wParam=%d, pos=%d)", (int)hbar, wParam, pos);
    wxNode *node = (wxNode *)wxScrollBarList.Find((long)hbar);
    if (!node)
	return;

    wxScrollBar *scrollBar = (wxScrollBar *)node->Data();
    int position = (int)WinSendMsg(hbar, SBM_QUERYPOS, (MPARAM)NULL, (MPARAM)NULL);

    int nScrollInc;
    switch ( wParam ) {
    case SB_LINEUP:
	nScrollInc = -1;
	break;
      
    case SB_LINEDOWN:
	nScrollInc = 1;
	break;
      
    case SB_PAGEUP:
	nScrollInc = -scrollBar->pageSize;
	break;
      
    case SB_PAGEDOWN:
	nScrollInc = scrollBar->pageSize;;
	break;
      
    case SB_SLIDERTRACK:
	nScrollInc = pos - position;
	break;
      
    default:
	nScrollInc = 0;
    }

    if (nScrollInc != 0) {
        int new_pos = position + nScrollInc;
#if XXX
        int minPos, maxPos;
        GetScrollRange(hbar, SB_CTL, &minPos, &maxPos);
        if (new_pos < 0)
            new_pos = 0;
        if (new_pos > maxPos)
            new_pos = maxPos;
#endif
        scrollBar->SetValue(new_pos);
        wxCommandEvent event(wxEVENT_TYPE_SCROLLBAR_COMMAND);
        event.commandInt = new_pos;
        event.eventObject = scrollBar;
        scrollBar->ProcessCommand(event);
    }
}

void wxScrollBar::SetValue(int viewStart)
{
TRACE("wxScrollBar::SetValue(%d) this=%#x", viewStart, (int)this);
	WinSendMsg(hwndItem, SBM_SETPOS, (MPARAM)viewStart, (MPARAM)0);
}

int wxScrollBar::GetValue(void)
{
  return (int)WinSendMsg(hwndItem, SBM_QUERYPOS, (MPARAM)NULL, (MPARAM)NULL);
}

void wxScrollBar::SetPageLength(int pageLength)
{
    pageSize = pageLength;
}

void wxScrollBar::SetObjectLength(int objectLength)
{
  WinSendMsg(hwndItem, SBM_SETSCROLLBAR, 
	     WinSendMsg(hwndItem, SBM_QUERYPOS, (MPARAM)NULL, (MPARAM)NULL),
	     MPFROM2SHORT(0, objectLength));
}

void wxScrollBar::SetViewLength(int viewLength)
{
    viewSize = viewLength;
}


void wxScrollBar::GetValues(int *viewStart, int *viewLength, int *objectLength,
           int *pageLength)
{
    MPARAM range = WinSendMsg(hwndItem, SBM_QUERYRANGE, (MPARAM)0, (MPARAM)0);
    *objectLength = SHORT2FROMMP(range) - SHORT1FROMMP(range) + 1;
    *viewStart = (int)WinSendMsg(hwndItem, SBM_QUERYPOS, (MPARAM)NULL, (MPARAM)NULL);
    *viewLength = 1;			// XXX Not correct
    *pageLength = pageSize;
}

char *wxScrollBar::GetLabel(void)
{
    return NULL;
}

void wxScrollBar::SetLabel(char *WXUNUSED(label))
{
}

#endif
