/*
 * File:        wx_timer.cc
 * Purpose:     wxTimer implementation
 * Author:      Julian Smart
 * Created:     1993
 * Updated:     August 1994
 * RCS_ID:      $Id: wx_timer.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:   (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#ifndef WX_PRECOMP
#include "common.h"
#endif

#include "wx_list.h"
#include "wx_timer.h"
#include "wx_stdev.h"
#include "wx_main.h"
#include "wx_utils.h"

extern HAB wxhAb;
extern char wxTimerClassName[];

IMPLEMENT_DYNAMIC_CLASS(wxTimer, wxObject)

wxTimer::wxTimer(void)
{
    id = 0;
    hwndTimer = NULLHANDLE;
}

wxTimer::~wxTimer(void)
{
    TRACE("wxTimer::~wxTimer(): id=%d, hwndTimer=%#x", id, (int)hwndTimer);
    if (hwndTimer) {
	Stop();
	WinDestroyWindow(hwndTimer);
    }
}

Bool wxTimer::Start(int milliseconds, Bool mode)
{
    if (hwndTimer == NULLHANDLE) {
	id = NewControlId();

	ULONG ctldata[3];
	ctldata[0] = sizeof ctldata;
	ctldata[1] = (int)this;
	ctldata[2] = 0;
	hwndTimer = WinCreateWindow(HWND_OBJECT, (PSZ)wxTimerClassName,
				     NULL, 0,
				     0, 0, 0, 0,
				     0,
				     HWND_BOTTOM, id, (PVOID)ctldata, NULL);
	TRACE("wxTimer:: id=%d, hwndTimer=%#x", id, (int)hwndTimer);
	TRACE("wxTimer::Start(milliseconds=%d, mode=%d) id=%d, hwndTimer=%#x", milliseconds, (int)mode, (int)hwndTimer, id);
    }
    oneShot = mode ;
    if (milliseconds < 0)
	milliseconds = lastMilli;

    if (milliseconds <= 0)
	return FALSE;

    lastMilli = milli = milliseconds;

    id = WinStartTimer(wxhAb, hwndTimer, (UINT)(id ? id : 1), (UINT)milliseconds);
    if (id > 0) {
	return TRUE;
    }
    return FALSE;
}

void wxTimer::Stop(void)
{
    TRACE("wxTimer::Stop() id=%d, hwndTimer=%#x", id, (int)hwndTimer);
    if (id)
	WinStopTimer(wxhAb, hwndTimer, (UINT)id);
    milli = 0;
}

MRESULT EXPENTRY
wxTimerProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxTimer *timer;
    if (msg == WM_CREATE) {
	ULONG *ctlptr = (ULONG *)PVOIDFROMMP(mp1);
	timer = (wxTimer *)ctlptr[1];
	TRACE("wxTimerProc: WM_CREATE, hwnd=%#x, timer=%#x", (int)hwnd, (int)timer);
	WinSetWindowPtr(hwnd, QWL_USER, timer);
	assert(timer->hwndTimer == NULLHANDLE);
	timer->hwndTimer = hwnd;
    } else
	timer = (wxTimer *)WinQueryWindowPtr(hwnd, QWL_USER);
    assert(timer != NULL);

#ifdef DEBUGPM
    wxTraceEvent("wxTimerProc", hwnd, msg, mp1, mp2, (PVOID)timer);
#endif

    switch (msg) {

    case WM_TIMER:
	assert(timer->id == (int)mp1);
	if (timer->oneShot)
	    timer->Stop() ;
	timer->Notify();
	break;
	
    default:
	return WinDefWindowProc(hwnd, msg, mp1, mp2);
    }
    return 0;
}
