/*
 * File:	wx_txt.cc
 * Purpose:	Single-line text item implementation
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	April 1995
 * Copyright:	(c) 1995, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include "wx_panel.h"
#include "wx_txt.h"
#include "wx_itemp.h"

// Text item

BOOL wxText::OS2Control(WORD param, WORD WXUNUSED(id))
{
/*
  // Debugging
  wxDebugMsg("Edit control %d: ", (int)id);
  switch (param)
  {
    case EN_SETFOCUS:
      wxDebugMsg("EN_SETFOCUS\n");
      break;
    case EN_KILLFOCUS:
      wxDebugMsg("EN_KILLFOCUS\n");
      break;
    case EN_CHANGE:
      wxDebugMsg("EN_CHANGE\n");
      break;
    case EN_UPDATE:
      wxDebugMsg("EN_UPDATE\n");
      break;
    case EN_ERRSPACE:
      wxDebugMsg("EN_ERRSPACE\n");
      break;
    case EN_MAXTEXT:
      wxDebugMsg("EN_MAXTEXT\n");
      break;
    case EN_HSCROLL:
      wxDebugMsg("EN_HSCROLL\n");
      break;
    case EN_VSCROLL:
      wxDebugMsg("EN_VSCROLL\n");
      break;
    default:
      wxDebugMsg("Unknown EDIT notification\n");
      break;
  }
*/
  WXTYPE eventTyp = 0;
  switch (param)
  {
    case EN_SETFOCUS:
      eventTyp = wxEVENT_TYPE_SET_FOCUS;
      break;
    case EN_KILLFOCUS:
      eventTyp = wxEVENT_TYPE_KILL_FOCUS;
      break;
    case EN_CHANGE:
      break;
#if 0      
    case EN_UPDATE:
      eventTyp = wxEVENT_TYPE_TEXT_COMMAND;
      break;
    case EN_ERRSPACE:
      break;
    case EN_MAXTEXT:
      break;
#endif      
    case EN_SCROLL:
       break;
    default:
      break;
  }
  if (eventTyp != 0)
  {
    wxCommandEvent event(eventTyp);
    event.commandString = GetValue();
    event.eventObject = this;
    ProcessCommand(event);
    return TRUE;
  }
  else
    return FALSE;
}

IMPLEMENT_DYNAMIC_CLASS(wxText, wxItem)

wxText::wxText(void)
{
}

wxText::wxText(wxPanel *panel, wxFunction Function, const char *label, const char *value,
               int x, int y, int width, int height, long style, const char *name):
  wxbText(panel, Function, label, value, x, y, width, height, style, name)
{
    Create(panel, Function, label, value, x, y, width, height, style, name);
}
  
Bool wxText::Create(wxPanel *panel, wxFunction Function, const char *label, const char *value,
               int x, int y, int width, int height, long style, const char *name)
{
    TRACE("wxText::Create(panel=%#x, Function=%#x, label='%s', value='%s', x=%d, y=%d, w=%d, h=%d, style=%#x, name='%s')", (int)panel, (int)Function, label, value, x, y, width, height, (int)style, name);
    SetName(name);
    if (panel) panel->AddChild(this);
    buttonFont = panel->buttonFont ;
    labelFont = panel->labelFont ;
    windowStyle = style;
    backColour = panel->backColour ;
    labelColour = panel->labelColour ;
    buttonColour = panel->buttonColour ;
    windowStyle = style;

//    labelPosition = panel->label_position;
    SetAppropriateLabelPosition();
    panel->GetValidPosition(&x, &y);

    // If label exists, create a static control for it.
    if (label)
	CreateLabel(panel->GetHWND(), label, style & wxFIXED_LENGTH);

    windows_id = (int)NewControlId();

    TRACE("\tpanel=%#x, panel->GetHWND()=%#x", (int)panel, (int)panel->GetHWND());
    hwndItem =  WinCreateWindow(panel->GetHWND(), WC_ENTRYFIELD, NULL, 
				ES_AUTOSCROLL | ES_LEFT | WS_TABSTOP,
				0, 0, 0, 0, panel->GetHWND(),
				HWND_TOP, windows_id, NULL, NULL);
    TRACE("wxText::Create: Window created, parent=%#x, handle=%#x", (int)panel->GetHWND(), (int)hwndItem);


/* The following code subclasses the EDIT control
 * to intercept the ENTER key, which only
 * works if the style wxPROCESS_ENTER has been used.
 */

    SubclassControl(hwndItem);

    if (buttonFont)
	wxSetWinFont(hwndItem, buttonFont);

    if ((width < 0) && value && strlen(value)) {
	char *the_value ;
	the_value = new char[strlen(value)+1] ;
	if (style & wxFIXED_LENGTH)
	    for (int i = 0; i < value[i]; i++)
		the_value[i] = MEANING_CHARACTER ;
	else
	    strcpy(the_value,value) ;
	the_value[strlen(value)] = '\0' ;

	int cx, cy;
	wxGetCharSize(hwndItem, &cx, &cy) ;
	float current_width, cyf;
	GetTextExtent(the_value, &current_width, &cyf, NULL, NULL, buttonFont) ;
	width = (int)current_width + 2*cx ;
	if (height <= 0)
	    height = (int)cyf;
	delete [] the_value ;
    }

    if (height <= 0)
	height = wxGetCharHeight(hwndItem);

    SetSize(x, y, width, height);
    Show();

    if (value)
	WinSetWindowText(hwndItem, (PSZ)value);

    panel->AdvanceCursor(this);
    Callback(Function);

    WinShowWindow(hwndItem, TRUE);
    return TRUE;
}

wxText::~wxText(void)
{
    TRACE("wxText::~wxText() this=%#x wxWindow=%#x", this, (wxWindow *)this);
}

char *wxText::GetValue(void)
{
    TRACE("wxText::GetValue()");
    int nchars = WinQueryWindowText(hwndItem, 500, (PCH)wxBuffer);
    wxBuffer[nchars] = 0;
    TRACE("wxText::GetValue: Returns '%s'", wxBuffer);
    return wxBuffer;
}

void wxText::SetValue(char *value)
{
    if (!value)
	value = "";
    
    // If newlines are denoted by just 10, must stick 13 in front.
    int singletons = 0;
    int len = strlen(value);
    int i;
    for (i = 0; i < len; i ++) {
	if ((i > 0) && (value[i] == 10) && (value[i-1] != 13))
	    singletons ++;
    }
    if (singletons > 0) {
	char *tmp = new char[len + singletons + 1];
	int j = 0;
	for (i = 0; i < len; i ++) {
	    if ((i > 0) && (value[i] == 10) && (value[i-1] != 13)) {
		tmp[j] = 13;
		j ++;
	    }
	    tmp[j] = value[i];
	    j ++;
	}
	tmp[j] = 0;
	WinSetWindowText(hwndItem, (PSZ)tmp);
	delete[] tmp;
    } else
	WinSetWindowText(hwndItem, (PSZ)value);
}

void wxText::GetAutoSize(int *cx, int *cy)
{
    *cx = DEFAULT_ITEM_WIDTH;
    *cy = wxGetCharHeight(hwndItem);
}

void wxText::SetFocus(void)
{
  wxItem::SetFocus();
}

// Clipboard operations
void wxText::Copy(void)
{
#if 0
  HWND hWnd = GetHWND();
  SendMessage(hWnd, WM_COPY, 0, 0L);
#endif  
}

void wxText::Cut(void)
{
#if 0
  HWND hWnd = GetHWND();
  SendMessage(hWnd, WM_CUT, 0, 0L);
#endif  
}

void wxText::Paste(void)
{
#if 0
  HWND hWnd = GetHWND();
  SendMessage(hWnd, WM_PASTE, 0, 0L);
#endif  
}

// Sub-classed edit control proc
#if 0
MRESULT EXPENTRY 
  wxSubclassedControlProc(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
//  wxItem *item = wxFindControlFromHandle(hWnd);

  if (!item)
  {
    wxDebugMsg("Panic! Cannot find wxItem for this HWND in wxSubclassedControlProc.\n");
    return FALSE;
  }

  switch (msg)
  {
    case WM_QUERYDLGCODE:
    {
#if 0
      if (item->GetWindowStyleFlag() & wxPROCESS_ENTER)
        return DLGC_WANTALLKEYS;
#endif        
      break;
    }
    case WM_CHAR: { // Always an ASCII character
	if ((CHARMSG(&mp1)->fs & KC_VIRTUALKEY) && (CHARMSG(&mp1)->vkey == VK_NEWLINE) {
	    wxCommandEvent event(wxEVENT_TYPE_TEXT_ENTER_COMMAND);
	    event.commandString = ((wxText *)item)->GetValue();
	    event.eventObject = item;
	    item->ProcessCommand(event);
	}
      break;
    }
    default:
      return item->oldWndProc(hWnd, msg, mp1, mp2);

  }
  return item->oldWndProc(hWnd, msg, mp1, mp2);
}

#endif

void wxText::SetEditable(Bool editable)
{
#if XXX
  HWND hWnd = GetHWND();
  SendMessage(hWnd, EM_SETREADONLY, (MPARAM)!editable, (MPARAM)0L);
#endif
}


void wxText::SetInsertionPoint(long pos)
{
#if XXX
    HWND hWnd = GetHWND();
#ifdef WIN32
    SendMessage(hWnd, EM_SETSEL, pos, pos);
    SendMessage(hWnd, EM_SCROLLCARET, (WPARAM)0, (LPARAM)0);
#else
    SendMessage(hWnd, EM_SETSEL, 0, MAKELPARAM(pos, pos));
#endif
    char *nothing = "";
    SendMessage(hWnd, EM_REPLACESEL, 0, (LPARAM)nothing);
#endif
}

void wxText::SetInsertionPointEnd(void)
{
#if XXX
  long pos = GetLastPosition();
  SetInsertionPoint(pos);
#endif
}

long wxText::GetInsertionPoint(void)
{
#if XXX
    DWORD Pos=(DWORD)SendMessage(GetHWND(), EM_GETSEL, 0, 0L);
    return Pos&0xFFFF;
#else
    return 0;
#endif
}

long wxText::GetLastPosition(void)
{
#if XXX
    HWND hWnd = GetHWND();
    
    // Will always return a number > 0 (according to docs)
    int noLines = (int)SendMessage(hWnd, EM_GETLINECOUNT, (WPARAM)0, (LPARAM)0L);

    // This gets the char index for the _beginning_ of the last line
    int charIndex = (int)SendMessage(hWnd, EM_LINEINDEX, (WPARAM)(noLines-1), (LPARAM)0L);
    
    // Get number of characters in the last line. We'll add this to the character
    // index for the last line, 1st position.
    int lineLength = (int)SendMessage(hWnd, EM_LINELENGTH, (WPARAM)charIndex, (LPARAM)0L);

    return (long)(charIndex + lineLength);
#else
    return 0;
#endif
}

void wxText::Replace(long from, long to, char *value)
{
#if XXX
#if USE_CLIPBOARD
    HWND hWnd = GetHWND();
    long fromChar = from;
    long toChar = to;
    
    // Set selection and remove it
#ifdef WIN32
    SendMessage(hWnd, EM_SETSEL, fromChar, toChar);
#else
    SendMessage(hWnd, EM_SETSEL, (WPARAM)0, (LPARAM)MAKELONG(fromChar, toChar));
#endif
    SendMessage(hWnd, WM_CUT, (WPARAM)0, (LPARAM)0);

    if (value)
    {
      // Now replace with 'value', by pasting.
      wxSetClipboardData(wxCF_TEXT, (wxObject *)value, 0, 0);

      // Paste into edit control
      SendMessage(hWnd, WM_PASTE, (WPARAM)0, (LPARAM)0L);
    }
#endif
#endif
}

void wxText::Remove(long from, long to)
{
#if XXX
    HWND hWnd = GetHWND();
    long fromChar = from;
    long toChar = to;
    
    // Cut all selected text
#ifdef WIN32
    SendMessage(hWnd, EM_SETSEL, fromChar, toChar);
#else
    SendMessage(hWnd, EM_SETSEL, (WPARAM)0, (LPARAM)MAKELONG(fromChar, toChar));
#endif
    SendMessage(hWnd, WM_CUT, (WPARAM)0, (LPARAM)0);
#endif
}

void wxText::SetSelection(long from, long to)
{
#if XXX
    HWND hWnd = GetHWND();
    long fromChar = from;
    long toChar = to;
    // if from and to are both -1, it means
    // (in wxWindows) that all text should be selected.
    // This translates into Windows convention
    if ((from == -1) && (to == -1))
    {
      fromChar = 0;
      toChar = -1;
    }
    
#ifdef WIN32
    SendMessage(hWnd, EM_SETSEL, (WPARAM)fromChar, (LPARAM)toChar);
    SendMessage(hWnd, EM_SCROLLCARET, (WPARAM)0, (LPARAM)0);
#else
    // WPARAM is 0: selection is scrolled into view
    SendMessage(hWnd, EM_SETSEL, (WPARAM)0, (LPARAM)MAKELONG(fromChar, toChar));
#endif
#endif
}

