/*
 * File:        wx_utils.cc
 * Purpose:     Various utilities
 * Author:      Julian Smart
 * Created:     1993
 * Updated:     August 1994
 * RCS_ID:      $Id: wx_utils.cc,v 1.2 1994/08/14 23:00:24 edz Exp edz $
 * Copyright:   (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

//#include "wx.h"
#pragma hdrstop

#include <iostream.h>
#include <fstream.h>
#include <ctype.h>

#if 0   // no direct.h for emx/gcc
#include <direct.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>

#define INCL_DOSFILEMGR   /* File Manager values */
#include "common.h"
#include "wx_utils.h"
#include "wx_main.h"
#include "wx_gdi.h"
#include "wx_timer.h"

// In the WIN.INI file
static const char WX_SECTION[] = "wxWindows";
static const char eHOSTNAME[]  = "HostName";
static const char eUSERID[]    = "UserId";
static const char eUSERNAME[]  = "UserName";


// For the following functions we SHOULD fill in support
// for Windows-NT (which I don't know) as I assume it begin
// a POSIX Unix (so claims MS) that it has some special
// functions beyond those provided by WinSock

// Get full hostname (eg. DoDo.BSn-Germany.crg.de)
Bool wxGetHostName(char *buf, int maxSize)
{
#if 0
// This code is Moving to the configuration utility....
# define MAXHOSTNAMELEN 128
  char hostname[MAXHOSTNAMELEN + 1];
  int nRC;

  if ((nRc = gethostname (hostname, MAXHOSTNAMELEN)) == 0)
    {
      struct hostent *hostptr;
      char fullname[MAXHOSTNAMELEN + 1];

      if ((hostptr = gethostbyname (hostname)) != NULL)
        {
          char *tp1, *tp2;
          if ((tp1 = strchr (hostname, '.')) == NULL)
            {
              if ((tp2 = strchr (hostptr->h_name, '.')) != NULL)
                {
                  sprintf (fullname, "%s.%s", hostname, tp2 + 1);
                  strcpy (hostname, fullname);
                }
              else
                hostname[0] = '\0';     /* better none than not complete */
            }
          /* else we have complete address */
        }
        strncpy(buf, hostname, maxSize - 1);
        buf[maxSize] = '\0';
        return TRUE;
    }
  return FALSE;
#else
  char *sysname;
//  const char *default_host = "noname";

  if ((sysname = getenv("SYSTEM_NAME")) == NULL) {
#if 0  
    GetProfileString(WX_SECTION, eHOSTNAME, default_host, buf, maxSize - 1);
#endif     
    strncpy(buf, "os/2", maxSize - 1);
  } else
    strncpy(buf, sysname, maxSize - 1);
  buf[maxSize] = '\0';
#endif
  return *buf ? TRUE : FALSE;
}

// Get user ID e.g. jacs
Bool wxGetUserId(char *buf, int maxSize)
{
  char *user;
//  const char *default_id = "anonymous";

  // Can't assume we have NIS (PC-NFS) or some other ID daemon
  // So we ...
  if (  (user = getenv("USER")) == NULL &&
        (user = getenv("LOGNAME")) == NULL ) {
     // Use wxWindows configuration data (comming soon)
#if 0     
    GetProfileString(WX_SECTION, eUSERID, default_id, buf, maxSize - 1);
#endif     
    strncpy(buf, "user", maxSize - 1);
  } else
    strncpy(buf, user, maxSize - 1);
  return *buf ? TRUE : FALSE;
}

// Get user name e.g. Julian Smart
Bool wxGetUserName(char *buf, int maxSize)
{
//  const char *default_name = "Unknown User";

  extern LHANDLE hPenOS2; // PenOS2 Running?
  if (hPenOS2) {
    // PenWindows Does have a user concept!
    // Get the current owner of the recognizer
#if 0    
    GetPrivateProfileString("Current", "User", default_name, wxBuffer, maxSize - 1, "PENWIN.INI");
    strncpy(buf, wxBuffer, maxSize - 1);
#endif    
    strncpy(buf, "unknown", maxSize - 1);
  } else {
    // Could use NIS, MS-Mail or other site specific programs
    // Use wxWindows configuration data 
#if 0    
    GetProfileString(WX_SECTION, eUSERNAME, default_name, buf, maxSize - 1);
#endif    
    strncpy(buf, "unknown", maxSize - 1);
  }
  return *buf ? TRUE : FALSE;
}

// Execute a command (e.g. another program) in a
// system-independent manner.

Bool wxExecute(char **argv, Bool Async)
{
  if (*argv == NULL)
    return FALSE;

  char command[1024];
  command[0] = '\0';

  for (int argc = 0; argv[argc]; argc++)
   {
    if (argc)
      strcat(command, " ");
    strcat(command, argv[argc]);
   }

  return wxExecute(command, Async);
}

Bool wxExecute(const char *command, Bool Async)
{
  if (command == NULL || *command == '\0')
    return FALSE;

#if XXX
  long Instance_ID = WinExec((LPCSTR)command, SW_SHOW);
  if (Instance_ID < 32) return(FALSE);
// WIN32 doesn't have GetModuleUsage!!
#ifndef WIN32
  if (Async) {
    int running;
    do {
      wxYield();
      running = GetModuleUsage((HANDLE)Instance_ID);
    } while (running);
  }
#endif
#endif
  return(TRUE);
}

//
// Execute a program in an Interactive Shell
//
Bool
wxShell(const char *command)
{
  char *shell;
  if ((shell = getenv("COMSPEC")) == NULL)
    shell = "CMD.EXE";

  char tmp[255];
  if (command && *command)
    sprintf(tmp, "%s /c %s", shell, command);
  else
    strcpy(tmp, shell);

  return wxExecute((char *)tmp, FALSE);
}



Bool wxRemoveFile(const char *file)
{
// Zortech -- what's the proper define?
#ifdef ZTC
  int flag = unlink(file);
#else
  int flag = remove(file);
#endif
  if (flag == 0) return TRUE;
  return FALSE;
}

Bool wxMkdir(const char *dir)
{
  return (mkdir(dir, 0) == 0);
}

Bool wxRmdir(const char *dir)
{
  return (rmdir(dir) == 0);
}

Bool wxDirExists(const char *dir)
{
#if XXX
  /* MATTHEW: [6] Always use same code for Win32, call FindClose */
#if defined(WIN32)
  WIN32_FIND_DATA fileInfo;
#else
#ifdef __BORLANDC__
  struct ffblk fileInfo;
#else
  struct find_t fileInfo;
#endif
#endif

#if defined(WIN32)
	HANDLE h = FindFirstFile((LPTSTR)dir,(LPWIN32_FIND_DATA)&fileInfo);

	if (h==INVALID_HANDLE_VALUE)
	 return FALSE;
	else {
	 FindClose(h);
	 return (fileInfo.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY);
	}
#else
  // In Borland findfirst has a different argument
  // ordering from _dos_findfirst. But _dos_findfirst
  // _should_ be ok in both MS and Borland... why not?
#ifdef __BORLANDC__
  return (findfirst(dir, &fileInfo, _A_SUBDIR) == 0  && (fileInfo.ff_attrib & _A_SUBDIR));
#else
  return ((_dos_findfirst(dir, _A_SUBDIR, &fileInfo) == 0) && (fileInfo.attrib & _A_SUBDIR));
#endif
#endif
#endif
  return FALSE;
}

// Get a temporary filename, opening and closing the file.
char *wxGetTempFileName(const char *prefix, char *buf)
{
#if XXX
  char tmp[64];
  ::GetTempFileName(0, prefix, 0, tmp);
  if (buf) strcpy(buf, tmp);
  else buf = copystring(tmp);
  return buf;
#endif
  return NULL;  
}

#if defined(WIN32)
// Sorry, not yet implemented for WIN32
#endif
#if defined(wx_os2)  // not yet implemented for OS/2
char *wxFindFirstFile(const char *spec, int flags)
{
// XXX
  return NULL;
}

char *wxFindNextFile(void)
{
  return NULL;
}
#else
static char *wxFileSpec = NULL;
static int wxFindFileFlags;

char *wxFindFirstFile(const char *spec, int flags)
{
  if (wxFileSpec)
	 delete[] wxFileSpec;
  wxFileSpec = copystring(spec);
  wxFindFileFlags = flags; /* MATTHEW: [5] Remember flags */

  // Find path only so we can concatenate
  // found file onto path
  char *p = wxPathOnly(wxFileSpec);
  if (p && (strlen(p) > 0))
	 strcpy(wxBuffer, p);
  else
	 wxBuffer[0] = 0;

#ifdef WIN32
  if (wxFileStrucHandle != INVALID_HANDLE_VALUE)
	 FindClose(wxFileStrucHandle);

  wxFileStrucHandle = FindFirstFile(spec, &wxFileStruc);

  if (wxFileStrucHandle == INVALID_HANDLE_VALUE)
	 return NULL;

  Bool isdir = !!(wxFileStruc.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY);

  if (isdir && !(flags & wxDIR))
	 return wxFindNextFile();
  else if (!isdir && flags && !(flags & wxFILE))
	 return wxFindNextFile();

  if (wxBuffer[0] != 0)
	 strcat(wxBuffer, "\\");
  strcat(wxBuffer, wxFileStruc.cFileName);
  return wxBuffer;
#else

  int flag = _A_NORMAL;
  if (flags & wxDIR) /* MATTHEW: [5] Use & */
    flag = _A_SUBDIR;

#ifdef __BORLANDC__
  if (findfirst(spec, &wxFileStruc, flag) == 0)
#else
  if (_dos_findfirst(spec, flag, &wxFileStruc) == 0)
#endif
  {
    /* MATTHEW: [5] Check directory flag */
    char attrib;

#ifdef __BORLANDC__
    attrib = wxFileStruc.ff_attrib;
#else
    attrib = wxFileStruc.attrib;
#endif

    if (attrib & _A_SUBDIR) {
      if (!(wxFindFileFlags & wxDIR))
	return wxFindNextFile();
    } else if (wxFindFileFlags && !(wxFindFileFlags & wxFILE))
		return wxFindNextFile();

	 if (wxBuffer[0] != 0)
		strcat(wxBuffer, "\\");

#ifdef __BORLANDC__
	 strcat(wxBuffer, wxFileStruc.ff_name);
#else
	 strcat(wxBuffer, wxFileStruc.name);
#endif
	 return wxBuffer;
  }
  else
    return NULL;
#endif // WIN32
}

char *wxFindNextFile(void)
{
  // Find path only so we can concatenate
  // found file onto path
  char *p = wxPathOnly(wxFileSpec);
  if (p && (strlen(p) > 0))
	 strcpy(wxBuffer, p);
  else
	 wxBuffer[0] = 0;
  
 try_again:

#ifdef WIN32
  if (wxFileStrucHandle == INVALID_HANDLE_VALUE)
	 return NULL;

  Bool success = FindNextFile(wxFileStrucHandle, &wxFileStruc);
  if (!success) {
		FindClose(wxFileStrucHandle);
      wxFileStrucHandle = INVALID_HANDLE_VALUE;
		return NULL;
  }

  Bool isdir = !!(wxFileStruc.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY);
  
  if (isdir && !(wxFindFileFlags & wxDIR))
    goto try_again;
  else if (!isdir && wxFindFileFlags && !(wxFindFileFlags & wxFILE))
	 goto try_again;

  if (wxBuffer[0] != 0)
    strcat(wxBuffer, "\\");
  strcat(wxBuffer, wxFileStruc.cFileName);
  return wxBuffer;  
#else

#ifdef __BORLANDC__
  if (findnext(&wxFileStruc) == 0)
#else
  if (_dos_findnext(&wxFileStruc) == 0)
#endif
  {
    /* MATTHEW: [5] Check directory flag */
    char attrib;

#ifdef __BORLANDC__
    attrib = wxFileStruc.ff_attrib;
#else
    attrib = wxFileStruc.attrib;
#endif

    if (attrib & _A_SUBDIR) {
      if (!(wxFindFileFlags & wxDIR))
	goto try_again;
    } else if (wxFindFileFlags && !(wxFindFileFlags & wxFILE))
      goto try_again;


	 if (wxBuffer[0] != 0)
      strcat(wxBuffer, "\\");
#ifdef __BORLANDC__
	 strcat(wxBuffer, wxFileStruc.ff_name);
#else
	 strcat(wxBuffer, wxFileStruc.name);
#endif
	 return wxBuffer;
  }
  else
    return NULL;
#endif
}
#endif

// Get current working directory.
// If buf is NULL, allocates space using new, else
// copies into buf.
char *wxGetWorkingDirectory(char *buf, int sz)
{
#if XXX
  if (!buf)
    buf = new char[1000];
#ifdef __BORLANDC__
  (void)getcwd(buf, sz);
#elif __WATCOMC__
  (void)getcwd(buf, sz);
#else
  (void)_getcwd(buf, sz);
#endif
  return buf;
#endif
  return NULL;  
}

Bool wxSetWorkingDirectory(char *d)
{
  Bool success = (_chdir2(d) == 0);

  return success;
}

// Get free memory in bytes, or -1 if cannot determine amount (e.g. on UNIX)
long wxGetFreeMemory(void)
{
#if 0
  return (long)GetFreeSpace(0);
#else
  return -1;
#endif    
}

// Sleep for nSecs seconds. Attempt a Windows implementation using timers.
static Bool inTimer = FALSE;
class wxSleepTimer: public wxTimer
{
 public:
  inline void Notify(void)
  {
    inTimer = FALSE;
    Stop();
  }
};
static wxTimer *wxTheSleepTimer = NULL;

void wxSleep(int nSecs)
{
  if (inTimer)
    return;

  wxTheSleepTimer = new wxSleepTimer;
  inTimer = TRUE;
  wxTheSleepTimer->Start(nSecs*1000);
  while (inTimer)
  {
    if (wxTheApp->Pending())
      wxTheApp->Dispatch();
}
  delete wxTheSleepTimer;
  wxTheSleepTimer = NULL;
}

// Consume all events until no more left
void wxFlushEvents(void)
{
}

// Output a debug mess., in a system dependent fashion.
void wxDebugMsg(const char *fmt ...)
{
  va_list ap;

  if (!wxTheApp->wantDebugOutput)
    return ;

#ifdef DEBUGPM
  char scratch[256];
  va_start(ap, fmt);
  vsprintf(scratch, fmt, ap);
  va_end(ap);
  printf(scratch);
#else
  FILE *fp;
  fp = fopen("std.err", "a");
  if (!fp) return;

  va_start(ap, fmt);
  vfprintf(fp,fmt,ap) ;
  va_end(ap);
  
  fclose(fp);
#endif
}

// Non-fatal error: pop up message box and (possibly) continue
void wxError(const char *msg, const char *title)
{
#if XXX
  if (WinMessageBox(HWND_DESKTOP,	// Parent
		    HWND_DESKTOP,	// Owner
		    msg,		// Text
		    title,		// Caption
		    ID_,		// Window ID
		    MB_OKCANCEL | MB_ICONEXCLAMATION) == MBID_CANCEL)
	  wxExit();
#endif
#if 0
  sprintf(wxBuffer, "%s\nContinue?", msg);
  if (MessageBox(NULL, (LPCSTR)wxBuffer, (LPCSTR)title,
             MB_ICONSTOP | MB_YESNO) == IDNO)
    wxExit();
#endif    
}

// Fatal error: pop up message box and abort
void wxFatalError(const char *msg, const char *title)
{
  sprintf(wxBuffer, "%s: %s", title, msg);
  WinMessageBox(HWND_DESKTOP,		// Parent
		HWND_DESKTOP,		// Owner
		(PSZ)msg,		// Text
		(PSZ)title,		// Caption
		4711,			// Window ID
		MB_CANCEL | MB_ICONEXCLAMATION);
  wxExit();
#if 0  
  FatalAppExit(0, (PSZ)wxBuffer);
#endif  
}

// Emit a beeeeeep
void wxBell(void)
{
// 1kHz during 1 sec.
	WinAlarm(HWND_DESKTOP, WA_NOTE);
}

int wxGetOsVersion(int *majorVsn, int *minorVsn)
{
  int retValue ;
  
  retValue = wxOS2_PM;

  ULONG   DataBuf[2];

  DosQuerySysInfo(QSV_VERSION_MAJOR, QSV_VERSION_MINOR, DataBuf, sizeof DataBuf);

  if (majorVsn) *majorVsn = DataBuf[0];
  if (minorVsn) *minorVsn = DataBuf[1];
  return retValue ;
}

// Reading and writing resources (eg WIN.INI, .Xdefaults)
#if USE_RESOURCES
Bool wxWriteResource(const char *section, const char *entry, char *value, const char *file)
{
  HINI hIni;

  if (file) {
    hIni = PrfOpenProfile(wxhAb, (PSZ)file);
    PrfWriteProfileData(hIni, (PSZ)section, (PSZ)entry, value, strlen(value)+1);
    PrfCloseProfile(hIni);
  }
  else {
    PrfWriteProfileData(HINI_USERPROFILE, (PSZ)section, (PSZ)entry, value, strlen(value)+1);
  }
  return TRUE;
}

Bool wxWriteResource(const char *section, const char *entry, float value, const char *file)
{
  char buf[50];
  sprintf(buf, "%.4f", value);
  return wxWriteResource(section, entry, buf, file);
}

Bool wxWriteResource(const char *section, const char *entry, long value, const char *file)
{
  char buf[50];
  sprintf(buf, "%ld", value);
  return wxWriteResource(section, entry, buf, file);
}

Bool wxWriteResource(const char *section, const char *entry, int value, const char *file)
{
  char buf[50];
  sprintf(buf, "%d", value);
  return wxWriteResource(section, entry, buf, file);
}

Bool wxGetResource(const char *section, const char *entry, char **value, const char *file)
{
TRACE("wxGetResource(section='%s', entry='%s', value=..., file='%s')", section, entry, file);
    HINI  hIni;
    ULONG len;
    Bool  res;

    res = FALSE;
    if (file) {
	hIni = PrfOpenProfile(wxhAb, (PSZ)file);
	if (PrfQueryProfileData(hIni, (PSZ)section, (PSZ)entry, wxBuffer, &len)) {
	    *value = copystring(wxBuffer);
	    res = TRUE;
	}
	PrfCloseProfile(hIni);
    } else {
	if (PrfQueryProfileData(HINI_USERPROFILE, (PSZ)section, (PSZ)entry, wxBuffer, &len)) {
	    *value = copystring(wxBuffer);
	    res = TRUE;
	}
    }
TRACE("wxGetResource(): res=%d, value='%s'", res, *value);
    return res;
}

Bool wxGetResource(const char *section, const char *entry, float *value, const char *file)
{
    char *s = NULL;
    Bool succ = wxGetResource(section, entry, &s, file);
    if (succ) {
	*value = (float)strtod(s, NULL);
	delete[] s;
	return TRUE;
    }
    return FALSE;
}

Bool wxGetResource(const char *section, const char *entry, long *value, const char *file)
{
    char *s = NULL;
    Bool succ = wxGetResource(section, entry, &s, file);
    if (succ) {
	*value = strtol(s, NULL, 10);
	delete[] s;
	return TRUE;
    }
    return FALSE;
}

Bool wxGetResource(const char *section, const char *entry, int *value, const char *file)
{
    char *s = NULL;
    Bool succ = wxGetResource(section, entry, &s, file);
    if (succ) {
	*value = (int)strtol(s, NULL, 10);
	delete[] s; 
	return TRUE;
    }
    return FALSE;
}
#endif // USE_RESOURCES

// Old cursor
static HPOINTER wxBusyCursorOld = NULLHANDLE;
static int wxBusyCursorCount = 0;
// Busy cursor
static HPOINTER wxBusyCursor = NULLHANDLE;

// Set the cursor to the busy cursor for all windows
void wxBeginBusyCursor(wxCursor *cursor)
{
    TRACE("wxBeginBusyCursor(cursor=%#x", (int)cursor);
    wxBusyCursorCount ++;
    if (wxBusyCursorCount == 1) {
	wxBusyCursorOld = WinQueryPointer(HWND_DESKTOP);
	wxBusyCursor = cursor->os2_cursor;
	TRACE("wxBeginBusyCursor...WinQueryPointer returns %#x", (int)wxBusyCursorOld);
	WinSetPointer(HWND_DESKTOP, wxBusyCursor);
    }
}

// Restore cursor to normal
void wxEndBusyCursor(void)
{
    TRACE("wxEndBusyCursor... wxBusyCursorOld=%#x", (int)wxBusyCursorOld);
    if (wxBusyCursorCount == 0)
	return;
    
    wxBusyCursorCount --;
    if (wxBusyCursorCount == 0) {
	WinSetPointer(HWND_DESKTOP, wxBusyCursorOld);
	wxBusyCursorOld = 0;
	wxBusyCursor = NULLHANDLE;
    }
}

// Treturn busy cursor handle if we're between the above two calls
HPOINTER wxIsBusy(void)
{
    return wxBusyCursor;
}    

// Hack for MS-DOS
char *wxGetUserHome (const char *user)
{
#if XXX
  char *home;
  if (user && *user) {
    char tmp[64];
    if (wxGetUserId(tmp, sizeof(tmp)/sizeof(char))) {
      // Guests belong in the temp dir
      if (stricmp(tmp, "annonymous") == 0) {
        if ((home = getenv("TMP")) != NULL ||
            (home = getenv("TMPDIR")) != NULL ||
            (home = getenv("TEMP")) != NULL)
          return *home ? home : "\\";
      }
      if (stricmp(tmp, user) == 0)
        user = NULL;
    }
  }
  if (user == NULL || *user == '\0')
    if ((home = getenv("HOME")) != NULL)
    {
      strcpy(wxBuffer, home);
      Unix2DosFilename(wxBuffer);
      return wxBuffer;
    }
  return NULL; // No home known!
#endif    
  return NULL; // No home known!
}

// Check whether this window wants to process messages, e.g. Stop button
// in long calculations.
Bool wxCheckForInterrupt(wxWindow *wnd)
{
#if XXX
	if(wnd){
		MSG msg;
		HWND win=wnd->GetHWND();
		while(PeekMessage(&msg,win,0,0,PM_REMOVE)){
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		return TRUE;//*** temporary?
	}
	else{
		wxError("wnd==NULL !!!");
		return FALSE;//*** temporary?
	}
#endif
	return FALSE;
}

