/*
 * File:        wx_win.cc
 * Purpose:     wxWindow class implementation
 * Author:      Julian Smart
 * Created:     1993
 * Updated:     August 1994
 * RCS_ID:      $Id: wx_win.cc,v 1.1 1994/08/14 21:59:17 edz Exp $
 * Copyright:   (c) 1993, AIAI, University of Edinburgh
 */

/* static const char sccsid[] = "%W% %G%"; */

#include <iostream.h>

// This file contais 'portable' message crackers. It is standard in NT
// If it also comes whith W3.1, we can consider to always use these
// macros.
// [currently used for MDIACTIVATE in Win32]

#include "wx_panel.h"
#include "wx_rbox.h"
#include "wx_txt.h"
#include "wx_text.h"
#include "wx_menu.h"
#include <iostream.h>
#include "wx_dcpan.h"
#include "wx_main.h"
#include "wx_itemp.h"

#if USE_SCROLLBAR
extern void wxScrollBarEvent(HWND hbar, int mp1, int pos);
#endif

#define WINDOW_MARGIN 3 // This defines sensitivity of Leave events

// Global variables
Bool wxShiftDown = FALSE;
Bool wxControlDown = FALSE;
Bool wxAltDown = FALSE;

wxMenu *wxCurrentPopupMenu = NULL;

extern HWND wxDesktopHandle;
extern char wxDialogClassName[];

extern "C" { int printf (__const__ char *format, ...); }

// Find an item given the MS Windows id
wxWindow *wxWindow::FindItem(int id)
{
    if (!children)
	return NULL;
    wxNode *current = children->First();
    while (current) {
	wxObject *obj = (wxObject *)current->Data() ;
	if (wxSubType(obj->__type,wxTYPE_PANEL)) {
	    // Do a recursive search.
	    wxPanel *panel = (wxPanel*)obj ;
	    wxWindow *wnd = panel->FindItem(id) ;
	    if (wnd)
		return wnd ;
	} else {
	    wxItem *item = (wxItem *)current->Data();
	    if (item->windows_id == id)
		return item;
	    else {
		// In case it's a 'virtual' control (e.g. radiobox)
		if (item->subControls.Member((wxObject *)id))
		    return item;
	    }
	}
	current = current->Next();
    }
    return NULL;
}

// Find an item given the OS/2 Windows handle
wxWindow *wxWindow::FindItemByHWND(HWND hWnd)
{
    if (!children)
	return NULL;
    wxNode *current = children->First();
    while (current) {
	wxObject *obj = (wxObject *)current->Data() ;
	if (wxSubType(obj->__type,wxTYPE_PANEL)) {
	    // Do a recursive search.
	    wxPanel *panel = (wxPanel*)obj ;
	    wxWindow *wnd = panel->FindItemByHWND(hWnd) ;
	    if (wnd)
		return wnd ;
	} else {
	    wxItem *item = (wxItem *)current->Data();
	    if (item->GetHWND() == hWnd)
		return item;
	    else {
		// In case it's a 'virtual' control (e.g. radiobox)
		if (item->__type == wxTYPE_RADIO_BOX) {
		    wxRadioBox *rbox = (wxRadioBox *)item;
		    for (int i = 0; i < rbox->no_items; i++)
			if (rbox->hwndButtons[i] == hWnd)
			    return item;
		}
	    }
	}
	current = current->Next();
    }
    return NULL;
}

// Find an item given the static label
wxWindow *wxWindow::FindItemByStatic(HWND hWnd)
{
    if (!children)
	return NULL;
    wxNode *current = children->First();
    while (current) {
	wxObject *obj = (wxObject *)current->Data() ;
	if (wxSubType(obj->__type,wxTYPE_PANEL)) {
	    // Do a recursive search.
	    wxPanel *panel = (wxPanel*)obj ;
	    wxWindow *wnd = panel->FindItemByStatic(hWnd) ;
	    if (wnd)
		return wnd ;
	} else {
	    wxItem *item = (wxItem *)current->Data();
	    if ((HWND)(item->GetLabelHWND()) == hWnd)
		return item;
#if 0
	    else {
		if (item->__type == wxTYPE_SLIDER) {
		    wxSlider *slider = (wxSlider *)item;
		    if (hWnd == slider->static_min || hWnd == slider->static_max || hWnd == slider->edit_value)
			return slider;
		}
	    }
#endif
	}
	current = current->Next();
    }
    return NULL;
}


// Default command handler
BOOL wxWindow::OS2Control(WORD WXUNUSED(param), WORD WXUNUSED(id))
{
    return FALSE;
}

HWND wxWindow::GetHWND(void)
{
    wxWnd *wnd = (wxWnd *)handle;
    if (wnd) {
//	HWND hWnd = wnd->hwndWnd;
	HWND hWnd = wnd->GetClientHWND();
	assert(hWnd != NULLHANDLE);
	return hWnd;
    }
    return NULLHANDLE;
}

// Constructor
IMPLEMENT_ABSTRACT_CLASS(wxWindow, wxEvtHandler)

wxWindow::wxWindow(void)
{
    TRACE("wxWindow::wxWindow");
    mouseInWindow = FALSE ;
    winEnabled = TRUE;
    windows_id = 0;
    mouseInWindow = FALSE;
    winEnabled = TRUE;
    caretWidth = 0; caretHeight = 0;
    caretEnabled = FALSE;
    caretShown = FALSE;
    inOnSize = FALSE;
    wxy = wxx = wxh = wxw = 0;
}

// Destructor
wxWindow::~wxWindow(void)
{
    TRACE("wxWindow::~wxWindow() this=%#x", this);
    if (window_parent)
	window_parent->RemoveChild(this);

    TRACE("wxWindow: GetHWND=>%#x", (int)GetHWND());
    wxbWindow::DestroyChildren();

    if (handle) {
	wxWnd *wnd = (wxWnd *)handle;
	handle = NULL;			// Try to avoid the loop
#if 1
	delete wnd;
#else
	switch (wnd->__type) {
	case wxTYPE_FRAME_WND: {
	    wxFrameWnd *wnd = (wxFrameWnd *)handle;
	    delete wnd;
	    break;
	}

	case wxTYPE_MDICHILD_WND: {
	    wxMDICHildWnd *wnd = (wxMDIChildWnd *)handle;
	    delete wnd;
	    break;
	}

	case wxTYPE_MDIFRAME_WND: {
	    wxMDIFrameWnd *wnd = (wxMDIFrameWnd *)handle;
	    delete handle;
	    break;
	}

	case wxTYPE_CANVAS_WND: {
	    wxCanvasWnd *wnd = (wxCanvasWnd *)handle;
	    delete handle;
	    break;
	}

	case wxTYPE_PANEL_WND: {
	    wxPanelWnd *wnd = (wxPanelWnd *)handle;
	    delete wxPanelWnd;
	    break;
	}

	case wxTYPE_DIALOG_WND: {
	    wxDialogWnd *wnd = (wxDialogWnd *)handle;
	    delete wnd;
	    break;
	}
	}
#endif
    }
    handle = NULL;

#if 0
case wxTYPE_XWND: {
    if (handle) {
	wxWnd *wnd = (wxWnd *)handle;

	wnd->DestroyWindow();
	delete wnd;
	handle = NULL;
    }
    break;
}
case wxTYPE_MDICHILD_WND: {
    wxMDIChildWnd *child = (wxMDIChildWnd *)handle;
    child->DestroyWindow();
    delete child;
    handle = NULL;
    break;
}
case wxTYPE_HWND: {
#if 1
    assert(GetHWND() == NULLHANDLE);
#else
    if (GetHWND()) {
	WinDestroyWindow(GetHWND());
	TRACE("wxWindow::~wxWindow: Destroying window %#x", (int)GetHWND());
    }
#endif
    handle = NULL;
    // Free global memory of editable text window
    if (wxSubType(__type, wxTYPE_TEXT_WINDOW))
    { 
//        wxTextWindow *textWin = (wxTextWindow *)this;
    }
    break;
}
default:
    break;
}
#endif

    delete children;
    children = NULL;
}

void wxWindow::SetFocus(void)
{
  HWND hWnd = GetHWND();
  if (hWnd)
	  WinSetFocus(HWND_DESKTOP, hWnd);
}

void wxWindow::Enable(Bool enable)
{
    TRACE("wxWindow::Enable(%d)", (int)enable);
    winEnabled = enable;
    HWND hWnd = GetHWND();
    if (hWnd)
	WinEnableWindow(hWnd, enable);
}

void wxWindow::CaptureMouse(void)
{
    HWND hWnd = GetHWND();
    if (hWnd && !winCaptured) {
	WinSetCapture(HWND_DESKTOP, hWnd);
	winCaptured = TRUE;
    }
}

void wxWindow::ReleaseMouse(void)
{
    if (winCaptured) {
	WinSetCapture(HWND_DESKTOP, NULLHANDLE);
	winCaptured = FALSE;
    }
}

void wxWindow::DragAcceptFiles(Bool accept)
{
#if XXX
  HWND hWnd = GetHWND();
  if (hWnd)
    ::DragAcceptFiles(hWnd, accept);
#endif    
}

// Get total size of entire window in pixels
void wxWindow::GetSize(int *x, int *y)
{
  wx_GetWindowSize(GetHWND(), x, y);
TRACE("wxWindow::GetSize() returns width=%d, height=%d", *x, *y);
}

void wxWindow::GetPosition(int *x, int *y)
{
    HWND hWnd = GetHWND();
    HWND hwndParent = NULLHANDLE;
    if (GetParent())
	hwndParent = GetParent()->GetHWND();
  
    wxRECTL rect;
    wx_WindowRect(hWnd, &rect, hwndParent);

#if 0
    // Since we now have the absolute screen coords,
    // if there's a parent we must subtract its top left corner
    wxPOINTL point = { rect.xLeft, rect.yTop };
    if (hwndParent)
	wx_ScreenToClient(hwndParent, &point);
    *x = point.x;
    *y = point.y;
#else
    *x = rect.xLeft;
    *y = rect.yTop;
#endif
    TRACE("wxWindow::GetPosition returns x=%d, y=%d", *x, *y);
}

void wxWindow::ScreenToClient(int *x, int *y)
{
	wxPOINTL pt = {*x, *y};
	wx_ScreenToClient(GetHWND(), &pt);
	*x = pt.x;
	*y = pt.y;
}

void wxWindow::ClientToScreen(int *x, int *y)
{
	wxPOINTL pt = {*x, *y};
	wx_ClientToScreen(GetHWND(), &pt);
	*x = pt.x;
	*y = pt.y;
}

wxCursor *wxWindow::SetCursor(wxCursor *cursor)
{
TRACE("wxWindow::SetCursor(cursor=%#x)", (int)cursor);
    wxCursor *old_cursor = wx_cursor;
    wx_cursor = cursor;
    if (wx_cursor) {
	wxWnd *wnd = (wxWnd *)handle;
	if (wnd) {
	    wnd->os2_cursor = wx_cursor->os2_cursor;
	    HWND hWnd = GetHWND();

	    // Change the cursor NOW if we're within the correct window
	    POINTL point;
	    WinQueryPointerPos(HWND_DESKTOP, &point);

	    RECTL rect;
	    wx_WindowRect(hWnd, &rect);

	    if (WinPtInRect(wxhAb, &rect, &point))
		WinSetPointer(HWND_DESKTOP, wnd->os2_cursor);
	}
    }
    wxFlushEvents();
    return old_cursor;
}

void wxWindow::SetColourMap(wxColourMap *WXUNUSED(cmap))
{
}

// Get size *available for subwindows* i.e. excluding menu bar etc.
// For XView, this is the same as GetSize
void wxWindow::GetClientSize(int *cx, int *cy)
{
#if 1
    HWND hWndClient = WinWindowFromID(GetHWND(), FID_CLIENT);
    assert(hWndClient == NULLHANDLE);
    RECTL rect;
    WinQueryWindowRect(GetHWND(), &rect);
    *cx = rcWidth(rect);
    *cy = rcHeight(rect);
#else
    wxRECTL rect;
    wx_ClientRect(GetHWND(), &rect);
    *cx = rcWidth(rect);
    *cy = rcHeight(rect);
#endif
    TRACE("wxWindow::GetClientSize() returns width=%d, height=%d", *cx, *cy);
}

void wxWindow::SetSize(int x, int y, int width, int height, int sizeFlags)
{
    TRACE("wxWindow::SetSize(x=%d, y=%d, w=%d, h=%d, sizeFlags=%#x)", x, y, width, height, sizeFlags);
    int oldX, oldY;
    GetPosition(&oldX, &oldY);
    int newWidth = width;
    int newHeight = height;
    if (x == -1)
	x = oldX;
    if (y == -1)
	y = oldY;

    int oldWidth,oldHeight;
    GetSize(&oldWidth, &oldHeight);
    if (width == -1)
	newWidth = oldWidth ;
    if (height == -1)
	newHeight = oldHeight ;

    HWND hWnd = GetHWND();

    wxWnd *wnd = (wxWnd *) handle;
    if (wnd && hWnd) {
	if (wnd->__type == wxTYPE_DIALOG_WND) {
	    RECTL rcl = {0, 0, newWidth, newHeight};

	    HWND hwnd1 = wnd->GetFrameHWND();
	    HWND hwnd2 = WinQueryWindow(hWnd, QW_PARENT);
	    assert(hwnd1 == hwnd2);

	    HWND hwndFrame = WinQueryWindow(hWnd, QW_PARENT);
	    assert(hwndFrame == wnd->GetFrameHWND());
	    TRACE("hwndFrame==%#x", hwndFrame);
	    WinCalcFrameRect(hwndFrame, &rcl, FALSE);
	    wx_MoveWindow(hwndFrame, x, y, rcWidth(rcl), rcHeight(rcl));
	    // Everything else will be changed by the WM_SIZE message following
	} else {
	    MoveWindow(x, y, newWidth, newHeight);

	    // OS/2 windows are upside down so we have to reposition all the children windows
	    int dheight = newHeight-oldHeight;
	    if (dheight) {
		HENUM henum = WinBeginEnumWindows(wnd->hwndWnd);
		while (HWND hwnd = WinGetNextWindow(henum)) {
		    SWP swp;
		    WinQueryWindowPos(hwnd, &swp);
		    WinSetWindowPos(hwnd, 0, swp.x, swp.y + dheight, 0, 0, SWP_MOVE);
		}
	    }
	    
	    if (!(width == -1) && (height == -1))
		GetEventHandler()->OnSize(width, height);
	}
    }
}

void wxWindow::SetClientSize(int width, int height)
{
    TRACE("wxWindow::SetClientsize(width=%d, height=%d)", width, height);
    wxFrameWnd *wnd = (wxFrameWnd *)handle;
    assert(wnd->__type == wxTYPE_FRAME_WND);
    HWND hwndClient = wnd->GetClientHWND();
    RECTL rcl = {0, 0, width, height};
    WinCalcFrameRect(wnd->hwndWnd, &rcl, FALSE);
    WinSetWindowPos(wnd->hwndWnd, HWND_TOP, 0, 0, rcWidth(rcl), rcHeight(rcl), SWP_SIZE);
    WinQueryWindowRect(wnd->hwndWnd, &rcl);
    WinQueryWindowRect(hwndClient, &rcl);

    // This appears to be necessary
    WinInvalidateRect(hwndClient, NULL, TRUE) ;

#if 1
    GetEventHandler()->OnSize(rcWidth(rcl), rcHeight(rcl));
#else
    GetEventHandler()->OnSize(width, height);
#endif
}

Bool wxWindow::Show(Bool show)
{
    HWND hWnd = GetHWND();
    if (show)
	WinSetWindowPos(hWnd, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE | SWP_ZORDER);
    else
	WinShowWindow(hWnd, FALSE);
    return TRUE;
}

float wxWindow::GetCharHeight(void)
{
    return (float)wxGetCharHeight(GetHWND());
}

float wxWindow::GetCharWidth(void)
{
    return (float)wxGetCharWidth(GetHWND());
}

void wxWindow::GetTextExtent(const char *string, float *x, float *y,
                           float *descent, float *externalLeading, wxFont *theFont, Bool use16bit)
{
    wxFont *fontToUse = theFont;
    if (!fontToUse)
	fontToUse = font;
    
    HWND hWnd = GetHWND();
    HPS ps = WinGetPS(hWnd);

    POINTL sizePt[TXTBOX_COUNT];
    FONTMETRICS fm;
    GpiQueryTextBox(ps, strlen(string), (PCH)string, TXTBOX_COUNT, sizePt);
    GpiQueryFontMetrics(ps, sizeof(FONTMETRICS),&fm);

    WinReleasePS(ps);

    *x = (float)sizePt[TXTBOX_TOPRIGHT].x;
    *y = (float)(sizePt[TXTBOX_TOPRIGHT].y - sizePt[TXTBOX_BOTTOMRIGHT].y);
    if (descent) *descent = (float)fm.lMaxDescender;
    if (externalLeading) *externalLeading = (float)fm.lExternalLeading;
}

void wxWindow::Refresh(void)
{
#ifdef XXX
  HWND hWnd = GetHWND();
  if (hWnd)
  {
    ::InvalidateRect(hWnd, NULL, TRUE);
  }
#endif
}

// Hook for new window just as it's being created,
// when the window isn't yet associated with the handle
wxWnd *wxWndHook = NULL;

// Main OS/2 window proc
MRESULT EXPENTRY wxWndProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxWnd *wnd;
    int width, height;

    if (msg == WM_CREATE) {
	ULONG *ctlptr = (ULONG *)PVOIDFROMMP(mp1);
	wnd = (wxWnd *)ctlptr[1];
	TRACE("wxWndProc: WM_CREATE, hwnd=%#x, wnd=%#x", (int)hwnd, (int)wnd);
	WinSetWindowPtr(hwnd, QWL_USER, wnd);
	assert(wnd->hwndWnd == NULLHANDLE);
	wnd->hwndWnd = hwnd;
	if (mp1 && mp2) {
	    PCREATESTRUCT p = (PCREATESTRUCT)PVOIDFROMMP(mp2);
	    p->pCtlData = (PVOID)ctlptr[2];
	    mp1 = (PVOID)ctlptr[2];
	}
    } else
	wnd = (wxWnd *)WinQueryWindowPtr(hwnd, QWL_USER);

#ifdef DEBUGPM
    wxTraceEvent("wxWndProc", hwnd, msg, mp1, mp2, (PVOID)wnd);
#endif

    assert(wnd != NULL && hwnd != NULLHANDLE);

    wnd->last_msg = msg;
    wnd->last_mp1 = mp1;
    wnd->last_mp2 = mp2;
#if 0
    if (msg == WM_SETFONT)
	return 0;
    else if (msg == WM_INITDIALOG)
	return TRUE;
#endif      

    switch (msg) {

    case WM_ACTIVATE:
    {
	WORD state = SHORT1FROMMP(mp1);
	HWND hwnd  = (HWND)mp2;
	WORD minimized;

	minimized = (WinQueryWindowULong(hwnd, QWL_STYLE) & WS_MINIMIZED) ? TRUE : FALSE;
	wnd->OnActivate(state, minimized, hwnd);
	return 0;
    }

    case WM_SETFOCUS: {
	HWND hwnd = (HWND)mp1;
	if (SHORT1FROMMP(mp2) == TRUE)
	    return (MRESULT)wnd->OnSetFocus(hwnd);
	else
	    return (MRESULT)wnd->OnKillFocus(hwnd);
	break;
    }

    case WM_CREATE:
	TRACE("WM_CREATE handle, calling wnd->OnCreate(%#x) = %#x", (int)mp2, (int)&wnd->OnCreate);
	wnd->OnCreate((PCREATESTRUCT)mp2);
	TRACE("returning from wnd->OnCreate");
	return 0;

    case WM_PAINT: {
	if (wnd->__type == wxTYPE_CANVAS_WND) {
	    wnd->hpsUpdate = WinBeginPaint(hwnd, ((wxCanvasWnd *)wnd)->hpsCanvas, &wnd->rectlUpdate);
	    wnd->OnPaint(hwnd);
	    WinEndPaint(wnd->hpsUpdate);
	    wnd->hpsUpdate = NULLHANDLE;
	} else {
	    wnd->hpsUpdate = WinBeginPaint(hwnd, NULLHANDLE, &wnd->rectlUpdate);
	    wxSetupPS(wnd->hpsUpdate);
	    wnd->OnPaint(hwnd);
	    WinEndPaint(wnd->hpsUpdate);
	    wnd->hpsUpdate = NULLHANDLE;
	}
	return 0;
    }

    case WM_SIZE: {
	ULONG style = WinQueryWindowULong(hwnd, QWL_STYLE);
	int width  = SHORT1FROMMP(mp2);
	int height = SHORT2FROMMP(mp2);
	wnd->OnSize(width, height, style);
	break;
    }

    case WM_BUTTON1DOWN: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->buttonstate |= BUTTON1;

	if (hwnd != WinQueryFocus(HWND_DESKTOP))
	    WinSetFocus(HWND_DESKTOP, hwnd);
	WinSetCapture(HWND_DESKTOP, hwnd);

	wnd->OnLButtonDown(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_BUTTON1UP: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->buttonstate &= ~BUTTON1;

	WinSetCapture(HWND_DESKTOP, NULLHANDLE);

	wnd->OnLButtonUp(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_BUTTON1DBLCLK: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->OnLButtonDClick(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_BUTTON2DOWN: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->buttonstate |= BUTTON2;

	if (hwnd != WinQueryFocus(HWND_DESKTOP))
	    WinSetFocus(HWND_DESKTOP, hwnd);
	WinSetCapture(HWND_DESKTOP, hwnd);

	wnd->OnRButtonDown(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_BUTTON2UP: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->buttonstate &= ~BUTTON2;

	WinSetCapture(HWND_DESKTOP, NULLHANDLE);

	wnd->OnRButtonUp(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_BUTTON2DBLCLK: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->OnRButtonDClick(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_BUTTON3DOWN: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->buttonstate |= BUTTON3;

	if (hwnd != WinQueryFocus(HWND_DESKTOP))
	    WinSetFocus(HWND_DESKTOP, hwnd);
	WinSetCapture(HWND_DESKTOP, hwnd);

	wnd->OnMButtonDown(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_BUTTON3UP: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->buttonstate &= ~BUTTON3;

	WinSetCapture(HWND_DESKTOP, NULLHANDLE);

	wnd->OnMButtonUp(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_BUTTON3DBLCLK: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->OnMButtonDClick(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_MOUSEMOVE: {
	int x = wnd->PmToWxX(SHORT1FROMMP(mp1));
	int y = wnd->PmToWxY(SHORT2FROMMP(mp1));
	wnd->OnMouseMove(x, y, SHORT2FROMMP(mp2));
	break;
    }

    case WM_DESTROY:
	if (wnd->OnDestroy())
	    return 0;
	else
	    return wnd->DefWindowProc(msg, mp1, mp2);
	break;
/*
  case WM_SYSCOMMAND:
  break;
  */
    case WM_CONTROL: {
	WORD id = SHORT1FROMMP(mp1);
	WORD usnotifycode = SHORT2FROMMP(mp1);
//	    ULONG ulcontrolspec = LONGFROMMP(mp2);
	wxWindow *item = wnd->wx_window->FindItem(id);
	if (item) {
	    BOOL value = item->OS2Control(usnotifycode, id);
	    return (MRESULT)value;
	}
	return FALSE;
    }

    case WM_COMMAND: {
	USHORT uscmd = SHORT1FROMMP(mp1);
	USHORT ussource = SHORT1FROMMP(mp2);
	USHORT uspointer = SHORT1FROMMP(mp2);

	if (!wnd->OS2Command(uscmd, ussource, uspointer))
	    return wnd->DefWindowProc(msg, mp1, mp2);
	break;
    }

#if 0
    case WM_MENUSELECT:
    {
//            WORD id       = SHORT1FROMMP(mp1);
	WORD flags    = SHORT2FROMMP(mp1);
	HWND sysmenu = (HWND)mp2;
	wnd->OnMenuSelect(0, flags, sysmenu);
	break;
    }
#endif

    case WM_CHAR:
    {
	USHORT fsKeyFlags = CHARMSG(&msg)->fs;
	wxShiftDown = fsKeyFlags & KC_SHIFT ? TRUE : FALSE;
	wxControlDown = fsKeyFlags & KC_CTRL ? TRUE : FALSE;
	wxAltDown = fsKeyFlags & KC_ALT ? TRUE : FALSE;
		
	if (!fsKeyFlags & KC_KEYUP) {
	    if (fsKeyFlags & KC_VIRTUALKEY)
		wnd->OnChar(SHORT2FROMMP(mp2), FALSE);
	    else if (fsKeyFlags & KC_CHAR)
		wnd->OnChar(SHORT1FROMMP(mp2), TRUE);
	}
	break;
    }

    case WM_HSCROLL:
    {
	int usidentifier = SHORT1FROMMP(mp1); // Scroll bar control window identifier
	int sslider = SHORT1FROMMP(mp2);	// Slider position
	int uscmd = SHORT2FROMMP(mp2);	// Command SB_
	TRACE("WM_HSCROLL: usidentifier=%d, sslider=%d, uscmd=%d", usidentifier, sslider, uscmd);
	wnd->OnHScroll(uscmd, sslider, usidentifier);
	break;
    }

    case WM_VSCROLL:
    {
	int usidentifier = SHORT1FROMMP(mp1); // Scroll bar control window identifier
	int sslider = SHORT1FROMMP(mp2);	// Slider position
	int uscmd = SHORT2FROMMP(mp2);	// Command SB_
	TRACE("WM_VSCROLL: usidentifier=%d, sslider=%d, uscmd=%d", usidentifier, sslider, uscmd);
	wnd->OnVScroll(uscmd, sslider, usidentifier);
	break;
    }

    case WM_SYSCOLORCHANGE:
    {
	return MPFROMLONG(wnd->OnColorChange(hwnd, msg, mp1, mp2));
	break;
    }

#if 0
    case WM_ERASEBACKGROUND:
    {
	// Prevents flicker when dragging
	if (WinQueryWindowULong(hwnd, QWL_STYLE) & WS_MINIMIZED) 
            return MPFROMSHORT(1);

	HPS hpsFrame = (HPS)LONGFROMMP(mp1);
	PRECTL pprcPaint = (PRECTL)PVOIDFROMMP(mp2);
	wxSetupPS(hpsFrame);
	return (MRESULT)(!wnd->OnEraseBkgnd(hpsFrame, pprcPaint));
    }
#endif

    case WM_CLOSE:
    {
	if (wnd->OnClose())
	    return 0L;
	else
	    return MPFROMSHORT(1);
	break;
    }

    case WM_WINDOWPOSCHANGED: {
	PSWP pswp = (PSWP)PVOIDFROMMP(mp1);
	TRACE("\tfl=%#x, cx=%d, cy=%d", (int)pswp->fl, (int)pswp->cx, (int)pswp->cy);
	if (pswp->fl & SWP_SIZE) {
	    TRACE("wxWndProc: win_height=%d, win_width=%d", wnd->win_height, wnd->win_width);

	    wnd->win_height = pswp->cy;
	    wnd->win_width = pswp->cx;
	}
	break;
    }


    case WM_PRESPARAMCHANGED: {
	ULONG idAttrType = LONGFROMMP(mp1);

	switch (idAttrType) {

	case PP_BACKGROUNDCOLOR:
	    wnd->OS2BackgroundColourChanged();
	    break;

	case PP_FONTNAMESIZE:
	    wnd->OS2FontChanged();
	    break;
	}
	return 0;
    }
	
    default:
//printf("Calling wnd->DefWindowProc\n");
	return wnd->DefWindowProc(msg, mp1, mp2);
    }
    return 0; // Success: we processed this command.
}

// Dialog window proc
MRESULT EXPENTRY wxDlgProc(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    wxWnd *wnd;
    if (msg == WM_CREATE) {
	ULONG *ctlptr = (ULONG *)PVOIDFROMMP(mp1);
	wnd = (wxWnd *)ctlptr[1];
	TRACE("wxDlgProc: WM_CREATE, hWnd=%#x, wnd=%#x", (int)hWnd, (int)wnd);
	WinSetWindowPtr(hWnd, QWL_USER, wnd);
	if (mp1 && mp2) {
	    PCREATESTRUCT p = (PCREATESTRUCT)PVOIDFROMMP(mp2);
	    p->pCtlData = (PVOID)ctlptr[2];
	    mp1 = (PVOID)ctlptr[2];
	}
    } else
	wnd = (wxWnd *)WinQueryWindowPtr(hWnd, QWL_USER);

#ifdef DEBUGPM
    wxTraceEvent("wxDlgProc", hWnd, msg, mp1, mp2, (PVOID)wnd);
#endif

    assert(wnd != NULL && hWnd != NULLHANDLE);

#if 0
    if (msg == WM_SETFONT)
	return 0;
    else if (msg == WM_INITDIALOG)
	return 0;
#endif

    switch (msg)
    {
    case WM_ACTIVATE:
    {
	WORD state = SHORT1FROMMP(mp1);
	HWND hwnd  = (HWND)mp2;
	WORD minimized;

	minimized = (WinQueryWindowULong(hwnd, QWL_STYLE) & WS_MINIMIZED) ? TRUE : FALSE;
	if (wnd) {
	    wnd->OnActivate(state, minimized, hwnd);
	}
	return 0;
	break;
    }

#if 1
    case WM_SETFOCUS:
    {
	HWND hwnd = (HWND)mp1;
	if (SHORT1FROMMP(mp2) == TRUE)
	    return (MRESULT)wnd->OnSetFocus(hwnd);
	else
	    return (MRESULT)wnd->OnKillFocus(hwnd);
	break;
    }
#endif

    case WM_CREATE:
	TRACE("wxDlgProc calling wnd->OnCreate(%#x) = %#x", (int)mp2, (int)&wnd->OnCreate);
	wnd->OnCreate((PCREATESTRUCT)mp2);
	TRACE("returning from wxDlgProc calling wnd->OnCreate");
	return 0;
            
    case WM_SIZE: {
	if (wnd) {
	    int width  = SHORT1FROMMP(mp2);
	    int height = SHORT2FROMMP(mp2);
	    wnd->OnSize(width, height, 0);
	}
	else return FALSE;
	break;
    }
/*
  case WM_DESTROY:
  if (wnd)
  {
  if (wnd->OnDestroy())
  return 0;
  }
  return FALSE;
  break;
  */
    case WM_COMMAND: {
	USHORT uscmd = SHORT1FROMMP(mp1);
	USHORT ussource = SHORT1FROMMP(mp2);
	USHORT uspointer = SHORT1FROMMP(mp2);

	if (!wnd->OS2Command(uscmd, ussource, uspointer))
	    return wnd->DefWindowProc(msg, mp1, mp2);
	break;
    }
    case WM_PAINT: {
	wnd->hpsUpdate = WinBeginPaint(hWnd, NULLHANDLE, &wnd->rectlUpdate);
	wxSetupPS(wnd->hpsUpdate);
//	    WinFillRect(wnd->cps, &wnd->rectlUpdate, *wxBLUE);
	wnd->OnPaint(hWnd);
	WinEndPaint(wnd->hpsUpdate);
	wnd->hpsUpdate = NULLHANDLE;
	return 0;
    }

#if 0
    case WM_RBUTTONDOWN:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnRButtonDown(x, y, mp1);
	break;
    }
    case WM_RBUTTONUP:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnRButtonUp(x, y, mp1);
	break;
    }
    case WM_RBUTTONDBLCLK:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnRButtonDClick(x, y, mp1);
	break;
    }
    case WM_MBUTTONDOWN:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnMButtonDown(x, y, mp1);
	break;
    }
    case WM_MBUTTONUP:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnMButtonUp(x, y, mp1);
	break;
    }
    case WM_MBUTTONDBLCLK:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnMButtonDClick(x, y, mp1);
	break;
    }
    case WM_LBUTTONDOWN:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnLButtonDown(x, y, mp1);
	break;
    }
    case WM_LBUTTONUP:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnLButtonUp(x, y, mp1);
	break;
    }
    case WM_LBUTTONDBLCLK:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnLButtonDClick(x, y, mp1);
	break;
    }
    case WM_MOUSEMOVE:
    {
	int x = wnd->PmToWxX(LOWORD(mp2));
	int y = wnd->PmToWxY(HIWORD(mp2));
	wnd->OnMouseMove(x, y, mp1);
	break;
    }
#endif

/* Do we intercept keystrokes from dialogs, or would this cause problems?
   case WM_KEYDOWN:
   {
   if (mp1 == VK_SHIFT)
   wxShiftDown = TRUE;
   else if (mp1 == VK_CONTROL)
   wxControlDown = TRUE;
   else if ((mp1 != VK_ESCAPE) && (mp1 != VK_SPACE) && (mp1 != VK_RETURN) && (mp1 != VK_DELETE))
   wnd->OnChar(mp1);
   break;
   }

   case WM_KEYUP:
   {
   if (mp1 == VK_SHIFT)
   wxShiftDown = FALSE;
   else if (mp1 == VK_CONTROL)
   wxControlDown = FALSE;
   break;
   }
   */
    case WM_HSCROLL:
    {
	int usidentifier = SHORT1FROMMP(mp1); // Scroll bar control window identifier
	int sslider = SHORT1FROMMP(mp2);	// Slider position
	int uscmd = SHORT2FROMMP(mp2);	// Command SB_
	wnd->OnHScroll(uscmd, sslider, usidentifier);
	break;
    }

    case WM_VSCROLL:
    {
	int usidentifier = SHORT1FROMMP(mp1); // Scroll bar control window identifier
	int sslider = SHORT1FROMMP(mp2);	// Slider position
	int uscmd = SHORT2FROMMP(mp2);	// Command SB_
	wnd->OnVScroll(uscmd, sslider, usidentifier);
	break;
    }
        
    case WM_SYSCOLORCHANGE:
    {
	if (wnd->userColours)
            return ::WinDefWindowProc( hWnd, msg, mp1, mp2);
	return ::WinDefWindowProc( hWnd, msg, mp1, mp2);
	break;
    }

#if 0
    case WM_ERASEBACKGROUND:
    {
	TRACE("wxDlgProc: wnd->OnEraseBkgnd(mp1=%#x)", (int)mp1);
	HPS hpsFrame = (HPS)LONGFROMMP(mp1);
	PRECTL pprcPaint = (PRECTL)PVOIDFROMMP(mp2);
	wxSetupPS(hpsFrame);
	return (MRESULT)(!wnd->OnEraseBkgnd(hpsFrame, pprcPaint));
    }
#endif
        
#if 0
    case WM_QUERYENDSESSION:
    {
	// Same as WM_CLOSE, but inverted results. Thx Microsoft :-)
	return wnd->OnClose();
	break;
    }
#endif
        
    case WM_CLOSE:
    {
	return MPFROMSHORT(!wnd->OnClose());
	break;
    }
        
#if 0
    case WM_DROPFILES:
    {
	wnd->OnDropFiles(mp1);
	break;
    }
#endif
        
    case WM_ADJUSTWINDOWPOS:
    {
	PSWP pswp = (PSWP)PVOIDFROMMP(mp1);
	TRACE("\thwnd=%d, hwndInsertBehind=%d, x=%ld, y=%ld, cx=%ld, cy=%ld, fl=%d", (int)pswp->hwnd, (int)pswp->hwndInsertBehind, pswp->x, pswp->y, pswp->cx, pswp->cy, (int)pswp->fl);
	goto deflt;
    }

    deflt:
    default:
	return WinDefWindowProc(hWnd, msg, mp1, mp2);
    }
    return FALSE;
}


/*
 * wxWnd class used to implement all Windows 3 windows
 */
wxWnd::wxWnd(void)
{
    TRACE("wxWnd::wxWnd()");
    x_scrolling_enabled = TRUE;
    y_scrolling_enabled = TRUE;
    calcScrolledOffset = TRUE;
    last_msg = 0;
    last_mp1 = 0;
    last_mp2 = 0;
    accelerator_table = NULLHANDLE;
    hwndMenu = 0;

    xscroll_pixels_per_line = yscroll_pixels_per_line = 0;
    xscroll_lines = yscroll_lines = 0;
    xscroll_lines_per_page = yscroll_lines_per_page = 0;
    xscroll_position = yscroll_position = 0;
    canDeleteBackgroundBrush = FALSE;
    userColours = FALSE;
    background_colour = 0;
    background_transparent = FALSE;
//    background_brush = NULL;
//    SetBackgroundBrush(wxLIGHT_GREY_BRUSH, FALSE);
//    SetBackgroundColour(wxLIGHT_GREY);

    last_x_pos = last_y_pos = -1.0;
    last_event = -1;
    hpsUpdate = NULLHANDLE;

    hwndWnd = NULLHANDLE;
    os2_cursor = NULLHANDLE;

    hwndMenu = NULLHANDLE;			// pgd
    wx_window = NULL;			// pgd
    buttonstate = 0L;

    OS2BackgroundColourChanged();	// Will setup to use the system background colour
}

wxWnd::~wxWnd(void)
{
    TRACE("wxWnd::~wxWnd() this=%#x __type=%s)", (int)this, wxGetTypeName(__type));
//#if !USE_GREY_BACKGROUND
//    if (background_brush && canDeleteBackgroundBrush)
//	::DeleteObject(background_brush) ;
//#endif
    /*
     * Will this call destroy hwndMenu also?
     * Should we destroy hwndMenu?
     * How about os2_cursor?
     */
    if (hwndWnd == NULLHANDLE)
	TRACE("wxWnd::~wxWnd() Window already destroyed");
    else {
	WinDestroyWindow(hwndWnd);
	hwndWnd = NULLHANDLE;
    }
}

// Default destroyer - override if you destroy it in some other way
// (e.g. with MDI child windows)
void wxWnd::DestroyWindow(void)
{
    DetachWindowMenu();
//  WinSetWindowPtr(hwndWnd, QWL_USER, 0);
//  HWND oldHandle = hwndWnd;
//  hwndWnd = NULLHANDLE;
//  os2_cursor = NULLHANDLE;
    // For some reason, wxWindows can activate another task altogether
    // when a frame is destroyed after a modal dialog has been invoked.
    // Try to bring the parent or main frame to the top.
    if (wx_window && (wxSubType(wx_window->__type, wxTYPE_FRAME) || wxSubType(wx_window->__type, wxTYPE_DIALOG_BOX)))
    {
	HWND hWnd = 0;
	if (wx_window->GetParent())
	    hWnd = wx_window->GetParent()->GetHWND();
//    else if (wxTheApp->wx_frame && (wxTheApp->wx_frame != wx_window))
//      hWnd = wxTheApp->wx_frame->GetHWND();
	if (hWnd)
	    WinShowWindow(hWnd, TRUE);
//      WinSetWindowPos(hWnd, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);
    }
  

    HWND hwnd1 = GetFrameHWND();
    HWND hwnd2 = WinQueryWindow(hwndWnd, QW_PARENT);
    assert(hwnd1 == hwnd2);

#if 0
    if (__type == wxTYPE_DIALOG_WND) {
	/*
	 * Destroy parent since parent is the frame window and
	 * hwndWnd points at the client window.
	 */
	WinDestroyWindow(WinQueryWindow(hwndWnd, QW_PARENT));
    } else
#endif
//      WinDestroyWindow(hwndWnd);
//TRACE("wxWnd::DestroyWindow: Destroying window %#x", (int)hwndWnd);
	hwndWnd = NULLHANDLE;
    os2_cursor = NULLHANDLE;

    // Menu is destroyed explicitly by wxMDIChildWnd::DestroyWindow,
    // or when Windows HWND is deleted if MDI parent or
    // SDI frame.
/*
  if (hwndMenu)
  {
  ::DestroyMenu(hwndMenu);
  hwndMenu = 0;
  }
  */
}

void wxWnd::Create(wxWnd *parent, char *wclass, wxWindow *wx_win, const char *title,
                    int x, int y, int width, int height,
                    DWORD style, char *dialog_template, DWORD extendedStyle)
{
    TRACE("wxWnd::Create(parent=%#x, wclass='%s', wx_win=%#x, title='%s', x=%d, y=%d, w=%d, h=%d, style=%#x, dialog_template=%s, extendedStyle=%#x)", (int)parent, wclass, (int)wx_win, title, x, y, width, height, (int)style, dialog_template, (int)extendedStyle);
    wx_window = wx_win;
    if (wx_window)
	wx_window->handle = (char *)this;

//  int x1 = 0;
//  int y1 = 0;
//  int x2 = 100;
//  int y2 = 100;
    if (wx_window && wx_window->GetWindowStyleFlag() & wxUSER_COLOURS)
	userColours = TRUE;
    else
	userColours = FALSE;

    ULONG ctldata[3];
    ctldata[0] = sizeof ctldata;
    ctldata[1] = (int)this;
    ctldata[2] = 0;

    wxRECTL cRect;
    HWND  hwndParent = HWND_DESKTOP;
    if (parent)
	hwndParent = parent->GetClientHWND();      // MUNGE: Used to get proper parent window
    TRACE("hwndParent=%#x", hwndParent);

    /*
     * Clip to parent window
     */
    wx_ClientRect(hwndParent, &cRect);
    TRACE("\tbefore clipping, x=%d, y=%d, width=%d, height=%d", x, y, width, height);
    assert(x >= 0 && y >= 0 && width >= 0 && height >= 0);
    if (x < cRect.xLeft)
	x = cRect.xLeft;
    else if (x > cRect.xRight)
	x = cRect.xRight;
    if (x + width > cRect.xRight)
	width = cRect.xRight - x;
    if (y < cRect.yTop)
	y = cRect.yTop;
    else if (y > cRect.yBottom)
	y = cRect.yBottom;
    if (y + height > cRect.yBottom)
	height = cRect.yBottom - y;
    TRACE("\tafter clipping, x=%d, y=%d, width=%d, height=%d", x, y, width, height);
    assert(x >= 0 && y >= 0 && width >= 0 && height >= 0);
		

    TRACE("wxWnd::Create creating window %s from class %s", title, wclass);
    TRACE("\tw=%d, h=%d", width, height);
    TRACE("\tthis = %#x", (int)this);
//style|=WS_VISIBLE;
    if (strcmp(wclass, wxDialogClassName) == 0) {
	TRACE("Creating dialog window");
	DWORD os2_flags = FCF_NOBYTEALIGN | FCF_DLGBORDER;// FCF_TITLEBAR | FCF_DLGBORDER
//  ULONG extendedStyle = 0;

	if (style & wxMINIMIZE_BOX)
	    os2_flags |= FCF_MINBUTTON;
	if (style & wxMAXIMIZE_BOX)
	    os2_flags |= FCF_MAXBUTTON;
	if (style & wxTHICK_FRAME)
	    os2_flags = (os2_flags & ~FCF_DLGBORDER) | FCF_SIZEBORDER;
	if (style & wxSYSTEM_MENU)
	    os2_flags |= FCF_SYSMENU;
	if (title)
	    os2_flags |= FCF_TITLEBAR;

      
	FRAMECDATA fcd;
	fcd.cb = sizeof(FRAMECDATA);
	fcd.flCreateFlags = os2_flags;
	fcd.hmodResources = NULLHANDLE;
	fcd.idResources   = NULLHANDLE;

	TRACE("Creating Dialog Frame window, parent=%#x", (int)hwndParent);
	hwndWnd = WinCreateWindow(hwndParent, WC_FRAME, (PSZ)title, 0,
				     0, 0, 0, 0,
				     0,
				     HWND_TOP, NewControlId(), &fcd, NULL);
	TRACE("Dialog Frame window handle=%#x", hwndWnd);
	WinSetWindowPtr(hwndWnd, QWL_USER, this);
	TRACE("Creating Dialog Client window, parent=%#x", (int)hwndWnd);
	HWND hwndClient = WinCreateWindow(hwndWnd, (PSZ)wclass, NULL,
				     WS_VISIBLE, 0, 0, 0, 0,
				     hwndWnd,
				     HWND_BOTTOM, FID_CLIENT, (PVOID)ctldata, NULL);

	((wxDialogWnd *)this)->hwndClient = hwndClient;

	TRACE("Dialog client window handle=%#x", hwndClient);

	background_colour = wxGetSysColour(SYSCLR_DIALOGBACKGROUND);
	wxSetBackgroundColour(hwndClient, background_colour);
	TRACE("wxWnd::Create: background_colour is %#06x", background_colour);

	/*
	 * Set client window size
	 */

//      wx_MoveWindow(hwndWnd, x, (cRect.yBottom-y-height), width, height, TRUE);
///		width = 600; height = 300;
	WinSetWindowPos(hwndWnd, HWND_TOP, 100, 100, width, height,
			SWP_SIZE|SWP_MOVE /*|SWP_ACTIVATE */|SWP_ZORDER /*|SWP_SHOW */);
	int fdx = WinQuerySysValue(HWND_DESKTOP, SV_CXDLGFRAME);
	int fdy = WinQuerySysValue(HWND_DESKTOP, SV_CYDLGFRAME);
//	int fdx = WinQuerySysValue(HWND_DESKTOP, SV_CXBORDER);
//	int fdy = WinQuerySysValue(HWND_DESKTOP, SV_CYBORDER);
	int tdy = WinQuerySysValue(HWND_DESKTOP, SV_CYTITLEBAR);
	WinSetWindowPos(hwndClient, HWND_TOP,
			fdx, fdy, width - 2*fdx, height - fdy - 2*tdy,
			SWP_MOVE | SWP_SIZE /* | SWP_SHOW  */ );
//		win_height = height - fdy - 2*tdy;
//      wx_MoveWindow(hwndWnd, x, (cRect.yBottom-y-height), width, height, TRUE);
//		WinShowWindow(hwndWnd, TRUE);
//		WinShowWindow(hwndClient, TRUE);

	TRACE("hwndWnd=%#x, hwndClient=%#x", (int)hwndWnd, (int)hwndClient);

    } else {
	TRACE("Creating non-dialog window");
	hwndWnd = WinCreateWindow(hwndParent, (PSZ)wclass, (PSZ)title, style,
				 x, (cRect.yBottom-y-height), width, height,
//		  0, 0, width, height,
				 hwndParent, HWND_TOP, NewControlId(), (PVOID)ctldata, NULL);
	win_height = height;
	TRACE("wxWnd::Create: Window created, parent=%#x, handle=%#x", (int)hwndParent, (int)hwndWnd);
	TRACE("\thandle=%#x", (int)hwndWnd);
	/*
	 * Pick up default presentation parameters
	 */
	background_colour = wxGetSysColour(SYSCLR_BACKGROUND);
	wxSetBackgroundColour(hwndWnd, background_colour);
	TRACE("wxWnd::Create: background_colour set to %#06x", background_colour);
    }
    assert(hwndWnd != NULLHANDLE);
//  WinShowWindow(hwndFrame, TRUE);
//  WinShowWindow(hwndWnd, TRUE);
}

void wxWnd::OnCreate(PCREATESTRUCT cs)
{
    TRACE("wxWnd::OnCreate(cs=%#x)", (int)cs);
}

BOOL wxWnd::OnPaint(HWND hwnd)
{
TRACE("wxWnd::OnPaint(hwnd=%#x)", (int)hwnd);
	return 1;
}

BOOL wxWnd::OnClose(void)
{
    TRACE("wxWND::OnClose()");
    return FALSE;
}

BOOL wxWnd::OnDestroy(void)
{
    TRACE("wxWND::OnDestroy()");
  return TRUE;
}

void wxWnd::OnSize(int x, int y, UINT style)
{
    TRACE("wxWND::OnSize(x=%d, y=%d, style=%#x)", x, y, (int)style);
//	win_height = y;
//	win_width = x;
}

// Deal with child commands from buttons etc.

BOOL wxWnd::OS2Command(int cmd, int source, int pointer)
{
    TRACE("wxWND::OS2Command(cmd=%#x, source=%d, pointer=%d)", cmd, source, pointer);
    return FALSE;
}

void wxWnd::OnMenuSelect(WORD item, WORD flags, HWND sysmenu)
{
    TRACE("wxWnd::OnMenuSelect(item=%#x, flags=%#x, sysmenu=%#x)", (int)item, (int)flags, (int)sysmenu);
}

BOOL wxWnd::OnActivate(BOOL state, BOOL minimized, HWND activate)
{
TRACE("wxWnd::OnActivate(state=%d, minimized=%d, activate=%#x)", state, minimized, (int)activate);
#if 0
  if (wx_window)
  {
    // Deal with caret
    if (wx_window->caretEnabled && (wx_window->caretWidth > 0) && (wx_window->caretHeight > 0))
    {
      ::CreateCaret(wx_window->GetHWND(), NULL, wx_window->caretWidth, wx_window->caretHeight);
      if (wx_window->caretShown)
        ::ShowCaret(wx_window->GetHWND());
    }
    
    wx_window->GetEventHandler()->OnActivate(((state == WA_ACTIVE) || (state == WA_CLICKACTIVE)));

    // If this window is an MDI parent, we must also send an OnActivate message
    // to the current child.
    if (wxSubType(wx_window->__type, wxTYPE_FRAME))
    {
      wxFrame *frame = (wxFrame *)wx_window;
      if (frame->frame_type == wxMDI_PARENT)
      {
        wxMDIFrameWnd *mdiFrame = (wxMDIFrameWnd *)this;
        if ((mdiFrame->current_child) && ((state == WA_ACTIVE) || (state == WA_CLICKACTIVE)))
          mdiFrame->current_child->wx_window->GetEventHandler()->OnActivate(TRUE);
      }
    }
    return 0;
  }
  else return TRUE;
#else
  return TRUE;
#endif
}

BOOL wxWnd::OnSetFocus(HWND hwnd)
{
    TRACE("wxWnd::OnSetFocus(hwnd=%#x)", (int)hwnd);
    if (wx_window) {
	wx_window->GetEventHandler()->OnSetFocus();
//    return 0;
	return TRUE;
    }
    else return FALSE;
}

BOOL wxWnd::OnKillFocus(HWND hwnd)
{
    TRACE("wxWnd::OnKillFocus(hwnd=%#x)", (int)hwnd);
    if (wx_window) {
	// Deal with caret
#if XXX
	if (wx_window->caretEnabled)
	{
	    ::DestroyCaret();
	}
#endif

	wx_window->GetEventHandler()->OnKillFocus();
	return TRUE;
    }
    else return FALSE;
}

void wxWnd::OnDropFiles(MPARAM mp1)
{
    TRACE("wxWnd::OnDropFiles(mp1=%#x)", (int)mp1);
#if 0

  HANDLE hFilesInfo = (HANDLE)mp1;
  POINTL dropPoint;
  DragQueryPoint(hFilesInfo, (LPPOINT) &dropPoint);

  // Get the total number of files dropped
  WORD gwFilesDropped = DragQueryFile (hFilesInfo,
                                   (UINT)-1,
                                   NULL,
                                   0);

  char **files = new char *[gwFilesDropped];
  for (int wIndex=0; wIndex < (int)gwFilesDropped; wIndex++)
  {
    DragQueryFile (hFilesInfo, wIndex, (LPSTR) wxBuffer, 1000);
    files[wIndex] = copystring(wxBuffer);
  }
  DragFinish (hFilesInfo);
  if (wx_window)
    wx_window->GetEventHandler()->OnDropFiles(gwFilesDropped, files, dropPoint.x, dropPoint.y);

  for (int i = 0; i < (int)gwFilesDropped; i++)
    delete[] files[i];
  delete[] files;
#endif  
}

#if 0
BOOL wxWnd::OnDrawItem(int id, DRAWITEMSTRUCT *itemStruct)
{
    TRACE("wxWnd::OnDrawItem(id=%#x, itemStruict=%#x)", id, (int)itemStruct);
#if 0
  wxWindow *item = wx_window->FindItem(id);
#if USE_DYNAMIC_CLASSES
  if (item && item->IsKindOf(CLASSINFO(wxItem)))
  {
    return ((wxItem *)item)->MSWOnDraw(itemStruct);
  }
  else
#endif
#endif
    return FALSE;
}
#endif

#if 0
BOOL wxWnd::OnMeasureItem(int id, MEASUREITEMSTRUCT *itemStruct)
{
    TRACE("wxWnd::onMeasureItem(id=%#x, itemStruct=%#x)", id, (int)itemStruct);
#if 0
  wxWindow *item = wx_window->FindItem(id);
#if USE_DYNAMIC_CLASSES
  if (item && item->IsKindOf(CLASSINFO(wxItem)))
  {
    return ((wxItem *)item)->MSWOnMeasure(itemStruct);
  }
  else
#endif
#endif
    return FALSE;
}
#endif

void wxWnd::OnVScroll(int uscmd, int sslider, int usidentifier)
{
TRACE("wxWnd::OnVScroll(uscmd=%d, sslider=%d, usidentifier=%d)", uscmd, sslider, usidentifier);
#if DEBUG > 1
  wxDebugMsg("wxWnd::OnVScroll %#x\n", hwndWnd);
#endif
}

void wxWnd::OnHScroll(int uscmd, int sslider, int usidentifier)
{
    TRACE("wxWnd::OnHscroll(uscmd=%d, sslider=%d, usidentifier=%d)", uscmd, sslider, usidentifier);
}

void wxWnd::CalcScrolledPosition(int x, int y, int *xx, int *yy)
{
    *xx = (calcScrolledOffset ? (x - xscroll_position * xscroll_pixels_per_line) : x);
    *yy = (calcScrolledOffset ? (y - yscroll_position * yscroll_pixels_per_line) : y);
}

void wxWnd::CalcUnscrolledPosition(int x, int y, float *xx, float *yy)
{
    *xx = (float)(calcScrolledOffset ? (x + xscroll_position * xscroll_pixels_per_line) : x);
    *yy = (float)(calcScrolledOffset ? (y + yscroll_position * yscroll_pixels_per_line) : y);
}


UINT wxWnd::OnCtlColor(HPS pDC, HWND pWnd, UINT nCtlColor, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    TRACE("wxWnd::OnCtlColor(pDC=%#x, pWnd=%#x, nCtlColor=%d, msg=%#x, mp1=%#x, mp2=%#x)", (int)pDC, (int)pWnd, (int)nCtlColor, (int)msg, (int)mp1, (int)mp2);
#if 0
#if DEBUG > 1
    wxDebugMsg("wxWnd::OnCtlColour %#x\n", hwndWnd);
#endif
    // Ignores CTL3DXXX and FAFA settings of background colour,
    // uses current background colour for background,
    // and COLOR_BTNFACE for foreground.
    if (userColours && wx_window) {
	// Is this OK for WIN32???
	wxWindow *item = wx_window->FindItemByHWND((HWND)LOWORD(mp2));
	if ((nCtlColor == CTLCOLOR_STATIC || nCtlColor == CTLCOLOR_BTN) && background_brush) {
	    if (background_transparent) {
		SetBkMode(pDC, TRANSPARENT);
	    } else {
		SetBkMode(pDC, OPAQUE);
		// Radio boxes, group boxes and check boxes are different kinds of buttons:
		// the text should be the same colour as the background, not button-coloured.
//        if ((nCtlColor == CTLCOLOR_BTN) && (!item ||
//             ((item->__type != wxTYPE_RADIO_BOX) && (item->__type != wxTYPE_GROUP_BOX) &&
//              (item->__type != wxTYPE_CHECK_BOX))))
//          SetBkColor(pDC, GetSysColor(COLOR_BTNFACE));
//        else
//          SetBkColor(pDC, background_colour);
	    }
	    return background_brush;
	}
	else return NULL;
    }

    if ((nCtlColor == CTLCOLOR_STATIC || nCtlColor == CTLCOLOR_BTN) && background_brush) {
	// After investigations, I found that CTL3DXXX use BTNFACE color
	// (which is ALWAYS grey :-))
	// So, respect the behavior!
	SetBkColor(pDC, GetSysColor(COLOR_BTNFACE)) ;
	return background_brush;
    } else
	return NULL;
#endif
    return NULLHANDLE;
}

void wxWnd::SetBackgroundColour(const wxColour& col)
{
    TRACE("wxWnd::SetBackgroundColour(...)");
    if (background_colour != col) {
	TRACE("wxWnd::SetBackgroundColour: Setting new background colour: %#06lx", (LONG)col);
	/*
	 * The order of the following two is important.
	 * Otherwise we will get infinite recursion.
	 */
	background_colour = col;
	TRACE("wxWnd::SetBackgroundColour: background_colour set to %#6x", background_colour);
	wxSetBackgroundColour(hwndWnd, background_colour);
    }
}

wxColour wxWnd::GetBackgroundColour(void)
{
    return wxColour(background_colour);
}


/*
 * This message called on receipt of a WM_PRESPARAMSCHANGED with
 * parameter PP_BACKGROUNDCOLOR
 */
void wxWnd::OS2BackgroundColourChanged(void)
{
    TRACE("wxWnd::OS2BackgroundColourChanged()");
    
    COLOR col = wxGetBackgroundColour(hwndWnd);
    if (col != -1) {
	wxColour new_colour = col;

	TRACE("new_colour=%#06lx, background_colour=%#06lx", (LONG)new_colour, background_colour);
	wxColour old_background_colour = background_colour;
	if (new_colour != background_colour) {
	    if (userColours) {
		/*
		 * The background colour has already been changed.
		 * If user want's control over the colours, we
		 * have to change back to the user colour.
		 */
		wxSetBackgroundColour(hwndWnd, background_colour);
	    }
	    
	    /*
	     * Inform higher level routines of the new background colour
	     */
	    TRACE("wx_window=%#x", wx_window);
	    if (wx_window)
		wx_window->GetEventHandler()->OnBackgroundColourChange(wxColour(new_colour));
	}
	
	/*
	 * Invalidate window only if colour changed.
	 */
	if (old_background_colour != background_colour)
	    WinInvalidateRect(GetClientHWND(), NULL, TRUE);
    }
}

/*
 * This message called on receipt of a WM_PRESPARAMSCHANGED with
 * parameter PP_FONTNAMESIZE
 */
void wxWnd::OS2FontChanged(void)
{
    TRACE("wxWND::OS2FontChanged()");
    ULONG len = WinQueryPresParam(hwndWnd, PP_FONTNAMESIZE, 0, NULL, 0, NULL, 0);
    char *fontnamesize = new char[len+1];
    WinQueryPresParam(hwndWnd, PP_FONTNAMESIZE, 0, NULL, len, MPFROMP(fontnamesize), 0);
    fontnamesize[len] = 0;
    TRACE("wxWnd:OS2FontChanged: new font='%s'", fontnamesize);

    /*
     * Inform higher level routines of the new font colour
     */
//    if (wx_window)
//	wx_window->GetEventHandler()->OnFontChange(fontnamesize);

    WinInvalidateRect(GetClientHWND(), NULL, TRUE);
}

BOOL wxWnd::OnColorChange(HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    TRACE("wxWnd::OnColorChange(hWnd=%#x, msg=%#x, mp1=%#x, mp2=%#x)", (int)hWnd, (int)msg, (int)mp1, (int)mp2);
    if (userColours)
	return (BOOL)::WinDefWindowProc(hWnd, msg, mp1, mp2);

    return (BOOL)::WinDefWindowProc(hWnd, msg, mp1, mp2);
}

BOOL wxWnd::OnEraseBkgnd(HPS pPS, PRECTL pprcPaint)
{
    TRACE("wxWnd::OnEraseBkgnd(pPS=%#x) this=%#x", (int)pPS, (int)this);
    TRACE("wxWnd::OnEraseBkgnd: background_colour=%#6x", background_colour);
    if (pprcPaint)
	WinFillRect(pPS, pprcPaint, background_colour);
    else {
	RECTL cRect;
	wx_ClientRect(hwndWnd, &cRect);
	WinFillRect(pPS, &cRect, background_colour);
    }
    return TRUE;
}

void wxWnd::OnLButtonDown(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnLButtonDown(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnLButtonUp(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnLButtonUp(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnLButtonDClick(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnLButtonDClick(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnMButtonDown(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnMButtonDown(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnMButtonUp(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnMButtonUp(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnMButtonDClick(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnMButtonDClick(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnRButtonDown(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnRButtonDown(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnRButtonUp(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnRButtonUp(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnRButtonDClick(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnRButtonDClick(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnMouseMove(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnMouseMove(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnMouseEnter(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnMouseEnter(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnMouseLeave(int x, int y, UINT flags)
{
    TRACE("wxWnd::OnMouseLeave(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
}

void wxWnd::OnChar(USHORT chvk, Bool isASCII)
{
    TRACE("wxWnd::OnChar(chvk=%d, isASCII=%d)", (int)chvk, isASCII);
}

MRESULT wxWnd::DefWindowProc(ULONG nMsg, MPARAM mp1, MPARAM mp2)
{
  return WinDefWindowProc(hwndWnd, nMsg, mp1, mp2);
}

BOOL wxWnd::ProcessMessage(QMSG* WXUNUSED(pMsg))
{
  return FALSE;
}

BOOL wxWnd::OnMDIActivate(BOOL flag, HWND activate, HWND deactivate)
{
    TRACE("wxWnd::OnMDIActivate(flag=%d, activate=%#x, deactivate=%#x)", flag, (int)activate, (int)deactivate);
#if DEBUG > 1
  wxDebugMsg("wxWnd::OnMDIActivate %#x\n", hwndWnd);
#endif
  return 1;
}

void wxWnd::DetachWindowMenu(void)
{
#if 0
  if (hwndMenu)
  {
    int N = GetMenuItemCount(hwndMenu);
    int i;
    for (i = 0; i < N; i++)
    {
      char buf[100];
      int chars = GetMenuString(hwndMenu, i, buf, 100, MF_BYPOSITION);
      if ((chars > 0) && (strcmp(buf, "&Window") == 0))
      {
        RemoveMenu(hwndMenu, i, MF_BYPOSITION);
        break;
      }
    }
  }
#endif
}


/*
 * Subwindow - used for panels and canvases
 *
 */

wxSubWnd::wxSubWnd(wxWnd *parent, char *wclass, wxWindow *wx_win,
                           int x, int y, int width, int height,
                           DWORD style, char *dialog_template)
{
    TRACE("wxSubWnd::wxSubWnd(parent=%#x, wclass='%s', wx_win=%#x, x=%d, y=%d, w=%d, h=%d, style=%#x, dialog_template='%s')", (int)parent, wclass, (int)wx_win, x, y, width, height, (int)style, dialog_template);
    Create(parent, wclass, wx_win, NULL, x, y, width, height, style, dialog_template);
    mouse_in_window = FALSE ;
}

wxSubWnd::~wxSubWnd(void)
{
    TRACE("wxSubWnd::~wxSubWnd() this=%#x, wxWnd=%#x", this, (wxWnd *)this);
}


BOOL wxSubWnd::OnPaint(HWND hwnd)
{
    TRACE("wxSubWnd::OnPaint(hwnd=%#x)", (int)hwnd);

    OnEraseBkgnd(hpsUpdate);
    if (wx_window)
	wx_window->GetEventHandler()->OnPaint();

    return TRUE;
}

void wxSubWnd::OnSize(int w, int h, UINT flag)
{
    TRACE("wxSubWnd::OnSize(w=%d, h=%d, flag=%d)", w, h, (int)flag);

    // Recalculate scroll bar range and position
    if (xscroll_lines > 0) {
	int nMaxWidth = xscroll_lines*xscroll_pixels_per_line;
#if 0    
	float noPositions = (float)((nMaxWidth - w)/(float)xscroll_pixels_per_line);
	if (noPositions > 0 && noPositions < 1)
	    noPositions = 1;
	else if (noPositions < 0)
	    noPositions = 0;
      
	int nHscrollMax = wxMax(0, (int)(0 + noPositions));
#endif    
	int nHscrollMax = wxMax(0, (int)(2 + (nMaxWidth - w)/xscroll_pixels_per_line));
	xscroll_position = wxMin(nHscrollMax, xscroll_position);

#if 1
	WinSendMsg(WinWindowFromID(hwndWnd, FID_HORZSCROLL), SBM_SETSCROLLBAR,
		   (MPARAM)xscroll_position, MPFROM2SHORT(0, nHscrollMax));
#else
	SetScrollRange(hwndWnd, SB_HORZ, 0, nHscrollMax, FALSE);
	SetScrollPos(hwndWnd, SB_HORZ, xscroll_position, TRUE);
#endif    
//    wxDebugMsg("w = %d\n", w);
//    wxDebugMsg("nMaxWidth = xscroll_lines (%d) * xscrollpixels_per_line (%d) = %d\n",
//      xscroll_lines, xscroll_pixels_per_line, nMaxWidth);
//    wxDebugMsg("nHscrollMax = %d, xscroll_position = %d\n", nHscrollMax, xscroll_position);
    }     
    if (yscroll_lines > 0) {
	int nMaxHeight = yscroll_lines*yscroll_pixels_per_line;
#if 0    
	float noPositions = (float)((nMaxHeight - h)/(float)yscroll_pixels_per_line);
	if (noPositions > 0 && noPositions < 1)
	    noPositions = 1;
	else if (noPositions < 0)
	    noPositions = 0;

	int nVscrollMax = wxMax(0, (int)(0 + noPositions));
#endif    
	int nVscrollMax = wxMax(0, (int)(2 + (nMaxHeight - h)/yscroll_pixels_per_line));
	yscroll_position = wxMin(nVscrollMax, yscroll_position);

#if 1
	WinSendMsg(WinWindowFromID(hwndWnd, FID_VERTSCROLL), SBM_SETSCROLLBAR, 
		   (MPARAM)yscroll_position, MPFROM2SHORT(0, nVscrollMax));
#else
	SetScrollRange(hwndWnd, SB_VERT, 0, nVscrollMax, FALSE);
	SetScrollPos(hwndWnd, SB_VERT, yscroll_position, TRUE);
#endif    
    }     

    // Store DC for duration of size message
    assert(hpsUpdate == NULLHANDLE);
    hpsUpdate = WinGetPS(hwndWnd);
    wxSetupPS(hpsUpdate);
    if (wx_window)
	wx_window->GetEventHandler()->OnSize(w, h);

    WinReleasePS(hpsUpdate);
    hpsUpdate = NULLHANDLE;
//    logical_function_set = pen_set = brush_set = background_brush_set = text_foreground_set = text_background_set = font_set = FALSE;
}

// Deal with child commands from buttons etc.
BOOL wxSubWnd::OS2Command(int cmd, int source, int pointer)
{
    TRACE("wxSubWnd::OS2Command(cmd=%#x, source=%d, pointer=%d)", cmd, source, pointer);

    TRACE("wxCurrentPopupMenu=%#x", wxCurrentPopupMenu);
    if (source == CMDSRC_MENU && wxCurrentPopupMenu) {
	TRACE("popup active, calling OS2Control for popup");
	Bool succ = wxCurrentPopupMenu->OS2Control(cmd, 0);
	wxCurrentPopupMenu = NULL;
	return succ;
    }

#if XXX
    wxWindow *item = wx_window->FindItem(id);
    if (item) {
	BOOL value = item->OS2Control(cmd, 0);
	return value;
    }
#endif

    return FALSE;
}

void wxSubWnd::OnLButtonDown(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnLButtonDown(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
#ifdef WIN32
    // DClick not clean supported on Win3.1, except if someone know
    // how to emulate Sleep()...
    // This means that your app will receive Down-Up-Dclick sequences
    // rather than Dclick
    if (wx_window && wx_window->doubleClickAllowed) {
	UINT time = GetDoubleClickTime() ;
	Sleep(time) ;
	MSG dummy ;
	if (PeekMessage(&dummy, hwndWnd,
			WM_LBUTTONDBLCLK,WM_LBUTTONDBLCLK,
			PM_NOREMOVE) ) {
	    PeekMessage(&dummy, hwndWnd, WM_LBUTTONUP,WM_LBUTTONUP,PM_REMOVE);
	    return; 
	}
    }
#endif
    wxMouseEvent event(wxEVENT_TYPE_LEFT_DOWN);

    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_LEFT_DOWN;
    if (wx_window)
	wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnLButtonUp(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnLButtonUp(x=%d, y=%d, flags=%#x)", x, y, (int)flags);

    wxMouseEvent event(wxEVENT_TYPE_LEFT_UP);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_LEFT_UP;

    if (wx_window)
	wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnLButtonDClick(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnLButtonDClick(x=%d, y=%d, flags=%#x)", x, y, (int)flags);

    wxMouseEvent event(wxEVENT_TYPE_LEFT_DCLICK);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_LEFT_DCLICK;

    if (wx_window && wx_window->doubleClickAllowed)
	wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnMButtonDown(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnMButtonDown(x=%d, y=%d, flags=%#x)", x, y, (int)flags);

#ifdef WIN32
    // DClick not clean supported on Win3.1, except if someone know
    // how to emulate Sleep()...
    // This means that your app will receive Down-Up-Dclick sequences
    // rather than Dclick
    if (wx_window && wx_window->doubleClickAllowed) {
	UINT time = GetDoubleClickTime() ;
	Sleep(time) ;
	MSG dummy ;
	if (PeekMessage(&dummy,hwndWnd,
			WM_MBUTTONDBLCLK,WM_MBUTTONDBLCLK,
			PM_NOREMOVE) ) {
	    PeekMessage(&dummy,hwndWnd,WM_MBUTTONUP,WM_MBUTTONUP,PM_REMOVE);
	    return; 
	}
    }
#endif

    wxMouseEvent event(wxEVENT_TYPE_MIDDLE_DOWN);

    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_LEFT_DOWN;
    if (wx_window) wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnMButtonUp(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnMButtonUp(x=%d, y=%d, flags=%#x)", x, y, (int)flags);

    wxMouseEvent event(wxEVENT_TYPE_MIDDLE_UP);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_LEFT_UP;
    if (wx_window) wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnMButtonDClick(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnMButtonDClick(x=%d, y=%d, flags=%#x)", x, y, (int)flags);

    wxMouseEvent event(wxEVENT_TYPE_MIDDLE_DCLICK);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_LEFT_DCLICK;
    if (wx_window && wx_window->doubleClickAllowed)
	wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnRButtonDown(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnRButtonDown(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
#ifdef WIN32
    // DClick not clean supported on Win3.1, except if someone know
    // how to emulate Sleep()...
    // This means that your app will receive Down-Up-Dclick sequences
    // rather than Dclick
    if (wx_window && wx_window->doubleClickAllowed) {
	UINT time = GetDoubleClickTime() ;
	Sleep(time) ;
	MSG dummy ;
	if (PeekMessage(&dummy,hwndWnd,
			WM_RBUTTONDBLCLK,WM_RBUTTONDBLCLK,
			PM_NOREMOVE)
	    ) {
	    PeekMessage(&dummy,hwndWnd,WM_RBUTTONUP,WM_RBUTTONUP,PM_REMOVE);
	    return; 
	}
    }
#endif

    wxMouseEvent event(wxEVENT_TYPE_RIGHT_DOWN);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON2);
    event.rightDown = !!(buttonstate & BUTTON3);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_RIGHT_DOWN;
    if (wx_window) wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnRButtonUp(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnRButtonUp(x=%d, y=%d, flags=%#x)", x, y, (int)flags);

    wxMouseEvent event(wxEVENT_TYPE_RIGHT_UP);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_RIGHT_UP;
    if (wx_window) wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnRButtonDClick(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnRButtonDClick(x=%d, y=%d, flags=%#x)", x, y, (int)flags);

    wxMouseEvent event(wxEVENT_TYPE_RIGHT_DCLICK);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_x_pos = event.x; last_y_pos = event.y; last_event = wxEVENT_TYPE_RIGHT_DCLICK;
    if (wx_window && wx_window->doubleClickAllowed)
	wx_window->GetEventHandler()->OnEvent(event);
}


void wxSubWnd::OnMouseMove(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnMouseMove(x=%d, y=%d, flags=%#x), wx_window=%#x", x, y, (int)flags, (int)wx_window);

    /*
     * Set cursor appropriate for window
     */
    HPOINTER cursor = wxIsBusy();
    if (!cursor)
	cursor = os2_cursor;
    if (cursor)
	WinSetPointer(HWND_DESKTOP, cursor);

#if MOUSE_EXIT_FIX //Should work now!!

    // Don't do the Leave/Enter fix if we've captured the window,
    // or SetCapture won't work properly.
    if (wx_window && !wx_window->winCaptured) {
	HWND hunder ;
	POINTL pt ;
	// See if we Leave/Enter the window.
	WinQueryPointerPos(HWND_DESKTOP, &pt);
	hunder = WinWindowFromPoint(HWND_DESKTOP, &pt, FALSE) ;

	if (hunder == hwndWnd) {
	    TRACE("I'm in the Window, but perhaps in NC area");
	    // I'm in the Window, but perhaps in NC area.
	    RECTL wind ;
	    RECTL nc ;

	    wx_ClientRect(hwndWnd, &wind);
	    wx_WindowRect(hwndWnd, &nc);
	    pt.x -= nc.xLeft ;
	    pt.y -= nc.yTop ;
	    wind.xLeft    += WINDOW_MARGIN ; // to be able to 'see' leave
	    wind.yTop     += WINDOW_MARGIN ; // to be able to 'see' leave
	    wind.xRight   -= WINDOW_MARGIN ; // to be able to 'see' leave
	    wind.yBottom  -= WINDOW_MARGIN ; // to be able to 'see' leave

	    if (!WinPtInRect(wxhAb, &wind, &pt)) {
		TRACE("simulate a Leave event");
		hunder = NULLHANDLE ; // So, I can simulate a Leave event...
	    }
	}

	if (hunder != hwndWnd) {
	    if (mouse_in_window) {
		mouse_in_window = FALSE ;
		// Capture/Release is no more needed...
		//ReleaseCapture() ;
		OnMouseLeave(x,y,flags) ;
		return ;
	    }
	    // We never want to see Enter or Motion in this part of the Window...
	    return ;
	} else {
	    TRACE("Event was triggered while I'm really into my client area.");
	    // Event was triggered while I'm really into my client area.
	    // Do an Enter if not done.
	    if (!mouse_in_window) {
		mouse_in_window = TRUE ;
		// Capture/Release is no more needed...
		//SetCapture(hwndWnd) ;
		// Set cursor, but only if we're not in 'busy' mode
//		if (wx_window->wx_cursor && !wxIsBusy())
//		    ::WinSetPointer(HWND_DESKTOP, wx_window->wx_cursor->os2_cursor);
		OnMouseEnter(x,y,flags) ;
		return ;
	    }
	}
    }
#endif //MOUSE_EXIT_FIX
    
    wxMouseEvent event(wxEVENT_TYPE_MOTION);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_event = wxEVENT_TYPE_MOTION;
    last_x_pos = event.x;
    last_y_pos = event.y;
    if (wx_window)
	wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnMouseEnter(int x, int y, UINT flags)
{
TRACE("wxSubWnd::OnMouseEnter(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
//wxDebugMsg("Client 0x%08x Enter %d,%d\n",this,x,y) ;

    // Set cursor, but only if we're not in 'busy' mode
//    if (wx_window->wx_cursor && !wxIsBusy())
//	::WinSetPointer(HWND_DESKTOP,wx_window->wx_cursor->os2_cursor);

    wxMouseEvent event(wxEVENT_TYPE_ENTER_WINDOW);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_event = wxEVENT_TYPE_ENTER_WINDOW;
    last_x_pos = event.x; last_y_pos = event.y;
    if (wx_window) wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnMouseLeave(int x, int y, UINT flags)
{
    TRACE("wxSubWnd::OnMouseLeave(x=%d, y=%d, flags=%#x)", x, y, (int)flags);
//wxDebugMsg("Client 0x%08x Leave %d,%d\n",this,x,y) ;

    // Set cursor, but only if we're not in 'busy' mode
//    if (wx_window->wx_cursor && !wxIsBusy())
//	::WinSetPointer(HWND_DESKTOP,wx_window->wx_cursor->os2_cursor);

    wxMouseEvent event(wxEVENT_TYPE_LEAVE_WINDOW);
    float px = (float)x;
    float py = (float)y;

    DeviceToLogical(&px, &py);
    CalcUnscrolledPosition((int)px, (int)py, &event.x, &event.y);

    event.shiftDown = (flags & KC_SHIFT);
    event.controlDown = (flags & KC_CTRL);
    event.altDown = (flags & KC_ALT);
    event.leftDown = !!(buttonstate & BUTTON1);
    event.middleDown = !!(buttonstate & BUTTON3);
    event.rightDown = !!(buttonstate & BUTTON2);
    event.SetTimestamp(WinQueryMsgTime(wxhAb));

    last_event = wxEVENT_TYPE_LEAVE_WINDOW;
    last_x_pos = event.x; last_y_pos = event.y;
    if (wx_window) wx_window->GetEventHandler()->OnEvent(event);
}

void wxSubWnd::OnChar(USHORT chvk, Bool isASCII)
{
    TRACE("wxSubWnd::OnChar(chvk=%d, isASCII=%d)", (int)chvk, isASCII);
    int id;
    Bool tempControlDown = FALSE;
    if (isASCII) {
	// If 1 -> 26, translate to CTRL plus a letter.
	if (wxControlDown) {
	    id = chvk;
	    switch (chvk) {
	    case 'M':
		id = WXK_RETURN;
		break;
				
	    case 'B':
		id = WXK_BACK;
		break;
				
	    case 'I':
		id = WXK_TAB;
		break;
				
	    default:
		tempControlDown = TRUE;
	    }
	}
    } else if ((id = wxCharCodeOS2ToWX(chvk)) == 0)
	id = -1;

    if ((id > -1) && wx_window) {
	wxKeyEvent event(wxEVENT_TYPE_CHAR);
	if (wxShiftDown)
	    event.shiftDown = TRUE;
	if (wxControlDown || tempControlDown)
	    event.controlDown = TRUE;
	if (wxAltDown)
	    event.altDown = TRUE;

	event.eventObject = wx_window;
	event.keyCode = id;

	POINTL pt ;
	WinQueryPointerPos(HWND_DESKTOP, &pt) ;
	RECTL rect;
	wx_WindowRect(hwndWnd, &rect);
	pt.x -= rect.xLeft ;
	pt.y -= rect.yTop ;
	float fx,fy ;
	fx = (float)pt.x ;
	fy = (float)pt.y ;
	DeviceToLogical(&fx,&fy) ;
	CalcUnscrolledPosition((int)fx,(int)fy,&event.x,&event.y) ;
		
	wx_window->OnChar(event);
    }
}

void wxSubWnd::OnVScroll(int uscmd, int sslider, int usidentifier)
{
    TRACE("wxSubWnd::OnVScroll(uscmd-%d, sslider=%d, usidentifier=%d)", uscmd, sslider, usidentifier);
    if (usidentifier) {
	wxNode *node = (wxNode *)wxScrollBarList.Find(usidentifier);
	if (!node)
	    return;
	wxWindow * win = (wxWindow *)node->Data();
	if (wxSubType(win->__type, wxTYPE_SLIDER)) {
	    wxSliderEvent(usidentifier, uscmd, sslider);
	    return;
	}
#if USE_SCROLLBAR
	else if (wxSubType(win->__type, wxTYPE_SCROLL_BAR)) {
	    wxScrollBarEvent(usidentifier, uscmd, sslider);
	    return;
	}
#endif
	return;
    }
	
    wxCommandEvent event;
    event.commandInt = sslider;
    event.extraLong = wxVERTICAL;
        
    switch ( uscmd )
    {
#if 0
    case SB_TOP:
	event.eventType = wxEVENT_TYPE_SCROLL_TOP;
	break;

    case SB_BOTTOM:
	event.eventType = yscroll_lines - yscroll_position;
	break;
#endif                        
    case SB_LINEUP:
	event.eventType = wxEVENT_TYPE_SCROLL_LINEUP;
	break;

    case SB_LINEDOWN:
	event.eventType = wxEVENT_TYPE_SCROLL_LINEDOWN;
	break;

    case SB_PAGEUP:
	event.eventType = wxEVENT_TYPE_SCROLL_PAGEUP;
	break;

    case SB_PAGEDOWN:
	event.eventType = wxEVENT_TYPE_SCROLL_PAGEDOWN;
	break;

    case SB_SLIDERTRACK:
	event.eventType = wxEVENT_TYPE_SCROLL_THUMBTRACK;
	break;

    default:
	return;
    }
    if (wx_window)
	wx_window->DoScroll(event);
}

void wxSubWnd::OnHScroll(int uscmd, int sslider, int usidentifier)
{
    TRACE("wxSubWnd::OnHscroll(uscmd=%d, sslider=%d, usidentifier=%d)", uscmd, sslider, usidentifier);
    if (usidentifier) {
	wxNode *node = (wxNode *)wxScrollBarList.Find(usidentifier);
	if (!node)
	    return;
	wxWindow * win = (wxWindow *)node->Data();
	if (wxSubType(win->__type, wxTYPE_SLIDER)) {
	    wxSliderEvent(usidentifier, uscmd, sslider);
	    return;
	}
    } else {
	wxCommandEvent event;
	event.commandInt = sslider;
	event.extraLong = wxHORIZONTAL;
	switch ( uscmd ) {
#if 0
	case SB_TOP:
	    event.eventType = wxEVENT_TYPE_SCROLL_TOP;
	    break;
			
	case SB_BOTTOM:
	    event.eventType = wxEVENT_TYPE_SCROLL_BOTTOM;
	    break;
#endif
	case SB_LINEUP:
	    event.eventType = wxEVENT_TYPE_SCROLL_LINEUP;
	    break;

	case SB_LINEDOWN:
	    event.eventType = wxEVENT_TYPE_SCROLL_LINEDOWN;
	    break;

	case SB_PAGEUP:
	    event.eventType = wxEVENT_TYPE_SCROLL_PAGEUP;
	    break;

	case SB_PAGEDOWN:
	    event.eventType = wxEVENT_TYPE_SCROLL_PAGEDOWN;
	    break;

	case SB_SLIDERTRACK:
	    event.eventType = wxEVENT_TYPE_SCROLL_THUMBTRACK;
	    break;

	default:
	    return;
        }
	if (wx_window)
	    wx_window->DoScroll(event);
    }
}

/*
 * Get character cell height
 * and average width of characters in the font
 */
void wxGetCharSize(HWND wnd, int *x, int *y)
{
    FONTMETRICS fm;

    HPS ps = WinGetPS(wnd);
    GpiQueryFontMetrics(ps, sizeof(FONTMETRICS), &fm);
    WinReleasePS(ps);
    *x = fm.lAveCharWidth;
    *y = fm.lMaxDescender + fm.lMaxAscender;
}

LONG wxGetCharHeight(HWND hwnd)
{
    int cx, cy;
    wxGetCharSize(hwnd, &cx, &cy);
    return cy;
}


LONG wxGetCharWidth(HWND hwnd)
{
    int cx, cy;
    wxGetCharSize(hwnd, &cx, &cy);
    return cx;
}


void wxGetTextExtents(HWND wnd, const char *str, int *cx, int *cy)
{
    POINTL aptl[TXTBOX_COUNT];
    HPS ps = WinGetPS(wnd);
    GpiQueryTextBox(ps, strlen(str), (PCH)str, TXTBOX_COUNT, aptl);
    if (cx)
	*cx = aptl[TXTBOX_TOPRIGHT].x - aptl[TXTBOX_TOPLEFT].x;
    if (cy)
	cy = aptl[TXTBOX_TOPRIGHT].y - aptl[TXTBOX_BOTTOMRIGHT].y;
    WinReleasePS(ps);
}

int wxGetTextWidth(HWND hwnd, const char *str)
{
    int cx;
    if (str == NULL) {
	char buf[300];
	WinQueryWindowText(hwnd, sizeof buf, (PCH)buf);
	wxGetTextExtents(hwnd, buf, &cx, NULL);
	TRACE("wxGetTextWidth(hwnd=%#x) buf='%s' = %d", hwnd, buf, cx);
    } else {
	wxGetTextExtents(hwnd, str, &cx, NULL);
	TRACE("wxGetTextWidth(hwnd=%#x, str='%s') = %d", hwnd, str, cx);
    }
    return cx;
}


// Returns 0 if was a normal ASCII value, not a special key. This indicates that
// the key should be ignored by WM_KEYDOWN and processed by WM_CHAR instead.
int wxCharCodeOS2ToWX(int keySym)
{
    int id = 0;
    switch (keySym) {
    case VK_BREAK:              id = WXK_CANCEL; break;
    case VK_BACKSPACE:          id = WXK_BACK; break;
    case VK_TAB:                id = WXK_TAB; break;
    case VK_CLEAR:              id = WXK_CLEAR; break;
    case VK_NEWLINE:            id = WXK_RETURN; break;
    case VK_SHIFT:              id = WXK_SHIFT; break;
    case VK_CTRL:               id = WXK_CONTROL; break;
/* VK_MENU is defined as VK_F10 in os2.h
   case VK_MENU :              id = WXK_MENU; break;
   */    
    case VK_PAUSE:              id = WXK_PAUSE; break;
    case VK_SPACE:              id = WXK_SPACE; break;
    case VK_ESC:                id = WXK_ESCAPE; break;
    case VK_PAGEUP:             id = WXK_PRIOR; break;
    case VK_PAGEDOWN :          id = WXK_NEXT; break;
    case VK_END:                id = WXK_END; break;
    case VK_HOME :              id = WXK_HOME; break;
    case VK_LEFT :              id = WXK_LEFT; break;
    case VK_UP:                 id = WXK_UP; break;
    case VK_RIGHT:              id = WXK_RIGHT; break;
    case VK_DOWN :              id = WXK_DOWN; break;
#if 0    
    case VK_SELECT:             id = WXK_SELECT; break;
    case VK_PRINT:              id = WXK_PRINT; break;
    case VK_EXECUTE:            id = WXK_EXECUTE; break;
#endif    
    case VK_INSERT:             id = WXK_INSERT; break;
    case VK_DELETE:             id = WXK_DELETE; break;
#if 0    
    case VK_HELP :              id = WXK_HELP; break;
    case VK_NUMPAD0:            id = WXK_NUMPAD0; break;
    case VK_NUMPAD1:            id = WXK_NUMPAD1; break;
    case VK_NUMPAD2:            id = WXK_NUMPAD2; break;
    case VK_NUMPAD3:            id = WXK_NUMPAD3; break;
    case VK_NUMPAD4:            id = WXK_NUMPAD4; break;
    case VK_NUMPAD5:            id = WXK_NUMPAD5; break;
    case VK_NUMPAD6:            id = WXK_NUMPAD6; break;
    case VK_NUMPAD7:            id = WXK_NUMPAD7; break;
    case VK_NUMPAD8:            id = WXK_NUMPAD8; break;
    case VK_NUMPAD9:            id = WXK_NUMPAD9; break;
    case VK_MULTIPLY:           id = WXK_MULTIPLY; break;
    case VK_ADD:                id = WXK_ADD; break;
    case VK_SUBTRACT:           id = WXK_SUBTRACT; break;
    case VK_DECIMAL:            id = WXK_DECIMAL; break;
    case VK_DIVIDE:             id = WXK_DIVIDE; break;
#endif    
    case VK_F1:                 id = WXK_F1; break;
    case VK_F2:                 id = WXK_F2; break;
    case VK_F3:                 id = WXK_F3; break;
    case VK_F4:                 id = WXK_F4; break;
    case VK_F5:                 id = WXK_F5; break;
    case VK_F6:                 id = WXK_F6; break;
    case VK_F7:                 id = WXK_F7; break;
    case VK_F8:                 id = WXK_F8; break;
    case VK_F9:                 id = WXK_F9; break;
    case VK_F10:                id = WXK_F10; break;
    case VK_F11:                id = WXK_F11; break;
    case VK_F12:                id = WXK_F12; break;
    case VK_F13:                id = WXK_F13; break;
    case VK_F14:                id = WXK_F14; break;
    case VK_F15:                id = WXK_F15; break;
    case VK_F16:                id = WXK_F16; break;
    case VK_F17:                id = WXK_F17; break;
    case VK_F18:                id = WXK_F18; break;
    case VK_F19:                id = WXK_F19; break;
    case VK_F20:                id = WXK_F20; break;
    case VK_F21:                id = WXK_F21; break;
    case VK_F22:                id = WXK_F22; break;
    case VK_F23:                id = WXK_F23; break;
    case VK_F24:                id = WXK_F24; break;
    case VK_NUMLOCK:            id = WXK_NUMLOCK; break;
    case VK_SCRLLOCK:           id = WXK_SCROLL; break;
    default:
    {
	return 0;
    }
    }
    return id;
}

int wxCharCodeWXToOS2(int id, Bool *isVirtual)
{
    *isVirtual = TRUE;
    int keySym = 0;
    switch (id) {
    case WXK_CANCEL:            keySym = VK_BREAK; break;
    case WXK_CLEAR:             keySym = VK_CLEAR; break;
    case WXK_SHIFT:             keySym = VK_SHIFT; break;
    case WXK_CONTROL:           keySym = VK_CTRL; break;
    case WXK_MENU :             keySym = VK_MENU; break;
    case WXK_PAUSE:             keySym = VK_PAUSE; break;
    case WXK_PRIOR:             keySym = VK_PAGEUP; break;
    case WXK_NEXT :             keySym = VK_PAGEDOWN; break;
    case WXK_END:               keySym = VK_END; break;
    case WXK_HOME :             keySym = VK_HOME; break;
    case WXK_LEFT :             keySym = VK_LEFT; break;
    case WXK_UP:                keySym = VK_UP; break;
    case WXK_RIGHT:             keySym = VK_RIGHT; break;
    case WXK_DOWN :             keySym = VK_DOWN; break;
#if 0    
    case WXK_SELECT:            keySym = VK_SELECT; break;
    case WXK_PRINT:             keySym = VK_PRINT; break;
    case WXK_EXECUTE:           keySym = VK_EXECUTE; break;
#endif    
    case WXK_INSERT:            keySym = VK_INSERT; break;
    case WXK_DELETE:            keySym = VK_DELETE; break;
#if 0    
    case WXK_HELP :             keySym = VK_HELP; break;
    case WXK_NUMPAD0:           keySym = VK_NUMPAD0; break;
    case WXK_NUMPAD1:           keySym = VK_NUMPAD1; break;
    case WXK_NUMPAD2:           keySym = VK_NUMPAD2; break;
    case WXK_NUMPAD3:           keySym = VK_NUMPAD3; break;
    case WXK_NUMPAD4:           keySym = VK_NUMPAD4; break;
    case WXK_NUMPAD5:           keySym = VK_NUMPAD5; break;
    case WXK_NUMPAD6:           keySym = VK_NUMPAD6; break;
    case WXK_NUMPAD7:           keySym = VK_NUMPAD7; break;
    case WXK_NUMPAD8:           keySym = VK_NUMPAD8; break;
    case WXK_NUMPAD9:           keySym = VK_NUMPAD9; break;
    case WXK_MULTIPLY:          keySym = VK_MULTIPLY; break;
    case WXK_ADD:               keySym = VK_ADD; break;
    case WXK_SUBTRACT:          keySym = VK_SUBTRACT; break;
    case WXK_DECIMAL:           keySym = VK_DECIMAL; break;
    case WXK_DIVIDE:            keySym = VK_DIVIDE; break;
#endif    
    case WXK_F1:                keySym = VK_F1; break;
    case WXK_F2:                keySym = VK_F2; break;
    case WXK_F3:                keySym = VK_F3; break;
    case WXK_F4:                keySym = VK_F4; break;
    case WXK_F5:                keySym = VK_F5; break;
    case WXK_F6:                keySym = VK_F6; break;
    case WXK_F7:                keySym = VK_F7; break;
    case WXK_F8:                keySym = VK_F8; break;
    case WXK_F9:                keySym = VK_F9; break;
    case WXK_F10:               keySym = VK_F10; break;
    case WXK_F11:               keySym = VK_F11; break;
    case WXK_F12:               keySym = VK_F12; break;
    case WXK_F13:               keySym = VK_F13; break;
    case WXK_F14:               keySym = VK_F14; break;
    case WXK_F15:               keySym = VK_F15; break;
    case WXK_F16:               keySym = VK_F16; break;
    case WXK_F17:               keySym = VK_F17; break;
    case WXK_F18:               keySym = VK_F18; break;
    case WXK_F19:               keySym = VK_F19; break;
    case WXK_F20:               keySym = VK_F20; break;
    case WXK_F21:               keySym = VK_F21; break;
    case WXK_F22:               keySym = VK_F22; break;
    case WXK_F23:               keySym = VK_F23; break;
    case WXK_F24:               keySym = VK_F24; break;
    case WXK_NUMLOCK:           keySym = VK_NUMLOCK; break;
    case WXK_SCROLL:            keySym = VK_SCRLLOCK; break;
    default:
    {
	*isVirtual = FALSE;
	keySym = id;
	break;
    }
    }
    return keySym;
}

void
wxSetupPS(HPS hps)
{
    /*
     * Load logical colour table from database
     */
    GpiCreateLogColorTable(hps, LCOL_RESET, LCOLF_CONSECRGB, 16,
			   wxTheColourDatabase->ncols, wxTheColourDatabase->alColor);
    GpiCreateLogColorTable(hps, 0, LCOLF_RGB, 0, 0, NULL);
}


void wxWindow::DoScroll(wxCommandEvent& event)
{
    long orient = event.extraLong;

    int nScrollInc = CalcScrollInc(event);
    if (nScrollInc == 0)
	return;

    wxWnd *wnd = (wxWnd *)handle;
    HWND hWnd = GetHWND();

    if (orient == wxHORIZONTAL) {
	int newPos = wnd->xscroll_position + nScrollInc;
	WinSendMsg(WinWindowFromID(hWnd, FID_HORZSCROLL),
		   SBM_SETPOS, MPFROMSHORT(newPos), (MPARAM)0);
    } else {
	int newPos = wnd->yscroll_position + nScrollInc;
	WinSendMsg(WinWindowFromID(hWnd, FID_VERTSCROLL),
		   SBM_SETPOS, MPFROMSHORT(newPos), (MPARAM)0);
    }

    OnScroll(event);

    if (orient == wxHORIZONTAL) {
	wnd->xscroll_position += nScrollInc;
    } else {
	wnd->yscroll_position += nScrollInc;
    }
}

int wxWindow::CalcScrollInc(wxCommandEvent& event)
{
    int pos = event.commandInt;
    long orient = event.extraLong;

    int nScrollInc = 0;
    wxWnd *wnd = (wxWnd *)handle;

    switch (event.GetEventType())
    {
    case wxEVENT_TYPE_SCROLL_TOP:
    {
	if (orient == wxHORIZONTAL)
	    nScrollInc = - wnd->xscroll_position;
	else
	    nScrollInc = - wnd->yscroll_position;
	break;
    }
    case wxEVENT_TYPE_SCROLL_BOTTOM:
    {
	if (orient == wxHORIZONTAL)
	    nScrollInc = wnd->xscroll_lines - wnd->xscroll_position;
	else
	    nScrollInc = wnd->yscroll_lines - wnd->yscroll_position;
	break;
    }
    case wxEVENT_TYPE_SCROLL_LINEUP:
    {
	nScrollInc = -1;
	break;
    }
    case wxEVENT_TYPE_SCROLL_LINEDOWN:
    {
	nScrollInc = 1;
	break;
    }
    case wxEVENT_TYPE_SCROLL_PAGEUP:
    {
	if (orient == wxHORIZONTAL)
	    nScrollInc = -wnd->xscroll_lines_per_page;
	else
	    nScrollInc = -wnd->yscroll_lines_per_page;
	break;
    }
    case wxEVENT_TYPE_SCROLL_PAGEDOWN:
    {
	if (orient == wxHORIZONTAL)
	    nScrollInc = wnd->xscroll_lines_per_page;
	else
	    nScrollInc = wnd->yscroll_lines_per_page;
	break;
    }
    case wxEVENT_TYPE_SCROLL_THUMBTRACK:
    {
	if (orient == wxHORIZONTAL)
	    nScrollInc = pos - wnd->xscroll_position;
	else
	    nScrollInc = pos - wnd->yscroll_position;
	break;
    }
    default:
    {
	break;
    }
    }
    HWND hWnd = GetHWND();
    if (orient == wxHORIZONTAL) {
	// We're scrolling automatically
	if (wnd->calcScrolledOffset) {
	    int w;
	    RECTL rect;
	    wx_ClientRect(hWnd, &rect);
	    w = rcWidth(rect);
	    int nMaxWidth = wnd->xscroll_lines*wnd->xscroll_pixels_per_line;

	    float noPositions = (float)((nMaxWidth - w)/(float)wnd->xscroll_pixels_per_line);

	    // Deal with situation where we need to scroll a fractional
	    // amount: scroll by 1 instead (ok, so we step slightly
	    // outside the scrollable area but tough.)
	    if (noPositions > 0 && noPositions < 1)
		noPositions = 1;
	    else if (noPositions < 0)
		noPositions = 0;

	    int nHscrollMax = wxMax(0, (int)(0 + noPositions));
	    nScrollInc = wxMax( -wnd->xscroll_position,
			      wxMin( nScrollInc, nHscrollMax - wnd->xscroll_position ));
	    return nScrollInc;
	}
	else
	{
	    // We're not scrolling automatically so we don't care about pixel-per-line
	    int newPosition = wnd->xscroll_position + nScrollInc;
	    if ((newPosition < 0) || (newPosition >= wnd->xscroll_lines))
		return 0;
	    else
		return nScrollInc;
	}
    } else {
	// We're scrolling automatically
	if (wnd->calcScrolledOffset) {
	    RECTL rect;
	    wx_ClientRect(hWnd, &rect);
	    int h = rcHeight(rect);

	    int nMaxHeight = wnd->yscroll_lines*wnd->yscroll_pixels_per_line;

	    float noPositions = (float)((nMaxHeight - h)/(float)wnd->yscroll_pixels_per_line);

	    // Deal with situation where we need to scroll a fractional
	    // amount: scroll by 1 instead (ok, so we step slightly
	    // outside the scrollable area but tough.)
	    if (noPositions > 0 && noPositions < 1)
		noPositions = 1;
	    else if (noPositions < 0)
		noPositions = 0;

	    int nVscrollMax = wxMax(0, (int)(0 + noPositions));

	    nScrollInc = wxMax( -wnd->yscroll_position,
			      wxMin( nScrollInc, nVscrollMax - wnd->yscroll_position ));
	    return nScrollInc;
	}
	else
	{
	    // We're not scrolling automatically so we don't care about pixel-per-line
	    int newPosition = wnd->yscroll_position + nScrollInc;
	    if ((newPosition < 0) || (newPosition >= wnd->yscroll_lines))
		return 0;
	    else
		return nScrollInc;
	}
    }
}

void wxWindow::OnScroll(wxCommandEvent& event)
{
    TRACE("wxWindow::OnScroll(...)");
    long orient = event.extraLong;

    int nScrollInc = CalcScrollInc(event);
    if (nScrollInc == 0)
	return;

    wxWnd *wnd = (wxWnd *)handle;
    HWND hWnd = GetHWND();
    
    if (orient == wxHORIZONTAL) {
	if (wnd->x_scrolling_enabled)
	    WinScrollWindow(hWnd, -wnd->xscroll_pixels_per_line * nScrollInc, 0,
			    (PRECTL)NULL, (PRECTL)NULL, 0, (PRECTL)NULL, 0);
	else {
	    RECTL  rcl;
	    WinQueryWindowRect(hWnd, &rcl);
	    WinInvalidateRect(hWnd, &rcl, FALSE);
	}
    } else {
	if (wnd->y_scrolling_enabled)
	    WinScrollWindow(hWnd, 0, -wnd->yscroll_pixels_per_line * nScrollInc,
			    (PRECTL)NULL, (PRECTL)NULL,
			    0, (PRECTL)NULL, 0);
	else {
	    RECTL  rcl;
	    WinQueryWindowRect(hWnd, &rcl);
	    WinInvalidateRect(hWnd, &rcl, FALSE);
	}
    }
}

void wxWindow::SetScrollPos(int orient, int pos)
{
	HWND hWnd = GetHWND();
	if (hWnd) {
		HWND hwndSb;
		if (orient == wxHORIZONTAL)
			hwndSb = WinWindowFromID(hWnd, FID_HORZSCROLL);
		else
			hwndSb = WinWindowFromID(hWnd, FID_VERTSCROLL);
		WinSendMsg(hwndSb, SBM_SETPOS, MPFROMSHORT(pos), (MPARAM)0);
	}
	wxWnd *wnd = (wxWnd *)handle;
	if (orient == wxHORIZONTAL)
		wnd->xscroll_position = pos;
	else
		wnd->yscroll_position = pos;
}

void wxWindow::SetScrollRange(int orient, int range)
{
	HWND hwnd = GetHWND();
	wxWnd *wnd = (wxWnd *)handle;
	if (orient == wxHORIZONTAL) {
		WinSendMsg(WinWindowFromID(hwnd, FID_HORZSCROLL),
			   SBM_SETSCROLLBAR, 
			   (MPARAM)wnd->xscroll_position, MPFROM2SHORT(0, range));
		wnd->xscroll_lines = range;
	} else {
		WinSendMsg(WinWindowFromID(hwnd, FID_VERTSCROLL),
			   SBM_SETSCROLLBAR, 
			   (MPARAM)wnd->yscroll_position, MPFROM2SHORT(0, range));
		wnd->yscroll_lines = range;
	}
}

void wxWindow::SetScrollPage(int orient, int page)
{
  wxWnd *wnd = (wxWnd *)handle;
  if (orient == wxHORIZONTAL)
    wnd->xscroll_lines_per_page = page;
  else
    wnd->yscroll_lines_per_page = page;
}

int wxWindow::GetScrollPos(int orient)
{
	HWND hWnd = GetHWND();
	if (hWnd) {
		if (orient == wxHORIZONTAL)
			return (int)WinSendMsg(WinWindowFromID(hWnd, FID_HORZSCROLL),
					       SBM_QUERYPOS, (MPARAM)0, (MPARAM)0);
		else
			return (int)WinSendMsg(WinWindowFromID(hWnd, FID_VERTSCROLL),
					       SBM_QUERYPOS, (MPARAM)0, (MPARAM)0);
	}
	return 0;
}

int wxWindow::GetScrollRange(int orient)
{
	HWND hWnd = GetHWND();
	if (hWnd) {
		if (orient == wxHORIZONTAL)
			return (int)WinSendMsg(WinWindowFromID(hWnd, FID_HORZSCROLL),
					       SBM_QUERYRANGE, (MPARAM)0, (MPARAM)0) >> 16;
		else
			return (int)WinSendMsg(WinWindowFromID(hWnd, FID_VERTSCROLL),
					       SBM_QUERYRANGE, (MPARAM)0, (MPARAM)0) >> 16;
	}
	return 0;
}

int wxWindow::GetScrollPage(int orient)
{
  wxWnd *wnd = (wxWnd *)handle;
  if (orient == wxHORIZONTAL)
    return wnd->xscroll_lines_per_page;
  else
    return wnd->yscroll_lines_per_page;
}

// Default OnSize resets scrollbars, if any
void wxWindow::OnSize(int w, int h)
{
    TRACE("wxWindow::OnSize(w=%d, h=%d)", w, h);
//    assert(!inOnSize);
    inOnSize = TRUE;
#if USE_CONSTRAINTS
    TRACE("autoLayout=%d", GetAutoLayout());
    if (GetAutoLayout())
	Layout();
#endif
    Layout();
  
    /*
     * Canvas is the only thing with automatic scroll bars -- so far
     * If this changes, we have to change this test.
     * Note that the scroll bar variables are in each wxWnd class.
     * If there are only scrollbars in canvases, then that should not be.
     */
    if (__type == wxTYPE_CANVAS) {
	wxWnd *wnd = (wxWnd *)handle;
	HWND hWnd = GetHWND();

	// Recalculate scroll bar range and position
	if (wnd->xscroll_lines > 0) {
	    int nMaxWidth = wnd->xscroll_lines*wnd->xscroll_pixels_per_line;
	    float noPositions = (float)((nMaxWidth - w)/(float)wnd->xscroll_pixels_per_line);
	    if (noPositions > 0 && noPositions < 1)
		noPositions = 1;
	    else if (noPositions < 0)
		noPositions = 0;
      
	    int nHscrollMax = wxMax(0, (int)(0 + noPositions));
	    wnd->xscroll_position = wxMin(nHscrollMax, wnd->xscroll_position);

#if 1
	    WinSendMsg(WinWindowFromID(hWnd, FID_HORZSCROLL),
		       SBM_SETSCROLLBAR, 
		       (MPARAM)wnd->xscroll_position, MPFROM2SHORT(0, nHscrollMax));
#else
	    ::SetScrollRange(hWnd, SB_HORZ, 0, nHscrollMax, FALSE);
	    ::SetScrollPos(hWnd, SB_HORZ, wnd->xscroll_position, TRUE);
#endif
//    wxDebugMsg("w = %d\n", w);
//    wxDebugMsg("nMaxWidth = xscroll_lines (%d) * xscrollpixels_per_line (%d) = %d\n",
//      xscroll_lines, xscroll_pixels_per_line, nMaxWidth);
//    wxDebugMsg("nHscrollMax = %d, xscroll_position = %d\n", nHscrollMax, xscroll_position);
	}	

	if (wnd->yscroll_lines > 0) {
	    int nMaxHeight = wnd->yscroll_lines*wnd->yscroll_pixels_per_line;
	    float noPositions = (float)((nMaxHeight - h)/(float)wnd->yscroll_pixels_per_line);
	    if (noPositions > 0 && noPositions < 1)
		noPositions = 1;
	    else if (noPositions < 0)
		noPositions = 0;

	    int nVscrollMax = wxMax(0, (int)(0 + noPositions));
	    wnd->yscroll_position = wxMin(nVscrollMax, wnd->yscroll_position);

#if 1
	    WinSendMsg(WinWindowFromID(hWnd, FID_VERTSCROLL),
		       SBM_SETSCROLLBAR, 
		       (MPARAM)wnd->yscroll_position, MPFROM2SHORT(0, nVscrollMax));
#else
	    ::SetScrollRange(hWnd, SB_VERT, 0, nVscrollMax, FALSE);
	    ::SetScrollPos(hWnd, SB_VERT, wnd->yscroll_position, TRUE);
#endif
	}
    }

    /*
     * Since OS/2 windows are upside down, we have to reposition all the children windows
     */
    wxNode *node;
    for (node = children->First(); node != NULL; node = node->Next()) {
	wxWindow *child;
	if ((child = (wxWindow *)node->Data()) != NULL) {
	    wx_MoveWindow(child->GetHWND(), child->wxx, child->wxy, child->wxw, child->wxh);
	}
    }
    inOnSize = FALSE;
}


// Caret manipulation
void wxWindow::CreateCaret(int w, int h)
{
  caretWidth = w;
  caretHeight = h;
  caretEnabled = TRUE;
}

void wxWindow::CreateCaret(wxBitmap *bitmap)
{
    // XXX
  // Not implemented
}

void wxWindow::ShowCaret(Bool show)
{
  if (caretEnabled)
  {
#if XXX
    if (show)
      ::ShowCaret(GetHWND());
    else
      ::HideCaret(GetHWND());
#endif
    caretShown = show;
  }
}

void wxWindow::DestroyCaret(void)
{
  caretEnabled = FALSE;
}

void wxWindow::SetCaretPos(int x, int y)
{
#if XXX
  ::SetCaretPos(x, y);
#endif
}

void wxWindow::GetCaretPos(int *x, int *y)
{
#if XXX
  POINTL point;
  ::GetCaretPos(&point);
  *x = point.x;
  *y = point.y;
#endif
}

/*
 * Update iterator. Use from within OnPaint.
 */
 
wxUpdateIterator::wxUpdateIterator(wxWindow* winp)
{
    current = 0;					//start somewhere...
    wxWnd *wnd = (wxWnd *)winp->handle;
    if (wnd) {
	rects = 1;
//	rp = *wnd->rectlUpdate;
    }
}

wxUpdateIterator::~wxUpdateIterator(void)
{
}

wxUpdateIterator::operator int (void)
{
    if (current < rects)
	return TRUE;
    else
	return FALSE;
}

wxUpdateIterator* wxUpdateIterator::operator ++(int)
{
    current++;
    return this;
}

void wxUpdateIterator::GetRect(wxRectangle *rect)
{
    rect->x = rectlUpdate.xLeft;
    rect->y = rectlUpdate.yTop;
    rect->width = rcWidth(rectlUpdate);
    rect->height = rcHeight(rectlUpdate);
}

int wxUpdateIterator::GetX()
{
    return rectlUpdate.xLeft;
}

int wxUpdateIterator::GetY()
{
    return rectlUpdate.yTop;
}

int wxUpdateIterator::GetW()
{
    return rectlUpdate.xRight-GetX();
}

int wxUpdateIterator::GetH()
{
    return rectlUpdate.yBottom-GetY();
}

wxWindow *wxGetActiveWindow(void)
{
    HWND hWnd = WinQueryActiveWindow(HWND_DESKTOP);
    if (hWnd != 0) {
	wxWnd *wnd = (wxWnd *)WinQueryWindowPtr(hWnd, QWL_USER);
	if (wnd && wnd->wx_window)
	    return wnd->wx_window;
    }
    return NULL;
}

#if USE_KEYBOARD_HOOKXXX
// Windows keyboard hook. Allows interception of e.g. F1, ESCAPE
// in active frames and dialogs, regardless of where the focus is.
static HHOOK wxTheKeyboardHook = 0;
static (*)() wxTheKeyboardHookProc = 0;
int
  wxKeyboardHook(int nCode, MPARAM mp1, MPARAM mp2);

void wxSetKeyboardHook(Bool doIt)
{
    if (doIt) {
	wxTheKeyboardHookProc = MakeProcInstance(wxKeyboardHook, wxhAb);
	wxTheKeyboardHook = SetWindowsHookEx(WH_KEYBOARD, wxTheKeyboardHookProc, wxhAb, GetCurrentTask());
    } else {
	UnhookWindowsHookEx(wxTheKeyboardHook);
	FreeProcInstance(wxTheKeyboardHookProc);
    }
}

int wxKeyboardHook(int nCode, MPARAM mp1, MPARAM mp2)
{
    DWORD hiWord = HIWORD(mp2);
    if (nCode != HC_NOREMOVE && ((hiWord & KF_UP) == 0)) {
	int id;
	if ((id = wxCharCodeMSWToWX(mp1)) != 0) {
	    wxKeyEvent event(wxEVENT_TYPE_CHAR);
	    if ((HIWORD(mp2) & KF_ALTDOWN) == KF_ALTDOWN)
		event.altDown = TRUE;
          
	    event.eventObject = NULL;
	    event.keyCode = id;

	    if (wxTheApp && wxTheApp->OnCharHook(event))
		return 1;
	}
    }
    return (int)CallNextHookEx(wxTheKeyboardHook, nCode, mp1, mp2);
}

#endif

/*
 * Help function to move window
 */
void wxWindow::MoveWindow(int x, int y, int w, int h, Bool show)
{
    TRACE("wxWindow::MoveWindow(x=%d, y=%d, w=%d, h=%d, show=%d)", x, y, w, h, (int)show);
    if (x != -1)
	wxx = x;
    if (y != -1)
	wxy = y;
    if (w != -1)
	wxw = w;
    if (h != -1)
	wxh = h;
    wx_MoveWindow(GetHWND(), wxx, wxy, wxw, wxh, show);
}

void wxWindow::SetBackgroundColour(const wxColour& col)
{
    TRACE("wxWindow::SetBackgroundColour(...)");
    wxWnd *wnd = (wxWnd *)handle;
    if (wnd)
	wnd->SetBackgroundColour(col);
}


void wxWindow::OnBackgroundColourChange(const wxColour& col)
{
    TRACE("wxWindow::OnBackgroundColourChange(...)");
    SetBackgroundColour(col);
}

void wxWindow::OnFontChange(const wxFont& fnt)
{
}

